<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs" version="3.0"
    xmlns:oxy="http://www.oxygenxml.com/ns">
    <xsl:output indent="yes" method="xml"/>
    <xsl:template match="node() | @*">
        <xsl:copy>
            <xsl:apply-templates select="node() | @*"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="glossgroup">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="title"/>
            <xsl:apply-templates select="glossentry">
                <xsl:sort select="oxy:toPinyin(string-join(glossterm//text()))"
                    case-order="lower-first"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>
    <!-- Convert Chinese characters to pinyin -->
    <xsl:function name="oxy:toPinyin">
        <xsl:param name="text"/>
        <xsl:variable name="map" as="map(xs:string, xs:string)">
            <xsl:map>
                <xsl:map-entry key="'&#x3007;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x4E00;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x4E01;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x4E02;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x4E03;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x4E04;'" select="'shang4	shang3'"/>
                <xsl:map-entry key="'&#x4E05;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x4E06;'" select="''"/>
                <xsl:map-entry key="'&#x4E07;'" select="'wan4	mo4'"/>
                <xsl:map-entry key="'&#x4E08;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x4E09;'" select="'san1'"/>
                <xsl:map-entry key="'&#x4E0A;'" select="'shang4	shang3'"/>
                <xsl:map-entry key="'&#x4E0B;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x4E0C;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x4E0D;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x4E0E;'" select="'yu3	yu4'"/>
                <xsl:map-entry key="'&#x4E0F;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x4E10;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x4E11;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x4E12;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x4E13;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x4E14;'" select="'qie3	ju1'"/>
                <xsl:map-entry key="'&#x4E15;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x4E16;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x4E17;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x4E18;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x4E19;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x4E1A;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x4E1B;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x4E1C;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x4E1D;'" select="'si1'"/>
                <xsl:map-entry key="'&#x4E1E;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x4E1F;'" select="'diu1'"/>
                <xsl:map-entry key="'&#x4E20;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x4E21;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x4E22;'" select="'diu1'"/>
                <xsl:map-entry key="'&#x4E23;'" select="'you3'"/>
                <xsl:map-entry key="'&#x4E24;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x4E25;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x4E26;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x4E27;'" select="'sang1	sang4'"/>
                <xsl:map-entry key="'&#x4E28;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x4E29;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x4E2A;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x4E2B;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x4E2C;'" select="'qiang2	pan2'"/>
                <xsl:map-entry key="'&#x4E2D;'" select="'zhong1	zhong4'"/>
                <xsl:map-entry key="'&#x4E2E;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x4E2F;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x4E30;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x4E31;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x4E32;'" select="'chuan4'"/>
                <xsl:map-entry key="'&#x4E33;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x4E34;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x4E35;'" select="'zhuo1'"/>
                <xsl:map-entry key="'&#x4E36;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x4E37;'" select="''"/>
                <xsl:map-entry key="'&#x4E38;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x4E39;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x4E3A;'" select="'wei4	wei2'"/>
                <xsl:map-entry key="'&#x4E3B;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x4E3C;'" select="'jing3	dan3'"/>
                <xsl:map-entry key="'&#x4E3D;'" select="'li4	li2'"/>
                <xsl:map-entry key="'&#x4E3E;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x4E3F;'" select="'pie3'"/>
                <xsl:map-entry key="'&#x4E40;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x4E41;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x4E42;'" select="'yi4	ai4'"/>
                <xsl:map-entry key="'&#x4E43;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x4E44;'" select="''"/>
                <xsl:map-entry key="'&#x4E45;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x4E46;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x4E47;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x4E48;'" select="'yao1	mo2'"/>
                <xsl:map-entry key="'&#x4E49;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x4E4A;'" select="''"/>
                <xsl:map-entry key="'&#x4E4B;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x4E4C;'" select="'wu1	wu4'"/>
                <xsl:map-entry key="'&#x4E4D;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x4E4E;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x4E4F;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x4E50;'" select="'le4	yue4'"/>
                <xsl:map-entry key="'&#x4E51;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x4E52;'" select="'ping1'"/>
                <xsl:map-entry key="'&#x4E53;'" select="'pang1'"/>
                <xsl:map-entry key="'&#x4E54;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x4E55;'" select="'hu3	hu4'"/>
                <xsl:map-entry key="'&#x4E56;'" select="'guai1'"/>
                <xsl:map-entry key="'&#x4E57;'" select="'cheng2	sheng4'"/>
                <xsl:map-entry key="'&#x4E58;'" select="'cheng2	sheng4'"/>
                <xsl:map-entry key="'&#x4E59;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x4E5A;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x4E5B;'" select="''"/>
                <xsl:map-entry key="'&#x4E5C;'" select="'mie1	nie4'"/>
                <xsl:map-entry key="'&#x4E5D;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x4E5E;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x4E5F;'" select="'ye3'"/>
                <xsl:map-entry key="'&#x4E60;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x4E61;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x4E62;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x4E63;'" select="'diu1'"/>
                <xsl:map-entry key="'&#x4E64;'" select="''"/>
                <xsl:map-entry key="'&#x4E65;'" select="''"/>
                <xsl:map-entry key="'&#x4E66;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x4E67;'" select="''"/>
                <xsl:map-entry key="'&#x4E68;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x4E69;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x4E6A;'" select="'nang1'"/>
                <xsl:map-entry key="'&#x4E6B;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x4E6C;'" select="''"/>
                <xsl:map-entry key="'&#x4E6D;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x4E6E;'" select="''"/>
                <xsl:map-entry key="'&#x4E6F;'" select="''"/>
                <xsl:map-entry key="'&#x4E70;'" select="'mai3'"/>
                <xsl:map-entry key="'&#x4E71;'" select="'luan4'"/>
                <xsl:map-entry key="'&#x4E72;'" select="''"/>
                <xsl:map-entry key="'&#x4E73;'" select="'ru3'"/>
                <xsl:map-entry key="'&#x4E74;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x4E75;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x4E76;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x4E77;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x4E78;'" select="'na3'"/>
                <xsl:map-entry key="'&#x4E79;'" select="'gan1	qian2'"/>
                <xsl:map-entry key="'&#x4E7A;'" select="''"/>
                <xsl:map-entry key="'&#x4E7B;'" select="''"/>
                <xsl:map-entry key="'&#x4E7C;'" select="''"/>
                <xsl:map-entry key="'&#x4E7D;'" select="''"/>
                <xsl:map-entry key="'&#x4E7E;'" select="'gan1	qian2'"/>
                <xsl:map-entry key="'&#x4E7F;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x4E80;'" select="'gui1	jun1	qiu1'"/>
                <xsl:map-entry key="'&#x4E81;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x4E82;'" select="'luan4'"/>
                <xsl:map-entry key="'&#x4E83;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x4E84;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x4E85;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x4E86;'" select="'liao3	le5'"/>
                <xsl:map-entry key="'&#x4E87;'" select="''"/>
                <xsl:map-entry key="'&#x4E88;'" select="'yu2	yu3'"/>
                <xsl:map-entry key="'&#x4E89;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x4E8A;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x4E8B;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x4E8C;'" select="'er4'"/>
                <xsl:map-entry key="'&#x4E8D;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x4E8E;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x4E8F;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x4E90;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x4E91;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x4E92;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x4E93;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x4E94;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x4E95;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x4E96;'" select="'si4'"/>
                <xsl:map-entry key="'&#x4E97;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x4E98;'" select="'gen4'"/>
                <xsl:map-entry key="'&#x4E99;'" select="'gen4	geng4'"/>
                <xsl:map-entry key="'&#x4E9A;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x4E9B;'" select="'xie1'"/>
                <xsl:map-entry key="'&#x4E9C;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x4E9D;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x4E9E;'" select="'ya4	ya3'"/>
                <xsl:map-entry key="'&#x4E9F;'" select="'ji2	qi4'"/>
                <xsl:map-entry key="'&#x4EA0;'" select="'tou2'"/>
                <xsl:map-entry key="'&#x4EA1;'" select="'wang2	wu2'"/>
                <xsl:map-entry key="'&#x4EA2;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x4EA3;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x4EA4;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x4EA5;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x4EA6;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x4EA7;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x4EA8;'" select="'heng1'"/>
                <xsl:map-entry key="'&#x4EA9;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x4EAA;'" select="''"/>
                <xsl:map-entry key="'&#x4EAB;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x4EAC;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x4EAD;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x4EAE;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x4EAF;'" select="'heng1'"/>
                <xsl:map-entry key="'&#x4EB0;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x4EB1;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x4EB2;'" select="'qin1	qing4'"/>
                <xsl:map-entry key="'&#x4EB3;'" select="'bo4'"/>
                <xsl:map-entry key="'&#x4EB4;'" select="'you4'"/>
                <xsl:map-entry key="'&#x4EB5;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x4EB6;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x4EB7;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x4EB8;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x4EB9;'" select="'wei3	wei4'"/>
                <xsl:map-entry key="'&#x4EBA;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x4EBB;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x4EBC;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x4EBD;'" select="''"/>
                <xsl:map-entry key="'&#x4EBE;'" select="'wang2'"/>
                <xsl:map-entry key="'&#x4EBF;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x4EC0;'" select="'shi2	shen2'"/>
                <xsl:map-entry key="'&#x4EC1;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x4EC2;'" select="'le4'"/>
                <xsl:map-entry key="'&#x4EC3;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x4EC4;'" select="'ze4'"/>
                <xsl:map-entry key="'&#x4EC5;'" select="'jin3	jin4'"/>
                <xsl:map-entry key="'&#x4EC6;'" select="'pu1	pu2'"/>
                <xsl:map-entry key="'&#x4EC7;'" select="'chou2	qiu2'"/>
                <xsl:map-entry key="'&#x4EC8;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x4EC9;'" select="'zhang3'"/>
                <xsl:map-entry key="'&#x4ECA;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x4ECB;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x4ECC;'" select="'bing1'"/>
                <xsl:map-entry key="'&#x4ECD;'" select="'reng2'"/>
                <xsl:map-entry key="'&#x4ECE;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x4ECF;'" select="'fo2	fu2'"/>
                <xsl:map-entry key="'&#x4ED0;'" select="'san3'"/>
                <xsl:map-entry key="'&#x4ED1;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x4ED2;'" select="''"/>
                <xsl:map-entry key="'&#x4ED3;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x4ED4;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x4ED5;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x4ED6;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x4ED7;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x4ED8;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x4ED9;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x4EDA;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x4EDB;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x4EDC;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x4EDD;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x4EDE;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x4EDF;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x4EE0;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x4EE1;'" select="'yi4	ge1'"/>
                <xsl:map-entry key="'&#x4EE2;'" select="'di2'"/>
                <xsl:map-entry key="'&#x4EE3;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x4EE4;'" select="'ling4	ling2	ling3'"/>
                <xsl:map-entry key="'&#x4EE5;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x4EE6;'" select="'chao4'"/>
                <xsl:map-entry key="'&#x4EE7;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x4EE8;'" select="'sa1'"/>
                <xsl:map-entry key="'&#x4EE9;'" select="'shang4'"/>
                <xsl:map-entry key="'&#x4EEA;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x4EEB;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x4EEC;'" select="'men5	men2'"/>
                <xsl:map-entry key="'&#x4EED;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x4EEE;'" select="'jia3	jia4'"/>
                <xsl:map-entry key="'&#x4EEF;'" select="'chao4'"/>
                <xsl:map-entry key="'&#x4EF0;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x4EF1;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x4EF2;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x4EF3;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x4EF4;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x4EF5;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x4EF6;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x4EF7;'" select="'jia4'"/>
                <xsl:map-entry key="'&#x4EF8;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x4EF9;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x4EFA;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x4EFB;'" select="'ren4	ren2'"/>
                <xsl:map-entry key="'&#x4EFC;'" select="'wang2'"/>
                <xsl:map-entry key="'&#x4EFD;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x4EFE;'" select="'di1'"/>
                <xsl:map-entry key="'&#x4EFF;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x4F00;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x4F01;'" select="'qi3	qi4'"/>
                <xsl:map-entry key="'&#x4F02;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x4F03;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x4F04;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x4F05;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x4F06;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x4F07;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x4F08;'" select="'xin3'"/>
                <xsl:map-entry key="'&#x4F09;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x4F0A;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x4F0B;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x4F0C;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x4F0D;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x4F0E;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x4F0F;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x4F10;'" select="'fa2	fa1'"/>
                <xsl:map-entry key="'&#x4F11;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x4F12;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x4F13;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x4F14;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x4F15;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x4F16;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x4F17;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x4F18;'" select="'you1'"/>
                <xsl:map-entry key="'&#x4F19;'" select="'huo3'"/>
                <xsl:map-entry key="'&#x4F1A;'" select="'hui4	kuai4'"/>
                <xsl:map-entry key="'&#x4F1B;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x4F1C;'" select="'cui4	zu2'"/>
                <xsl:map-entry key="'&#x4F1D;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x4F1E;'" select="'san3'"/>
                <xsl:map-entry key="'&#x4F1F;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x4F20;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x4F21;'" select="'che1'"/>
                <xsl:map-entry key="'&#x4F22;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x4F23;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x4F24;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x4F25;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x4F26;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x4F27;'" select="'cang1	chen5'"/>
                <xsl:map-entry key="'&#x4F28;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x4F29;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x4F2A;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x4F2B;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x4F2C;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x4F2D;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x4F2E;'" select="'nao2	nu3'"/>
                <xsl:map-entry key="'&#x4F2F;'" select="'bo2	bai3'"/>
                <xsl:map-entry key="'&#x4F30;'" select="'gu1	gu4'"/>
                <xsl:map-entry key="'&#x4F31;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x4F32;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x4F33;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x4F34;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x4F35;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x4F36;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x4F37;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x4F38;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x4F39;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x4F3A;'" select="'si4	ci4'"/>
                <xsl:map-entry key="'&#x4F3B;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x4F3C;'" select="'si4	shi4'"/>
                <xsl:map-entry key="'&#x4F3D;'" select="'jia1	qie2'"/>
                <xsl:map-entry key="'&#x4F3E;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x4F3F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x4F40;'" select="'si4	shi4'"/>
                <xsl:map-entry key="'&#x4F41;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x4F42;'" select="'zheng1	zheng4'"/>
                <xsl:map-entry key="'&#x4F43;'" select="'dian4	tian2'"/>
                <xsl:map-entry key="'&#x4F44;'" select="'han2'"/>
                <xsl:map-entry key="'&#x4F45;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x4F46;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x4F47;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x4F48;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x4F49;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x4F4A;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x4F4B;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x4F4C;'" select="'ci3'"/>
                <xsl:map-entry key="'&#x4F4D;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x4F4E;'" select="'di1'"/>
                <xsl:map-entry key="'&#x4F4F;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x4F50;'" select="'zuo3'"/>
                <xsl:map-entry key="'&#x4F51;'" select="'you4'"/>
                <xsl:map-entry key="'&#x4F52;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x4F53;'" select="'ti3'"/>
                <xsl:map-entry key="'&#x4F54;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x4F55;'" select="'he2	he4'"/>
                <xsl:map-entry key="'&#x4F56;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x4F57;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x4F58;'" select="'she2'"/>
                <xsl:map-entry key="'&#x4F59;'" select="'yu2	tu2'"/>
                <xsl:map-entry key="'&#x4F5A;'" select="'yi4	die2'"/>
                <xsl:map-entry key="'&#x4F5B;'" select="'fo2	fu2'"/>
                <xsl:map-entry key="'&#x4F5C;'" select="'zuo1'"/>
                <xsl:map-entry key="'&#x4F5D;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x4F5E;'" select="'ning4'"/>
                <xsl:map-entry key="'&#x4F5F;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x4F60;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x4F61;'" select="'xuan1	san3'"/>
                <xsl:map-entry key="'&#x4F62;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x4F63;'" select="'yong4	yong1'"/>
                <xsl:map-entry key="'&#x4F64;'" select="'wa3'"/>
                <xsl:map-entry key="'&#x4F65;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x4F66;'" select="''"/>
                <xsl:map-entry key="'&#x4F67;'" select="'ka3'"/>
                <xsl:map-entry key="'&#x4F68;'" select="''"/>
                <xsl:map-entry key="'&#x4F69;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x4F6A;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x4F6B;'" select="'he4'"/>
                <xsl:map-entry key="'&#x4F6C;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x4F6D;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x4F6E;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x4F6F;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x4F70;'" select="'bai3'"/>
                <xsl:map-entry key="'&#x4F71;'" select="'fa3'"/>
                <xsl:map-entry key="'&#x4F72;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x4F73;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x4F74;'" select="'er4	nai4'"/>
                <xsl:map-entry key="'&#x4F75;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x4F76;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x4F77;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x4F78;'" select="'huo2'"/>
                <xsl:map-entry key="'&#x4F79;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x4F7A;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x4F7B;'" select="'tiao1	tiao2'"/>
                <xsl:map-entry key="'&#x4F7C;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x4F7D;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x4F7E;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x4F7F;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x4F80;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x4F81;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x4F82;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x4F83;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x4F84;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x4F85;'" select="'gai1	kai1'"/>
                <xsl:map-entry key="'&#x4F86;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x4F87;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x4F88;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x4F89;'" select="'kua1	kua3'"/>
                <xsl:map-entry key="'&#x4F8A;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x4F8B;'" select="'li4'"/>
                <xsl:map-entry key="'&#x4F8C;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x4F8D;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x4F8E;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x4F8F;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x4F90;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x4F91;'" select="'you4'"/>
                <xsl:map-entry key="'&#x4F92;'" select="'an1'"/>
                <xsl:map-entry key="'&#x4F93;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x4F94;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x4F95;'" select="'er2'"/>
                <xsl:map-entry key="'&#x4F96;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x4F97;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x4F98;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x4F99;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x4F9A;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x4F9B;'" select="'gong1	gong4'"/>
                <xsl:map-entry key="'&#x4F9C;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x4F9D;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x4F9E;'" select="'ru3'"/>
                <xsl:map-entry key="'&#x4F9F;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x4FA0;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x4FA1;'" select="'jia4	jie4'"/>
                <xsl:map-entry key="'&#x4FA2;'" select="'zai4'"/>
                <xsl:map-entry key="'&#x4FA3;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x4FA4;'" select="''"/>
                <xsl:map-entry key="'&#x4FA5;'" select="'jiao3	yao2'"/>
                <xsl:map-entry key="'&#x4FA6;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x4FA7;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x4FA8;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x4FA9;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x4FAA;'" select="'chai2'"/>
                <xsl:map-entry key="'&#x4FAB;'" select="'ning4'"/>
                <xsl:map-entry key="'&#x4FAC;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x4FAD;'" select="'jin3	jin4'"/>
                <xsl:map-entry key="'&#x4FAE;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x4FAF;'" select="'hou2	hou4'"/>
                <xsl:map-entry key="'&#x4FB0;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x4FB1;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x4FB2;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x4FB3;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x4FB4;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x4FB5;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x4FB6;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x4FB7;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x4FB8;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x4FB9;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x4FBA;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x4FBB;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x4FBC;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x4FBD;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x4FBE;'" select="'hao1'"/>
                <xsl:map-entry key="'&#x4FBF;'" select="'bian4	pian2'"/>
                <xsl:map-entry key="'&#x4FC0;'" select="'tui3'"/>
                <xsl:map-entry key="'&#x4FC1;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x4FC2;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x4FC3;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x4FC4;'" select="'e2	e4'"/>
                <xsl:map-entry key="'&#x4FC5;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x4FC6;'" select="'xu2'"/>
                <xsl:map-entry key="'&#x4FC7;'" select="'kuang3'"/>
                <xsl:map-entry key="'&#x4FC8;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x4FC9;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x4FCA;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x4FCB;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x4FCC;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x4FCD;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x4FCE;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x4FCF;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x4FD0;'" select="'li4'"/>
                <xsl:map-entry key="'&#x4FD1;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x4FD2;'" select="'hun4'"/>
                <xsl:map-entry key="'&#x4FD3;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x4FD4;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x4FD5;'" select="'san4'"/>
                <xsl:map-entry key="'&#x4FD6;'" select="'pai3'"/>
                <xsl:map-entry key="'&#x4FD7;'" select="'su2'"/>
                <xsl:map-entry key="'&#x4FD8;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x4FD9;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x4FDA;'" select="'li3'"/>
                <xsl:map-entry key="'&#x4FDB;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x4FDC;'" select="'ping1	ping2'"/>
                <xsl:map-entry key="'&#x4FDD;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x4FDE;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x4FDF;'" select="'si4	qi2'"/>
                <xsl:map-entry key="'&#x4FE0;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x4FE1;'" select="'xin4	shen1'"/>
                <xsl:map-entry key="'&#x4FE2;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x4FE3;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x4FE4;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x4FE5;'" select="'che1'"/>
                <xsl:map-entry key="'&#x4FE6;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x4FE7;'" select="''"/>
                <xsl:map-entry key="'&#x4FE8;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x4FE9;'" select="'lia3	liang3'"/>
                <xsl:map-entry key="'&#x4FEA;'" select="'li4'"/>
                <xsl:map-entry key="'&#x4FEB;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x4FEC;'" select="'si1'"/>
                <xsl:map-entry key="'&#x4FED;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x4FEE;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x4FEF;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x4FF0;'" select="'he2'"/>
                <xsl:map-entry key="'&#x4FF1;'" select="'ju4	ju1'"/>
                <xsl:map-entry key="'&#x4FF2;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x4FF3;'" select="'pai2'"/>
                <xsl:map-entry key="'&#x4FF4;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x4FF5;'" select="'biao3'"/>
                <xsl:map-entry key="'&#x4FF6;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x4FF7;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x4FF8;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x4FF9;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x4FFA;'" select="'an3'"/>
                <xsl:map-entry key="'&#x4FFB;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x4FFC;'" select="'yu4	zhou1'"/>
                <xsl:map-entry key="'&#x4FFD;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x4FFE;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x4FFF;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x5000;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x5001;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x5002;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x5003;'" select="'zan2'"/>
                <xsl:map-entry key="'&#x5004;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5005;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x5006;'" select="'lia3	liang3'"/>
                <xsl:map-entry key="'&#x5007;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x5008;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x5009;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x500A;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x500B;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x500C;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x500D;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x500E;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x500F;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x5010;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x5011;'" select="'men5	men2'"/>
                <xsl:map-entry key="'&#x5012;'" select="'dao3	dao4'"/>
                <xsl:map-entry key="'&#x5013;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x5014;'" select="'jue2	jue4'"/>
                <xsl:map-entry key="'&#x5015;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x5016;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x5017;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x5018;'" select="'tang3	chang2'"/>
                <xsl:map-entry key="'&#x5019;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x501A;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x501B;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x501C;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x501D;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x501E;'" select="'jing4	liang4'"/>
                <xsl:map-entry key="'&#x501F;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x5020;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x5021;'" select="'chang4	chang1'"/>
                <xsl:map-entry key="'&#x5022;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5023;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x5024;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x5025;'" select="'kong1	kong3'"/>
                <xsl:map-entry key="'&#x5026;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x5027;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x5028;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5029;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x502A;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x502B;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x502C;'" select="'zhuo1	zhuo2'"/>
                <xsl:map-entry key="'&#x502D;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x502E;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x502F;'" select="'song1'"/>
                <xsl:map-entry key="'&#x5030;'" select="'leng2'"/>
                <xsl:map-entry key="'&#x5031;'" select="'hun4'"/>
                <xsl:map-entry key="'&#x5032;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x5033;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x5034;'" select="'ben4'"/>
                <xsl:map-entry key="'&#x5035;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x5036;'" select="'ju4	ju1'"/>
                <xsl:map-entry key="'&#x5037;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x5038;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x5039;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x503A;'" select="'zhai4'"/>
                <xsl:map-entry key="'&#x503B;'" select="'ye1'"/>
                <xsl:map-entry key="'&#x503C;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x503D;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x503E;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x503F;'" select="''"/>
                <xsl:map-entry key="'&#x5040;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x5041;'" select="'cheng1	cheng4'"/>
                <xsl:map-entry key="'&#x5042;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x5043;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5044;'" select="'nuan4'"/>
                <xsl:map-entry key="'&#x5045;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x5046;'" select="'chun3'"/>
                <xsl:map-entry key="'&#x5047;'" select="'jia3	jia4'"/>
                <xsl:map-entry key="'&#x5048;'" select="'jie2	ji4'"/>
                <xsl:map-entry key="'&#x5049;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x504A;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x504B;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x504C;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x504D;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x504E;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x504F;'" select="'pian1'"/>
                <xsl:map-entry key="'&#x5050;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5051;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x5052;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x5053;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x5054;'" select="'e4'"/>
                <xsl:map-entry key="'&#x5055;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x5056;'" select="'che3'"/>
                <xsl:map-entry key="'&#x5057;'" select="'sheng3'"/>
                <xsl:map-entry key="'&#x5058;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x5059;'" select="'di4'"/>
                <xsl:map-entry key="'&#x505A;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x505B;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x505C;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x505D;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x505E;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x505F;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x5060;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x5061;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x5062;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x5063;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x5064;'" select="'you2'"/>
                <xsl:map-entry key="'&#x5065;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x5066;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x5067;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x5068;'" select="'chai1'"/>
                <xsl:map-entry key="'&#x5069;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x506A;'" select="'bi1'"/>
                <xsl:map-entry key="'&#x506B;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x506C;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x506D;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x506E;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x506F;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x5070;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5071;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x5072;'" select="'si1'"/>
                <xsl:map-entry key="'&#x5073;'" select="'duan1'"/>
                <xsl:map-entry key="'&#x5074;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x5075;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x5076;'" select="'ou3'"/>
                <xsl:map-entry key="'&#x5077;'" select="'tou1'"/>
                <xsl:map-entry key="'&#x5078;'" select="'tou1'"/>
                <xsl:map-entry key="'&#x5079;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x507A;'" select="'za2	zan2'"/>
                <xsl:map-entry key="'&#x507B;'" select="'lu:3	lou2'"/>
                <xsl:map-entry key="'&#x507C;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x507D;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x507E;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x507F;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x5080;'" select="'gui1	kui3'"/>
                <xsl:map-entry key="'&#x5081;'" select="'sou3'"/>
                <xsl:map-entry key="'&#x5082;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x5083;'" select="'su4'"/>
                <xsl:map-entry key="'&#x5084;'" select="'xia1'"/>
                <xsl:map-entry key="'&#x5085;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5086;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x5087;'" select="'rong3'"/>
                <xsl:map-entry key="'&#x5088;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5089;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x508A;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x508B;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x508C;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x508D;'" select="'bang4	bang1'"/>
                <xsl:map-entry key="'&#x508E;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x508F;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x5090;'" select="'hao1'"/>
                <xsl:map-entry key="'&#x5091;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5092;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5093;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x5094;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x5095;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5096;'" select="'cang1	chen5'"/>
                <xsl:map-entry key="'&#x5097;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x5098;'" select="'san3'"/>
                <xsl:map-entry key="'&#x5099;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x509A;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x509B;'" select="'yong2'"/>
                <xsl:map-entry key="'&#x509C;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x509D;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x509E;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x509F;'" select="'wang1'"/>
                <xsl:map-entry key="'&#x50A0;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x50A1;'" select="'bing4	bing1'"/>
                <xsl:map-entry key="'&#x50A2;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x50A3;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x50A4;'" select="'zai4'"/>
                <xsl:map-entry key="'&#x50A5;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x50A6;'" select="''"/>
                <xsl:map-entry key="'&#x50A7;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x50A8;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x50A9;'" select="'nuo2'"/>
                <xsl:map-entry key="'&#x50AA;'" select="'zan1'"/>
                <xsl:map-entry key="'&#x50AB;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x50AC;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x50AD;'" select="'yong1	yong4'"/>
                <xsl:map-entry key="'&#x50AE;'" select="'zao1'"/>
                <xsl:map-entry key="'&#x50AF;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x50B0;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x50B1;'" select="'song3'"/>
                <xsl:map-entry key="'&#x50B2;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x50B3;'" select="'chuan2	zhuan4'"/>
                <xsl:map-entry key="'&#x50B4;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x50B5;'" select="'zhai4'"/>
                <xsl:map-entry key="'&#x50B6;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x50B7;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x50B8;'" select="'qiang3'"/>
                <xsl:map-entry key="'&#x50B9;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x50BA;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x50BB;'" select="'sha3'"/>
                <xsl:map-entry key="'&#x50BC;'" select="'han4'"/>
                <xsl:map-entry key="'&#x50BD;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x50BE;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x50BF;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x50C0;'" select="'di4'"/>
                <xsl:map-entry key="'&#x50C1;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x50C2;'" select="'lu:3	lou2'"/>
                <xsl:map-entry key="'&#x50C3;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x50C4;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x50C5;'" select="'jin3	jin4'"/>
                <xsl:map-entry key="'&#x50C6;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x50C7;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x50C8;'" select="'man4'"/>
                <xsl:map-entry key="'&#x50C9;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x50CA;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x50CB;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x50CC;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x50CD;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x50CE;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x50CF;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x50D0;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x50D1;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x50D2;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x50D3;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x50D4;'" select="'zun3'"/>
                <xsl:map-entry key="'&#x50D5;'" select="'pu2	pu1'"/>
                <xsl:map-entry key="'&#x50D6;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x50D7;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x50D8;'" select="'chang3'"/>
                <xsl:map-entry key="'&#x50D9;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x50DA;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x50DB;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x50DC;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x50DD;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x50DE;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x50DF;'" select="'zhang3'"/>
                <xsl:map-entry key="'&#x50E0;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x50E1;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x50E2;'" select="'chuan3'"/>
                <xsl:map-entry key="'&#x50E3;'" select="'tie3'"/>
                <xsl:map-entry key="'&#x50E4;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x50E5;'" select="'jiao3	yao2'"/>
                <xsl:map-entry key="'&#x50E6;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x50E7;'" select="'seng1'"/>
                <xsl:map-entry key="'&#x50E8;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x50E9;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x50EA;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x50EB;'" select="'e4'"/>
                <xsl:map-entry key="'&#x50EC;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x50ED;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x50EE;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x50EF;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x50F0;'" select="'bo2	fu2'"/>
                <xsl:map-entry key="'&#x50F1;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x50F2;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x50F3;'" select="'su4'"/>
                <xsl:map-entry key="'&#x50F4;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x50F5;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x50F6;'" select="'min3'"/>
                <xsl:map-entry key="'&#x50F7;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x50F8;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x50F9;'" select="'jia4'"/>
                <xsl:map-entry key="'&#x50FA;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x50FB;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x50FC;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x50FD;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x50FE;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x50FF;'" select="'sai4'"/>
                <xsl:map-entry key="'&#x5100;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5101;'" select="'jun4	juan4'"/>
                <xsl:map-entry key="'&#x5102;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x5103;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x5104;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5105;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x5106;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x5107;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x5108;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x5109;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x510A;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x510B;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x510C;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x510D;'" select="'sha3'"/>
                <xsl:map-entry key="'&#x510E;'" select="'zai4	zai3'"/>
                <xsl:map-entry key="'&#x510F;'" select="''"/>
                <xsl:map-entry key="'&#x5110;'" select="'bin4	bin1'"/>
                <xsl:map-entry key="'&#x5111;'" select="'an4'"/>
                <xsl:map-entry key="'&#x5112;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x5113;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x5114;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x5115;'" select="'chai2'"/>
                <xsl:map-entry key="'&#x5116;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x5117;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x5118;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x5119;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x511A;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x511B;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x511C;'" select="'neng2'"/>
                <xsl:map-entry key="'&#x511D;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x511E;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x511F;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x5120;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x5121;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x5122;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x5123;'" select="'kuang3'"/>
                <xsl:map-entry key="'&#x5124;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x5125;'" select="'du2'"/>
                <xsl:map-entry key="'&#x5126;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x5127;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x5128;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x5129;'" select="'si4'"/>
                <xsl:map-entry key="'&#x512A;'" select="'you1'"/>
                <xsl:map-entry key="'&#x512B;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x512C;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x512D;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x512E;'" select="'li4'"/>
                <xsl:map-entry key="'&#x512F;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x5130;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x5131;'" select="'long2'"/>
                <xsl:map-entry key="'&#x5132;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x5133;'" select="'chan4'"/>
                <xsl:map-entry key="'&#x5134;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x5135;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x5136;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5137;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5138;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x5139;'" select="'zan3	zuan3'"/>
                <xsl:map-entry key="'&#x513A;'" select="'nuo2'"/>
                <xsl:map-entry key="'&#x513B;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x513C;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x513D;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x513E;'" select="'nang4'"/>
                <xsl:map-entry key="'&#x513F;'" select="'er2	ren2'"/>
                <xsl:map-entry key="'&#x5140;'" select="'wu4	wu1'"/>
                <xsl:map-entry key="'&#x5141;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x5142;'" select="'zan1'"/>
                <xsl:map-entry key="'&#x5143;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x5144;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x5145;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x5146;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x5147;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x5148;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x5149;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x514A;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x514B;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x514C;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x514D;'" select="'mian3	wen4'"/>
                <xsl:map-entry key="'&#x514E;'" select="'tu4'"/>
                <xsl:map-entry key="'&#x514F;'" select="'chang2	zhang3'"/>
                <xsl:map-entry key="'&#x5150;'" select="'er2'"/>
                <xsl:map-entry key="'&#x5151;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x5152;'" select="'er2	er1'"/>
                <xsl:map-entry key="'&#x5153;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x5154;'" select="'tu4'"/>
                <xsl:map-entry key="'&#x5155;'" select="'si4'"/>
                <xsl:map-entry key="'&#x5156;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5157;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5158;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x5159;'" select="'shi2ke4'"/>
                <xsl:map-entry key="'&#x515A;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x515B;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x515C;'" select="'dou1'"/>
                <xsl:map-entry key="'&#x515D;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x515E;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x515F;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x5160;'" select="'dou1'"/>
                <xsl:map-entry key="'&#x5161;'" select="'bai3ke4'"/>
                <xsl:map-entry key="'&#x5162;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x5163;'" select="'li3'"/>
                <xsl:map-entry key="'&#x5164;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x5165;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x5166;'" select="'wang2	wu2'"/>
                <xsl:map-entry key="'&#x5167;'" select="'nei4'"/>
                <xsl:map-entry key="'&#x5168;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x5169;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x516A;'" select="'yu2	shu4'"/>
                <xsl:map-entry key="'&#x516B;'" select="'ba1	ba2'"/>
                <xsl:map-entry key="'&#x516C;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x516D;'" select="'liu4	lu4'"/>
                <xsl:map-entry key="'&#x516E;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x516F;'" select="''"/>
                <xsl:map-entry key="'&#x5170;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x5171;'" select="'gong4'"/>
                <xsl:map-entry key="'&#x5172;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x5173;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x5174;'" select="'xing1	xing4'"/>
                <xsl:map-entry key="'&#x5175;'" select="'bing1'"/>
                <xsl:map-entry key="'&#x5176;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x5177;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5178;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x5179;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x517A;'" select="''"/>
                <xsl:map-entry key="'&#x517B;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x517C;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x517D;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x517E;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x517F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5180;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5181;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x5182;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x5183;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x5184;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x5185;'" select="'nei4'"/>
                <xsl:map-entry key="'&#x5186;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x5187;'" select="'mao3	mou3'"/>
                <xsl:map-entry key="'&#x5188;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x5189;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x518A;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x518B;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x518C;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x518D;'" select="'zai4'"/>
                <xsl:map-entry key="'&#x518E;'" select="'gua3'"/>
                <xsl:map-entry key="'&#x518F;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x5190;'" select="'mao4	mo4'"/>
                <xsl:map-entry key="'&#x5191;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x5192;'" select="'mao4	mo4'"/>
                <xsl:map-entry key="'&#x5193;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x5194;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x5195;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x5196;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5197;'" select="'rong3'"/>
                <xsl:map-entry key="'&#x5198;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5199;'" select="'xie3'"/>
                <xsl:map-entry key="'&#x519A;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x519B;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x519C;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x519D;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x519E;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x519F;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x51A0;'" select="'guan1	guan4'"/>
                <xsl:map-entry key="'&#x51A1;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x51A2;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x51A3;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x51A4;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x51A5;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x51A6;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x51A7;'" select="''"/>
                <xsl:map-entry key="'&#x51A8;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x51A9;'" select="'xie3'"/>
                <xsl:map-entry key="'&#x51AA;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x51AB;'" select="'bing1'"/>
                <xsl:map-entry key="'&#x51AC;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x51AD;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x51AE;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x51AF;'" select="'feng2	ping2'"/>
                <xsl:map-entry key="'&#x51B0;'" select="'bing1'"/>
                <xsl:map-entry key="'&#x51B1;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x51B2;'" select="'chong1	chong4'"/>
                <xsl:map-entry key="'&#x51B3;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x51B4;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x51B5;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x51B6;'" select="'ye3'"/>
                <xsl:map-entry key="'&#x51B7;'" select="'leng3'"/>
                <xsl:map-entry key="'&#x51B8;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x51B9;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x51BA;'" select="'min3'"/>
                <xsl:map-entry key="'&#x51BB;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x51BC;'" select="'sheng3'"/>
                <xsl:map-entry key="'&#x51BD;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x51BE;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x51BF;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x51C0;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x51C1;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x51C2;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x51C3;'" select="'shang4'"/>
                <xsl:map-entry key="'&#x51C4;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x51C5;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x51C6;'" select="'zhun3'"/>
                <xsl:map-entry key="'&#x51C7;'" select="'song1'"/>
                <xsl:map-entry key="'&#x51C8;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x51C9;'" select="'liang2	liang4'"/>
                <xsl:map-entry key="'&#x51CA;'" select="'qing4	jing4'"/>
                <xsl:map-entry key="'&#x51CB;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x51CC;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x51CD;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x51CE;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x51CF;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x51D0;'" select="'yin1	yin2'"/>
                <xsl:map-entry key="'&#x51D1;'" select="'cou4'"/>
                <xsl:map-entry key="'&#x51D2;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x51D3;'" select="'li4'"/>
                <xsl:map-entry key="'&#x51D4;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x51D5;'" select="'ming3'"/>
                <xsl:map-entry key="'&#x51D6;'" select="'zhun3'"/>
                <xsl:map-entry key="'&#x51D7;'" select="'cui2'"/>
                <xsl:map-entry key="'&#x51D8;'" select="'si1'"/>
                <xsl:map-entry key="'&#x51D9;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x51DA;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x51DB;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x51DC;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x51DD;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x51DE;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x51DF;'" select="'du2'"/>
                <xsl:map-entry key="'&#x51E0;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x51E1;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x51E2;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x51E3;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x51E4;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x51E5;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x51E6;'" select="'chu3	chu4'"/>
                <xsl:map-entry key="'&#x51E7;'" select="''"/>
                <xsl:map-entry key="'&#x51E8;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x51E9;'" select="''"/>
                <xsl:map-entry key="'&#x51EA;'" select="''"/>
                <xsl:map-entry key="'&#x51EB;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x51EC;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x51ED;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x51EE;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x51EF;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x51F0;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x51F1;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x51F2;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x51F3;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x51F4;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x51F5;'" select="'qu1	kan3'"/>
                <xsl:map-entry key="'&#x51F6;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x51F7;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x51F8;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x51F9;'" select="'ao1	wa1'"/>
                <xsl:map-entry key="'&#x51FA;'" select="'chu1'"/>
                <xsl:map-entry key="'&#x51FB;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x51FC;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x51FD;'" select="'han2'"/>
                <xsl:map-entry key="'&#x51FE;'" select="'han2'"/>
                <xsl:map-entry key="'&#x51FF;'" select="'zao2	zuo4'"/>
                <xsl:map-entry key="'&#x5200;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x5201;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x5202;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x5203;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x5204;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x5205;'" select="'chuang1'"/>
                <xsl:map-entry key="'&#x5206;'" select="'fen1	fen4'"/>
                <xsl:map-entry key="'&#x5207;'" select="'qie1	qie4'"/>
                <xsl:map-entry key="'&#x5208;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5209;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x520A;'" select="'kan1'"/>
                <xsl:map-entry key="'&#x520B;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x520C;'" select="'cun3'"/>
                <xsl:map-entry key="'&#x520D;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x520E;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x520F;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x5210;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x5211;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x5212;'" select="'hua2	hua4'"/>
                <xsl:map-entry key="'&#x5213;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x5214;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5215;'" select="'li2'"/>
                <xsl:map-entry key="'&#x5216;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x5217;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x5218;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x5219;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x521A;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x521B;'" select="'chuang4	chuang1'"/>
                <xsl:map-entry key="'&#x521C;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x521D;'" select="'chu1'"/>
                <xsl:map-entry key="'&#x521E;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x521F;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x5220;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x5221;'" select="'min3'"/>
                <xsl:map-entry key="'&#x5222;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x5223;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x5224;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x5225;'" select="'bie2'"/>
                <xsl:map-entry key="'&#x5226;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5227;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5228;'" select="'bao4	pao2'"/>
                <xsl:map-entry key="'&#x5229;'" select="'li4'"/>
                <xsl:map-entry key="'&#x522A;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x522B;'" select="'bie2	bie4'"/>
                <xsl:map-entry key="'&#x522C;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x522D;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x522E;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x522F;'" select="'gen1'"/>
                <xsl:map-entry key="'&#x5230;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x5231;'" select="'chuang4'"/>
                <xsl:map-entry key="'&#x5232;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x5233;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x5234;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x5235;'" select="'er4'"/>
                <xsl:map-entry key="'&#x5236;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5237;'" select="'shua1	shua4'"/>
                <xsl:map-entry key="'&#x5238;'" select="'quan4	xuan4'"/>
                <xsl:map-entry key="'&#x5239;'" select="'cha4	sha1'"/>
                <xsl:map-entry key="'&#x523A;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x523B;'" select="'ke4	ke1'"/>
                <xsl:map-entry key="'&#x523C;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x523D;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x523E;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x523F;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x5240;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x5241;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x5242;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5243;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x5244;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x5245;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x5246;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x5247;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x5248;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x5249;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x524A;'" select="'xue1	xiao1'"/>
                <xsl:map-entry key="'&#x524B;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x524C;'" select="'la4	la2'"/>
                <xsl:map-entry key="'&#x524D;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x524E;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x524F;'" select="'chuan4'"/>
                <xsl:map-entry key="'&#x5250;'" select="'gua3'"/>
                <xsl:map-entry key="'&#x5251;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x5252;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x5253;'" select="'li2'"/>
                <xsl:map-entry key="'&#x5254;'" select="'ti1'"/>
                <xsl:map-entry key="'&#x5255;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x5256;'" select="'pou1'"/>
                <xsl:map-entry key="'&#x5257;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x5258;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x5259;'" select="'chuang4'"/>
                <xsl:map-entry key="'&#x525A;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x525B;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x525C;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x525D;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x525E;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x525F;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x5260;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x5261;'" select="'yan3	shan4'"/>
                <xsl:map-entry key="'&#x5262;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x5263;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x5264;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5265;'" select="'bo1	bao1'"/>
                <xsl:map-entry key="'&#x5266;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x5267;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5268;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x5269;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x526A;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x526B;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x526C;'" select="'duan1'"/>
                <xsl:map-entry key="'&#x526D;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x526E;'" select="'gua3'"/>
                <xsl:map-entry key="'&#x526F;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5270;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x5271;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x5272;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x5273;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x5274;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x5275;'" select="'chuang4	chuang1'"/>
                <xsl:map-entry key="'&#x5276;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x5277;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x5278;'" select="'tuan2	zhuan1'"/>
                <xsl:map-entry key="'&#x5279;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x527A;'" select="'li2'"/>
                <xsl:map-entry key="'&#x527B;'" select="'fou2'"/>
                <xsl:map-entry key="'&#x527C;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x527D;'" select="'piao4	piao1'"/>
                <xsl:map-entry key="'&#x527E;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x527F;'" select="'jiao3	chao1'"/>
                <xsl:map-entry key="'&#x5280;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x5281;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x5282;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5283;'" select="'hua4	hua2'"/>
                <xsl:map-entry key="'&#x5284;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x5285;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x5286;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5287;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5288;'" select="'pi1	pi3'"/>
                <xsl:map-entry key="'&#x5289;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x528A;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x528B;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x528C;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x528D;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x528E;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x528F;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x5290;'" select="'huo1'"/>
                <xsl:map-entry key="'&#x5291;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5292;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x5293;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5294;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x5295;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5296;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x5297;'" select="'cuan2'"/>
                <xsl:map-entry key="'&#x5298;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x5299;'" select="'li2'"/>
                <xsl:map-entry key="'&#x529A;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x529B;'" select="'li4'"/>
                <xsl:map-entry key="'&#x529C;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x529D;'" select="'quan4'"/>
                <xsl:map-entry key="'&#x529E;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x529F;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x52A0;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x52A1;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x52A2;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x52A3;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x52A4;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x52A5;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x52A6;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x52A7;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x52A8;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x52A9;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x52AA;'" select="'nu3	nao2'"/>
                <xsl:map-entry key="'&#x52AB;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x52AC;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x52AD;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x52AE;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x52AF;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x52B0;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x52B1;'" select="'li4'"/>
                <xsl:map-entry key="'&#x52B2;'" select="'jing4	jin4'"/>
                <xsl:map-entry key="'&#x52B3;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x52B4;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x52B5;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x52B6;'" select="'kou3'"/>
                <xsl:map-entry key="'&#x52B7;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x52B8;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x52B9;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x52BA;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x52BB;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x52BC;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x52BD;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x52BE;'" select="'he2'"/>
                <xsl:map-entry key="'&#x52BF;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x52C0;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x52C1;'" select="'jing4	jin4'"/>
                <xsl:map-entry key="'&#x52C2;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x52C3;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x52C4;'" select="'min3'"/>
                <xsl:map-entry key="'&#x52C5;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x52C6;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x52C7;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x52C8;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x52C9;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x52CA;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x52CB;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x52CC;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x52CD;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x52CE;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x52CF;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x52D0;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x52D1;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x52D2;'" select="'le4	lei1'"/>
                <xsl:map-entry key="'&#x52D3;'" select="'kai4'"/>
                <xsl:map-entry key="'&#x52D4;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x52D5;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x52D6;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x52D7;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x52D8;'" select="'kan1	kan4'"/>
                <xsl:map-entry key="'&#x52D9;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x52DA;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x52DB;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x52DC;'" select="'weng3'"/>
                <xsl:map-entry key="'&#x52DD;'" select="'sheng4	sheng1'"/>
                <xsl:map-entry key="'&#x52DE;'" select="'lao2	lao4'"/>
                <xsl:map-entry key="'&#x52DF;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x52E0;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x52E1;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x52E2;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x52E3;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x52E4;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x52E5;'" select="'qiang3'"/>
                <xsl:map-entry key="'&#x52E6;'" select="'jiao3	chao1'"/>
                <xsl:map-entry key="'&#x52E7;'" select="'quan4'"/>
                <xsl:map-entry key="'&#x52E8;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x52E9;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x52EA;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x52EB;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x52EC;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x52ED;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x52EE;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x52EF;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x52F0;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x52F1;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x52F2;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x52F3;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x52F4;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x52F5;'" select="'li4'"/>
                <xsl:map-entry key="'&#x52F6;'" select="'che4'"/>
                <xsl:map-entry key="'&#x52F7;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x52F8;'" select="'quan4'"/>
                <xsl:map-entry key="'&#x52F9;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x52FA;'" select="'shao2'"/>
                <xsl:map-entry key="'&#x52FB;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x52FC;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x52FD;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x52FE;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x52FF;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5300;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x5301;'" select="''"/>
                <xsl:map-entry key="'&#x5302;'" select="''"/>
                <xsl:map-entry key="'&#x5303;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x5304;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x5305;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x5306;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x5307;'" select="''"/>
                <xsl:map-entry key="'&#x5308;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x5309;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x530A;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x530B;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x530C;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x530D;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x530E;'" select="'an4'"/>
                <xsl:map-entry key="'&#x530F;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x5310;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5311;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x5312;'" select="'da2'"/>
                <xsl:map-entry key="'&#x5313;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x5314;'" select="'qiong1'"/>
                <xsl:map-entry key="'&#x5315;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x5316;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x5317;'" select="'bei3'"/>
                <xsl:map-entry key="'&#x5318;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x5319;'" select="'chi2	shi5'"/>
                <xsl:map-entry key="'&#x531A;'" select="'fang1	xi3'"/>
                <xsl:map-entry key="'&#x531B;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x531C;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x531D;'" select="'za1'"/>
                <xsl:map-entry key="'&#x531E;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x531F;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x5320;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x5321;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x5322;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5323;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x5324;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x5325;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x5326;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x5327;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x5328;'" select="'cang2	zang4'"/>
                <xsl:map-entry key="'&#x5329;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x532A;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x532B;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x532C;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x532D;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x532E;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x532F;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5330;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x5331;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x5332;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5333;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5334;'" select="'suan3'"/>
                <xsl:map-entry key="'&#x5335;'" select="'du2'"/>
                <xsl:map-entry key="'&#x5336;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x5337;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x5338;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x5339;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x533A;'" select="'qu1	ou1'"/>
                <xsl:map-entry key="'&#x533B;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x533C;'" select="'an4'"/>
                <xsl:map-entry key="'&#x533D;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x533E;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x533F;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x5340;'" select="'qu1	ou1'"/>
                <xsl:map-entry key="'&#x5341;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x5342;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x5343;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x5344;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x5345;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x5346;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x5347;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x5348;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x5349;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x534A;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x534B;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x534C;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x534D;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x534E;'" select="'hua2	hua4'"/>
                <xsl:map-entry key="'&#x534F;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x5350;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x5351;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x5352;'" select="'zu2	cu4'"/>
                <xsl:map-entry key="'&#x5353;'" select="'zhuo1	zhuo2'"/>
                <xsl:map-entry key="'&#x5354;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x5355;'" select="'dan1	chan2'"/>
                <xsl:map-entry key="'&#x5356;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x5357;'" select="'nan2	na1'"/>
                <xsl:map-entry key="'&#x5358;'" select="'dan1	chan2'"/>
                <xsl:map-entry key="'&#x5359;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x535A;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x535B;'" select="'shuai4	lu:4'"/>
                <xsl:map-entry key="'&#x535C;'" select="'bu3	bo5'"/>
                <xsl:map-entry key="'&#x535D;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x535E;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x535F;'" select="'bu3'"/>
                <xsl:map-entry key="'&#x5360;'" select="'zhan1	zhan4'"/>
                <xsl:map-entry key="'&#x5361;'" select="'qia3	ka3'"/>
                <xsl:map-entry key="'&#x5362;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x5363;'" select="'you3	you4'"/>
                <xsl:map-entry key="'&#x5364;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x5365;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5366;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x5367;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x5368;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5369;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x536A;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x536B;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x536C;'" select="'ang2	yang3'"/>
                <xsl:map-entry key="'&#x536D;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x536E;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x536F;'" select="'mao3'"/>
                <xsl:map-entry key="'&#x5370;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x5371;'" select="'wei1	wei2'"/>
                <xsl:map-entry key="'&#x5372;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x5373;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5374;'" select="'que4'"/>
                <xsl:map-entry key="'&#x5375;'" select="'luan3'"/>
                <xsl:map-entry key="'&#x5376;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x5377;'" select="'juan4	juan3'"/>
                <xsl:map-entry key="'&#x5378;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5379;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x537A;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x537B;'" select="'que4'"/>
                <xsl:map-entry key="'&#x537C;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x537D;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x537E;'" select="'e4'"/>
                <xsl:map-entry key="'&#x537F;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x5380;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5381;'" select="''"/>
                <xsl:map-entry key="'&#x5382;'" select="'chang3'"/>
                <xsl:map-entry key="'&#x5383;'" select="'han3'"/>
                <xsl:map-entry key="'&#x5384;'" select="'e4'"/>
                <xsl:map-entry key="'&#x5385;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x5386;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5387;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x5388;'" select="'an1	chang3'"/>
                <xsl:map-entry key="'&#x5389;'" select="'li4'"/>
                <xsl:map-entry key="'&#x538A;'" select="'ya3'"/>
                <xsl:map-entry key="'&#x538B;'" select="'ya1	ya4'"/>
                <xsl:map-entry key="'&#x538C;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x538D;'" select="'she4'"/>
                <xsl:map-entry key="'&#x538E;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x538F;'" select="'zha3'"/>
                <xsl:map-entry key="'&#x5390;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x5391;'" select="''"/>
                <xsl:map-entry key="'&#x5392;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x5393;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x5394;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5395;'" select="'ce4	si5'"/>
                <xsl:map-entry key="'&#x5396;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x5397;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x5398;'" select="'li2'"/>
                <xsl:map-entry key="'&#x5399;'" select="'she4'"/>
                <xsl:map-entry key="'&#x539A;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x539B;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x539C;'" select="'zui1'"/>
                <xsl:map-entry key="'&#x539D;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x539E;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x539F;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x53A0;'" select="'ce4	si5'"/>
                <xsl:map-entry key="'&#x53A1;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x53A2;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x53A3;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x53A4;'" select="'li4'"/>
                <xsl:map-entry key="'&#x53A5;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x53A6;'" select="'sha4	xia4'"/>
                <xsl:map-entry key="'&#x53A7;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x53A8;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x53A9;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x53AA;'" select="'qin2	jin3'"/>
                <xsl:map-entry key="'&#x53AB;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x53AC;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x53AD;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x53AE;'" select="'si1'"/>
                <xsl:map-entry key="'&#x53AF;'" select="'li4'"/>
                <xsl:map-entry key="'&#x53B0;'" select="'chang3	an1'"/>
                <xsl:map-entry key="'&#x53B1;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x53B2;'" select="'li4'"/>
                <xsl:map-entry key="'&#x53B3;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x53B4;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x53B5;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x53B6;'" select="'si1'"/>
                <xsl:map-entry key="'&#x53B7;'" select="'si1'"/>
                <xsl:map-entry key="'&#x53B8;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x53B9;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x53BA;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x53BB;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x53BC;'" select="''"/>
                <xsl:map-entry key="'&#x53BD;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x53BE;'" select="'du1'"/>
                <xsl:map-entry key="'&#x53BF;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x53C0;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x53C1;'" select="'san1'"/>
                <xsl:map-entry key="'&#x53C2;'" select="'can1	cen1	shen1'"/>
                <xsl:map-entry key="'&#x53C3;'" select="'can1	cen1	shen1'"/>
                <xsl:map-entry key="'&#x53C4;'" select="'san1'"/>
                <xsl:map-entry key="'&#x53C5;'" select="'can1	cen1	shen1'"/>
                <xsl:map-entry key="'&#x53C6;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x53C7;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x53C8;'" select="'you4'"/>
                <xsl:map-entry key="'&#x53C9;'" select="'cha1	cha2	cha3	cha4'"/>
                <xsl:map-entry key="'&#x53CA;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x53CB;'" select="'you3'"/>
                <xsl:map-entry key="'&#x53CC;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x53CD;'" select="'fan3'"/>
                <xsl:map-entry key="'&#x53CE;'" select="'shou1'"/>
                <xsl:map-entry key="'&#x53CF;'" select="'guai4'"/>
                <xsl:map-entry key="'&#x53D0;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x53D1;'" select="'fa1	fa4'"/>
                <xsl:map-entry key="'&#x53D2;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x53D3;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x53D4;'" select="'shu1	shu2'"/>
                <xsl:map-entry key="'&#x53D5;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x53D6;'" select="'qu3'"/>
                <xsl:map-entry key="'&#x53D7;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x53D8;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x53D9;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x53DA;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x53DB;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x53DC;'" select="'sou3'"/>
                <xsl:map-entry key="'&#x53DD;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x53DE;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x53DF;'" select="'sou3'"/>
                <xsl:map-entry key="'&#x53E0;'" select="'die2'"/>
                <xsl:map-entry key="'&#x53E1;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x53E2;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x53E3;'" select="'kou3'"/>
                <xsl:map-entry key="'&#x53E4;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x53E5;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x53E6;'" select="'ling4'"/>
                <xsl:map-entry key="'&#x53E7;'" select="'gua3'"/>
                <xsl:map-entry key="'&#x53E8;'" select="'tao1	dao1	dao2'"/>
                <xsl:map-entry key="'&#x53E9;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x53EA;'" select="'zhi3	zhi1'"/>
                <xsl:map-entry key="'&#x53EB;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x53EC;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x53ED;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x53EE;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x53EF;'" select="'ke3	ke4'"/>
                <xsl:map-entry key="'&#x53F0;'" select="'tai2	tai1'"/>
                <xsl:map-entry key="'&#x53F1;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x53F2;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x53F3;'" select="'you4'"/>
                <xsl:map-entry key="'&#x53F4;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x53F5;'" select="'po3'"/>
                <xsl:map-entry key="'&#x53F6;'" select="'ye4	xie2'"/>
                <xsl:map-entry key="'&#x53F7;'" select="'hao4	hao2'"/>
                <xsl:map-entry key="'&#x53F8;'" select="'si1'"/>
                <xsl:map-entry key="'&#x53F9;'" select="'tan4'"/>
                <xsl:map-entry key="'&#x53FA;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x53FB;'" select="'le4'"/>
                <xsl:map-entry key="'&#x53FC;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x53FD;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x53FE;'" select="''"/>
                <xsl:map-entry key="'&#x53FF;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x5400;'" select="'mie1'"/>
                <xsl:map-entry key="'&#x5401;'" select="'xu1	yu4'"/>
                <xsl:map-entry key="'&#x5402;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x5403;'" select="'chi1	ji2'"/>
                <xsl:map-entry key="'&#x5404;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x5405;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x5406;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x5407;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x5408;'" select="'he2'"/>
                <xsl:map-entry key="'&#x5409;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x540A;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x540B;'" select="'cun4'"/>
                <xsl:map-entry key="'&#x540C;'" select="'tong2	tong4'"/>
                <xsl:map-entry key="'&#x540D;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x540E;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x540F;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5410;'" select="'tu3	tu4'"/>
                <xsl:map-entry key="'&#x5411;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x5412;'" select="'zha4	zha1'"/>
                <xsl:map-entry key="'&#x5413;'" select="'xia4	he4'"/>
                <xsl:map-entry key="'&#x5414;'" select="'ye3'"/>
                <xsl:map-entry key="'&#x5415;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x5416;'" select="'a1'"/>
                <xsl:map-entry key="'&#x5417;'" select="'ma5	ma3'"/>
                <xsl:map-entry key="'&#x5418;'" select="'ou3'"/>
                <xsl:map-entry key="'&#x5419;'" select="'xue1'"/>
                <xsl:map-entry key="'&#x541A;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x541B;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x541C;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x541D;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x541E;'" select="'tun1'"/>
                <xsl:map-entry key="'&#x541F;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5420;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x5421;'" select="'bi3	pi3'"/>
                <xsl:map-entry key="'&#x5422;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x5423;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x5424;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x5425;'" select="'pou1'"/>
                <xsl:map-entry key="'&#x5426;'" select="'fou3	pi3'"/>
                <xsl:map-entry key="'&#x5427;'" select="'ba5	ba1'"/>
                <xsl:map-entry key="'&#x5428;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x5429;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x542A;'" select="'e2'"/>
                <xsl:map-entry key="'&#x542B;'" select="'han2'"/>
                <xsl:map-entry key="'&#x542C;'" select="'ting1	yin3'"/>
                <xsl:map-entry key="'&#x542D;'" select="'hang2	keng1'"/>
                <xsl:map-entry key="'&#x542E;'" select="'shun3'"/>
                <xsl:map-entry key="'&#x542F;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5430;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5431;'" select="'zhi1	zi1'"/>
                <xsl:map-entry key="'&#x5432;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x5433;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x5434;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x5435;'" select="'chao3	chao1'"/>
                <xsl:map-entry key="'&#x5436;'" select="'na4'"/>
                <xsl:map-entry key="'&#x5437;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x5438;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5439;'" select="'chui1	chui4'"/>
                <xsl:map-entry key="'&#x543A;'" select="'dou1'"/>
                <xsl:map-entry key="'&#x543B;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x543C;'" select="'hou3'"/>
                <xsl:map-entry key="'&#x543D;'" select="'ou2	hong1'"/>
                <xsl:map-entry key="'&#x543E;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x543F;'" select="'gao4	gu4'"/>
                <xsl:map-entry key="'&#x5440;'" select="'ya1	ya5'"/>
                <xsl:map-entry key="'&#x5441;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x5442;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x5443;'" select="'e4	e5'"/>
                <xsl:map-entry key="'&#x5444;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x5445;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x5446;'" select="'dai1'"/>
                <xsl:map-entry key="'&#x5447;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5448;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x5449;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x544A;'" select="'gao4	gu4'"/>
                <xsl:map-entry key="'&#x544B;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x544C;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x544D;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x544E;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x544F;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x5450;'" select="'na4'"/>
                <xsl:map-entry key="'&#x5451;'" select="'tun1'"/>
                <xsl:map-entry key="'&#x5452;'" select="'m2'"/>
                <xsl:map-entry key="'&#x5453;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5454;'" select="'tai3	tai4'"/>
                <xsl:map-entry key="'&#x5455;'" select="'ou3	ou4'"/>
                <xsl:map-entry key="'&#x5456;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5457;'" select="'bai4	bei5'"/>
                <xsl:map-entry key="'&#x5458;'" select="'yuan2	yun2	yun4'"/>
                <xsl:map-entry key="'&#x5459;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x545A;'" select="''"/>
                <xsl:map-entry key="'&#x545B;'" select="'qiang1	qiang4'"/>
                <xsl:map-entry key="'&#x545C;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x545D;'" select="'e4'"/>
                <xsl:map-entry key="'&#x545E;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x545F;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x5460;'" select="'pen3'"/>
                <xsl:map-entry key="'&#x5461;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x5462;'" select="'ni2	ne5'"/>
                <xsl:map-entry key="'&#x5463;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x5464;'" select="'ling2	ling4'"/>
                <xsl:map-entry key="'&#x5465;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x5466;'" select="'you1'"/>
                <xsl:map-entry key="'&#x5467;'" select="'di3'"/>
                <xsl:map-entry key="'&#x5468;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x5469;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x546A;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x546B;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x546C;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x546D;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x546E;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x546F;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x5470;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x5471;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x5472;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x5473;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x5474;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x5475;'" select="'he1	ke1'"/>
                <xsl:map-entry key="'&#x5476;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x5477;'" select="'xia1	xia2'"/>
                <xsl:map-entry key="'&#x5478;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x5479;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x547A;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x547B;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x547C;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x547D;'" select="'ming4'"/>
                <xsl:map-entry key="'&#x547E;'" select="'da2'"/>
                <xsl:map-entry key="'&#x547F;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x5480;'" select="'ju3	zui3'"/>
                <xsl:map-entry key="'&#x5481;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x5482;'" select="'za1'"/>
                <xsl:map-entry key="'&#x5483;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x5484;'" select="'duo1'"/>
                <xsl:map-entry key="'&#x5485;'" select="'pou3'"/>
                <xsl:map-entry key="'&#x5486;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x5487;'" select="'bie2'"/>
                <xsl:map-entry key="'&#x5488;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5489;'" select="'bi4	fu2'"/>
                <xsl:map-entry key="'&#x548A;'" select="'he2	he4	huo2	huo4	huo5'"/>
                <xsl:map-entry key="'&#x548B;'" select="'za3	ze2	zha1'"/>
                <xsl:map-entry key="'&#x548C;'" select="'he2	he4	huo2	huo4	huo5'"/>
                <xsl:map-entry key="'&#x548D;'" select="'hai1'"/>
                <xsl:map-entry key="'&#x548E;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x548F;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x5490;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5491;'" select="'da1'"/>
                <xsl:map-entry key="'&#x5492;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x5493;'" select="'wa3'"/>
                <xsl:map-entry key="'&#x5494;'" select="'ka3	ka1'"/>
                <xsl:map-entry key="'&#x5495;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x5496;'" select="'ka1'"/>
                <xsl:map-entry key="'&#x5497;'" select="'zuo3'"/>
                <xsl:map-entry key="'&#x5498;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x5499;'" select="'long2'"/>
                <xsl:map-entry key="'&#x549A;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x549B;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x549C;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x549D;'" select="'si1'"/>
                <xsl:map-entry key="'&#x549E;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x549F;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x54A0;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x54A1;'" select="'er4'"/>
                <xsl:map-entry key="'&#x54A2;'" select="'e4'"/>
                <xsl:map-entry key="'&#x54A3;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x54A4;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x54A5;'" select="'xi4	die2'"/>
                <xsl:map-entry key="'&#x54A6;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x54A7;'" select="'lie3	lie1'"/>
                <xsl:map-entry key="'&#x54A8;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x54A9;'" select="'mie1'"/>
                <xsl:map-entry key="'&#x54AA;'" select="'mi1'"/>
                <xsl:map-entry key="'&#x54AB;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x54AC;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x54AD;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x54AE;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x54AF;'" select="'ge1	ka3	lo5	luo4'"/>
                <xsl:map-entry key="'&#x54B0;'" select="'shuai4'"/>
                <xsl:map-entry key="'&#x54B1;'" select="'zan2'"/>
                <xsl:map-entry key="'&#x54B2;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x54B3;'" select="'ke2	hai1	ka3'"/>
                <xsl:map-entry key="'&#x54B4;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x54B5;'" select="'kua1'"/>
                <xsl:map-entry key="'&#x54B6;'" select="'huai4'"/>
                <xsl:map-entry key="'&#x54B7;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x54B8;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x54B9;'" select="'e4'"/>
                <xsl:map-entry key="'&#x54BA;'" select="'xuan3'"/>
                <xsl:map-entry key="'&#x54BB;'" select="'xiu1	xu3'"/>
                <xsl:map-entry key="'&#x54BC;'" select="'guo1	kuai1'"/>
                <xsl:map-entry key="'&#x54BD;'" select="'yan1	yan4	ye4'"/>
                <xsl:map-entry key="'&#x54BE;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x54BF;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x54C0;'" select="'ai1'"/>
                <xsl:map-entry key="'&#x54C1;'" select="'pin3'"/>
                <xsl:map-entry key="'&#x54C2;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x54C3;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x54C4;'" select="'hong1	hong3	hong4'"/>
                <xsl:map-entry key="'&#x54C5;'" select="'xiong1	hong1'"/>
                <xsl:map-entry key="'&#x54C6;'" select="'duo1'"/>
                <xsl:map-entry key="'&#x54C7;'" select="'wa1	wa5'"/>
                <xsl:map-entry key="'&#x54C8;'" select="'ha1	ha3	ha4'"/>
                <xsl:map-entry key="'&#x54C9;'" select="'zai1'"/>
                <xsl:map-entry key="'&#x54CA;'" select="'you4'"/>
                <xsl:map-entry key="'&#x54CB;'" select="'di4'"/>
                <xsl:map-entry key="'&#x54CC;'" select="'pai4'"/>
                <xsl:map-entry key="'&#x54CD;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x54CE;'" select="'ai1'"/>
                <xsl:map-entry key="'&#x54CF;'" select="'gen2'"/>
                <xsl:map-entry key="'&#x54D0;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x54D1;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x54D2;'" select="'da1'"/>
                <xsl:map-entry key="'&#x54D3;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x54D4;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x54D5;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x54D6;'" select="''"/>
                <xsl:map-entry key="'&#x54D7;'" select="'hua1	hua2	ye4'"/>
                <xsl:map-entry key="'&#x54D8;'" select="''"/>
                <xsl:map-entry key="'&#x54D9;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x54DA;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x54DB;'" select="''"/>
                <xsl:map-entry key="'&#x54DC;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x54DD;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x54DE;'" select="'mou1	mou2'"/>
                <xsl:map-entry key="'&#x54DF;'" select="'yo5	yo1'"/>
                <xsl:map-entry key="'&#x54E0;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x54E1;'" select="'yuan2	yun2	yun4'"/>
                <xsl:map-entry key="'&#x54E2;'" select="'long4'"/>
                <xsl:map-entry key="'&#x54E3;'" select="'pou3'"/>
                <xsl:map-entry key="'&#x54E4;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x54E5;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x54E6;'" select="'e2	o2	o4'"/>
                <xsl:map-entry key="'&#x54E7;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x54E8;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x54E9;'" select="'li1'"/>
                <xsl:map-entry key="'&#x54EA;'" select="'na3	na5	ne2'"/>
                <xsl:map-entry key="'&#x54EB;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x54EC;'" select="'he1'"/>
                <xsl:map-entry key="'&#x54ED;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x54EE;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x54EF;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x54F0;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x54F1;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x54F2;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x54F3;'" select="'zha1	zha2'"/>
                <xsl:map-entry key="'&#x54F4;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x54F5;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x54F6;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x54F7;'" select="'le4'"/>
                <xsl:map-entry key="'&#x54F8;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x54F9;'" select="'fou2'"/>
                <xsl:map-entry key="'&#x54FA;'" select="'bu3'"/>
                <xsl:map-entry key="'&#x54FB;'" select="'han4'"/>
                <xsl:map-entry key="'&#x54FC;'" select="'heng1'"/>
                <xsl:map-entry key="'&#x54FD;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x54FE;'" select="'shuo1'"/>
                <xsl:map-entry key="'&#x54FF;'" select="'ge3'"/>
                <xsl:map-entry key="'&#x5500;'" select="'you4'"/>
                <xsl:map-entry key="'&#x5501;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5502;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x5503;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x5504;'" select="'bai4	bei5'"/>
                <xsl:map-entry key="'&#x5505;'" select="'han1'"/>
                <xsl:map-entry key="'&#x5506;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x5507;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x5508;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5509;'" select="'ai1	ai4'"/>
                <xsl:map-entry key="'&#x550A;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x550B;'" select="'tu3'"/>
                <xsl:map-entry key="'&#x550C;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x550D;'" select="'guan1	guan3'"/>
                <xsl:map-entry key="'&#x550E;'" select="'li4'"/>
                <xsl:map-entry key="'&#x550F;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5510;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x5511;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x5512;'" select="'miu1'"/>
                <xsl:map-entry key="'&#x5513;'" select="'che1'"/>
                <xsl:map-entry key="'&#x5514;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x5515;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x5516;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x5517;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x5518;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5519;'" select="'di2'"/>
                <xsl:map-entry key="'&#x551A;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x551B;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x551C;'" select="''"/>
                <xsl:map-entry key="'&#x551D;'" select="'gong4'"/>
                <xsl:map-entry key="'&#x551E;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x551F;'" select="''"/>
                <xsl:map-entry key="'&#x5520;'" select="'lao2	lao4'"/>
                <xsl:map-entry key="'&#x5521;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x5522;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x5523;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x5524;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x5525;'" select="''"/>
                <xsl:map-entry key="'&#x5526;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x5527;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x5528;'" select="'zuo3'"/>
                <xsl:map-entry key="'&#x5529;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x552A;'" select="'feng3'"/>
                <xsl:map-entry key="'&#x552B;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x552C;'" select="'hu3	xia4'"/>
                <xsl:map-entry key="'&#x552D;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x552E;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x552F;'" select="'wei2	wei3'"/>
                <xsl:map-entry key="'&#x5530;'" select="'shua1'"/>
                <xsl:map-entry key="'&#x5531;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x5532;'" select="'er2'"/>
                <xsl:map-entry key="'&#x5533;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5534;'" select="'qiang4'"/>
                <xsl:map-entry key="'&#x5535;'" select="'an3'"/>
                <xsl:map-entry key="'&#x5536;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x5537;'" select="'yo1'"/>
                <xsl:map-entry key="'&#x5538;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x5539;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x553A;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x553B;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x553C;'" select="'sha4	za1'"/>
                <xsl:map-entry key="'&#x553D;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x553E;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x553F;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5540;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x5541;'" select="'zhou1	zhao1	zou1'"/>
                <xsl:map-entry key="'&#x5542;'" select="'nou4'"/>
                <xsl:map-entry key="'&#x5543;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x5544;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x5545;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x5546;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x5547;'" select="'di1'"/>
                <xsl:map-entry key="'&#x5548;'" select="'heng4'"/>
                <xsl:map-entry key="'&#x5549;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x554A;'" select="'a5	a1	a2	a3	a4'"/>
                <xsl:map-entry key="'&#x554B;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x554C;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x554D;'" select="'tun1'"/>
                <xsl:map-entry key="'&#x554E;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x554F;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x5550;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x5551;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x5552;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5553;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5554;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5555;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x5556;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x5557;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x5558;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x5559;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x555A;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x555B;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x555C;'" select="'chuo4	chuai4'"/>
                <xsl:map-entry key="'&#x555D;'" select="'he2'"/>
                <xsl:map-entry key="'&#x555E;'" select="'ya3	ya1'"/>
                <xsl:map-entry key="'&#x555F;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5560;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x5561;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x5562;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x5563;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x5564;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x5565;'" select="'sha4	sha2'"/>
                <xsl:map-entry key="'&#x5566;'" select="'la5	la1'"/>
                <xsl:map-entry key="'&#x5567;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x5568;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x5569;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x556A;'" select="'pa1	pai1'"/>
                <xsl:map-entry key="'&#x556B;'" select="'zhe3'"/>
                <xsl:map-entry key="'&#x556C;'" select="'se4'"/>
                <xsl:map-entry key="'&#x556D;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x556E;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x556F;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x5570;'" select="'luo1'"/>
                <xsl:map-entry key="'&#x5571;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x5572;'" select="'di4'"/>
                <xsl:map-entry key="'&#x5573;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x5574;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x5575;'" select="'bo5	bo1'"/>
                <xsl:map-entry key="'&#x5576;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x5577;'" select="'lang1'"/>
                <xsl:map-entry key="'&#x5578;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x5579;'" select="''"/>
                <xsl:map-entry key="'&#x557A;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x557B;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x557C;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x557D;'" select="'an2'"/>
                <xsl:map-entry key="'&#x557E;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x557F;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x5580;'" select="'ke4	ka1	ka4'"/>
                <xsl:map-entry key="'&#x5581;'" select="'yong2'"/>
                <xsl:map-entry key="'&#x5582;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x5583;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x5584;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x5585;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5586;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x5587;'" select="'la3	la1	la2'"/>
                <xsl:map-entry key="'&#x5588;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x5589;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x558A;'" select="'han3'"/>
                <xsl:map-entry key="'&#x558B;'" select="'die2	zha2'"/>
                <xsl:map-entry key="'&#x558C;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x558D;'" select="'chai2'"/>
                <xsl:map-entry key="'&#x558E;'" select="'kuai1'"/>
                <xsl:map-entry key="'&#x558F;'" select="'re3'"/>
                <xsl:map-entry key="'&#x5590;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5591;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x5592;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x5593;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x5594;'" select="'wo1	o1'"/>
                <xsl:map-entry key="'&#x5595;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x5596;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x5597;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x5598;'" select="'chuan3'"/>
                <xsl:map-entry key="'&#x5599;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x559A;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x559B;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x559C;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x559D;'" select="'he1	he4'"/>
                <xsl:map-entry key="'&#x559E;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x559F;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x55A0;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x55A1;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x55A2;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x55A3;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x55A4;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x55A5;'" select="'du4'"/>
                <xsl:map-entry key="'&#x55A6;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x55A7;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x55A8;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x55A9;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x55AA;'" select="'sang1	sang4'"/>
                <xsl:map-entry key="'&#x55AB;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x55AC;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x55AD;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x55AE;'" select="'dan1	chan2'"/>
                <xsl:map-entry key="'&#x55AF;'" select="'pen1'"/>
                <xsl:map-entry key="'&#x55B0;'" select="'shi2	si4'"/>
                <xsl:map-entry key="'&#x55B1;'" select="'li2'"/>
                <xsl:map-entry key="'&#x55B2;'" select="'yo5	yo1'"/>
                <xsl:map-entry key="'&#x55B3;'" select="'zha1	cha1'"/>
                <xsl:map-entry key="'&#x55B4;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x55B5;'" select="'miao1'"/>
                <xsl:map-entry key="'&#x55B6;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x55B7;'" select="'pen1	pen4'"/>
                <xsl:map-entry key="'&#x55B8;'" select="''"/>
                <xsl:map-entry key="'&#x55B9;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x55BA;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x55BB;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x55BC;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x55BD;'" select="'lou5	lou2'"/>
                <xsl:map-entry key="'&#x55BE;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x55BF;'" select="'cao1'"/>
                <xsl:map-entry key="'&#x55C0;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x55C1;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x55C2;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x55C3;'" select="'he4'"/>
                <xsl:map-entry key="'&#x55C4;'" select="'a2	sha4'"/>
                <xsl:map-entry key="'&#x55C5;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x55C6;'" select="'qiang1	qiang4'"/>
                <xsl:map-entry key="'&#x55C7;'" select="'se4'"/>
                <xsl:map-entry key="'&#x55C8;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x55C9;'" select="'su4'"/>
                <xsl:map-entry key="'&#x55CA;'" select="'hong3'"/>
                <xsl:map-entry key="'&#x55CB;'" select="'xie1'"/>
                <xsl:map-entry key="'&#x55CC;'" select="'yi4	ai4'"/>
                <xsl:map-entry key="'&#x55CD;'" select="'suo1	shuo4'"/>
                <xsl:map-entry key="'&#x55CE;'" select="'ma5	ma3'"/>
                <xsl:map-entry key="'&#x55CF;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x55D0;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x55D1;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x55D2;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x55D3;'" select="'sang3'"/>
                <xsl:map-entry key="'&#x55D4;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x55D5;'" select="'ru4	nou4'"/>
                <xsl:map-entry key="'&#x55D6;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x55D7;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x55D8;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x55D9;'" select="'pang3'"/>
                <xsl:map-entry key="'&#x55DA;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x55DB;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x55DC;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x55DD;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x55DE;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x55DF;'" select="'jie1	jue1'"/>
                <xsl:map-entry key="'&#x55E0;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x55E1;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x55E2;'" select="'wa4'"/>
                <xsl:map-entry key="'&#x55E3;'" select="'si4'"/>
                <xsl:map-entry key="'&#x55E4;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x55E5;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x55E6;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x55E7;'" select="'jia1lun2'"/>
                <xsl:map-entry key="'&#x55E8;'" select="'hai1	hai4	hei1'"/>
                <xsl:map-entry key="'&#x55E9;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x55EA;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x55EB;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x55EC;'" select="'he1'"/>
                <xsl:map-entry key="'&#x55ED;'" select="''"/>
                <xsl:map-entry key="'&#x55EE;'" select="'sai4'"/>
                <xsl:map-entry key="'&#x55EF;'" select="'ng4	ng2	ng3'"/>
                <xsl:map-entry key="'&#x55F0;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x55F1;'" select="'na2'"/>
                <xsl:map-entry key="'&#x55F2;'" select="'dia3	dia1'"/>
                <xsl:map-entry key="'&#x55F3;'" select="'ai4	ai3'"/>
                <xsl:map-entry key="'&#x55F4;'" select="''"/>
                <xsl:map-entry key="'&#x55F5;'" select="'tong1'"/>
                <xsl:map-entry key="'&#x55F6;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x55F7;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x55F8;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x55F9;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x55FA;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x55FB;'" select="'zhe1'"/>
                <xsl:map-entry key="'&#x55FC;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x55FD;'" select="'sou4'"/>
                <xsl:map-entry key="'&#x55FE;'" select="'sou3	zu2'"/>
                <xsl:map-entry key="'&#x55FF;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x5600;'" select="'di2	di1'"/>
                <xsl:map-entry key="'&#x5601;'" select="'qi1	za1'"/>
                <xsl:map-entry key="'&#x5602;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x5603;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x5604;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x5605;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x5606;'" select="'tan4'"/>
                <xsl:map-entry key="'&#x5607;'" select="'san1'"/>
                <xsl:map-entry key="'&#x5608;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x5609;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x560A;'" select="'yai1'"/>
                <xsl:map-entry key="'&#x560B;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x560C;'" select="'biao1	piao4'"/>
                <xsl:map-entry key="'&#x560D;'" select="'lou5	lou2'"/>
                <xsl:map-entry key="'&#x560E;'" select="'ga1	ga3'"/>
                <xsl:map-entry key="'&#x560F;'" select="'gu3	jia3'"/>
                <xsl:map-entry key="'&#x5610;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x5611;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5612;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5613;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x5614;'" select="'ou1	ou3	ou4'"/>
                <xsl:map-entry key="'&#x5615;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x5616;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x5617;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x5618;'" select="'xu1	shi1'"/>
                <xsl:map-entry key="'&#x5619;'" select="'po2'"/>
                <xsl:map-entry key="'&#x561A;'" select="'de2'"/>
                <xsl:map-entry key="'&#x561B;'" select="'ma5	ma2'"/>
                <xsl:map-entry key="'&#x561C;'" select="'ma4	ma3	mo4'"/>
                <xsl:map-entry key="'&#x561D;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x561E;'" select="'lei5'"/>
                <xsl:map-entry key="'&#x561F;'" select="'du1'"/>
                <xsl:map-entry key="'&#x5620;'" select="'ga1'"/>
                <xsl:map-entry key="'&#x5621;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x5622;'" select="'ye3'"/>
                <xsl:map-entry key="'&#x5623;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x5624;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x5625;'" select="''"/>
                <xsl:map-entry key="'&#x5626;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x5627;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5628;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x5629;'" select="'hua1	hua2	ye4'"/>
                <xsl:map-entry key="'&#x562A;'" select="'mai3'"/>
                <xsl:map-entry key="'&#x562B;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x562C;'" select="'zuo1	chuai4	zhuai4'"/>
                <xsl:map-entry key="'&#x562D;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x562E;'" select="'lao2	lao4'"/>
                <xsl:map-entry key="'&#x562F;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x5630;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x5631;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x5632;'" select="'chao2	zhao1'"/>
                <xsl:map-entry key="'&#x5633;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x5634;'" select="'zui3'"/>
                <xsl:map-entry key="'&#x5635;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x5636;'" select="'si1'"/>
                <xsl:map-entry key="'&#x5637;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x5638;'" select="'fu3	m2'"/>
                <xsl:map-entry key="'&#x5639;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x563A;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x563B;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x563C;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x563D;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x563E;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x563F;'" select="'mo4	hai1	hei1'"/>
                <xsl:map-entry key="'&#x5640;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x5641;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5642;'" select="'zun3'"/>
                <xsl:map-entry key="'&#x5643;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x5644;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x5645;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x5646;'" select="'can3'"/>
                <xsl:map-entry key="'&#x5647;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x5648;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x5649;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x564A;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x564B;'" select="'tun1'"/>
                <xsl:map-entry key="'&#x564C;'" select="'cheng1	ceng1'"/>
                <xsl:map-entry key="'&#x564D;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x564E;'" select="'ye1'"/>
                <xsl:map-entry key="'&#x564F;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5650;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x5651;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x5652;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5653;'" select="'xu1	shi1'"/>
                <xsl:map-entry key="'&#x5654;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x5655;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x5656;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5657;'" select="'pu1'"/>
                <xsl:map-entry key="'&#x5658;'" select="'jue1'"/>
                <xsl:map-entry key="'&#x5659;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x565A;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x565B;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x565C;'" select="'lu1'"/>
                <xsl:map-entry key="'&#x565D;'" select="'si1'"/>
                <xsl:map-entry key="'&#x565E;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x565F;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x5660;'" select="'da1'"/>
                <xsl:map-entry key="'&#x5661;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x5662;'" select="'yu3	o1	ou4'"/>
                <xsl:map-entry key="'&#x5663;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x5664;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x5665;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x5666;'" select="'hui4	yue1'"/>
                <xsl:map-entry key="'&#x5667;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5668;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x5669;'" select="'e4'"/>
                <xsl:map-entry key="'&#x566A;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x566B;'" select="'yi1	yi4'"/>
                <xsl:map-entry key="'&#x566C;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x566D;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x566E;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x566F;'" select="'ai4	ai3'"/>
                <xsl:map-entry key="'&#x5670;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x5671;'" select="'jue2	xue1	xue2'"/>
                <xsl:map-entry key="'&#x5672;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x5673;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x5674;'" select="'pen1	pen4'"/>
                <xsl:map-entry key="'&#x5675;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x5676;'" select="'ga2'"/>
                <xsl:map-entry key="'&#x5677;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x5678;'" select="'dun1	dun4'"/>
                <xsl:map-entry key="'&#x5679;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x567A;'" select="''"/>
                <xsl:map-entry key="'&#x567B;'" select="'sai1'"/>
                <xsl:map-entry key="'&#x567C;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x567D;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x567E;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x567F;'" select="'zui3'"/>
                <xsl:map-entry key="'&#x5680;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x5681;'" select="'di2'"/>
                <xsl:map-entry key="'&#x5682;'" select="'han3'"/>
                <xsl:map-entry key="'&#x5683;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x5684;'" select="'huo4	huo1'"/>
                <xsl:map-entry key="'&#x5685;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x5686;'" select="'hao1'"/>
                <xsl:map-entry key="'&#x5687;'" select="'xia4	he4'"/>
                <xsl:map-entry key="'&#x5688;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5689;'" select="'duo1'"/>
                <xsl:map-entry key="'&#x568A;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x568B;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x568C;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x568D;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x568E;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x568F;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x5690;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x5691;'" select="''"/>
                <xsl:map-entry key="'&#x5692;'" select="''"/>
                <xsl:map-entry key="'&#x5693;'" select="'ca1	cha1'"/>
                <xsl:map-entry key="'&#x5694;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x5695;'" select="'lu1'"/>
                <xsl:map-entry key="'&#x5696;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5697;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x5698;'" select="'you1'"/>
                <xsl:map-entry key="'&#x5699;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x569A;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x569B;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x569C;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x569D;'" select="'huang1'"/>
                <xsl:map-entry key="'&#x569E;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x569F;'" select="'li2'"/>
                <xsl:map-entry key="'&#x56A0;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x56A1;'" select="''"/>
                <xsl:map-entry key="'&#x56A2;'" select="'nang2	nang1'"/>
                <xsl:map-entry key="'&#x56A3;'" select="'xiao1	ao2'"/>
                <xsl:map-entry key="'&#x56A4;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x56A5;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x56A6;'" select="'li4'"/>
                <xsl:map-entry key="'&#x56A7;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x56A8;'" select="'long2'"/>
                <xsl:map-entry key="'&#x56A9;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x56AA;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x56AB;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x56AC;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x56AD;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x56AE;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x56AF;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x56B0;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x56B1;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x56B2;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x56B3;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x56B4;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x56B5;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x56B6;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x56B7;'" select="'rang3	rang1'"/>
                <xsl:map-entry key="'&#x56B8;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x56B9;'" select="'la1'"/>
                <xsl:map-entry key="'&#x56BA;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x56BB;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x56BC;'" select="'jiao2	jiao4	jue2'"/>
                <xsl:map-entry key="'&#x56BD;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x56BE;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x56BF;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x56C0;'" select="'zhuan4	zhuan3'"/>
                <xsl:map-entry key="'&#x56C1;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x56C2;'" select="'xiao1	ao2'"/>
                <xsl:map-entry key="'&#x56C3;'" select="'ca4'"/>
                <xsl:map-entry key="'&#x56C4;'" select="'li2'"/>
                <xsl:map-entry key="'&#x56C5;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x56C6;'" select="'chai4'"/>
                <xsl:map-entry key="'&#x56C7;'" select="'li4'"/>
                <xsl:map-entry key="'&#x56C8;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x56C9;'" select="'luo1	luo2'"/>
                <xsl:map-entry key="'&#x56CA;'" select="'nang2	nang1'"/>
                <xsl:map-entry key="'&#x56CB;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x56CC;'" select="'su1'"/>
                <xsl:map-entry key="'&#x56CD;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x56CE;'" select="''"/>
                <xsl:map-entry key="'&#x56CF;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x56D0;'" select="'za2'"/>
                <xsl:map-entry key="'&#x56D1;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x56D2;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x56D3;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x56D4;'" select="'nang1'"/>
                <xsl:map-entry key="'&#x56D5;'" select="''"/>
                <xsl:map-entry key="'&#x56D6;'" select="''"/>
                <xsl:map-entry key="'&#x56D7;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x56D8;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x56D9;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x56DA;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x56DB;'" select="'si4'"/>
                <xsl:map-entry key="'&#x56DC;'" select="'nin2'"/>
                <xsl:map-entry key="'&#x56DD;'" select="'jian3	nan1'"/>
                <xsl:map-entry key="'&#x56DE;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x56DF;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x56E0;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x56E1;'" select="'nan1'"/>
                <xsl:map-entry key="'&#x56E2;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x56E3;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x56E4;'" select="'dun4	tun2'"/>
                <xsl:map-entry key="'&#x56E5;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x56E6;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x56E7;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x56E8;'" select="'pian1'"/>
                <xsl:map-entry key="'&#x56E9;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x56EA;'" select="'cong1	chuang1'"/>
                <xsl:map-entry key="'&#x56EB;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x56EC;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x56ED;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x56EE;'" select="'e2'"/>
                <xsl:map-entry key="'&#x56EF;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x56F0;'" select="'kun4'"/>
                <xsl:map-entry key="'&#x56F1;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x56F2;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x56F3;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x56F4;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x56F5;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x56F6;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x56F7;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x56F8;'" select="'ri4'"/>
                <xsl:map-entry key="'&#x56F9;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x56FA;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x56FB;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x56FC;'" select="'tai1'"/>
                <xsl:map-entry key="'&#x56FD;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x56FE;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x56FF;'" select="'you4'"/>
                <xsl:map-entry key="'&#x5700;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x5701;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5702;'" select="'hun4'"/>
                <xsl:map-entry key="'&#x5703;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x5704;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x5705;'" select="'han2'"/>
                <xsl:map-entry key="'&#x5706;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x5707;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x5708;'" select="'quan1	juan1	juan4'"/>
                <xsl:map-entry key="'&#x5709;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x570A;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x570B;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x570C;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x570D;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x570E;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x570F;'" select="'quan1	juan1	juan4'"/>
                <xsl:map-entry key="'&#x5710;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x5711;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x5712;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x5713;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x5714;'" select="'e4'"/>
                <xsl:map-entry key="'&#x5715;'" select="'tu2shu1guan3'"/>
                <xsl:map-entry key="'&#x5716;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5717;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5718;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x5719;'" select="'lu:e4'"/>
                <xsl:map-entry key="'&#x571A;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x571B;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x571C;'" select="'yuan2	huan2'"/>
                <xsl:map-entry key="'&#x571D;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x571E;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x571F;'" select="'tu3'"/>
                <xsl:map-entry key="'&#x5720;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x5721;'" select="'tu3'"/>
                <xsl:map-entry key="'&#x5722;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x5723;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x5724;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5725;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x5726;'" select="''"/>
                <xsl:map-entry key="'&#x5727;'" select="'ya1	ya4'"/>
                <xsl:map-entry key="'&#x5728;'" select="'zai4'"/>
                <xsl:map-entry key="'&#x5729;'" select="'wei2	xu1	yu2'"/>
                <xsl:map-entry key="'&#x572A;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x572B;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x572C;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x572D;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x572E;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x572F;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5730;'" select="'di4	de5'"/>
                <xsl:map-entry key="'&#x5731;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x5732;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x5733;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x5734;'" select="'zhuo2	shao2'"/>
                <xsl:map-entry key="'&#x5735;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x5736;'" select="'qia4'"/>
                <xsl:map-entry key="'&#x5737;'" select="''"/>
                <xsl:map-entry key="'&#x5738;'" select="''"/>
                <xsl:map-entry key="'&#x5739;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x573A;'" select="'chang2	chang3'"/>
                <xsl:map-entry key="'&#x573B;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x573C;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x573D;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x573E;'" select="'ji2	ji1	se4'"/>
                <xsl:map-entry key="'&#x573F;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x5740;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x5741;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x5742;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x5743;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x5744;'" select="'tou2'"/>
                <xsl:map-entry key="'&#x5745;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x5746;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x5747;'" select="'jun1	yun4'"/>
                <xsl:map-entry key="'&#x5748;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x5749;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x574A;'" select="'fang1	fang2'"/>
                <xsl:map-entry key="'&#x574B;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x574C;'" select="'ben4'"/>
                <xsl:map-entry key="'&#x574D;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x574E;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x574F;'" select="'pi1	huai4	pei1'"/>
                <xsl:map-entry key="'&#x5750;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x5751;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x5752;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5753;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x5754;'" select="'di4'"/>
                <xsl:map-entry key="'&#x5755;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x5756;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5757;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x5758;'" select="'di3'"/>
                <xsl:map-entry key="'&#x5759;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x575A;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x575B;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x575C;'" select="'li4'"/>
                <xsl:map-entry key="'&#x575D;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x575E;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x575F;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x5760;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x5761;'" select="'po1'"/>
                <xsl:map-entry key="'&#x5762;'" select="'pan3'"/>
                <xsl:map-entry key="'&#x5763;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x5764;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x5765;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x5766;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x5767;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x5768;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x5769;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x576A;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x576B;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x576C;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x576D;'" select="'ni2	ni3'"/>
                <xsl:map-entry key="'&#x576E;'" select="'tai2	tai1'"/>
                <xsl:map-entry key="'&#x576F;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x5770;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x5771;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x5772;'" select="'fo2'"/>
                <xsl:map-entry key="'&#x5773;'" select="'ao4	ao1'"/>
                <xsl:map-entry key="'&#x5774;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x5775;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x5776;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x5777;'" select="'ke3	ke1'"/>
                <xsl:map-entry key="'&#x5778;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x5779;'" select="'xue4'"/>
                <xsl:map-entry key="'&#x577A;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x577B;'" select="'chi2	di3'"/>
                <xsl:map-entry key="'&#x577C;'" select="'che4'"/>
                <xsl:map-entry key="'&#x577D;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x577E;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x577F;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5780;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5781;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5782;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x5783;'" select="'la1	le4'"/>
                <xsl:map-entry key="'&#x5784;'" select="'long3'"/>
                <xsl:map-entry key="'&#x5785;'" select="'long3'"/>
                <xsl:map-entry key="'&#x5786;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x5787;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x5788;'" select="''"/>
                <xsl:map-entry key="'&#x5789;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x578A;'" select="''"/>
                <xsl:map-entry key="'&#x578B;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x578C;'" select="'tong2	tong3'"/>
                <xsl:map-entry key="'&#x578D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x578E;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x578F;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x5790;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x5791;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x5792;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x5793;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x5794;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x5795;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x5796;'" select="'dui1'"/>
                <xsl:map-entry key="'&#x5797;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x5798;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5799;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x579A;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x579B;'" select="'duo3	duo4'"/>
                <xsl:map-entry key="'&#x579C;'" select="'duo3	duo4'"/>
                <xsl:map-entry key="'&#x579D;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x579E;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x579F;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x57A0;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x57A1;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x57A2;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x57A3;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x57A4;'" select="'die2'"/>
                <xsl:map-entry key="'&#x57A5;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x57A6;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x57A7;'" select="'shang3'"/>
                <xsl:map-entry key="'&#x57A8;'" select="'shou3'"/>
                <xsl:map-entry key="'&#x57A9;'" select="'e4'"/>
                <xsl:map-entry key="'&#x57AA;'" select="''"/>
                <xsl:map-entry key="'&#x57AB;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x57AC;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x57AD;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x57AE;'" select="'kua3'"/>
                <xsl:map-entry key="'&#x57AF;'" select="'da5'"/>
                <xsl:map-entry key="'&#x57B0;'" select="''"/>
                <xsl:map-entry key="'&#x57B1;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x57B2;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x57B3;'" select="''"/>
                <xsl:map-entry key="'&#x57B4;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x57B5;'" select="'an1'"/>
                <xsl:map-entry key="'&#x57B6;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x57B7;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x57B8;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x57B9;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x57BA;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x57BB;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x57BC;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x57BD;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x57BE;'" select="'han4'"/>
                <xsl:map-entry key="'&#x57BF;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x57C0;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x57C1;'" select="'cen2'"/>
                <xsl:map-entry key="'&#x57C2;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x57C3;'" select="'ai1'"/>
                <xsl:map-entry key="'&#x57C4;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x57C5;'" select="'fang2'"/>
                <xsl:map-entry key="'&#x57C6;'" select="'que4'"/>
                <xsl:map-entry key="'&#x57C7;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x57C8;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x57C9;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x57CA;'" select="'di4'"/>
                <xsl:map-entry key="'&#x57CB;'" select="'mai2	man2'"/>
                <xsl:map-entry key="'&#x57CC;'" select="'lang4'"/>
                <xsl:map-entry key="'&#x57CD;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x57CE;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x57CF;'" select="'yan2	shan1'"/>
                <xsl:map-entry key="'&#x57D0;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x57D1;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x57D2;'" select="'le4	lie4'"/>
                <xsl:map-entry key="'&#x57D3;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x57D4;'" select="'bu4	bu3	pu3'"/>
                <xsl:map-entry key="'&#x57D5;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x57D6;'" select="''"/>
                <xsl:map-entry key="'&#x57D7;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x57D8;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x57D9;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x57DA;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x57DB;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x57DC;'" select="'ye3'"/>
                <xsl:map-entry key="'&#x57DD;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x57DE;'" select="'di1'"/>
                <xsl:map-entry key="'&#x57DF;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x57E0;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x57E1;'" select="'wu4	ya1'"/>
                <xsl:map-entry key="'&#x57E2;'" select="'juan3'"/>
                <xsl:map-entry key="'&#x57E3;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x57E4;'" select="'pi2	bei1	bi4'"/>
                <xsl:map-entry key="'&#x57E5;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x57E6;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x57E7;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x57E8;'" select="'lun3'"/>
                <xsl:map-entry key="'&#x57E9;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x57EA;'" select="'kong1'"/>
                <xsl:map-entry key="'&#x57EB;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x57EC;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x57ED;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x57EE;'" select="'tan4'"/>
                <xsl:map-entry key="'&#x57EF;'" select="'an3'"/>
                <xsl:map-entry key="'&#x57F0;'" select="'cai4'"/>
                <xsl:map-entry key="'&#x57F1;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x57F2;'" select="'beng3'"/>
                <xsl:map-entry key="'&#x57F3;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x57F4;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x57F5;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x57F6;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x57F7;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x57F8;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x57F9;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x57FA;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x57FB;'" select="'zhun3'"/>
                <xsl:map-entry key="'&#x57FC;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x57FD;'" select="'sao4	sao3'"/>
                <xsl:map-entry key="'&#x57FE;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x57FF;'" select="'ni2	ni4'"/>
                <xsl:map-entry key="'&#x5800;'" select="'ku1	jue2'"/>
                <xsl:map-entry key="'&#x5801;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x5802;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x5803;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x5804;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x5805;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x5806;'" select="'dui1	zui1'"/>
                <xsl:map-entry key="'&#x5807;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x5808;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x5809;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x580A;'" select="'e4'"/>
                <xsl:map-entry key="'&#x580B;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x580C;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x580D;'" select="'tu4'"/>
                <xsl:map-entry key="'&#x580E;'" select="'leng4	ling2'"/>
                <xsl:map-entry key="'&#x580F;'" select="''"/>
                <xsl:map-entry key="'&#x5810;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x5811;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x5812;'" select="''"/>
                <xsl:map-entry key="'&#x5813;'" select="'an4'"/>
                <xsl:map-entry key="'&#x5814;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x5815;'" select="'duo4	hui1'"/>
                <xsl:map-entry key="'&#x5816;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x5817;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x5818;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x5819;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x581A;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x581B;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x581C;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x581D;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x581E;'" select="'die2'"/>
                <xsl:map-entry key="'&#x581F;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x5820;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x5821;'" select="'bao3	bu3	pu4'"/>
                <xsl:map-entry key="'&#x5822;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x5823;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5824;'" select="'di1'"/>
                <xsl:map-entry key="'&#x5825;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x5826;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x5827;'" select="'ruan2'"/>
                <xsl:map-entry key="'&#x5828;'" select="'e4	ai4'"/>
                <xsl:map-entry key="'&#x5829;'" select="'geng4'"/>
                <xsl:map-entry key="'&#x582A;'" select="'kan1'"/>
                <xsl:map-entry key="'&#x582B;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x582C;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x582D;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x582E;'" select="'e4'"/>
                <xsl:map-entry key="'&#x582F;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5830;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5831;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x5832;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5833;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x5834;'" select="'chang2	chang3'"/>
                <xsl:map-entry key="'&#x5835;'" select="'du3'"/>
                <xsl:map-entry key="'&#x5836;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x5837;'" select="'an3'"/>
                <xsl:map-entry key="'&#x5838;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x5839;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x583A;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x583B;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x583C;'" select="'heng4'"/>
                <xsl:map-entry key="'&#x583D;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x583E;'" select="'chuan3'"/>
                <xsl:map-entry key="'&#x583F;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x5840;'" select="''"/>
                <xsl:map-entry key="'&#x5841;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x5842;'" select="'gang3'"/>
                <xsl:map-entry key="'&#x5843;'" select="'huang1'"/>
                <xsl:map-entry key="'&#x5844;'" select="'leng2'"/>
                <xsl:map-entry key="'&#x5845;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x5846;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x5847;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x5848;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5849;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x584A;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x584B;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x584C;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x584D;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x584E;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x584F;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x5850;'" select="'su4'"/>
                <xsl:map-entry key="'&#x5851;'" select="'su4'"/>
                <xsl:map-entry key="'&#x5852;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x5853;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5854;'" select="'ta3	da5'"/>
                <xsl:map-entry key="'&#x5855;'" select="'weng3'"/>
                <xsl:map-entry key="'&#x5856;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x5857;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5858;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x5859;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x585A;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x585B;'" select="'li4'"/>
                <xsl:map-entry key="'&#x585C;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x585D;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x585E;'" select="'sai1	sai4	se4'"/>
                <xsl:map-entry key="'&#x585F;'" select="'zang4'"/>
                <xsl:map-entry key="'&#x5860;'" select="'dui1'"/>
                <xsl:map-entry key="'&#x5861;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x5862;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5863;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x5864;'" select="'xun1	xuan1'"/>
                <xsl:map-entry key="'&#x5865;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x5866;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x5867;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x5868;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x5869;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x586A;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x586B;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x586C;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x586D;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x586E;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x586F;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x5870;'" select="''"/>
                <xsl:map-entry key="'&#x5871;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x5872;'" select="'chang2	chang3'"/>
                <xsl:map-entry key="'&#x5873;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x5874;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x5875;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x5876;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x5877;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x5878;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x5879;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x587A;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x587B;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x587C;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x587D;'" select="'shuang3'"/>
                <xsl:map-entry key="'&#x587E;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x587F;'" select="'lou3'"/>
                <xsl:map-entry key="'&#x5880;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x5881;'" select="'man4'"/>
                <xsl:map-entry key="'&#x5882;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x5883;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x5884;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x5885;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x5886;'" select="'di4'"/>
                <xsl:map-entry key="'&#x5887;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x5888;'" select="'kan4'"/>
                <xsl:map-entry key="'&#x5889;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x588A;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x588B;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x588C;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x588D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x588E;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x588F;'" select="'qiang3'"/>
                <xsl:map-entry key="'&#x5890;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x5891;'" select="'di1'"/>
                <xsl:map-entry key="'&#x5892;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x5893;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x5894;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x5895;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5896;'" select="'ta3	da5'"/>
                <xsl:map-entry key="'&#x5897;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x5898;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x5899;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x589A;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x589B;'" select="''"/>
                <xsl:map-entry key="'&#x589C;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x589D;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x589E;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x589F;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x58A0;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x58A1;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x58A2;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x58A3;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x58A4;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x58A5;'" select="'dong3'"/>
                <xsl:map-entry key="'&#x58A6;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x58A7;'" select="'que4'"/>
                <xsl:map-entry key="'&#x58A8;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x58A9;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x58AA;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x58AB;'" select="'zun1	zun3'"/>
                <xsl:map-entry key="'&#x58AC;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x58AD;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x58AE;'" select="'duo4	hui1'"/>
                <xsl:map-entry key="'&#x58AF;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x58B0;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x58B1;'" select="'deng4	yan4'"/>
                <xsl:map-entry key="'&#x58B2;'" select="'mu2'"/>
                <xsl:map-entry key="'&#x58B3;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x58B4;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x58B5;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x58B6;'" select="'da5'"/>
                <xsl:map-entry key="'&#x58B7;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x58B8;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x58B9;'" select="''"/>
                <xsl:map-entry key="'&#x58BA;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x58BB;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x58BC;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x58BD;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x58BE;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x58BF;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x58C0;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x58C1;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x58C2;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x58C3;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x58C4;'" select="'ye3'"/>
                <xsl:map-entry key="'&#x58C5;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x58C6;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x58C7;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x58C8;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x58C9;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x58CA;'" select="'huai4	pi1'"/>
                <xsl:map-entry key="'&#x58CB;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x58CC;'" select="'rang3'"/>
                <xsl:map-entry key="'&#x58CD;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x58CE;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x58CF;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x58D0;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x58D1;'" select="'he4'"/>
                <xsl:map-entry key="'&#x58D2;'" select="'kai4'"/>
                <xsl:map-entry key="'&#x58D3;'" select="'ya1	ya4'"/>
                <xsl:map-entry key="'&#x58D4;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x58D5;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x58D6;'" select="'ruan2'"/>
                <xsl:map-entry key="'&#x58D7;'" select="''"/>
                <xsl:map-entry key="'&#x58D8;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x58D9;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x58DA;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x58DB;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x58DC;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x58DD;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x58DE;'" select="'huai4	pi1'"/>
                <xsl:map-entry key="'&#x58DF;'" select="'long3'"/>
                <xsl:map-entry key="'&#x58E0;'" select="'long3'"/>
                <xsl:map-entry key="'&#x58E1;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x58E2;'" select="'li4'"/>
                <xsl:map-entry key="'&#x58E3;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x58E4;'" select="'rang3'"/>
                <xsl:map-entry key="'&#x58E5;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x58E6;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x58E7;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x58E8;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x58E9;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x58EA;'" select="''"/>
                <xsl:map-entry key="'&#x58EB;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x58EC;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x58ED;'" select="''"/>
                <xsl:map-entry key="'&#x58EE;'" select="'zhuang4'"/>
                <xsl:map-entry key="'&#x58EF;'" select="'zhuang4'"/>
                <xsl:map-entry key="'&#x58F0;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x58F1;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x58F2;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x58F3;'" select="'ke2	qiao4'"/>
                <xsl:map-entry key="'&#x58F4;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x58F5;'" select="'zhuang4'"/>
                <xsl:map-entry key="'&#x58F6;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x58F7;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x58F8;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x58F9;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x58FA;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x58FB;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x58FC;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x58FD;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x58FE;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x58FF;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x5900;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x5901;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x5902;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x5903;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x5904;'" select="'chu4	chu3'"/>
                <xsl:map-entry key="'&#x5905;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x5906;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x5907;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x5908;'" select="''"/>
                <xsl:map-entry key="'&#x5909;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x590A;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x590B;'" select="'qun1'"/>
                <xsl:map-entry key="'&#x590C;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x590D;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x590E;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x590F;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x5910;'" select="'xiong4'"/>
                <xsl:map-entry key="'&#x5911;'" select="''"/>
                <xsl:map-entry key="'&#x5912;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x5913;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x5914;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x5915;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5916;'" select="'wai4'"/>
                <xsl:map-entry key="'&#x5917;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x5918;'" select="'mao3'"/>
                <xsl:map-entry key="'&#x5919;'" select="'su4'"/>
                <xsl:map-entry key="'&#x591A;'" select="'duo1'"/>
                <xsl:map-entry key="'&#x591B;'" select="'duo1'"/>
                <xsl:map-entry key="'&#x591C;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x591D;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x591E;'" select="''"/>
                <xsl:map-entry key="'&#x591F;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x5920;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x5921;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x5922;'" select="'meng4'"/>
                <xsl:map-entry key="'&#x5923;'" select="'meng4'"/>
                <xsl:map-entry key="'&#x5924;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5925;'" select="'huo3'"/>
                <xsl:map-entry key="'&#x5926;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x5927;'" select="'da4	dai4'"/>
                <xsl:map-entry key="'&#x5928;'" select="'ze4'"/>
                <xsl:map-entry key="'&#x5929;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x592A;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x592B;'" select="'fu1	fu2'"/>
                <xsl:map-entry key="'&#x592C;'" select="'guai4'"/>
                <xsl:map-entry key="'&#x592D;'" select="'yao3	yao1'"/>
                <xsl:map-entry key="'&#x592E;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x592F;'" select="'hang1	ben4'"/>
                <xsl:map-entry key="'&#x5930;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x5931;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x5932;'" select="'ben3'"/>
                <xsl:map-entry key="'&#x5933;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x5934;'" select="'tou2'"/>
                <xsl:map-entry key="'&#x5935;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5936;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x5937;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5938;'" select="'kua1'"/>
                <xsl:map-entry key="'&#x5939;'" select="'jia1	jia2'"/>
                <xsl:map-entry key="'&#x593A;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x593B;'" select="''"/>
                <xsl:map-entry key="'&#x593C;'" select="'kuang3'"/>
                <xsl:map-entry key="'&#x593D;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x593E;'" select="'jia1	jia2'"/>
                <xsl:map-entry key="'&#x593F;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x5940;'" select="'en1	mang2'"/>
                <xsl:map-entry key="'&#x5941;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5942;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x5943;'" select="'di1'"/>
                <xsl:map-entry key="'&#x5944;'" select="'yan3	yan1'"/>
                <xsl:map-entry key="'&#x5945;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x5946;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x5947;'" select="'qi2	ji1'"/>
                <xsl:map-entry key="'&#x5948;'" select="'nai4	nai3'"/>
                <xsl:map-entry key="'&#x5949;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x594A;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x594B;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x594C;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x594D;'" select="''"/>
                <xsl:map-entry key="'&#x594E;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x594F;'" select="'zou4'"/>
                <xsl:map-entry key="'&#x5950;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x5951;'" select="'qi4	xie4'"/>
                <xsl:map-entry key="'&#x5952;'" select="'kai1'"/>
                <xsl:map-entry key="'&#x5953;'" select="'she1'"/>
                <xsl:map-entry key="'&#x5954;'" select="'ben1	ben4'"/>
                <xsl:map-entry key="'&#x5955;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5956;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x5957;'" select="'tao4'"/>
                <xsl:map-entry key="'&#x5958;'" select="'zang4	zhuang3'"/>
                <xsl:map-entry key="'&#x5959;'" select="'ben3'"/>
                <xsl:map-entry key="'&#x595A;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x595B;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x595C;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x595D;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x595E;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x595F;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x5960;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x5961;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x5962;'" select="'she1'"/>
                <xsl:map-entry key="'&#x5963;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x5964;'" select="'pan3'"/>
                <xsl:map-entry key="'&#x5965;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x5966;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5967;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x5968;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x5969;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x596A;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x596B;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x596C;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x596D;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x596E;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x596F;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x5970;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x5971;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5972;'" select="'che3'"/>
                <xsl:map-entry key="'&#x5973;'" select="'nu:3	ru3'"/>
                <xsl:map-entry key="'&#x5974;'" select="'nu2'"/>
                <xsl:map-entry key="'&#x5975;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x5976;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x5977;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x5978;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x5979;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x597A;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x597B;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x597C;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x597D;'" select="'hao3	hao4'"/>
                <xsl:map-entry key="'&#x597E;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x597F;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x5980;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x5981;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x5982;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x5983;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x5984;'" select="'wang4'"/>
                <xsl:map-entry key="'&#x5985;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x5986;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x5987;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5988;'" select="'ma1'"/>
                <xsl:map-entry key="'&#x5989;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x598A;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x598B;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x598C;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x598D;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x598E;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x598F;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x5990;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x5991;'" select="'pa1'"/>
                <xsl:map-entry key="'&#x5992;'" select="'du4'"/>
                <xsl:map-entry key="'&#x5993;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5994;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x5995;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x5996;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x5997;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x5998;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x5999;'" select="'miao4'"/>
                <xsl:map-entry key="'&#x599A;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x599B;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x599C;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x599D;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x599E;'" select="'niu1'"/>
                <xsl:map-entry key="'&#x599F;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x59A0;'" select="'na4'"/>
                <xsl:map-entry key="'&#x59A1;'" select="'xin1	xin4'"/>
                <xsl:map-entry key="'&#x59A2;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x59A3;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x59A4;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x59A5;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x59A6;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x59A7;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x59A8;'" select="'fang2	fang1'"/>
                <xsl:map-entry key="'&#x59A9;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x59AA;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x59AB;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x59AC;'" select="'du4'"/>
                <xsl:map-entry key="'&#x59AD;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x59AE;'" select="'ni1	ni2'"/>
                <xsl:map-entry key="'&#x59AF;'" select="'zhou2'"/>
                <xsl:map-entry key="'&#x59B0;'" select="'zhou2'"/>
                <xsl:map-entry key="'&#x59B1;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x59B2;'" select="'da2'"/>
                <xsl:map-entry key="'&#x59B3;'" select="'nai3	ni3'"/>
                <xsl:map-entry key="'&#x59B4;'" select="'yuan3'"/>
                <xsl:map-entry key="'&#x59B5;'" select="'tou3'"/>
                <xsl:map-entry key="'&#x59B6;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x59B7;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x59B8;'" select="'e1'"/>
                <xsl:map-entry key="'&#x59B9;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x59BA;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x59BB;'" select="'qi1	qi4'"/>
                <xsl:map-entry key="'&#x59BC;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x59BD;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x59BE;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x59BF;'" select="'e1'"/>
                <xsl:map-entry key="'&#x59C0;'" select="'he2'"/>
                <xsl:map-entry key="'&#x59C1;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x59C2;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x59C3;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x59C4;'" select="'ni1'"/>
                <xsl:map-entry key="'&#x59C5;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x59C6;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x59C7;'" select="'fu1	fu4'"/>
                <xsl:map-entry key="'&#x59C8;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x59C9;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x59CA;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x59CB;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x59CC;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x59CD;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x59CE;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x59CF;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x59D0;'" select="'jie3'"/>
                <xsl:map-entry key="'&#x59D1;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x59D2;'" select="'si4'"/>
                <xsl:map-entry key="'&#x59D3;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x59D4;'" select="'wei3	wei1'"/>
                <xsl:map-entry key="'&#x59D5;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x59D6;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x59D7;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x59D8;'" select="'pin1'"/>
                <xsl:map-entry key="'&#x59D9;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x59DA;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x59DB;'" select="'tong3'"/>
                <xsl:map-entry key="'&#x59DC;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x59DD;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x59DE;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x59DF;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x59E0;'" select="'shang4'"/>
                <xsl:map-entry key="'&#x59E1;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x59E2;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x59E3;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x59E4;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x59E5;'" select="'mu3	lao3'"/>
                <xsl:map-entry key="'&#x59E6;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x59E7;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x59E8;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x59E9;'" select="'nian2'"/>
                <xsl:map-entry key="'&#x59EA;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x59EB;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x59EC;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x59ED;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x59EE;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x59EF;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x59F0;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x59F1;'" select="'kua1'"/>
                <xsl:map-entry key="'&#x59F2;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x59F3;'" select="'ming3'"/>
                <xsl:map-entry key="'&#x59F4;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x59F5;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x59F6;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x59F7;'" select="'you4'"/>
                <xsl:map-entry key="'&#x59F8;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x59F9;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x59FA;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x59FB;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x59FC;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x59FD;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x59FE;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x59FF;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x5A00;'" select="'song1'"/>
                <xsl:map-entry key="'&#x5A01;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x5A02;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x5A03;'" select="'wa2'"/>
                <xsl:map-entry key="'&#x5A04;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x5A05;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x5A06;'" select="'rao3	rao2'"/>
                <xsl:map-entry key="'&#x5A07;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x5A08;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x5A09;'" select="'ping1'"/>
                <xsl:map-entry key="'&#x5A0A;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x5A0B;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x5A0C;'" select="'li3'"/>
                <xsl:map-entry key="'&#x5A0D;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x5A0E;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5A0F;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x5A10;'" select="'fu'"/>
                <xsl:map-entry key="'&#x5A11;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x5A12;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x5A13;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x5A14;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x5A15;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x5A16;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x5A17;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x5A18;'" select="'niang2'"/>
                <xsl:map-entry key="'&#x5A19;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x5A1A;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x5A1B;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5A1C;'" select="'nuo2	na4'"/>
                <xsl:map-entry key="'&#x5A1D;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x5A1E;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x5A1F;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x5A20;'" select="'shen1	chen2	zhen4'"/>
                <xsl:map-entry key="'&#x5A21;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5A22;'" select="'han2'"/>
                <xsl:map-entry key="'&#x5A23;'" select="'di4'"/>
                <xsl:map-entry key="'&#x5A24;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x5A25;'" select="'e2'"/>
                <xsl:map-entry key="'&#x5A26;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x5A27;'" select="'tui4'"/>
                <xsl:map-entry key="'&#x5A28;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x5A29;'" select="'mian3	wan3'"/>
                <xsl:map-entry key="'&#x5A2A;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x5A2B;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5A2C;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x5A2D;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5A2E;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5A2F;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5A30;'" select="'si4'"/>
                <xsl:map-entry key="'&#x5A31;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5A32;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x5A33;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5A34;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x5A35;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x5A36;'" select="'qu3	qu4'"/>
                <xsl:map-entry key="'&#x5A37;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x5A38;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x5A39;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x5A3A;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x5A3B;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x5A3C;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x5A3D;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x5A3E;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x5A3F;'" select="'e1'"/>
                <xsl:map-entry key="'&#x5A40;'" select="'e1'"/>
                <xsl:map-entry key="'&#x5A41;'" select="'lou2	lu:3'"/>
                <xsl:map-entry key="'&#x5A42;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x5A43;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x5A44;'" select="'pou3'"/>
                <xsl:map-entry key="'&#x5A45;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x5A46;'" select="'po2'"/>
                <xsl:map-entry key="'&#x5A47;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x5A48;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x5A49;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x5A4A;'" select="'biao3'"/>
                <xsl:map-entry key="'&#x5A4B;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x5A4C;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x5A4D;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5A4E;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x5A4F;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5A50;'" select="'wo3'"/>
                <xsl:map-entry key="'&#x5A51;'" select="'rui2'"/>
                <xsl:map-entry key="'&#x5A52;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x5A53;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x5A54;'" select="'wei'"/>
                <xsl:map-entry key="'&#x5A55;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5A56;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x5A57;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x5A58;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x5A59;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x5A5A;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x5A5B;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x5A5C;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x5A5D;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x5A5E;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x5A5F;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x5A60;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x5A61;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x5A62;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5A63;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x5A64;'" select="'chou1	zhou1'"/>
                <xsl:map-entry key="'&#x5A65;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x5A66;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5A67;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x5A68;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x5A69;'" select="'yan4	an4'"/>
                <xsl:map-entry key="'&#x5A6A;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x5A6B;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x5A6C;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5A6D;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x5A6E;'" select="'ju'"/>
                <xsl:map-entry key="'&#x5A6F;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5A70;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x5A71;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x5A72;'" select="''"/>
                <xsl:map-entry key="'&#x5A73;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x5A74;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x5A75;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x5A76;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x5A77;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x5A78;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x5A79;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x5A7A;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5A7B;'" select="'nan4'"/>
                <xsl:map-entry key="'&#x5A7C;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x5A7D;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x5A7E;'" select="'tou1'"/>
                <xsl:map-entry key="'&#x5A7F;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x5A80;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5A81;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x5A82;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x5A83;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x5A84;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x5A85;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x5A86;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x5A87;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x5A88;'" select="'hui'"/>
                <xsl:map-entry key="'&#x5A89;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x5A8A;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x5A8B;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x5A8C;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x5A8D;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5A8E;'" select="'jie3'"/>
                <xsl:map-entry key="'&#x5A8F;'" select="'duan1'"/>
                <xsl:map-entry key="'&#x5A90;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5A91;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x5A92;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x5A93;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x5A94;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x5A95;'" select="'an1'"/>
                <xsl:map-entry key="'&#x5A96;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x5A97;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x5A98;'" select="'jie'"/>
                <xsl:map-entry key="'&#x5A99;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x5A9A;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x5A9B;'" select="'yuan4	yuan2'"/>
                <xsl:map-entry key="'&#x5A9C;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x5A9D;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x5A9E;'" select="'ti2	shi4'"/>
                <xsl:map-entry key="'&#x5A9F;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5AA0;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x5AA1;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x5AA2;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x5AA3;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x5AA4;'" select="'si1'"/>
                <xsl:map-entry key="'&#x5AA5;'" select="'pian1'"/>
                <xsl:map-entry key="'&#x5AA6;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x5AA7;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x5AA8;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x5AA9;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x5AAA;'" select="'ao3'"/>
                <xsl:map-entry key="'&#x5AAB;'" select="''"/>
                <xsl:map-entry key="'&#x5AAC;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x5AAD;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x5AAE;'" select="'tou1	tou3'"/>
                <xsl:map-entry key="'&#x5AAF;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x5AB0;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x5AB1;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5AB2;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x5AB3;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x5AB4;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x5AB5;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x5AB6;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x5AB7;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x5AB8;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x5AB9;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x5ABA;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x5ABB;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x5ABC;'" select="'ao3'"/>
                <xsl:map-entry key="'&#x5ABD;'" select="'ma1'"/>
                <xsl:map-entry key="'&#x5ABE;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x5ABF;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x5AC0;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x5AC1;'" select="'jia4'"/>
                <xsl:map-entry key="'&#x5AC2;'" select="'sao3'"/>
                <xsl:map-entry key="'&#x5AC3;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x5AC4;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x5AC5;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x5AC6;'" select="'yong2'"/>
                <xsl:map-entry key="'&#x5AC7;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x5AC8;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x5AC9;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5ACA;'" select="'su4'"/>
                <xsl:map-entry key="'&#x5ACB;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x5ACC;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x5ACD;'" select="'tao1	yao3'"/>
                <xsl:map-entry key="'&#x5ACE;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x5ACF;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x5AD0;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x5AD1;'" select="'bao2'"/>
                <xsl:map-entry key="'&#x5AD2;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x5AD3;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x5AD4;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x5AD5;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5AD6;'" select="'piao2'"/>
                <xsl:map-entry key="'&#x5AD7;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5AD8;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x5AD9;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x5ADA;'" select="'man4'"/>
                <xsl:map-entry key="'&#x5ADB;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x5ADC;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x5ADD;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x5ADE;'" select="'yong2'"/>
                <xsl:map-entry key="'&#x5ADF;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x5AE0;'" select="'li2'"/>
                <xsl:map-entry key="'&#x5AE1;'" select="'di2'"/>
                <xsl:map-entry key="'&#x5AE2;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x5AE3;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x5AE4;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x5AE5;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x5AE6;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x5AE7;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x5AE8;'" select="'han1	ran3'"/>
                <xsl:map-entry key="'&#x5AE9;'" select="'nen4'"/>
                <xsl:map-entry key="'&#x5AEA;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x5AEB;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x5AEC;'" select="'zhe1'"/>
                <xsl:map-entry key="'&#x5AED;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x5AEE;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x5AEF;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x5AF0;'" select="'nen4'"/>
                <xsl:map-entry key="'&#x5AF1;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x5AF2;'" select="''"/>
                <xsl:map-entry key="'&#x5AF3;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5AF4;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x5AF5;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x5AF6;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x5AF7;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x5AF8;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x5AF9;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x5AFA;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x5AFB;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x5AFC;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x5AFD;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x5AFE;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5AFF;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x5B00;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x5B01;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x5B02;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x5B03;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x5B04;'" select="'yi'"/>
                <xsl:map-entry key="'&#x5B05;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x5B06;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5B07;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5B08;'" select="'rao3	rao2'"/>
                <xsl:map-entry key="'&#x5B09;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5B0A;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5B0B;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x5B0C;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x5B0D;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x5B0E;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x5B0F;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x5B10;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x5B11;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5B12;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x5B13;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x5B14;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x5B15;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x5B16;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5B17;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x5B18;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x5B19;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x5B1A;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5B1B;'" select="'huan2	qiong2	xuan1'"/>
                <xsl:map-entry key="'&#x5B1C;'" select="'xin'"/>
                <xsl:map-entry key="'&#x5B1D;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x5B1E;'" select="'dong3'"/>
                <xsl:map-entry key="'&#x5B1F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5B20;'" select="'can2'"/>
                <xsl:map-entry key="'&#x5B21;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x5B22;'" select="'niang2'"/>
                <xsl:map-entry key="'&#x5B23;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x5B24;'" select="'ma1'"/>
                <xsl:map-entry key="'&#x5B25;'" select="'tiao3'"/>
                <xsl:map-entry key="'&#x5B26;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x5B27;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x5B28;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x5B29;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5B2A;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x5B2B;'" select="'yong'"/>
                <xsl:map-entry key="'&#x5B2C;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x5B2D;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x5B2E;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x5B2F;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x5B30;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x5B31;'" select="'can2'"/>
                <xsl:map-entry key="'&#x5B32;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x5B33;'" select="'wo'"/>
                <xsl:map-entry key="'&#x5B34;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x5B35;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x5B36;'" select="''"/>
                <xsl:map-entry key="'&#x5B37;'" select="'ma1'"/>
                <xsl:map-entry key="'&#x5B38;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x5B39;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x5B3A;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x5B3B;'" select="'du2'"/>
                <xsl:map-entry key="'&#x5B3C;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x5B3D;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x5B3E;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x5B3F;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5B40;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x5B41;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x5B42;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x5B43;'" select="'niang2'"/>
                <xsl:map-entry key="'&#x5B44;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x5B45;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x5B46;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x5B47;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x5B48;'" select="'shuai1'"/>
                <xsl:map-entry key="'&#x5B49;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x5B4A;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x5B4B;'" select="'li2'"/>
                <xsl:map-entry key="'&#x5B4C;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x5B4D;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5B4E;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x5B4F;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x5B50;'" select="'zi3	zi2	zi5'"/>
                <xsl:map-entry key="'&#x5B51;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5B52;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5B53;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5B54;'" select="'kong3'"/>
                <xsl:map-entry key="'&#x5B55;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x5B56;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x5B57;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x5B58;'" select="'cun2'"/>
                <xsl:map-entry key="'&#x5B59;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x5B5A;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5B5B;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x5B5C;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x5B5D;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x5B5E;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x5B5F;'" select="'meng4'"/>
                <xsl:map-entry key="'&#x5B60;'" select="'si4'"/>
                <xsl:map-entry key="'&#x5B61;'" select="'tai1'"/>
                <xsl:map-entry key="'&#x5B62;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x5B63;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5B64;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x5B65;'" select="'nu2'"/>
                <xsl:map-entry key="'&#x5B66;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x5B67;'" select="''"/>
                <xsl:map-entry key="'&#x5B68;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x5B69;'" select="'hai2'"/>
                <xsl:map-entry key="'&#x5B6A;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x5B6B;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x5B6C;'" select="'huai4	nao1'"/>
                <xsl:map-entry key="'&#x5B6D;'" select="'mie1'"/>
                <xsl:map-entry key="'&#x5B6E;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x5B6F;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x5B70;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x5B71;'" select="'chan2	can4'"/>
                <xsl:map-entry key="'&#x5B72;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x5B73;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x5B74;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x5B75;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x5B76;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x5B77;'" select="'li2'"/>
                <xsl:map-entry key="'&#x5B78;'" select="'xue2	xiao2'"/>
                <xsl:map-entry key="'&#x5B79;'" select="'bo4'"/>
                <xsl:map-entry key="'&#x5B7A;'" select="'ru2	ru4'"/>
                <xsl:map-entry key="'&#x5B7B;'" select="'nai2'"/>
                <xsl:map-entry key="'&#x5B7C;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x5B7D;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x5B7E;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x5B7F;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x5B80;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x5B81;'" select="'zhu4	ning2	ning4'"/>
                <xsl:map-entry key="'&#x5B82;'" select="'rong3'"/>
                <xsl:map-entry key="'&#x5B83;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x5B84;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x5B85;'" select="'zhai2	zhe4'"/>
                <xsl:map-entry key="'&#x5B86;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x5B87;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x5B88;'" select="'shou3'"/>
                <xsl:map-entry key="'&#x5B89;'" select="'an1'"/>
                <xsl:map-entry key="'&#x5B8A;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x5B8B;'" select="'song4'"/>
                <xsl:map-entry key="'&#x5B8C;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x5B8D;'" select="'rou4'"/>
                <xsl:map-entry key="'&#x5B8E;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x5B8F;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x5B90;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5B91;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x5B92;'" select="'zhun1'"/>
                <xsl:map-entry key="'&#x5B93;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5B94;'" select="'guai1'"/>
                <xsl:map-entry key="'&#x5B95;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x5B96;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x5B97;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x5B98;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x5B99;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x5B9A;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x5B9B;'" select="'wan3	yuan1'"/>
                <xsl:map-entry key="'&#x5B9C;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5B9D;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x5B9E;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x5B9F;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x5BA0;'" select="'chong3'"/>
                <xsl:map-entry key="'&#x5BA1;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x5BA2;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x5BA3;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x5BA4;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x5BA5;'" select="'you4'"/>
                <xsl:map-entry key="'&#x5BA6;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x5BA7;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5BA8;'" select="'tiao3'"/>
                <xsl:map-entry key="'&#x5BA9;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x5BAA;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x5BAB;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x5BAC;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x5BAD;'" select="'qun2'"/>
                <xsl:map-entry key="'&#x5BAE;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x5BAF;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x5BB0;'" select="'zai3'"/>
                <xsl:map-entry key="'&#x5BB1;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x5BB2;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x5BB3;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x5BB4;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5BB5;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x5BB6;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x5BB7;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x5BB8;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x5BB9;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x5BBA;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x5BBB;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5BBC;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x5BBD;'" select="'kuan1'"/>
                <xsl:map-entry key="'&#x5BBE;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x5BBF;'" select="'su4	xiu3	xiu4'"/>
                <xsl:map-entry key="'&#x5BC0;'" select="'cai3	shen3'"/>
                <xsl:map-entry key="'&#x5BC1;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x5BC2;'" select="'ji4	ji2'"/>
                <xsl:map-entry key="'&#x5BC3;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x5BC4;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5BC5;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5BC6;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5BC7;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x5BC8;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x5BC9;'" select="'he4'"/>
                <xsl:map-entry key="'&#x5BCA;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x5BCB;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x5BCC;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5BCD;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x5BCE;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x5BCF;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x5BD0;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x5BD1;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x5BD2;'" select="'han2'"/>
                <xsl:map-entry key="'&#x5BD3;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5BD4;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x5BD5;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x5BD6;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x5BD7;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x5BD8;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5BD9;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x5BDA;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x5BDB;'" select="'kuan1'"/>
                <xsl:map-entry key="'&#x5BDC;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x5BDD;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x5BDE;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x5BDF;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x5BE0;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5BE1;'" select="'gua3'"/>
                <xsl:map-entry key="'&#x5BE2;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x5BE3;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5BE4;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5BE5;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x5BE6;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x5BE7;'" select="'ning2	ning4'"/>
                <xsl:map-entry key="'&#x5BE8;'" select="'zhai4'"/>
                <xsl:map-entry key="'&#x5BE9;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x5BEA;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x5BEB;'" select="'xie3'"/>
                <xsl:map-entry key="'&#x5BEC;'" select="'kuan1'"/>
                <xsl:map-entry key="'&#x5BED;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5BEE;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x5BEF;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x5BF0;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x5BF1;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5BF2;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5BF3;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x5BF4;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x5BF5;'" select="'chong3'"/>
                <xsl:map-entry key="'&#x5BF6;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x5BF7;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x5BF8;'" select="'cun4'"/>
                <xsl:map-entry key="'&#x5BF9;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x5BFA;'" select="'si4'"/>
                <xsl:map-entry key="'&#x5BFB;'" select="'xun2	xin2'"/>
                <xsl:map-entry key="'&#x5BFC;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x5BFD;'" select="'lu:3	luo1'"/>
                <xsl:map-entry key="'&#x5BFE;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x5BFF;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x5C00;'" select="'po3'"/>
                <xsl:map-entry key="'&#x5C01;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x5C02;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x5C03;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x5C04;'" select="'she4'"/>
                <xsl:map-entry key="'&#x5C05;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x5C06;'" select="'jiang1	jiang4'"/>
                <xsl:map-entry key="'&#x5C07;'" select="'jiang1	jiang4'"/>
                <xsl:map-entry key="'&#x5C08;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x5C09;'" select="'wei4	yu4'"/>
                <xsl:map-entry key="'&#x5C0A;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x5C0B;'" select="'xun2	xin2'"/>
                <xsl:map-entry key="'&#x5C0C;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x5C0D;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x5C0E;'" select="'dao3	dao4'"/>
                <xsl:map-entry key="'&#x5C0F;'" select="'xiao3'"/>
                <xsl:map-entry key="'&#x5C10;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x5C11;'" select="'shao3	shao4'"/>
                <xsl:map-entry key="'&#x5C12;'" select="'er3'"/>
                <xsl:map-entry key="'&#x5C13;'" select="'er3'"/>
                <xsl:map-entry key="'&#x5C14;'" select="'er3'"/>
                <xsl:map-entry key="'&#x5C15;'" select="'ga3'"/>
                <xsl:map-entry key="'&#x5C16;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x5C17;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x5C18;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x5C19;'" select="'shang4'"/>
                <xsl:map-entry key="'&#x5C1A;'" select="'shang4'"/>
                <xsl:map-entry key="'&#x5C1B;'" select="'mo'"/>
                <xsl:map-entry key="'&#x5C1C;'" select="'ga2'"/>
                <xsl:map-entry key="'&#x5C1D;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x5C1E;'" select="'liao2	liao3'"/>
                <xsl:map-entry key="'&#x5C1F;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x5C20;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x5C21;'" select="''"/>
                <xsl:map-entry key="'&#x5C22;'" select="'wang1	you2'"/>
                <xsl:map-entry key="'&#x5C23;'" select="'wang1	you2'"/>
                <xsl:map-entry key="'&#x5C24;'" select="'you2'"/>
                <xsl:map-entry key="'&#x5C25;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x5C26;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x5C27;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5C28;'" select="'mang2	pang2'"/>
                <xsl:map-entry key="'&#x5C29;'" select="'wang1'"/>
                <xsl:map-entry key="'&#x5C2A;'" select="'wang1'"/>
                <xsl:map-entry key="'&#x5C2B;'" select="'wang1'"/>
                <xsl:map-entry key="'&#x5C2C;'" select="'ga4'"/>
                <xsl:map-entry key="'&#x5C2D;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5C2E;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x5C2F;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x5C30;'" select="'zhong3	zhong4'"/>
                <xsl:map-entry key="'&#x5C31;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x5C32;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x5C33;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x5C34;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x5C35;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x5C36;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x5C37;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x5C38;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x5C39;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x5C3A;'" select="'chi3	che3'"/>
                <xsl:map-entry key="'&#x5C3B;'" select="'kao1'"/>
                <xsl:map-entry key="'&#x5C3C;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x5C3D;'" select="'jin3	jin4'"/>
                <xsl:map-entry key="'&#x5C3E;'" select="'wei3	yi3'"/>
                <xsl:map-entry key="'&#x5C3F;'" select="'niao4	ni4	sui1'"/>
                <xsl:map-entry key="'&#x5C40;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x5C41;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x5C42;'" select="'ceng2'"/>
                <xsl:map-entry key="'&#x5C43;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x5C44;'" select="'bi1	bi3'"/>
                <xsl:map-entry key="'&#x5C45;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x5C46;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x5C47;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x5C48;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x5C49;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x5C4A;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x5C4B;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x5C4C;'" select="'diao3'"/>
                <xsl:map-entry key="'&#x5C4D;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x5C4E;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x5C4F;'" select="'ping2	bing3'"/>
                <xsl:map-entry key="'&#x5C50;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x5C51;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5C52;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x5C53;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x5C54;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x5C55;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x5C56;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5C57;'" select="''"/>
                <xsl:map-entry key="'&#x5C58;'" select="'man3'"/>
                <xsl:map-entry key="'&#x5C59;'" select="'e1'"/>
                <xsl:map-entry key="'&#x5C5A;'" select="'lou4'"/>
                <xsl:map-entry key="'&#x5C5B;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x5C5C;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x5C5D;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x5C5E;'" select="'shu3	zhu3'"/>
                <xsl:map-entry key="'&#x5C5F;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5C60;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5C61;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x5C62;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x5C63;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x5C64;'" select="'ceng2'"/>
                <xsl:map-entry key="'&#x5C65;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x5C66;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5C67;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5C68;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5C69;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5C6A;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x5C6B;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5C6C;'" select="'shu3	zhu3'"/>
                <xsl:map-entry key="'&#x5C6D;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x5C6E;'" select="'che4'"/>
                <xsl:map-entry key="'&#x5C6F;'" select="'tun2	zhun1'"/>
                <xsl:map-entry key="'&#x5C70;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x5C71;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x5C72;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x5C73;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x5C74;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5C75;'" select="'e4'"/>
                <xsl:map-entry key="'&#x5C76;'" select="''"/>
                <xsl:map-entry key="'&#x5C77;'" select="''"/>
                <xsl:map-entry key="'&#x5C78;'" select="'long2'"/>
                <xsl:map-entry key="'&#x5C79;'" select="'yi4	ge1'"/>
                <xsl:map-entry key="'&#x5C7A;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5C7B;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x5C7C;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5C7D;'" select="'han4'"/>
                <xsl:map-entry key="'&#x5C7E;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x5C7F;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x5C80;'" select="'chu1'"/>
                <xsl:map-entry key="'&#x5C81;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x5C82;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x5C83;'" select="''"/>
                <xsl:map-entry key="'&#x5C84;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x5C85;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x5C86;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x5C87;'" select="'ang2'"/>
                <xsl:map-entry key="'&#x5C88;'" select="'ya2	xia1'"/>
                <xsl:map-entry key="'&#x5C89;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5C8A;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5C8B;'" select="'e4'"/>
                <xsl:map-entry key="'&#x5C8C;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5C8D;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x5C8E;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x5C8F;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x5C90;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x5C91;'" select="'cen2'"/>
                <xsl:map-entry key="'&#x5C92;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x5C93;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x5C94;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x5C95;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x5C96;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x5C97;'" select="'gang3'"/>
                <xsl:map-entry key="'&#x5C98;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x5C99;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x5C9A;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x5C9B;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x5C9C;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x5C9D;'" select="'zhai3'"/>
                <xsl:map-entry key="'&#x5C9E;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x5C9F;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x5CA0;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5CA1;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x5CA2;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x5CA3;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x5CA4;'" select="'xue4'"/>
                <xsl:map-entry key="'&#x5CA5;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x5CA6;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5CA7;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x5CA8;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x5CA9;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5CAA;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5CAB;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x5CAC;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x5CAD;'" select="'ling2	ling3'"/>
                <xsl:map-entry key="'&#x5CAE;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x5CAF;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x5CB0;'" select="'you3'"/>
                <xsl:map-entry key="'&#x5CB1;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x5CB2;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x5CB3;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x5CB4;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x5CB5;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x5CB6;'" select="'po4'"/>
                <xsl:map-entry key="'&#x5CB7;'" select="'min2'"/>
                <xsl:map-entry key="'&#x5CB8;'" select="'an4'"/>
                <xsl:map-entry key="'&#x5CB9;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x5CBA;'" select="'ling3'"/>
                <xsl:map-entry key="'&#x5CBB;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x5CBC;'" select="''"/>
                <xsl:map-entry key="'&#x5CBD;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x5CBE;'" select="''"/>
                <xsl:map-entry key="'&#x5CBF;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x5CC0;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x5CC1;'" select="'mao3'"/>
                <xsl:map-entry key="'&#x5CC2;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x5CC3;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x5CC4;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5CC5;'" select="''"/>
                <xsl:map-entry key="'&#x5CC6;'" select="'he1'"/>
                <xsl:map-entry key="'&#x5CC7;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x5CC8;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x5CC9;'" select="'e1'"/>
                <xsl:map-entry key="'&#x5CCA;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5CCB;'" select="'xun2	xun1'"/>
                <xsl:map-entry key="'&#x5CCC;'" select="'die2'"/>
                <xsl:map-entry key="'&#x5CCD;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x5CCE;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x5CCF;'" select="'er3'"/>
                <xsl:map-entry key="'&#x5CD0;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x5CD1;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x5CD2;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x5CD3;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5CD4;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x5CD5;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x5CD6;'" select="'an1'"/>
                <xsl:map-entry key="'&#x5CD7;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x5CD8;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5CD9;'" select="'zhi4	shi4'"/>
                <xsl:map-entry key="'&#x5CDA;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5CDB;'" select="'li3'"/>
                <xsl:map-entry key="'&#x5CDC;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x5CDD;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x5CDE;'" select="'kui3'"/>
                <xsl:map-entry key="'&#x5CDF;'" select="'you4'"/>
                <xsl:map-entry key="'&#x5CE0;'" select="''"/>
                <xsl:map-entry key="'&#x5CE1;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x5CE2;'" select="'li3'"/>
                <xsl:map-entry key="'&#x5CE3;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5CE4;'" select="'jiao4	jiao2	qiao2'"/>
                <xsl:map-entry key="'&#x5CE5;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x5CE6;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x5CE7;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x5CE8;'" select="'e2'"/>
                <xsl:map-entry key="'&#x5CE9;'" select="'e2'"/>
                <xsl:map-entry key="'&#x5CEA;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5CEB;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x5CEC;'" select="'bu1'"/>
                <xsl:map-entry key="'&#x5CED;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x5CEE;'" select="'qun1'"/>
                <xsl:map-entry key="'&#x5CEF;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x5CF0;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x5CF1;'" select="'nao2	nao1'"/>
                <xsl:map-entry key="'&#x5CF2;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5CF3;'" select="'you2'"/>
                <xsl:map-entry key="'&#x5CF4;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x5CF5;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x5CF6;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x5CF7;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x5CF8;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x5CF9;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5CFA;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x5CFB;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x5CFC;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x5CFD;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x5CFE;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x5CFF;'" select="'wu2	yu1'"/>
                <xsl:map-entry key="'&#x5D00;'" select="'lang4'"/>
                <xsl:map-entry key="'&#x5D01;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x5D02;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x5D03;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x5D04;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x5D05;'" select="'que4'"/>
                <xsl:map-entry key="'&#x5D06;'" select="'kong1'"/>
                <xsl:map-entry key="'&#x5D07;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x5D08;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x5D09;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x5D0A;'" select="'lin'"/>
                <xsl:map-entry key="'&#x5D0B;'" select="'hua2	hua4'"/>
                <xsl:map-entry key="'&#x5D0C;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x5D0D;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x5D0E;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x5D0F;'" select="'min2'"/>
                <xsl:map-entry key="'&#x5D10;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x5D11;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x5D12;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x5D13;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x5D14;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x5D15;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x5D16;'" select="'ya2	ai2'"/>
                <xsl:map-entry key="'&#x5D17;'" select="'gang3	gang1'"/>
                <xsl:map-entry key="'&#x5D18;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x5D19;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x5D1A;'" select="'leng2'"/>
                <xsl:map-entry key="'&#x5D1B;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5D1C;'" select="'duo1'"/>
                <xsl:map-entry key="'&#x5D1D;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x5D1E;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x5D1F;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5D20;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x5D21;'" select="'han2'"/>
                <xsl:map-entry key="'&#x5D22;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x5D23;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x5D24;'" select="'yao2	xiao2'"/>
                <xsl:map-entry key="'&#x5D25;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x5D26;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x5D27;'" select="'song1'"/>
                <xsl:map-entry key="'&#x5D28;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5D29;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x5D2A;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x5D2B;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5D2C;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x5D2D;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x5D2E;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x5D2F;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x5D30;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x5D31;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x5D32;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x5D33;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5D34;'" select="'wei1	wai3'"/>
                <xsl:map-entry key="'&#x5D35;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x5D36;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x5D37;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x5D38;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x5D39;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x5D3A;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x5D3B;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5D3C;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x5D3D;'" select="'zai3'"/>
                <xsl:map-entry key="'&#x5D3E;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x5D3F;'" select="'e4'"/>
                <xsl:map-entry key="'&#x5D40;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x5D41;'" select="'kan1'"/>
                <xsl:map-entry key="'&#x5D42;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x5D43;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5D44;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x5D45;'" select="'gan1	gan4'"/>
                <xsl:map-entry key="'&#x5D46;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x5D47;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x5D48;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x5D49;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x5D4A;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x5D4B;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x5D4C;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x5D4D;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5D4E;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5D4F;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x5D50;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x5D51;'" select="'jie2	he2'"/>
                <xsl:map-entry key="'&#x5D52;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5D53;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5D54;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x5D55;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x5D56;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x5D57;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x5D58;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x5D59;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x5D5A;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x5D5B;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x5D5C;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x5D5D;'" select="'lou3'"/>
                <xsl:map-entry key="'&#x5D5E;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5D5F;'" select="'dui1'"/>
                <xsl:map-entry key="'&#x5D60;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5D61;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x5D62;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x5D63;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x5D64;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x5D65;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5D66;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x5D67;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x5D68;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x5D69;'" select="'song1'"/>
                <xsl:map-entry key="'&#x5D6A;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x5D6B;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x5D6C;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x5D6D;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x5D6E;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x5D6F;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x5D70;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x5D71;'" select="'yong2'"/>
                <xsl:map-entry key="'&#x5D72;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x5D73;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x5D74;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5D75;'" select="''"/>
                <xsl:map-entry key="'&#x5D76;'" select="''"/>
                <xsl:map-entry key="'&#x5D77;'" select="'song3'"/>
                <xsl:map-entry key="'&#x5D78;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x5D79;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x5D7A;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x5D7B;'" select="'kang'"/>
                <xsl:map-entry key="'&#x5D7C;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x5D7D;'" select="'di4'"/>
                <xsl:map-entry key="'&#x5D7E;'" select="'cen1'"/>
                <xsl:map-entry key="'&#x5D7F;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x5D80;'" select="'tu1	die2'"/>
                <xsl:map-entry key="'&#x5D81;'" select="'lou3'"/>
                <xsl:map-entry key="'&#x5D82;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x5D83;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x5D84;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x5D85;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x5D86;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x5D87;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x5D88;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x5D89;'" select="'zui1'"/>
                <xsl:map-entry key="'&#x5D8A;'" select="'zui3'"/>
                <xsl:map-entry key="'&#x5D8B;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x5D8C;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x5D8D;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x5D8E;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5D8F;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x5D90;'" select="'long2'"/>
                <xsl:map-entry key="'&#x5D91;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x5D92;'" select="'ceng2'"/>
                <xsl:map-entry key="'&#x5D93;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x5D94;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x5D95;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x5D96;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5D97;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x5D98;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x5D99;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x5D9A;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x5D9B;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x5D9C;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x5D9D;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x5D9E;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x5D9F;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x5DA0;'" select="'jiao4	jiao2	qiao2'"/>
                <xsl:map-entry key="'&#x5DA1;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x5DA2;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5DA3;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x5DA4;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5DA5;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5DA6;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x5DA7;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5DA8;'" select="'xue1'"/>
                <xsl:map-entry key="'&#x5DA9;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x5DAA;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x5DAB;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x5DAC;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x5DAD;'" select="'e4'"/>
                <xsl:map-entry key="'&#x5DAE;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x5DAF;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5DB0;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5DB1;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x5DB2;'" select="'sui3'"/>
                <xsl:map-entry key="'&#x5DB3;'" select="'di4'"/>
                <xsl:map-entry key="'&#x5DB4;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x5DB5;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x5DB6;'" select="''"/>
                <xsl:map-entry key="'&#x5DB7;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5DB8;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x5DB9;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x5DBA;'" select="'ling3'"/>
                <xsl:map-entry key="'&#x5DBB;'" select="'za2'"/>
                <xsl:map-entry key="'&#x5DBC;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x5DBD;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x5DBE;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x5DBF;'" select="''"/>
                <xsl:map-entry key="'&#x5DC0;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x5DC1;'" select="'li4'"/>
                <xsl:map-entry key="'&#x5DC2;'" select="'sui3	xi1'"/>
                <xsl:map-entry key="'&#x5DC3;'" select="'long2'"/>
                <xsl:map-entry key="'&#x5DC4;'" select="'long2'"/>
                <xsl:map-entry key="'&#x5DC5;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x5DC6;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x5DC7;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5DC8;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x5DC9;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x5DCA;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x5DCB;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x5DCC;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5DCD;'" select="'wei1	wei2'"/>
                <xsl:map-entry key="'&#x5DCE;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x5DCF;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x5DD0;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x5DD1;'" select="'cuan2'"/>
                <xsl:map-entry key="'&#x5DD2;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x5DD3;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x5DD4;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x5DD5;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x5DD6;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5DD7;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5DD8;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5DD9;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x5DDA;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5DDB;'" select="'chuan1'"/>
                <xsl:map-entry key="'&#x5DDC;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x5DDD;'" select="'chuan1'"/>
                <xsl:map-entry key="'&#x5DDE;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x5DDF;'" select="'huang1'"/>
                <xsl:map-entry key="'&#x5DE0;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x5DE1;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x5DE2;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x5DE3;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x5DE4;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x5DE5;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x5DE6;'" select="'zuo3'"/>
                <xsl:map-entry key="'&#x5DE7;'" select="'qiao3'"/>
                <xsl:map-entry key="'&#x5DE8;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x5DE9;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x5DEA;'" select="''"/>
                <xsl:map-entry key="'&#x5DEB;'" select="'wu1	wu2'"/>
                <xsl:map-entry key="'&#x5DEC;'" select="''"/>
                <xsl:map-entry key="'&#x5DED;'" select="''"/>
                <xsl:map-entry key="'&#x5DEE;'" select="'chai1	cha1	cha4	ci1'"/>
                <xsl:map-entry key="'&#x5DEF;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x5DF0;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x5DF1;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x5DF2;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x5DF3;'" select="'si4'"/>
                <xsl:map-entry key="'&#x5DF4;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x5DF5;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x5DF6;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x5DF7;'" select="'xiang4	hang4'"/>
                <xsl:map-entry key="'&#x5DF8;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5DF9;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x5DFA;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x5DFB;'" select="'juan4	juan3'"/>
                <xsl:map-entry key="'&#x5DFC;'" select="''"/>
                <xsl:map-entry key="'&#x5DFD;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x5DFE;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x5DFF;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5E00;'" select="'za1'"/>
                <xsl:map-entry key="'&#x5E01;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5E02;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x5E03;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x5E04;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x5E05;'" select="'shuai4'"/>
                <xsl:map-entry key="'&#x5E06;'" select="'fan1	fan2'"/>
                <xsl:map-entry key="'&#x5E07;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x5E08;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x5E09;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x5E0A;'" select="'pa4'"/>
                <xsl:map-entry key="'&#x5E0B;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x5E0C;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5E0D;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x5E0E;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x5E0F;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x5E10;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x5E11;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x5E12;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x5E13;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x5E14;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x5E15;'" select="'pa4'"/>
                <xsl:map-entry key="'&#x5E16;'" select="'tie1	tie3	tie4'"/>
                <xsl:map-entry key="'&#x5E17;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5E18;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5E19;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5E1A;'" select="'zhou3'"/>
                <xsl:map-entry key="'&#x5E1B;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x5E1C;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5E1D;'" select="'di4'"/>
                <xsl:map-entry key="'&#x5E1E;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x5E1F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5E20;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5E21;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x5E22;'" select="'qia4'"/>
                <xsl:map-entry key="'&#x5E23;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x5E24;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x5E25;'" select="'shuai4	shuo4'"/>
                <xsl:map-entry key="'&#x5E26;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x5E27;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x5E28;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x5E29;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x5E2A;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x5E2B;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x5E2C;'" select="'qun2'"/>
                <xsl:map-entry key="'&#x5E2D;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x5E2E;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x5E2F;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x5E30;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x5E31;'" select="'chou2	dao4'"/>
                <xsl:map-entry key="'&#x5E32;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x5E33;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x5E34;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x5E35;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x5E36;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x5E37;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x5E38;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x5E39;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x5E3A;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x5E3B;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x5E3C;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x5E3D;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x5E3E;'" select="'du3'"/>
                <xsl:map-entry key="'&#x5E3F;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x5E40;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x5E41;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x5E42;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5E43;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x5E44;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x5E45;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5E46;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5E47;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x5E48;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x5E49;'" select="''"/>
                <xsl:map-entry key="'&#x5E4A;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x5E4B;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x5E4C;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x5E4D;'" select="'dao1	tao1'"/>
                <xsl:map-entry key="'&#x5E4E;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5E4F;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x5E50;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x5E51;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x5E52;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x5E53;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x5E54;'" select="'man4'"/>
                <xsl:map-entry key="'&#x5E55;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x5E56;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x5E57;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x5E58;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x5E59;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x5E5A;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x5E5B;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x5E5C;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x5E5D;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x5E5E;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5E5F;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5E60;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5E61;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x5E62;'" select="'chuang2	zhuang4'"/>
                <xsl:map-entry key="'&#x5E63;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5E64;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5E65;'" select="''"/>
                <xsl:map-entry key="'&#x5E66;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x5E67;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x5E68;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x5E69;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x5E6A;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x5E6B;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x5E6C;'" select="'chou2	dao4'"/>
                <xsl:map-entry key="'&#x5E6D;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x5E6E;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x5E6F;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x5E70;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x5E71;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x5E72;'" select="'gan1	gan4'"/>
                <xsl:map-entry key="'&#x5E73;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x5E74;'" select="'nian2'"/>
                <xsl:map-entry key="'&#x5E75;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x5E76;'" select="'bing4	bing1'"/>
                <xsl:map-entry key="'&#x5E77;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x5E78;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x5E79;'" select="'gan4	gan1'"/>
                <xsl:map-entry key="'&#x5E7A;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x5E7B;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x5E7C;'" select="'you4'"/>
                <xsl:map-entry key="'&#x5E7D;'" select="'you1'"/>
                <xsl:map-entry key="'&#x5E7E;'" select="'ji3	ji1'"/>
                <xsl:map-entry key="'&#x5E7F;'" select="'guang3'"/>
                <xsl:map-entry key="'&#x5E80;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x5E81;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x5E82;'" select="'ze4'"/>
                <xsl:map-entry key="'&#x5E83;'" select="'guang3	an1'"/>
                <xsl:map-entry key="'&#x5E84;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x5E85;'" select="'mo5'"/>
                <xsl:map-entry key="'&#x5E86;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x5E87;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5E88;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x5E89;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x5E8A;'" select="'chuang2'"/>
                <xsl:map-entry key="'&#x5E8B;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x5E8C;'" select="'ya3'"/>
                <xsl:map-entry key="'&#x5E8D;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x5E8E;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x5E8F;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x5E90;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x5E91;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x5E92;'" select="''"/>
                <xsl:map-entry key="'&#x5E93;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x5E94;'" select="'ying4	ying1'"/>
                <xsl:map-entry key="'&#x5E95;'" select="'di3	de5'"/>
                <xsl:map-entry key="'&#x5E96;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x5E97;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x5E98;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x5E99;'" select="'miao4'"/>
                <xsl:map-entry key="'&#x5E9A;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x5E9B;'" select="'ci1'"/>
                <xsl:map-entry key="'&#x5E9C;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x5E9D;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x5E9E;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x5E9F;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x5EA0;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x5EA1;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x5EA2;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5EA3;'" select="'tiao1'"/>
                <xsl:map-entry key="'&#x5EA4;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5EA5;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x5EA6;'" select="'du4	duo2'"/>
                <xsl:map-entry key="'&#x5EA7;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x5EA8;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x5EA9;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5EAA;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x5EAB;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x5EAC;'" select="'pang2	mang3'"/>
                <xsl:map-entry key="'&#x5EAD;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x5EAE;'" select="'you3'"/>
                <xsl:map-entry key="'&#x5EAF;'" select="'bu1'"/>
                <xsl:map-entry key="'&#x5EB0;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x5EB1;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x5EB2;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x5EB3;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5EB4;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5EB5;'" select="'an1'"/>
                <xsl:map-entry key="'&#x5EB6;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x5EB7;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x5EB8;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x5EB9;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x5EBA;'" select="'song1'"/>
                <xsl:map-entry key="'&#x5EBB;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x5EBC;'" select="'qing3'"/>
                <xsl:map-entry key="'&#x5EBD;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5EBE;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x5EBF;'" select="'miao4'"/>
                <xsl:map-entry key="'&#x5EC0;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x5EC1;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x5EC2;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x5EC3;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x5EC4;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x5EC5;'" select="'he2'"/>
                <xsl:map-entry key="'&#x5EC6;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5EC7;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x5EC8;'" select="'sha4	xia4'"/>
                <xsl:map-entry key="'&#x5EC9;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x5ECA;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x5ECB;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x5ECC;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x5ECD;'" select="'pou3'"/>
                <xsl:map-entry key="'&#x5ECE;'" select="'qing3'"/>
                <xsl:map-entry key="'&#x5ECF;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x5ED0;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x5ED1;'" select="'qin2	jin3'"/>
                <xsl:map-entry key="'&#x5ED2;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x5ED3;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x5ED4;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x5ED5;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x5ED6;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x5ED7;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x5ED8;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x5ED9;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5EDA;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x5EDB;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x5EDC;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5EDD;'" select="'si1'"/>
                <xsl:map-entry key="'&#x5EDE;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x5EDF;'" select="'miao4'"/>
                <xsl:map-entry key="'&#x5EE0;'" select="'chang3	an1'"/>
                <xsl:map-entry key="'&#x5EE1;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x5EE2;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x5EE3;'" select="'guang3	an1'"/>
                <xsl:map-entry key="'&#x5EE4;'" select="''"/>
                <xsl:map-entry key="'&#x5EE5;'" select="'guai4'"/>
                <xsl:map-entry key="'&#x5EE6;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5EE7;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x5EE8;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5EE9;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x5EEA;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x5EEB;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x5EEC;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x5EED;'" select="''"/>
                <xsl:map-entry key="'&#x5EEE;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x5EEF;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x5EF0;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x5EF1;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x5EF2;'" select="'li2'"/>
                <xsl:map-entry key="'&#x5EF3;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x5EF4;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x5EF5;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x5EF6;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x5EF7;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x5EF8;'" select="'di2'"/>
                <xsl:map-entry key="'&#x5EF9;'" select="'po4'"/>
                <xsl:map-entry key="'&#x5EFA;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x5EFB;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x5EFC;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x5EFD;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x5EFE;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x5EFF;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x5F00;'" select="'kai1'"/>
                <xsl:map-entry key="'&#x5F01;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x5F02;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5F03;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x5F04;'" select="'nong4	long4'"/>
                <xsl:map-entry key="'&#x5F05;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x5F06;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x5F07;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x5F08;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5F09;'" select="'zang4	zhuang3'"/>
                <xsl:map-entry key="'&#x5F0A;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5F0B;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5F0C;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x5F0D;'" select="'er4'"/>
                <xsl:map-entry key="'&#x5F0E;'" select="'san1'"/>
                <xsl:map-entry key="'&#x5F0F;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x5F10;'" select="'er4'"/>
                <xsl:map-entry key="'&#x5F11;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x5F12;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x5F13;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x5F14;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x5F15;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x5F16;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x5F17;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5F18;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x5F19;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x5F1A;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x5F1B;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x5F1C;'" select="'qiang2	qiang3'"/>
                <xsl:map-entry key="'&#x5F1D;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x5F1E;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x5F1F;'" select="'di4'"/>
                <xsl:map-entry key="'&#x5F20;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x5F21;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5F22;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x5F23;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x5F24;'" select="'di3'"/>
                <xsl:map-entry key="'&#x5F25;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x5F26;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x5F27;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x5F28;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x5F29;'" select="'nu3'"/>
                <xsl:map-entry key="'&#x5F2A;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x5F2B;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x5F2C;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5F2D;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x5F2E;'" select="'quan1'"/>
                <xsl:map-entry key="'&#x5F2F;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x5F30;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x5F31;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x5F32;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x5F33;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x5F34;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x5F35;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x5F36;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x5F37;'" select="'qiang2	jiang4	qiang3'"/>
                <xsl:map-entry key="'&#x5F38;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x5F39;'" select="'tan'"/>
                <xsl:map-entry key="'&#x5F3A;'" select="'qiang2	qiang3'"/>
                <xsl:map-entry key="'&#x5F3B;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5F3C;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5F3D;'" select="'she4'"/>
                <xsl:map-entry key="'&#x5F3E;'" select="'dan4	tan2'"/>
                <xsl:map-entry key="'&#x5F3F;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x5F40;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x5F41;'" select="''"/>
                <xsl:map-entry key="'&#x5F42;'" select="'fa1'"/>
                <xsl:map-entry key="'&#x5F43;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5F44;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x5F45;'" select="''"/>
                <xsl:map-entry key="'&#x5F46;'" select="'bie4'"/>
                <xsl:map-entry key="'&#x5F47;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x5F48;'" select="'dan4	tan2'"/>
                <xsl:map-entry key="'&#x5F49;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x5F4A;'" select="'qiang2	jiang4'"/>
                <xsl:map-entry key="'&#x5F4B;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x5F4C;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x5F4D;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x5F4E;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x5F4F;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x5F50;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5F51;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5F52;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x5F53;'" select="'dang1	dang4'"/>
                <xsl:map-entry key="'&#x5F54;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x5F55;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x5F56;'" select="'tuan4'"/>
                <xsl:map-entry key="'&#x5F57;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5F58;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5F59;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5F5A;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x5F5B;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5F5C;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5F5D;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5F5E;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x5F5F;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x5F60;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x5F61;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x5F62;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x5F63;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x5F64;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x5F65;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5F66;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x5F67;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5F68;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x5F69;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x5F6A;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x5F6B;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x5F6C;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x5F6D;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x5F6E;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x5F6F;'" select="'piao4'"/>
                <xsl:map-entry key="'&#x5F70;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x5F71;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x5F72;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x5F73;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x5F74;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x5F75;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x5F76;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5F77;'" select="'pang2	fang3'"/>
                <xsl:map-entry key="'&#x5F78;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x5F79;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5F7A;'" select="'wang2'"/>
                <xsl:map-entry key="'&#x5F7B;'" select="'che4'"/>
                <xsl:map-entry key="'&#x5F7C;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x5F7D;'" select="'di1'"/>
                <xsl:map-entry key="'&#x5F7E;'" select="'ling3'"/>
                <xsl:map-entry key="'&#x5F7F;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x5F80;'" select="'wang3	wang4'"/>
                <xsl:map-entry key="'&#x5F81;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x5F82;'" select="'cu2'"/>
                <xsl:map-entry key="'&#x5F83;'" select="'wang3	wang4'"/>
                <xsl:map-entry key="'&#x5F84;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x5F85;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x5F86;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5F87;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x5F88;'" select="'hen3'"/>
                <xsl:map-entry key="'&#x5F89;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x5F8A;'" select="'huai2	hui2'"/>
                <xsl:map-entry key="'&#x5F8B;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x5F8C;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x5F8D;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x5F8E;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x5F8F;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5F90;'" select="'xu2'"/>
                <xsl:map-entry key="'&#x5F91;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x5F92;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x5F93;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x5F94;'" select="''"/>
                <xsl:map-entry key="'&#x5F95;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x5F96;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x5F97;'" select="'de2	de5	dei3'"/>
                <xsl:map-entry key="'&#x5F98;'" select="'pai2'"/>
                <xsl:map-entry key="'&#x5F99;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x5F9A;'" select="''"/>
                <xsl:map-entry key="'&#x5F9B;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x5F9C;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x5F9D;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5F9E;'" select="'cong2	cong1	zong4'"/>
                <xsl:map-entry key="'&#x5F9F;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x5FA0;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x5FA1;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5FA2;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x5FA3;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x5FA4;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x5FA5;'" select="'chi2	shi4'"/>
                <xsl:map-entry key="'&#x5FA6;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x5FA7;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x5FA8;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x5FA9;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x5FAA;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x5FAB;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x5FAC;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x5FAD;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x5FAE;'" select="'wei1	wei2'"/>
                <xsl:map-entry key="'&#x5FAF;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5FB0;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x5FB1;'" select="'piao4'"/>
                <xsl:map-entry key="'&#x5FB2;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x5FB3;'" select="'de2'"/>
                <xsl:map-entry key="'&#x5FB4;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x5FB5;'" select="'zheng1	zhi3'"/>
                <xsl:map-entry key="'&#x5FB6;'" select="'bie2'"/>
                <xsl:map-entry key="'&#x5FB7;'" select="'de2'"/>
                <xsl:map-entry key="'&#x5FB8;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x5FB9;'" select="'che4'"/>
                <xsl:map-entry key="'&#x5FBA;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x5FBB;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x5FBC;'" select="'jiao4	jia3	jiao3'"/>
                <xsl:map-entry key="'&#x5FBD;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x5FBE;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x5FBF;'" select="'long4'"/>
                <xsl:map-entry key="'&#x5FC0;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x5FC1;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x5FC2;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x5FC3;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x5FC4;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x5FC5;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x5FC6;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x5FC7;'" select="'le4'"/>
                <xsl:map-entry key="'&#x5FC8;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x5FC9;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x5FCA;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x5FCB;'" select="'gai3'"/>
                <xsl:map-entry key="'&#x5FCC;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x5FCD;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x5FCE;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x5FCF;'" select="'chan4	qian1	qian3'"/>
                <xsl:map-entry key="'&#x5FD0;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x5FD1;'" select="'te4'"/>
                <xsl:map-entry key="'&#x5FD2;'" select="'te4	tei1	tui1'"/>
                <xsl:map-entry key="'&#x5FD3;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x5FD4;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x5FD5;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x5FD6;'" select="'cun3'"/>
                <xsl:map-entry key="'&#x5FD7;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5FD8;'" select="'wang4	wang2'"/>
                <xsl:map-entry key="'&#x5FD9;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x5FDA;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x5FDB;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x5FDC;'" select="'ying1	ying4'"/>
                <xsl:map-entry key="'&#x5FDD;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x5FDE;'" select="'min2'"/>
                <xsl:map-entry key="'&#x5FDF;'" select="'min2'"/>
                <xsl:map-entry key="'&#x5FE0;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x5FE1;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x5FE2;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x5FE3;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x5FE4;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x5FE5;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x5FE6;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x5FE7;'" select="'you1'"/>
                <xsl:map-entry key="'&#x5FE8;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x5FE9;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x5FEA;'" select="'zhong1	song1'"/>
                <xsl:map-entry key="'&#x5FEB;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x5FEC;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x5FED;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x5FEE;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x5FEF;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x5FF0;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x5FF1;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x5FF2;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x5FF3;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x5FF4;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x5FF5;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x5FF6;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x5FF7;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x5FF8;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x5FF9;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x5FFA;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x5FFB;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x5FFC;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x5FFD;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x5FFE;'" select="'kai4'"/>
                <xsl:map-entry key="'&#x5FFF;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x6000;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x6001;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x6002;'" select="'song3'"/>
                <xsl:map-entry key="'&#x6003;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x6004;'" select="'ou4'"/>
                <xsl:map-entry key="'&#x6005;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x6006;'" select="'chuang4'"/>
                <xsl:map-entry key="'&#x6007;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x6008;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6009;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x600A;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x600B;'" select="'min2'"/>
                <xsl:map-entry key="'&#x600C;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x600D;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x600E;'" select="'zen3	ze3'"/>
                <xsl:map-entry key="'&#x600F;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x6010;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x6011;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x6012;'" select="'nu4'"/>
                <xsl:map-entry key="'&#x6013;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x6014;'" select="'zheng1	zheng4'"/>
                <xsl:map-entry key="'&#x6015;'" select="'pa4'"/>
                <xsl:map-entry key="'&#x6016;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x6017;'" select="'tie1'"/>
                <xsl:map-entry key="'&#x6018;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6019;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x601A;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x601B;'" select="'da2'"/>
                <xsl:map-entry key="'&#x601C;'" select="'lian2	ling2'"/>
                <xsl:map-entry key="'&#x601D;'" select="'si1	sai1'"/>
                <xsl:map-entry key="'&#x601E;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x601F;'" select="'di4'"/>
                <xsl:map-entry key="'&#x6020;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x6021;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6022;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x6023;'" select="'you2'"/>
                <xsl:map-entry key="'&#x6024;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x6025;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6026;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x6027;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x6028;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x6029;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x602A;'" select="'guai4'"/>
                <xsl:map-entry key="'&#x602B;'" select="'fu2	fei4'"/>
                <xsl:map-entry key="'&#x602C;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x602D;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x602E;'" select="'you1'"/>
                <xsl:map-entry key="'&#x602F;'" select="'qie4	que4'"/>
                <xsl:map-entry key="'&#x6030;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x6031;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x6032;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x6033;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x6034;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6035;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x6036;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x6037;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6038;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6039;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x603A;'" select="''"/>
                <xsl:map-entry key="'&#x603B;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x603C;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x603D;'" select="''"/>
                <xsl:map-entry key="'&#x603E;'" select="''"/>
                <xsl:map-entry key="'&#x603F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6040;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x6041;'" select="'nen4'"/>
                <xsl:map-entry key="'&#x6042;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6043;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x6044;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x6045;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x6046;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x6047;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x6048;'" select="'mou2	mu4'"/>
                <xsl:map-entry key="'&#x6049;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x604A;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x604B;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x604C;'" select="'tiao1'"/>
                <xsl:map-entry key="'&#x604D;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x604E;'" select="'die2'"/>
                <xsl:map-entry key="'&#x604F;'" select="'hao3'"/>
                <xsl:map-entry key="'&#x6050;'" select="'kong3'"/>
                <xsl:map-entry key="'&#x6051;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x6052;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x6053;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6054;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x6055;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6056;'" select="'sai1	si1'"/>
                <xsl:map-entry key="'&#x6057;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x6058;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x6059;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x605A;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x605B;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x605C;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x605D;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x605E;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x605F;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x6060;'" select="'guai4'"/>
                <xsl:map-entry key="'&#x6061;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x6062;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x6063;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x6064;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6065;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x6066;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x6067;'" select="'nu:4'"/>
                <xsl:map-entry key="'&#x6068;'" select="'hen4'"/>
                <xsl:map-entry key="'&#x6069;'" select="'en1'"/>
                <xsl:map-entry key="'&#x606A;'" select="'ke4	que4'"/>
                <xsl:map-entry key="'&#x606B;'" select="'tong1	dong4'"/>
                <xsl:map-entry key="'&#x606C;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x606D;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x606E;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x606F;'" select="'xi1	xi2'"/>
                <xsl:map-entry key="'&#x6070;'" select="'qia4'"/>
                <xsl:map-entry key="'&#x6071;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x6072;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x6073;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x6074;'" select="'de2'"/>
                <xsl:map-entry key="'&#x6075;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6076;'" select="'e4	e3	wu4'"/>
                <xsl:map-entry key="'&#x6077;'" select="''"/>
                <xsl:map-entry key="'&#x6078;'" select="'tong4'"/>
                <xsl:map-entry key="'&#x6079;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x607A;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x607B;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x607C;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x607D;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x607E;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x607F;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x6080;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x6081;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x6082;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x6083;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x6084;'" select="'qiao3	qiao1'"/>
                <xsl:map-entry key="'&#x6085;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x6086;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6087;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6088;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x6089;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x608A;'" select="'zhe2	qi1'"/>
                <xsl:map-entry key="'&#x608B;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x608C;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x608D;'" select="'han4'"/>
                <xsl:map-entry key="'&#x608E;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x608F;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x6090;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x6091;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x6092;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6093;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x6094;'" select="'hui3'"/>
                <xsl:map-entry key="'&#x6095;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6096;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x6097;'" select="'man2'"/>
                <xsl:map-entry key="'&#x6098;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6099;'" select="'heng1'"/>
                <xsl:map-entry key="'&#x609A;'" select="'song3'"/>
                <xsl:map-entry key="'&#x609B;'" select="'quan1'"/>
                <xsl:map-entry key="'&#x609C;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x609D;'" select="'kui1	li3'"/>
                <xsl:map-entry key="'&#x609E;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x609F;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x60A0;'" select="'you1'"/>
                <xsl:map-entry key="'&#x60A1;'" select="'li2'"/>
                <xsl:map-entry key="'&#x60A2;'" select="'liang4	lang3'"/>
                <xsl:map-entry key="'&#x60A3;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x60A4;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x60A5;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x60A6;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x60A7;'" select="'li4'"/>
                <xsl:map-entry key="'&#x60A8;'" select="'nin2'"/>
                <xsl:map-entry key="'&#x60A9;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x60AA;'" select="'e4	e3	wu4'"/>
                <xsl:map-entry key="'&#x60AB;'" select="'que4'"/>
                <xsl:map-entry key="'&#x60AC;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x60AD;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x60AE;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x60AF;'" select="'min3'"/>
                <xsl:map-entry key="'&#x60B0;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x60B1;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x60B2;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x60B3;'" select="'de2'"/>
                <xsl:map-entry key="'&#x60B4;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x60B5;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x60B6;'" select="'men4	men1'"/>
                <xsl:map-entry key="'&#x60B7;'" select="'li4'"/>
                <xsl:map-entry key="'&#x60B8;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x60B9;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x60BA;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x60BB;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x60BC;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x60BD;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x60BE;'" select="'kong1'"/>
                <xsl:map-entry key="'&#x60BF;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x60C0;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x60C1;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x60C2;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x60C3;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x60C4;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x60C5;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x60C6;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x60C7;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x60C8;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x60C9;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x60CA;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x60CB;'" select="'wan3	wan4'"/>
                <xsl:map-entry key="'&#x60CC;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x60CD;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x60CE;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x60CF;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x60D0;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x60D1;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x60D2;'" select="'he2'"/>
                <xsl:map-entry key="'&#x60D3;'" select="'quan1'"/>
                <xsl:map-entry key="'&#x60D4;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x60D5;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x60D6;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x60D7;'" select="'nie1'"/>
                <xsl:map-entry key="'&#x60D8;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x60D9;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x60DA;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x60DB;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x60DC;'" select="'xi1	xi2'"/>
                <xsl:map-entry key="'&#x60DD;'" select="'tang3	chang3'"/>
                <xsl:map-entry key="'&#x60DE;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x60DF;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x60E0;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x60E1;'" select="'e4	e3	wu4'"/>
                <xsl:map-entry key="'&#x60E2;'" select="'rui3'"/>
                <xsl:map-entry key="'&#x60E3;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x60E4;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x60E5;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x60E6;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x60E7;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x60E8;'" select="'can3	can4'"/>
                <xsl:map-entry key="'&#x60E9;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x60EA;'" select="'de2'"/>
                <xsl:map-entry key="'&#x60EB;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x60EC;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x60ED;'" select="'can2'"/>
                <xsl:map-entry key="'&#x60EE;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x60EF;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x60F0;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x60F1;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x60F2;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x60F3;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x60F4;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x60F5;'" select="'die2'"/>
                <xsl:map-entry key="'&#x60F6;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x60F7;'" select="'chun3'"/>
                <xsl:map-entry key="'&#x60F8;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x60F9;'" select="'re3'"/>
                <xsl:map-entry key="'&#x60FA;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x60FB;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x60FC;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x60FD;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x60FE;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x60FF;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x6100;'" select="'qiao3'"/>
                <xsl:map-entry key="'&#x6101;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x6102;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x6103;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x6104;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x6105;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x6106;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6107;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x6108;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6109;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x610A;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x610B;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x610C;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x610D;'" select="'min3'"/>
                <xsl:map-entry key="'&#x610E;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x610F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6110;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x6111;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x6112;'" select="'kai4	qi4'"/>
                <xsl:map-entry key="'&#x6113;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x6114;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x6115;'" select="'e4'"/>
                <xsl:map-entry key="'&#x6116;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x6117;'" select="'mou4'"/>
                <xsl:map-entry key="'&#x6118;'" select="'qia4'"/>
                <xsl:map-entry key="'&#x6119;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x611A;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x611B;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x611C;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x611D;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x611E;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x611F;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x6120;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x6121;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x6122;'" select="'sai1'"/>
                <xsl:map-entry key="'&#x6123;'" select="'leng4'"/>
                <xsl:map-entry key="'&#x6124;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x6125;'" select="''"/>
                <xsl:map-entry key="'&#x6126;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x6127;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x6128;'" select="'que4'"/>
                <xsl:map-entry key="'&#x6129;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x612A;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x612B;'" select="'su4'"/>
                <xsl:map-entry key="'&#x612C;'" select="'su4'"/>
                <xsl:map-entry key="'&#x612D;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x612E;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x612F;'" select="'song3'"/>
                <xsl:map-entry key="'&#x6130;'" select="'huang4'"/>
                <xsl:map-entry key="'&#x6131;'" select="'shitsu'"/>
                <xsl:map-entry key="'&#x6132;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x6133;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x6134;'" select="'chuang4'"/>
                <xsl:map-entry key="'&#x6135;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x6136;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x6137;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x6138;'" select="'zheng3'"/>
                <xsl:map-entry key="'&#x6139;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x613A;'" select="'cao3'"/>
                <xsl:map-entry key="'&#x613B;'" select="'sun4'"/>
                <xsl:map-entry key="'&#x613C;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x613D;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x613E;'" select="'kai4'"/>
                <xsl:map-entry key="'&#x613F;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x6140;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x6141;'" select="'hun4'"/>
                <xsl:map-entry key="'&#x6142;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x6143;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x6144;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6145;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x6146;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x6147;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x6148;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x6149;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x614A;'" select="'qian4	qie4'"/>
                <xsl:map-entry key="'&#x614B;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x614C;'" select="'huang1'"/>
                <xsl:map-entry key="'&#x614D;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x614E;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x614F;'" select="'ming3'"/>
                <xsl:map-entry key="'&#x6150;'" select="''"/>
                <xsl:map-entry key="'&#x6151;'" select="'she4'"/>
                <xsl:map-entry key="'&#x6152;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x6153;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x6154;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x6155;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x6156;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x6157;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x6158;'" select="'can3	can4'"/>
                <xsl:map-entry key="'&#x6159;'" select="'can2'"/>
                <xsl:map-entry key="'&#x615A;'" select="'can2'"/>
                <xsl:map-entry key="'&#x615B;'" select="'cui2'"/>
                <xsl:map-entry key="'&#x615C;'" select="'min3'"/>
                <xsl:map-entry key="'&#x615D;'" select="'te4	ni4'"/>
                <xsl:map-entry key="'&#x615E;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x615F;'" select="'tong4'"/>
                <xsl:map-entry key="'&#x6160;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x6161;'" select="'shuang3'"/>
                <xsl:map-entry key="'&#x6162;'" select="'man4'"/>
                <xsl:map-entry key="'&#x6163;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x6164;'" select="'que4'"/>
                <xsl:map-entry key="'&#x6165;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x6166;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x6167;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6168;'" select="'kai3	kai4'"/>
                <xsl:map-entry key="'&#x6169;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x616A;'" select="'ou4	ou1'"/>
                <xsl:map-entry key="'&#x616B;'" select="'song3'"/>
                <xsl:map-entry key="'&#x616C;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x616D;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x616E;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x616F;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x6170;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x6171;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x6172;'" select="'man2'"/>
                <xsl:map-entry key="'&#x6173;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6174;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x6175;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x6176;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x6177;'" select="'kang1	kang3'"/>
                <xsl:map-entry key="'&#x6178;'" select="'di4'"/>
                <xsl:map-entry key="'&#x6179;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x617A;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x617B;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x617C;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x617D;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x617E;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x617F;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x6180;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x6181;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x6182;'" select="'you1'"/>
                <xsl:map-entry key="'&#x6183;'" select="'chuang1'"/>
                <xsl:map-entry key="'&#x6184;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6185;'" select="'tong4'"/>
                <xsl:map-entry key="'&#x6186;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x6187;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6188;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x6189;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x618A;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x618B;'" select="'bie1'"/>
                <xsl:map-entry key="'&#x618C;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x618D;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x618E;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x618F;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x6190;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x6191;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x6192;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x6193;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6194;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x6195;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x6196;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x6197;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x6198;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x6199;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x619A;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x619B;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x619C;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x619D;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x619E;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x619F;'" select="'su4'"/>
                <xsl:map-entry key="'&#x61A0;'" select="'jue4'"/>
                <xsl:map-entry key="'&#x61A1;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x61A2;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x61A3;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x61A4;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x61A5;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x61A6;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x61A7;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x61A8;'" select="'han1'"/>
                <xsl:map-entry key="'&#x61A9;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x61AA;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x61AB;'" select="'min3'"/>
                <xsl:map-entry key="'&#x61AC;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x61AD;'" select="'liao3'"/>
                <xsl:map-entry key="'&#x61AE;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x61AF;'" select="'can3'"/>
                <xsl:map-entry key="'&#x61B0;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x61B1;'" select="'chou4'"/>
                <xsl:map-entry key="'&#x61B2;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x61B3;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x61B4;'" select="'sheng2'"/>
                <xsl:map-entry key="'&#x61B5;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x61B6;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x61B7;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x61B8;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x61B9;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x61BA;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x61BB;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x61BC;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x61BD;'" select="'song1'"/>
                <xsl:map-entry key="'&#x61BE;'" select="'han4'"/>
                <xsl:map-entry key="'&#x61BF;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x61C0;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x61C1;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x61C2;'" select="'dong3'"/>
                <xsl:map-entry key="'&#x61C3;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x61C4;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x61C5;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x61C6;'" select="'cao3'"/>
                <xsl:map-entry key="'&#x61C7;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x61C8;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x61C9;'" select="'ying4	ying1'"/>
                <xsl:map-entry key="'&#x61CA;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x61CB;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x61CC;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x61CD;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x61CE;'" select="'se4'"/>
                <xsl:map-entry key="'&#x61CF;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x61D0;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x61D1;'" select="'men4'"/>
                <xsl:map-entry key="'&#x61D2;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x61D3;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x61D4;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x61D5;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x61D6;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x61D7;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x61D8;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x61D9;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x61DA;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x61DB;'" select="'dai1'"/>
                <xsl:map-entry key="'&#x61DC;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x61DD;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x61DE;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x61DF;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x61E0;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x61E1;'" select="'mo3'"/>
                <xsl:map-entry key="'&#x61E2;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x61E3;'" select="'men4'"/>
                <xsl:map-entry key="'&#x61E4;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x61E5;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x61E6;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x61E7;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x61E8;'" select="'yan1	yan4'"/>
                <xsl:map-entry key="'&#x61E9;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x61EA;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x61EB;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x61EC;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x61ED;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x61EE;'" select="'you1'"/>
                <xsl:map-entry key="'&#x61EF;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x61F0;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x61F1;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x61F2;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x61F3;'" select="''"/>
                <xsl:map-entry key="'&#x61F4;'" select="'chan4'"/>
                <xsl:map-entry key="'&#x61F5;'" select="'meng2	meng3'"/>
                <xsl:map-entry key="'&#x61F6;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x61F7;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x61F8;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x61F9;'" select="'rang4'"/>
                <xsl:map-entry key="'&#x61FA;'" select="'chan4'"/>
                <xsl:map-entry key="'&#x61FB;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x61FC;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x61FD;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x61FE;'" select="'she4	zhe2'"/>
                <xsl:map-entry key="'&#x61FF;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6200;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x6201;'" select="'nan3'"/>
                <xsl:map-entry key="'&#x6202;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x6203;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x6204;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6205;'" select="'gang4'"/>
                <xsl:map-entry key="'&#x6206;'" select="'gang4	zhuang4'"/>
                <xsl:map-entry key="'&#x6207;'" select="'zhuang4'"/>
                <xsl:map-entry key="'&#x6208;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x6209;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x620A;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x620B;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x620C;'" select="'xu1	qu5'"/>
                <xsl:map-entry key="'&#x620D;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x620E;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x620F;'" select="'xi4	hu1'"/>
                <xsl:map-entry key="'&#x6210;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x6211;'" select="'wo3'"/>
                <xsl:map-entry key="'&#x6212;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x6213;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x6214;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x6215;'" select="'qiang1	qiang2'"/>
                <xsl:map-entry key="'&#x6216;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x6217;'" select="'qiang1	qiang4'"/>
                <xsl:map-entry key="'&#x6218;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x6219;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x621A;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x621B;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x621C;'" select="'die2'"/>
                <xsl:map-entry key="'&#x621D;'" select="'cai2'"/>
                <xsl:map-entry key="'&#x621E;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x621F;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x6220;'" select="'shi4	chi4'"/>
                <xsl:map-entry key="'&#x6221;'" select="'kan1'"/>
                <xsl:map-entry key="'&#x6222;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6223;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x6224;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x6225;'" select="'deng3'"/>
                <xsl:map-entry key="'&#x6226;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x6227;'" select="'chuang1	qiang1	qiang4'"/>
                <xsl:map-entry key="'&#x6228;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x6229;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x622A;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x622B;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x622C;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x622D;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x622E;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x622F;'" select="'xi4	hu1'"/>
                <xsl:map-entry key="'&#x6230;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x6231;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x6232;'" select="'xi4	hu1'"/>
                <xsl:map-entry key="'&#x6233;'" select="'chuo1'"/>
                <xsl:map-entry key="'&#x6234;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x6235;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x6236;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6237;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6238;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6239;'" select="'e4'"/>
                <xsl:map-entry key="'&#x623A;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x623B;'" select="'li4'"/>
                <xsl:map-entry key="'&#x623C;'" select="'mao3'"/>
                <xsl:map-entry key="'&#x623D;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x623E;'" select="'li4'"/>
                <xsl:map-entry key="'&#x623F;'" select="'fang2'"/>
                <xsl:map-entry key="'&#x6240;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x6241;'" select="'bian3	pian1'"/>
                <xsl:map-entry key="'&#x6242;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x6243;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x6244;'" select="'shang3'"/>
                <xsl:map-entry key="'&#x6245;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6246;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x6247;'" select="'shan4	shan1'"/>
                <xsl:map-entry key="'&#x6248;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6249;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x624A;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x624B;'" select="'shou3'"/>
                <xsl:map-entry key="'&#x624C;'" select="'shou3'"/>
                <xsl:map-entry key="'&#x624D;'" select="'cai2'"/>
                <xsl:map-entry key="'&#x624E;'" select="'zha1	za1	zha2'"/>
                <xsl:map-entry key="'&#x624F;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6250;'" select="'le4'"/>
                <xsl:map-entry key="'&#x6251;'" select="'pu1'"/>
                <xsl:map-entry key="'&#x6252;'" select="'ba1	pa1	pa2'"/>
                <xsl:map-entry key="'&#x6253;'" select="'da3'"/>
                <xsl:map-entry key="'&#x6254;'" select="'reng1	reng3'"/>
                <xsl:map-entry key="'&#x6255;'" select="'fu2	bi4'"/>
                <xsl:map-entry key="'&#x6256;'" select="''"/>
                <xsl:map-entry key="'&#x6257;'" select="'zai4'"/>
                <xsl:map-entry key="'&#x6258;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x6259;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x625A;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x625B;'" select="'kang2	gang1'"/>
                <xsl:map-entry key="'&#x625C;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x625D;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x625E;'" select="'han4'"/>
                <xsl:map-entry key="'&#x625F;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x6260;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x6261;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x6262;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x6263;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x6264;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x6265;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x6266;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6267;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x6268;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x6269;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x626A;'" select="'men2'"/>
                <xsl:map-entry key="'&#x626B;'" select="'sao3	sao4'"/>
                <xsl:map-entry key="'&#x626C;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x626D;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x626E;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x626F;'" select="'che3'"/>
                <xsl:map-entry key="'&#x6270;'" select="'rao3	you4'"/>
                <xsl:map-entry key="'&#x6271;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6272;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x6273;'" select="'ban1	pan1'"/>
                <xsl:map-entry key="'&#x6274;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x6275;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6276;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x6277;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x6278;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6279;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x627A;'" select="'di3'"/>
                <xsl:map-entry key="'&#x627B;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x627C;'" select="'e4'"/>
                <xsl:map-entry key="'&#x627D;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x627E;'" select="'zhao3'"/>
                <xsl:map-entry key="'&#x627F;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x6280;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6281;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x6282;'" select="'kuang2'"/>
                <xsl:map-entry key="'&#x6283;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x6284;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x6285;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x6286;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x6287;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x6288;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x6289;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x628A;'" select="'ba3	ba4'"/>
                <xsl:map-entry key="'&#x628B;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x628C;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x628D;'" select="'zheng3'"/>
                <xsl:map-entry key="'&#x628E;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x628F;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x6290;'" select="'na4'"/>
                <xsl:map-entry key="'&#x6291;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6292;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6293;'" select="'zhua1'"/>
                <xsl:map-entry key="'&#x6294;'" select="'pou2'"/>
                <xsl:map-entry key="'&#x6295;'" select="'tou2'"/>
                <xsl:map-entry key="'&#x6296;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x6297;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x6298;'" select="'zhe2	she2	zhe1'"/>
                <xsl:map-entry key="'&#x6299;'" select="'pou2'"/>
                <xsl:map-entry key="'&#x629A;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x629B;'" select="'pao1'"/>
                <xsl:map-entry key="'&#x629C;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x629D;'" select="'ao3'"/>
                <xsl:map-entry key="'&#x629E;'" select="'ze2	zhai2'"/>
                <xsl:map-entry key="'&#x629F;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x62A0;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x62A1;'" select="'lun2	lun1'"/>
                <xsl:map-entry key="'&#x62A2;'" select="'qiang3	qiang1'"/>
                <xsl:map-entry key="'&#x62A3;'" select="''"/>
                <xsl:map-entry key="'&#x62A4;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x62A5;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x62A6;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x62A7;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x62A8;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x62A9;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x62AA;'" select="'pu1'"/>
                <xsl:map-entry key="'&#x62AB;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x62AC;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x62AD;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x62AE;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x62AF;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x62B0;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x62B1;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x62B2;'" select="'he1'"/>
                <xsl:map-entry key="'&#x62B3;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x62B4;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x62B5;'" select="'di3'"/>
                <xsl:map-entry key="'&#x62B6;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x62B7;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x62B8;'" select="'za1'"/>
                <xsl:map-entry key="'&#x62B9;'" select="'mo3	ma1	mo4'"/>
                <xsl:map-entry key="'&#x62BA;'" select="'mo3'"/>
                <xsl:map-entry key="'&#x62BB;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x62BC;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x62BD;'" select="'chou1'"/>
                <xsl:map-entry key="'&#x62BE;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x62BF;'" select="'min3'"/>
                <xsl:map-entry key="'&#x62C0;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x62C1;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x62C2;'" select="'fu2	bi4'"/>
                <xsl:map-entry key="'&#x62C3;'" select="'zha3'"/>
                <xsl:map-entry key="'&#x62C4;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x62C5;'" select="'dan4	dan1	dan3'"/>
                <xsl:map-entry key="'&#x62C6;'" select="'chai1	ca1'"/>
                <xsl:map-entry key="'&#x62C7;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x62C8;'" select="'nian1	nian3'"/>
                <xsl:map-entry key="'&#x62C9;'" select="'la1	la2	la3'"/>
                <xsl:map-entry key="'&#x62CA;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x62CB;'" select="'pao1'"/>
                <xsl:map-entry key="'&#x62CC;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x62CD;'" select="'pai1'"/>
                <xsl:map-entry key="'&#x62CE;'" select="'ling1'"/>
                <xsl:map-entry key="'&#x62CF;'" select="'na2'"/>
                <xsl:map-entry key="'&#x62D0;'" select="'guai3'"/>
                <xsl:map-entry key="'&#x62D1;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x62D2;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x62D3;'" select="'tuo4	ta4'"/>
                <xsl:map-entry key="'&#x62D4;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x62D5;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x62D6;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x62D7;'" select="'ao4	niu4'"/>
                <xsl:map-entry key="'&#x62D8;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x62D9;'" select="'zhuo2	zhuo1'"/>
                <xsl:map-entry key="'&#x62DA;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x62DB;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x62DC;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x62DD;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x62DE;'" select="'di3'"/>
                <xsl:map-entry key="'&#x62DF;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x62E0;'" select="'ju4	ju1'"/>
                <xsl:map-entry key="'&#x62E1;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x62E2;'" select="'long3'"/>
                <xsl:map-entry key="'&#x62E3;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x62E4;'" select="'qia2'"/>
                <xsl:map-entry key="'&#x62E5;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x62E6;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x62E7;'" select="'ning2	ning3	ning4'"/>
                <xsl:map-entry key="'&#x62E8;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x62E9;'" select="'ze2	zhai2'"/>
                <xsl:map-entry key="'&#x62EA;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x62EB;'" select="'hen2'"/>
                <xsl:map-entry key="'&#x62EC;'" select="'gua1	kuo4'"/>
                <xsl:map-entry key="'&#x62ED;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x62EE;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x62EF;'" select="'zheng3'"/>
                <xsl:map-entry key="'&#x62F0;'" select="'nin3'"/>
                <xsl:map-entry key="'&#x62F1;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x62F2;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x62F3;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x62F4;'" select="'shuan1'"/>
                <xsl:map-entry key="'&#x62F5;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x62F6;'" select="'zan3	za1'"/>
                <xsl:map-entry key="'&#x62F7;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x62F8;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x62F9;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x62FA;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x62FB;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x62FC;'" select="'pin1'"/>
                <xsl:map-entry key="'&#x62FD;'" select="'zhuai4'"/>
                <xsl:map-entry key="'&#x62FE;'" select="'shi2	she4'"/>
                <xsl:map-entry key="'&#x62FF;'" select="'na2'"/>
                <xsl:map-entry key="'&#x6300;'" select="'bo4'"/>
                <xsl:map-entry key="'&#x6301;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x6302;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x6303;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6304;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x6305;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x6306;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x6307;'" select="'zhi3	zhi1'"/>
                <xsl:map-entry key="'&#x6308;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x6309;'" select="'an4'"/>
                <xsl:map-entry key="'&#x630A;'" select="'nong4	long4'"/>
                <xsl:map-entry key="'&#x630B;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x630C;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x630D;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x630E;'" select="'ku1	kua4'"/>
                <xsl:map-entry key="'&#x630F;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x6310;'" select="'ru2	na2'"/>
                <xsl:map-entry key="'&#x6311;'" select="'tiao1	tiao3'"/>
                <xsl:map-entry key="'&#x6312;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x6313;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x6314;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x6315;'" select="'die2'"/>
                <xsl:map-entry key="'&#x6316;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x6317;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6318;'" select="''"/>
                <xsl:map-entry key="'&#x6319;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x631A;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x631B;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x631C;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x631D;'" select="'zhua1	wo1'"/>
                <xsl:map-entry key="'&#x631E;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x631F;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x6320;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x6321;'" select="'dang3	dang4'"/>
                <xsl:map-entry key="'&#x6322;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x6323;'" select="'zheng1	zheng4'"/>
                <xsl:map-entry key="'&#x6324;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x6325;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x6326;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6327;'" select="''"/>
                <xsl:map-entry key="'&#x6328;'" select="'ai1	ai2'"/>
                <xsl:map-entry key="'&#x6329;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x632A;'" select="'nuo2'"/>
                <xsl:map-entry key="'&#x632B;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x632C;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x632D;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x632E;'" select="'ti3'"/>
                <xsl:map-entry key="'&#x632F;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x6330;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x6331;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x6332;'" select="'suo1	sa1'"/>
                <xsl:map-entry key="'&#x6333;'" select="'keng1	keng3'"/>
                <xsl:map-entry key="'&#x6334;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x6335;'" select="'long4	nong4'"/>
                <xsl:map-entry key="'&#x6336;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x6337;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x6338;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x6339;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x633A;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x633B;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x633C;'" select="'nuo4	ruo2'"/>
                <xsl:map-entry key="'&#x633D;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x633E;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x633F;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x6340;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x6341;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x6342;'" select="'wu3	wu2'"/>
                <xsl:map-entry key="'&#x6343;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x6344;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x6345;'" select="'tong3'"/>
                <xsl:map-entry key="'&#x6346;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x6347;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x6348;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x6349;'" select="'zhuo1'"/>
                <xsl:map-entry key="'&#x634A;'" select="'pou2'"/>
                <xsl:map-entry key="'&#x634B;'" select="'le4	lu:3	luo1'"/>
                <xsl:map-entry key="'&#x634C;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x634D;'" select="'han4'"/>
                <xsl:map-entry key="'&#x634E;'" select="'shao1	shao4'"/>
                <xsl:map-entry key="'&#x634F;'" select="'nie1'"/>
                <xsl:map-entry key="'&#x6350;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x6351;'" select="'she4'"/>
                <xsl:map-entry key="'&#x6352;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6353;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x6354;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6355;'" select="'bu3'"/>
                <xsl:map-entry key="'&#x6356;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x6357;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x6358;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x6359;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x635A;'" select="'zhai1'"/>
                <xsl:map-entry key="'&#x635B;'" select="'ly3'"/>
                <xsl:map-entry key="'&#x635C;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x635D;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x635E;'" select="'lao1'"/>
                <xsl:map-entry key="'&#x635F;'" select="'sun3'"/>
                <xsl:map-entry key="'&#x6360;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x6361;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x6362;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x6363;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x6364;'" select="''"/>
                <xsl:map-entry key="'&#x6365;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x6366;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x6367;'" select="'peng3'"/>
                <xsl:map-entry key="'&#x6368;'" select="'she3	she4'"/>
                <xsl:map-entry key="'&#x6369;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x636A;'" select="'min2'"/>
                <xsl:map-entry key="'&#x636B;'" select="'men2'"/>
                <xsl:map-entry key="'&#x636C;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x636D;'" select="'bai3'"/>
                <xsl:map-entry key="'&#x636E;'" select="'ju4	ju1'"/>
                <xsl:map-entry key="'&#x636F;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x6370;'" select="'wo3'"/>
                <xsl:map-entry key="'&#x6371;'" select="'ai2	ai1'"/>
                <xsl:map-entry key="'&#x6372;'" select="'juan3'"/>
                <xsl:map-entry key="'&#x6373;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x6374;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x6375;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x6376;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x6377;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6378;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x6379;'" select="'ben4'"/>
                <xsl:map-entry key="'&#x637A;'" select="'na4'"/>
                <xsl:map-entry key="'&#x637B;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x637C;'" select="'nuo2'"/>
                <xsl:map-entry key="'&#x637D;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x637E;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x637F;'" select="'xi1	qi1'"/>
                <xsl:map-entry key="'&#x6380;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x6381;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x6382;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x6383;'" select="'sao3	sao4'"/>
                <xsl:map-entry key="'&#x6384;'" select="'lun2	lun1'"/>
                <xsl:map-entry key="'&#x6385;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x6386;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x6387;'" select="'duo2	duo1'"/>
                <xsl:map-entry key="'&#x6388;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x6389;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x638A;'" select="'pou2	pou3'"/>
                <xsl:map-entry key="'&#x638B;'" select="'di3'"/>
                <xsl:map-entry key="'&#x638C;'" select="'zhang3'"/>
                <xsl:map-entry key="'&#x638D;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x638E;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x638F;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x6390;'" select="'qia1'"/>
                <xsl:map-entry key="'&#x6391;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x6392;'" select="'pai2	pai3'"/>
                <xsl:map-entry key="'&#x6393;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x6394;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6395;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x6396;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x6397;'" select="'ya3'"/>
                <xsl:map-entry key="'&#x6398;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6399;'" select="'zheng1	zheng4'"/>
                <xsl:map-entry key="'&#x639A;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x639B;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x639C;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x639D;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x639E;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x639F;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x63A0;'" select="'lu:e4'"/>
                <xsl:map-entry key="'&#x63A1;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x63A2;'" select="'tan4'"/>
                <xsl:map-entry key="'&#x63A3;'" select="'che4'"/>
                <xsl:map-entry key="'&#x63A4;'" select="'bing1'"/>
                <xsl:map-entry key="'&#x63A5;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x63A6;'" select="'ti1'"/>
                <xsl:map-entry key="'&#x63A7;'" select="'kong4'"/>
                <xsl:map-entry key="'&#x63A8;'" select="'tui1'"/>
                <xsl:map-entry key="'&#x63A9;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x63AA;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x63AB;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x63AC;'" select="'ju2	ju1'"/>
                <xsl:map-entry key="'&#x63AD;'" select="'tian4'"/>
                <xsl:map-entry key="'&#x63AE;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x63AF;'" select="'ken4'"/>
                <xsl:map-entry key="'&#x63B0;'" select="'bai1	bo4'"/>
                <xsl:map-entry key="'&#x63B1;'" select="'shou3	pa2'"/>
                <xsl:map-entry key="'&#x63B2;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x63B3;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x63B4;'" select="'guo2	guai1'"/>
                <xsl:map-entry key="'&#x63B5;'" select="''"/>
                <xsl:map-entry key="'&#x63B6;'" select="''"/>
                <xsl:map-entry key="'&#x63B7;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x63B8;'" select="'dan3	shan3	shan4'"/>
                <xsl:map-entry key="'&#x63B9;'" select="''"/>
                <xsl:map-entry key="'&#x63BA;'" select="'can4	chan1	shan3'"/>
                <xsl:map-entry key="'&#x63BB;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x63BC;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x63BD;'" select="'peng4'"/>
                <xsl:map-entry key="'&#x63BE;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x63BF;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x63C0;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x63C1;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x63C2;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x63C3;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x63C4;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x63C5;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x63C6;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x63C7;'" select="'nan3'"/>
                <xsl:map-entry key="'&#x63C8;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x63C9;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x63CA;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x63CB;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x63CC;'" select="'sai1'"/>
                <xsl:map-entry key="'&#x63CD;'" select="'zou4'"/>
                <xsl:map-entry key="'&#x63CE;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x63CF;'" select="'miao2'"/>
                <xsl:map-entry key="'&#x63D0;'" select="'ti2	di1'"/>
                <xsl:map-entry key="'&#x63D1;'" select="'nie1'"/>
                <xsl:map-entry key="'&#x63D2;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x63D3;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x63D4;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x63D5;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x63D6;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x63D7;'" select="'shun3'"/>
                <xsl:map-entry key="'&#x63D8;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x63D9;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x63DA;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x63DB;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x63DC;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x63DD;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x63DE;'" select="'an3'"/>
                <xsl:map-entry key="'&#x63DF;'" select="'xu1	ju1'"/>
                <xsl:map-entry key="'&#x63E0;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x63E1;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x63E2;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x63E3;'" select="'chuai3	chuai1	chuai4'"/>
                <xsl:map-entry key="'&#x63E4;'" select="'ji2	jie2'"/>
                <xsl:map-entry key="'&#x63E5;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x63E6;'" select="'la2'"/>
                <xsl:map-entry key="'&#x63E7;'" select="'la4'"/>
                <xsl:map-entry key="'&#x63E8;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x63E9;'" select="'kai1'"/>
                <xsl:map-entry key="'&#x63EA;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x63EB;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x63EC;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x63ED;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x63EE;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x63EF;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x63F0;'" select="'chong4'"/>
                <xsl:map-entry key="'&#x63F1;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x63F2;'" select="'she2	die2'"/>
                <xsl:map-entry key="'&#x63F3;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x63F4;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x63F5;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x63F6;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x63F7;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x63F8;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x63F9;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x63FA;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x63FB;'" select="''"/>
                <xsl:map-entry key="'&#x63FC;'" select="''"/>
                <xsl:map-entry key="'&#x63FD;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x63FE;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x63FF;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x6400;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x6401;'" select="'ge1	ge2'"/>
                <xsl:map-entry key="'&#x6402;'" select="'lou3	lou1'"/>
                <xsl:map-entry key="'&#x6403;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x6404;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x6405;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x6406;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x6407;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x6408;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x6409;'" select="'que4'"/>
                <xsl:map-entry key="'&#x640A;'" select="'chou1'"/>
                <xsl:map-entry key="'&#x640B;'" select="'chi3	chuai1'"/>
                <xsl:map-entry key="'&#x640C;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x640D;'" select="'sun3'"/>
                <xsl:map-entry key="'&#x640E;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x640F;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x6410;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x6411;'" select="'rong3'"/>
                <xsl:map-entry key="'&#x6412;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x6413;'" select="'cuo1'"/>
                <xsl:map-entry key="'&#x6414;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x6415;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x6416;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x6417;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x6418;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x6419;'" select="'nu4'"/>
                <xsl:map-entry key="'&#x641A;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x641B;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x641C;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x641D;'" select="'qiu3'"/>
                <xsl:map-entry key="'&#x641E;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x641F;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x6420;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x6421;'" select="'sang3'"/>
                <xsl:map-entry key="'&#x6422;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x6423;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x6424;'" select="'e4'"/>
                <xsl:map-entry key="'&#x6425;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x6426;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x6427;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x6428;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x6429;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x642A;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x642B;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x642C;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x642D;'" select="'da1'"/>
                <xsl:map-entry key="'&#x642E;'" select="'li4'"/>
                <xsl:map-entry key="'&#x642F;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x6430;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x6431;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6432;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x6433;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x6434;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6435;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x6436;'" select="'qiang3	chuang3	qiang1'"/>
                <xsl:map-entry key="'&#x6437;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x6438;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x6439;'" select="'e4'"/>
                <xsl:map-entry key="'&#x643A;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x643B;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x643C;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x643D;'" select="'cha2	ca1'"/>
                <xsl:map-entry key="'&#x643E;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x643F;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x6440;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x6441;'" select="'en4'"/>
                <xsl:map-entry key="'&#x6442;'" select="'she4'"/>
                <xsl:map-entry key="'&#x6443;'" select="'gong4'"/>
                <xsl:map-entry key="'&#x6444;'" select="'she4'"/>
                <xsl:map-entry key="'&#x6445;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6446;'" select="'bai3'"/>
                <xsl:map-entry key="'&#x6447;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x6448;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x6449;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x644A;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x644B;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x644C;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x644D;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x644E;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x644F;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x6450;'" select="'chuang1'"/>
                <xsl:map-entry key="'&#x6451;'" select="'guo2	guai1'"/>
                <xsl:map-entry key="'&#x6452;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x6453;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x6454;'" select="'shuai1'"/>
                <xsl:map-entry key="'&#x6455;'" select="'di4'"/>
                <xsl:map-entry key="'&#x6456;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6457;'" select="'sou'"/>
                <xsl:map-entry key="'&#x6458;'" select="'zhai1	zhe2'"/>
                <xsl:map-entry key="'&#x6459;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x645A;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x645B;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x645C;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x645D;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x645E;'" select="'luo2	luo4'"/>
                <xsl:map-entry key="'&#x645F;'" select="'lou3	lou1'"/>
                <xsl:map-entry key="'&#x6460;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x6461;'" select="'gai4	xi4'"/>
                <xsl:map-entry key="'&#x6462;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6463;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x6464;'" select="'chuang3'"/>
                <xsl:map-entry key="'&#x6465;'" select="'tang4'"/>
                <xsl:map-entry key="'&#x6466;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x6467;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x6468;'" select="'nai2'"/>
                <xsl:map-entry key="'&#x6469;'" select="'mo2	ma1'"/>
                <xsl:map-entry key="'&#x646A;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x646B;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x646C;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x646D;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x646E;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x646F;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6470;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x6471;'" select="'man2'"/>
                <xsl:map-entry key="'&#x6472;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x6473;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x6474;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6475;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x6476;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x6477;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x6478;'" select="'mo1	mo2'"/>
                <xsl:map-entry key="'&#x6479;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x647A;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x647B;'" select="'chan1	shan3'"/>
                <xsl:map-entry key="'&#x647C;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x647D;'" select="'piao3	biao1'"/>
                <xsl:map-entry key="'&#x647E;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x647F;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x6480;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x6481;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6482;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x6483;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x6484;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x6485;'" select="'jue1'"/>
                <xsl:map-entry key="'&#x6486;'" select="'pie1'"/>
                <xsl:map-entry key="'&#x6487;'" select="'pie3	pie1'"/>
                <xsl:map-entry key="'&#x6488;'" select="'lao1'"/>
                <xsl:map-entry key="'&#x6489;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x648A;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x648B;'" select="'ruan2'"/>
                <xsl:map-entry key="'&#x648C;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x648D;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x648E;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x648F;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6490;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x6491;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x6492;'" select="'sa3	sa1'"/>
                <xsl:map-entry key="'&#x6493;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x6494;'" select="'heng4'"/>
                <xsl:map-entry key="'&#x6495;'" select="'si1'"/>
                <xsl:map-entry key="'&#x6496;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x6497;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x6498;'" select="'da1'"/>
                <xsl:map-entry key="'&#x6499;'" select="'zun3'"/>
                <xsl:map-entry key="'&#x649A;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x649B;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x649C;'" select="'zheng3'"/>
                <xsl:map-entry key="'&#x649D;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x649E;'" select="'zhuang4	chuang2'"/>
                <xsl:map-entry key="'&#x649F;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x64A0;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x64A1;'" select="'cao1'"/>
                <xsl:map-entry key="'&#x64A2;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x64A3;'" select="'dan3	shan3	shan4'"/>
                <xsl:map-entry key="'&#x64A4;'" select="'che4'"/>
                <xsl:map-entry key="'&#x64A5;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x64A6;'" select="'che3'"/>
                <xsl:map-entry key="'&#x64A7;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x64A8;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x64A9;'" select="'liao2	liao1	liao4'"/>
                <xsl:map-entry key="'&#x64AA;'" select="'ben4'"/>
                <xsl:map-entry key="'&#x64AB;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x64AC;'" select="'qiao4	qiao1'"/>
                <xsl:map-entry key="'&#x64AD;'" select="'bo4	bo1'"/>
                <xsl:map-entry key="'&#x64AE;'" select="'cuo1	zuo3'"/>
                <xsl:map-entry key="'&#x64AF;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x64B0;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x64B1;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x64B2;'" select="'pu1'"/>
                <xsl:map-entry key="'&#x64B3;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x64B4;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x64B5;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x64B6;'" select="''"/>
                <xsl:map-entry key="'&#x64B7;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x64B8;'" select="'lu1'"/>
                <xsl:map-entry key="'&#x64B9;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x64BA;'" select="'cuan1'"/>
                <xsl:map-entry key="'&#x64BB;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x64BC;'" select="'han4'"/>
                <xsl:map-entry key="'&#x64BD;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x64BE;'" select="'zhua1	wo1'"/>
                <xsl:map-entry key="'&#x64BF;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x64C0;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x64C1;'" select="'yong3	yong1'"/>
                <xsl:map-entry key="'&#x64C2;'" select="'lei2	lei1	lei4'"/>
                <xsl:map-entry key="'&#x64C3;'" select="'kuo3'"/>
                <xsl:map-entry key="'&#x64C4;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x64C5;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x64C6;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x64C7;'" select="'ze2	zhai2'"/>
                <xsl:map-entry key="'&#x64C8;'" select="'pu1'"/>
                <xsl:map-entry key="'&#x64C9;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x64CA;'" select="'ji2	ji1'"/>
                <xsl:map-entry key="'&#x64CB;'" select="'dang3	dang4'"/>
                <xsl:map-entry key="'&#x64CC;'" select="'se4'"/>
                <xsl:map-entry key="'&#x64CD;'" select="'cao1	cao4'"/>
                <xsl:map-entry key="'&#x64CE;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x64CF;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x64D0;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x64D1;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x64D2;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x64D3;'" select="'kuai3'"/>
                <xsl:map-entry key="'&#x64D4;'" select="'dan1	dan3	dan4'"/>
                <xsl:map-entry key="'&#x64D5;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x64D6;'" select="'ge3'"/>
                <xsl:map-entry key="'&#x64D7;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x64D8;'" select="'bo4'"/>
                <xsl:map-entry key="'&#x64D9;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x64DA;'" select="'ju4	ju1'"/>
                <xsl:map-entry key="'&#x64DB;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x64DC;'" select="''"/>
                <xsl:map-entry key="'&#x64DD;'" select="''"/>
                <xsl:map-entry key="'&#x64DE;'" select="'sou3	sou4'"/>
                <xsl:map-entry key="'&#x64DF;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x64E0;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x64E1;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x64E2;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x64E3;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x64E4;'" select="'xing3'"/>
                <xsl:map-entry key="'&#x64E5;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x64E6;'" select="'ca1'"/>
                <xsl:map-entry key="'&#x64E7;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x64E8;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x64E9;'" select="'ru3'"/>
                <xsl:map-entry key="'&#x64EA;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x64EB;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x64EC;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x64ED;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x64EE;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x64EF;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x64F0;'" select="'ning2	ning3	ning4'"/>
                <xsl:map-entry key="'&#x64F1;'" select="'ge1	ge2'"/>
                <xsl:map-entry key="'&#x64F2;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x64F3;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x64F4;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x64F5;'" select="'mo2	ma1'"/>
                <xsl:map-entry key="'&#x64F6;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x64F7;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x64F8;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x64F9;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x64FA;'" select="'bai3'"/>
                <xsl:map-entry key="'&#x64FB;'" select="'sou3	sou4'"/>
                <xsl:map-entry key="'&#x64FC;'" select="'lu1'"/>
                <xsl:map-entry key="'&#x64FD;'" select="'lu:e4'"/>
                <xsl:map-entry key="'&#x64FE;'" select="'rao3'"/>
                <xsl:map-entry key="'&#x64FF;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x6500;'" select="'pan1'"/>
                <xsl:map-entry key="'&#x6501;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x6502;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x6503;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x6504;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6505;'" select="'zan3	cuan2'"/>
                <xsl:map-entry key="'&#x6506;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x6507;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x6508;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x6509;'" select="'huo4	huo1'"/>
                <xsl:map-entry key="'&#x650A;'" select="'lye4'"/>
                <xsl:map-entry key="'&#x650B;'" select="'la4'"/>
                <xsl:map-entry key="'&#x650C;'" select="'han4'"/>
                <xsl:map-entry key="'&#x650D;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x650E;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x650F;'" select="'long3'"/>
                <xsl:map-entry key="'&#x6510;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6511;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6512;'" select="'zan3	cuan2'"/>
                <xsl:map-entry key="'&#x6513;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6514;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x6515;'" select="'san1'"/>
                <xsl:map-entry key="'&#x6516;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x6517;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x6518;'" select="'rang2	rang3'"/>
                <xsl:map-entry key="'&#x6519;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x651A;'" select="''"/>
                <xsl:map-entry key="'&#x651B;'" select="'cuan1'"/>
                <xsl:map-entry key="'&#x651C;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x651D;'" select="'she4'"/>
                <xsl:map-entry key="'&#x651E;'" select="'luo1'"/>
                <xsl:map-entry key="'&#x651F;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x6520;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x6521;'" select="'li2'"/>
                <xsl:map-entry key="'&#x6522;'" select="'zan3	cuan2'"/>
                <xsl:map-entry key="'&#x6523;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x6524;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x6525;'" select="'zuan4'"/>
                <xsl:map-entry key="'&#x6526;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6527;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x6528;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x6529;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x652A;'" select="'jiao3	gao3	jia3'"/>
                <xsl:map-entry key="'&#x652B;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x652C;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x652D;'" select="'li4'"/>
                <xsl:map-entry key="'&#x652E;'" select="'nang3'"/>
                <xsl:map-entry key="'&#x652F;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x6530;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x6531;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x6532;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x6533;'" select="'xin2'"/>
                <xsl:map-entry key="'&#x6534;'" select="'po1'"/>
                <xsl:map-entry key="'&#x6535;'" select="'po1'"/>
                <xsl:map-entry key="'&#x6536;'" select="'shou1'"/>
                <xsl:map-entry key="'&#x6537;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x6538;'" select="'you1'"/>
                <xsl:map-entry key="'&#x6539;'" select="'gai3'"/>
                <xsl:map-entry key="'&#x653A;'" select="'gai3'"/>
                <xsl:map-entry key="'&#x653B;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x653C;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x653D;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x653E;'" select="'fang4'"/>
                <xsl:map-entry key="'&#x653F;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x6540;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x6541;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x6542;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x6543;'" select="'min3'"/>
                <xsl:map-entry key="'&#x6544;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x6545;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x6546;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x6547;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x6548;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x6549;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x654A;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x654B;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x654C;'" select="'di2'"/>
                <xsl:map-entry key="'&#x654D;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x654E;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x654F;'" select="'min3'"/>
                <xsl:map-entry key="'&#x6550;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x6551;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x6552;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x6553;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x6554;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x6555;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x6556;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x6557;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x6558;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6559;'" select="'jiao4	jiao1'"/>
                <xsl:map-entry key="'&#x655A;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x655B;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x655C;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x655D;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x655E;'" select="'chang3'"/>
                <xsl:map-entry key="'&#x655F;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x6560;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x6561;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6562;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x6563;'" select="'san4	san3'"/>
                <xsl:map-entry key="'&#x6564;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x6565;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x6566;'" select="'dun1	dui4'"/>
                <xsl:map-entry key="'&#x6567;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x6568;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x6569;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x656A;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x656B;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x656C;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x656D;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x656E;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x656F;'" select="'hun1	min3'"/>
                <xsl:map-entry key="'&#x6570;'" select="'shu4	shu3	shuo4'"/>
                <xsl:map-entry key="'&#x6571;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x6572;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x6573;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x6574;'" select="'zheng3'"/>
                <xsl:map-entry key="'&#x6575;'" select="'di2'"/>
                <xsl:map-entry key="'&#x6576;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x6577;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x6578;'" select="'shu4	shu3	shuo4'"/>
                <xsl:map-entry key="'&#x6579;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x657A;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x657B;'" select="'xiong4'"/>
                <xsl:map-entry key="'&#x657C;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x657D;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x657E;'" select="''"/>
                <xsl:map-entry key="'&#x657F;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x6580;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x6581;'" select="'yi4	du4'"/>
                <xsl:map-entry key="'&#x6582;'" select="'lian4	lian3'"/>
                <xsl:map-entry key="'&#x6583;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6584;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6585;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x6586;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x6587;'" select="'wen2	wen4'"/>
                <xsl:map-entry key="'&#x6588;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x6589;'" select="'qi2	ji4	qi4'"/>
                <xsl:map-entry key="'&#x658A;'" select="'qi2	ji4	qi4'"/>
                <xsl:map-entry key="'&#x658B;'" select="'zhai1'"/>
                <xsl:map-entry key="'&#x658C;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x658D;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x658E;'" select="'zhai1'"/>
                <xsl:map-entry key="'&#x658F;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x6590;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x6591;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x6592;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x6593;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x6594;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x6595;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x6596;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x6597;'" select="'dou3	dou4'"/>
                <xsl:map-entry key="'&#x6598;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x6599;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x659A;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x659B;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x659C;'" select="'xie2	xia2'"/>
                <xsl:map-entry key="'&#x659D;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x659E;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x659F;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x65A0;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x65A1;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x65A2;'" select="'tiao3	tou4'"/>
                <xsl:map-entry key="'&#x65A3;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x65A4;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x65A5;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x65A6;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x65A7;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x65A8;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x65A9;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x65AA;'" select="'qu2	ju1'"/>
                <xsl:map-entry key="'&#x65AB;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x65AC;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x65AD;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x65AE;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x65AF;'" select="'si1'"/>
                <xsl:map-entry key="'&#x65B0;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x65B1;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x65B2;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x65B3;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x65B4;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x65B5;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x65B6;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x65B7;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x65B8;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x65B9;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x65BA;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x65BB;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x65BC;'" select="'yu2	wu1	yu1'"/>
                <xsl:map-entry key="'&#x65BD;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x65BE;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x65BF;'" select="'you2'"/>
                <xsl:map-entry key="'&#x65C0;'" select="''"/>
                <xsl:map-entry key="'&#x65C1;'" select="'pang2	bang4'"/>
                <xsl:map-entry key="'&#x65C2;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x65C3;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x65C4;'" select="'mao2	mao4'"/>
                <xsl:map-entry key="'&#x65C5;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x65C6;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x65C7;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x65C8;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x65C9;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x65CA;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x65CB;'" select="'xuan2	xuan4'"/>
                <xsl:map-entry key="'&#x65CC;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x65CD;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x65CE;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x65CF;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x65D0;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x65D1;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x65D2;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x65D3;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x65D4;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x65D5;'" select="''"/>
                <xsl:map-entry key="'&#x65D6;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x65D7;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x65D8;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x65D9;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x65DA;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x65DB;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x65DC;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x65DD;'" select="'guai4'"/>
                <xsl:map-entry key="'&#x65DE;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x65DF;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x65E0;'" select="'wu2	mo2'"/>
                <xsl:map-entry key="'&#x65E1;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x65E2;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x65E3;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x65E4;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x65E5;'" select="'ri4'"/>
                <xsl:map-entry key="'&#x65E6;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x65E7;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x65E8;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x65E9;'" select="'zao3'"/>
                <xsl:map-entry key="'&#x65EA;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x65EB;'" select="'tiao1'"/>
                <xsl:map-entry key="'&#x65EC;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x65ED;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x65EE;'" select="'ga1'"/>
                <xsl:map-entry key="'&#x65EF;'" select="'la2'"/>
                <xsl:map-entry key="'&#x65F0;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x65F1;'" select="'han4'"/>
                <xsl:map-entry key="'&#x65F2;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x65F3;'" select="'di4'"/>
                <xsl:map-entry key="'&#x65F4;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x65F5;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x65F6;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x65F7;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x65F8;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x65F9;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x65FA;'" select="'wang4'"/>
                <xsl:map-entry key="'&#x65FB;'" select="'min2'"/>
                <xsl:map-entry key="'&#x65FC;'" select="'min2'"/>
                <xsl:map-entry key="'&#x65FD;'" select="'tun1'"/>
                <xsl:map-entry key="'&#x65FE;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x65FF;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x6600;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x6601;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x6602;'" select="'ang2'"/>
                <xsl:map-entry key="'&#x6603;'" select="'ze4'"/>
                <xsl:map-entry key="'&#x6604;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x6605;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6606;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x6607;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x6608;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6609;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x660A;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x660B;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x660C;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x660D;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x660E;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x660F;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x6610;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x6611;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x6612;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x6613;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6614;'" select="'xi2	xi1'"/>
                <xsl:map-entry key="'&#x6615;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x6616;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x6617;'" select="'ze4'"/>
                <xsl:map-entry key="'&#x6618;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x6619;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x661A;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x661B;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x661C;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x661D;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x661E;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x661F;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x6620;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x6621;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x6622;'" select="'pei3'"/>
                <xsl:map-entry key="'&#x6623;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x6624;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x6625;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x6626;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x6627;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x6628;'" select="'zuo2'"/>
                <xsl:map-entry key="'&#x6629;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x662A;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x662B;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x662C;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x662D;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x662E;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x662F;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x6630;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x6631;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6632;'" select="'fei4	fu2'"/>
                <xsl:map-entry key="'&#x6633;'" select="'die2'"/>
                <xsl:map-entry key="'&#x6634;'" select="'mao3'"/>
                <xsl:map-entry key="'&#x6635;'" select="'ni4	ni3'"/>
                <xsl:map-entry key="'&#x6636;'" select="'chang3'"/>
                <xsl:map-entry key="'&#x6637;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x6638;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x6639;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x663A;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x663B;'" select="'ang2'"/>
                <xsl:map-entry key="'&#x663C;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x663D;'" select="'long2'"/>
                <xsl:map-entry key="'&#x663E;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x663F;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x6640;'" select="'tiao3'"/>
                <xsl:map-entry key="'&#x6641;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x6642;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x6643;'" select="'huang3	huang4'"/>
                <xsl:map-entry key="'&#x6644;'" select="'huang3	huang4'"/>
                <xsl:map-entry key="'&#x6645;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x6646;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x6647;'" select="'xu1	kua1'"/>
                <xsl:map-entry key="'&#x6648;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x6649;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x664A;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x664B;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x664C;'" select="'shang3'"/>
                <xsl:map-entry key="'&#x664D;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x664E;'" select="'hong3'"/>
                <xsl:map-entry key="'&#x664F;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x6650;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x6651;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x6652;'" select="'shai4'"/>
                <xsl:map-entry key="'&#x6653;'" select="'xiao3'"/>
                <xsl:map-entry key="'&#x6654;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x6655;'" select="'yun1	yun4'"/>
                <xsl:map-entry key="'&#x6656;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x6657;'" select="'han2'"/>
                <xsl:map-entry key="'&#x6658;'" select="'han4'"/>
                <xsl:map-entry key="'&#x6659;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x665A;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x665B;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x665C;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x665D;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x665E;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x665F;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x6660;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x6661;'" select="'bu1'"/>
                <xsl:map-entry key="'&#x6662;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x6663;'" select="'zhe1'"/>
                <xsl:map-entry key="'&#x6664;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x6665;'" select="'han4'"/>
                <xsl:map-entry key="'&#x6666;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6667;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x6668;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x6669;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x666A;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x666B;'" select="'zhuo1'"/>
                <xsl:map-entry key="'&#x666C;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x666D;'" select="'zhou3'"/>
                <xsl:map-entry key="'&#x666E;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x666F;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x6670;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6671;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x6672;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x6673;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6674;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x6675;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x6676;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x6677;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x6678;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x6679;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x667A;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x667B;'" select="'an3'"/>
                <xsl:map-entry key="'&#x667C;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x667D;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x667E;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x667F;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x6680;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x6681;'" select="'xiao3'"/>
                <xsl:map-entry key="'&#x6682;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x6683;'" select="''"/>
                <xsl:map-entry key="'&#x6684;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x6685;'" select="'xuan3'"/>
                <xsl:map-entry key="'&#x6686;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6687;'" select="'xia2	xia4'"/>
                <xsl:map-entry key="'&#x6688;'" select="'yun1	yun4'"/>
                <xsl:map-entry key="'&#x6689;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x668A;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x668B;'" select="'min3	min2'"/>
                <xsl:map-entry key="'&#x668C;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x668D;'" select="'he4'"/>
                <xsl:map-entry key="'&#x668E;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x668F;'" select="'du3'"/>
                <xsl:map-entry key="'&#x6690;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x6691;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x6692;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x6693;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x6694;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x6695;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x6696;'" select="'nuan3'"/>
                <xsl:map-entry key="'&#x6697;'" select="'an4'"/>
                <xsl:map-entry key="'&#x6698;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x6699;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x669A;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x669B;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x669C;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x669D;'" select="'ming2	ming4'"/>
                <xsl:map-entry key="'&#x669E;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x669F;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x66A0;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x66A1;'" select="'weng3'"/>
                <xsl:map-entry key="'&#x66A2;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x66A3;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x66A4;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x66A5;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x66A6;'" select="'li4'"/>
                <xsl:map-entry key="'&#x66A7;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x66A8;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x66A9;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x66AA;'" select="'men3'"/>
                <xsl:map-entry key="'&#x66AB;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x66AC;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x66AD;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x66AE;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x66AF;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x66B0;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x66B1;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x66B2;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x66B3;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x66B4;'" select="'bao4	pu4'"/>
                <xsl:map-entry key="'&#x66B5;'" select="'han4'"/>
                <xsl:map-entry key="'&#x66B6;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x66B7;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x66B8;'" select="'liao3'"/>
                <xsl:map-entry key="'&#x66B9;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x66BA;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x66BB;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x66BC;'" select="'pie1'"/>
                <xsl:map-entry key="'&#x66BD;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x66BE;'" select="'tun1'"/>
                <xsl:map-entry key="'&#x66BF;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x66C0;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x66C1;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x66C2;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x66C3;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x66C4;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x66C5;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x66C6;'" select="'li4'"/>
                <xsl:map-entry key="'&#x66C7;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x66C8;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x66C9;'" select="'xiao3'"/>
                <xsl:map-entry key="'&#x66CA;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x66CB;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x66CC;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x66CD;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x66CE;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x66CF;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x66D0;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x66D1;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x66D2;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x66D3;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x66D4;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x66D5;'" select="'yian4'"/>
                <xsl:map-entry key="'&#x66D6;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x66D7;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x66D8;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x66D9;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x66DA;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x66DB;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x66DC;'" select="'yao4	yue4'"/>
                <xsl:map-entry key="'&#x66DD;'" select="'pu4	bao4'"/>
                <xsl:map-entry key="'&#x66DE;'" select="'li4'"/>
                <xsl:map-entry key="'&#x66DF;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x66E0;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x66E1;'" select="'die2'"/>
                <xsl:map-entry key="'&#x66E2;'" select="''"/>
                <xsl:map-entry key="'&#x66E3;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x66E4;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x66E5;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x66E6;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x66E7;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x66E8;'" select="'long2'"/>
                <xsl:map-entry key="'&#x66E9;'" select="'nang3'"/>
                <xsl:map-entry key="'&#x66EA;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x66EB;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x66EC;'" select="'shai4'"/>
                <xsl:map-entry key="'&#x66ED;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x66EE;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x66EF;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x66F0;'" select="'yue1'"/>
                <xsl:map-entry key="'&#x66F1;'" select="'yue1'"/>
                <xsl:map-entry key="'&#x66F2;'" select="'qu3	qu1'"/>
                <xsl:map-entry key="'&#x66F3;'" select="'zhuai4'"/>
                <xsl:map-entry key="'&#x66F4;'" select="'geng4	geng1'"/>
                <xsl:map-entry key="'&#x66F5;'" select="'zhuai4'"/>
                <xsl:map-entry key="'&#x66F6;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x66F7;'" select="'he2'"/>
                <xsl:map-entry key="'&#x66F8;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x66F9;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x66FA;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x66FB;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x66FC;'" select="'man4'"/>
                <xsl:map-entry key="'&#x66FD;'" select="'ceng2	zeng1'"/>
                <xsl:map-entry key="'&#x66FE;'" select="'ceng2	zeng1'"/>
                <xsl:map-entry key="'&#x66FF;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x6700;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x6701;'" select="'can3'"/>
                <xsl:map-entry key="'&#x6702;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6703;'" select="'hui4	hui3	kuai4'"/>
                <xsl:map-entry key="'&#x6704;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x6705;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x6706;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x6707;'" select="'pi2	bi4'"/>
                <xsl:map-entry key="'&#x6708;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x6709;'" select="'you3	you4'"/>
                <xsl:map-entry key="'&#x670A;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x670B;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x670C;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x670D;'" select="'fu2	fu4'"/>
                <xsl:map-entry key="'&#x670E;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x670F;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x6710;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x6711;'" select="''"/>
                <xsl:map-entry key="'&#x6712;'" select="'nu:4'"/>
                <xsl:map-entry key="'&#x6713;'" select="'tiao4	tiao3'"/>
                <xsl:map-entry key="'&#x6714;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x6715;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x6716;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x6717;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x6718;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x6719;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x671A;'" select="'huang1'"/>
                <xsl:map-entry key="'&#x671B;'" select="'wang4'"/>
                <xsl:map-entry key="'&#x671C;'" select="'tun1'"/>
                <xsl:map-entry key="'&#x671D;'" select="'zhao1	chao2'"/>
                <xsl:map-entry key="'&#x671E;'" select="'ji1	qi1'"/>
                <xsl:map-entry key="'&#x671F;'" select="'ji1	qi1'"/>
                <xsl:map-entry key="'&#x6720;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x6721;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x6722;'" select="'wang4'"/>
                <xsl:map-entry key="'&#x6723;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x6724;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x6725;'" select="''"/>
                <xsl:map-entry key="'&#x6726;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x6727;'" select="'long2'"/>
                <xsl:map-entry key="'&#x6728;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x6729;'" select="'deng3'"/>
                <xsl:map-entry key="'&#x672A;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x672B;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x672C;'" select="'ben3'"/>
                <xsl:map-entry key="'&#x672D;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x672E;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x672F;'" select="'zhu2	shu4'"/>
                <xsl:map-entry key="'&#x6730;'" select="''"/>
                <xsl:map-entry key="'&#x6731;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x6732;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x6733;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x6734;'" select="'po4	piao2	pu3'"/>
                <xsl:map-entry key="'&#x6735;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x6736;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x6737;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x6738;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6739;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x673A;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x673B;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x673C;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x673D;'" select="'xiu3'"/>
                <xsl:map-entry key="'&#x673E;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x673F;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x6740;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x6741;'" select="''"/>
                <xsl:map-entry key="'&#x6742;'" select="'za2'"/>
                <xsl:map-entry key="'&#x6743;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x6744;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6745;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6746;'" select="'gan1	gan3'"/>
                <xsl:map-entry key="'&#x6747;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x6748;'" select="'cha1	cha4'"/>
                <xsl:map-entry key="'&#x6749;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x674A;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x674B;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x674C;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x674D;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x674E;'" select="'li3'"/>
                <xsl:map-entry key="'&#x674F;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x6750;'" select="'cai2'"/>
                <xsl:map-entry key="'&#x6751;'" select="'cun1'"/>
                <xsl:map-entry key="'&#x6752;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x6753;'" select="'shao2	biao1'"/>
                <xsl:map-entry key="'&#x6754;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x6755;'" select="'di4'"/>
                <xsl:map-entry key="'&#x6756;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x6757;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x6758;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x6759;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x675A;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x675B;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x675C;'" select="'du4'"/>
                <xsl:map-entry key="'&#x675D;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x675E;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x675F;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6760;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x6761;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x6762;'" select="''"/>
                <xsl:map-entry key="'&#x6763;'" select="''"/>
                <xsl:map-entry key="'&#x6764;'" select="''"/>
                <xsl:map-entry key="'&#x6765;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x6766;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x6767;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x6768;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x6769;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x676A;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x676B;'" select="'si4'"/>
                <xsl:map-entry key="'&#x676C;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x676D;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x676E;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x676F;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x6770;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6771;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x6772;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x6773;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x6774;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x6775;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x6776;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x6777;'" select="'pa2	ba5'"/>
                <xsl:map-entry key="'&#x6778;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6779;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x677A;'" select="'xin2'"/>
                <xsl:map-entry key="'&#x677B;'" select="'chou3	niu3'"/>
                <xsl:map-entry key="'&#x677C;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x677D;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x677E;'" select="'song1'"/>
                <xsl:map-entry key="'&#x677F;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x6780;'" select="'song1'"/>
                <xsl:map-entry key="'&#x6781;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6782;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x6783;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x6784;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x6785;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x6786;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x6787;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x6788;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6789;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x678A;'" select="'ang4'"/>
                <xsl:map-entry key="'&#x678B;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x678C;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x678D;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x678E;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x678F;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x6790;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6791;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6792;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x6793;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x6794;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6795;'" select="'zhen3	zhen4'"/>
                <xsl:map-entry key="'&#x6796;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x6797;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x6798;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x6799;'" select="'e4'"/>
                <xsl:map-entry key="'&#x679A;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x679B;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x679C;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x679D;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x679E;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x679F;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x67A0;'" select="''"/>
                <xsl:map-entry key="'&#x67A1;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x67A2;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x67A3;'" select="'zao3'"/>
                <xsl:map-entry key="'&#x67A4;'" select="''"/>
                <xsl:map-entry key="'&#x67A5;'" select="'li4'"/>
                <xsl:map-entry key="'&#x67A6;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x67A7;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x67A8;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x67A9;'" select="'song1'"/>
                <xsl:map-entry key="'&#x67AA;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x67AB;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x67AC;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x67AD;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x67AE;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x67AF;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x67B0;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x67B1;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x67B2;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x67B3;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x67B4;'" select="'guai3'"/>
                <xsl:map-entry key="'&#x67B5;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x67B6;'" select="'jia4'"/>
                <xsl:map-entry key="'&#x67B7;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x67B8;'" select="'gou3	gou1	ju3'"/>
                <xsl:map-entry key="'&#x67B9;'" select="'bao1	fu2'"/>
                <xsl:map-entry key="'&#x67BA;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x67BB;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x67BC;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x67BD;'" select="'sang1'"/>
                <xsl:map-entry key="'&#x67BE;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x67BF;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x67C0;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x67C1;'" select="'duo4	duo1	tuo2'"/>
                <xsl:map-entry key="'&#x67C2;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x67C3;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x67C4;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x67C5;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x67C6;'" select="'la1'"/>
                <xsl:map-entry key="'&#x67C7;'" select="'he2'"/>
                <xsl:map-entry key="'&#x67C8;'" select="'pan2	ban4'"/>
                <xsl:map-entry key="'&#x67C9;'" select="'fan2	bian1'"/>
                <xsl:map-entry key="'&#x67CA;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x67CB;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x67CC;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x67CD;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x67CE;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x67CF;'" select="'bo2	bai3'"/>
                <xsl:map-entry key="'&#x67D0;'" select="'mou3'"/>
                <xsl:map-entry key="'&#x67D1;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x67D2;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x67D3;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x67D4;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x67D5;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x67D6;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x67D7;'" select="'song1'"/>
                <xsl:map-entry key="'&#x67D8;'" select="'zhe4'"/>
                <xsl:map-entry key="'&#x67D9;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x67DA;'" select="'you4	you2'"/>
                <xsl:map-entry key="'&#x67DB;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x67DC;'" select="'ju3	gui4'"/>
                <xsl:map-entry key="'&#x67DD;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x67DE;'" select="'zuo4	zha4'"/>
                <xsl:map-entry key="'&#x67DF;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x67E0;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x67E1;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x67E2;'" select="'di3'"/>
                <xsl:map-entry key="'&#x67E3;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x67E4;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x67E5;'" select="'cha2	zha1'"/>
                <xsl:map-entry key="'&#x67E6;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x67E7;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x67E8;'" select="'pu'"/>
                <xsl:map-entry key="'&#x67E9;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x67EA;'" select="'ao1'"/>
                <xsl:map-entry key="'&#x67EB;'" select="'fu2	bi4'"/>
                <xsl:map-entry key="'&#x67EC;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x67ED;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x67EE;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x67EF;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x67F0;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x67F1;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x67F2;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x67F3;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x67F4;'" select="'chai2'"/>
                <xsl:map-entry key="'&#x67F5;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x67F6;'" select="'si4'"/>
                <xsl:map-entry key="'&#x67F7;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x67F8;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x67F9;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x67FA;'" select="'guai3'"/>
                <xsl:map-entry key="'&#x67FB;'" select="'cha2	zha1'"/>
                <xsl:map-entry key="'&#x67FC;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x67FD;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x67FE;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x67FF;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x6800;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x6801;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x6802;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x6803;'" select="''"/>
                <xsl:map-entry key="'&#x6804;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x6805;'" select="'zha4	shan1'"/>
                <xsl:map-entry key="'&#x6806;'" select="''"/>
                <xsl:map-entry key="'&#x6807;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x6808;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x6809;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x680A;'" select="'long2'"/>
                <xsl:map-entry key="'&#x680B;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x680C;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x680D;'" select="''"/>
                <xsl:map-entry key="'&#x680E;'" select="'li4	yue4'"/>
                <xsl:map-entry key="'&#x680F;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x6810;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x6811;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6812;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6813;'" select="'shuan1'"/>
                <xsl:map-entry key="'&#x6814;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6815;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x6816;'" select="'qi1	xi1'"/>
                <xsl:map-entry key="'&#x6817;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6818;'" select="'chi2	yi2'"/>
                <xsl:map-entry key="'&#x6819;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x681A;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x681B;'" select="'li4'"/>
                <xsl:map-entry key="'&#x681C;'" select="'su4'"/>
                <xsl:map-entry key="'&#x681D;'" select="'gua1	kuo4'"/>
                <xsl:map-entry key="'&#x681E;'" select="'kan1'"/>
                <xsl:map-entry key="'&#x681F;'" select="'bing1'"/>
                <xsl:map-entry key="'&#x6820;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x6821;'" select="'xiao4	jiao4'"/>
                <xsl:map-entry key="'&#x6822;'" select="'bo2	bai3'"/>
                <xsl:map-entry key="'&#x6823;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x6824;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x6825;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x6826;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x6827;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6828;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x6829;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x682A;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x682B;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x682C;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x682D;'" select="'er2'"/>
                <xsl:map-entry key="'&#x682E;'" select="'er3'"/>
                <xsl:map-entry key="'&#x682F;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6830;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x6831;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x6832;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x6833;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x6834;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x6835;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6836;'" select="'yin'"/>
                <xsl:map-entry key="'&#x6837;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x6838;'" select="'he2	hu2'"/>
                <xsl:map-entry key="'&#x6839;'" select="'gen1'"/>
                <xsl:map-entry key="'&#x683A;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x683B;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x683C;'" select="'ge2	ge1'"/>
                <xsl:map-entry key="'&#x683D;'" select="'zai1'"/>
                <xsl:map-entry key="'&#x683E;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x683F;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x6840;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6841;'" select="'hang2	heng2'"/>
                <xsl:map-entry key="'&#x6842;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x6843;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x6844;'" select="'guang4	guang1'"/>
                <xsl:map-entry key="'&#x6845;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6846;'" select="'kuang4	kuang1'"/>
                <xsl:map-entry key="'&#x6847;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x6848;'" select="'an4'"/>
                <xsl:map-entry key="'&#x6849;'" select="'an4	an1'"/>
                <xsl:map-entry key="'&#x684A;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x684B;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x684C;'" select="'zhuo1'"/>
                <xsl:map-entry key="'&#x684D;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x684E;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x684F;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x6850;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x6851;'" select="'sang1'"/>
                <xsl:map-entry key="'&#x6852;'" select="'sang1'"/>
                <xsl:map-entry key="'&#x6853;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x6854;'" select="'jie2	ju2'"/>
                <xsl:map-entry key="'&#x6855;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x6856;'" select="'xue4'"/>
                <xsl:map-entry key="'&#x6857;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x6858;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x6859;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x685A;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x685B;'" select="''"/>
                <xsl:map-entry key="'&#x685C;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x685D;'" select="''"/>
                <xsl:map-entry key="'&#x685E;'" select="''"/>
                <xsl:map-entry key="'&#x685F;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x6860;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x6861;'" select="'rao2'"/>
                <xsl:map-entry key="'&#x6862;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x6863;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x6864;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x6865;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x6866;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x6867;'" select="'gui4	hui4'"/>
                <xsl:map-entry key="'&#x6868;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x6869;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x686A;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x686B;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x686C;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x686D;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x686E;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x686F;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x6870;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x6871;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x6872;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x6873;'" select="'ben4'"/>
                <xsl:map-entry key="'&#x6874;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x6875;'" select="'rui3'"/>
                <xsl:map-entry key="'&#x6876;'" select="'tong3'"/>
                <xsl:map-entry key="'&#x6877;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6878;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6879;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x687A;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x687B;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x687C;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x687D;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x687E;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x687F;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x6880;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x6881;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x6882;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6883;'" select="'ting3	ting4'"/>
                <xsl:map-entry key="'&#x6884;'" select="'you3'"/>
                <xsl:map-entry key="'&#x6885;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x6886;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x6887;'" select="'long4'"/>
                <xsl:map-entry key="'&#x6888;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x6889;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x688A;'" select="'di4'"/>
                <xsl:map-entry key="'&#x688B;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x688C;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x688D;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x688E;'" select="'ao1'"/>
                <xsl:map-entry key="'&#x688F;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x6890;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6891;'" select="'di2'"/>
                <xsl:map-entry key="'&#x6892;'" select="'han2'"/>
                <xsl:map-entry key="'&#x6893;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x6894;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x6895;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x6896;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x6897;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x6898;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x6899;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x689A;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x689B;'" select="'nuo2'"/>
                <xsl:map-entry key="'&#x689C;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x689D;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x689E;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x689F;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x68A0;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x68A1;'" select="'kuan3'"/>
                <xsl:map-entry key="'&#x68A2;'" select="'shao1	sao4'"/>
                <xsl:map-entry key="'&#x68A3;'" select="'cen2'"/>
                <xsl:map-entry key="'&#x68A4;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x68A5;'" select="'song1'"/>
                <xsl:map-entry key="'&#x68A6;'" select="'meng4'"/>
                <xsl:map-entry key="'&#x68A7;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x68A8;'" select="'li2'"/>
                <xsl:map-entry key="'&#x68A9;'" select="'li2'"/>
                <xsl:map-entry key="'&#x68AA;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x68AB;'" select="'cen1'"/>
                <xsl:map-entry key="'&#x68AC;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x68AD;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x68AE;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x68AF;'" select="'ti1'"/>
                <xsl:map-entry key="'&#x68B0;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x68B1;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x68B2;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x68B3;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x68B4;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x68B5;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x68B6;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x68B7;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x68B8;'" select="'li2'"/>
                <xsl:map-entry key="'&#x68B9;'" select="'bing1	bin1'"/>
                <xsl:map-entry key="'&#x68BA;'" select="''"/>
                <xsl:map-entry key="'&#x68BB;'" select="''"/>
                <xsl:map-entry key="'&#x68BC;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x68BD;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x68BE;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x68BF;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x68C0;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x68C1;'" select="'zhuo1'"/>
                <xsl:map-entry key="'&#x68C2;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x68C3;'" select="'li2'"/>
                <xsl:map-entry key="'&#x68C4;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x68C5;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x68C6;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x68C7;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x68C8;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x68C9;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x68CA;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x68CB;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x68CC;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x68CD;'" select="'gun4'"/>
                <xsl:map-entry key="'&#x68CE;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x68CF;'" select="'de2'"/>
                <xsl:map-entry key="'&#x68D0;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x68D1;'" select="'pai2'"/>
                <xsl:map-entry key="'&#x68D2;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x68D3;'" select="'pou3	bang4'"/>
                <xsl:map-entry key="'&#x68D4;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x68D5;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x68D6;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x68D7;'" select="'zao3'"/>
                <xsl:map-entry key="'&#x68D8;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x68D9;'" select="'li4'"/>
                <xsl:map-entry key="'&#x68DA;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x68DB;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x68DC;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x68DD;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x68DE;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x68DF;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x68E0;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x68E1;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x68E2;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x68E3;'" select="'di4'"/>
                <xsl:map-entry key="'&#x68E4;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x68E5;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x68E6;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x68E7;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x68E8;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x68E9;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x68EA;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x68EB;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x68EC;'" select="'quan1'"/>
                <xsl:map-entry key="'&#x68ED;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x68EE;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x68EF;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x68F0;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x68F1;'" select="'leng2	leng1	ling2'"/>
                <xsl:map-entry key="'&#x68F2;'" select="'qi1	xi1'"/>
                <xsl:map-entry key="'&#x68F3;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x68F4;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x68F5;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x68F6;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x68F7;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x68F8;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x68F9;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x68FA;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x68FB;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x68FC;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x68FD;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x68FE;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x68FF;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x6900;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x6901;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x6902;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x6903;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x6904;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x6905;'" select="'yi3	yi1'"/>
                <xsl:map-entry key="'&#x6906;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x6907;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x6908;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x6909;'" select="'cheng2	sheng4'"/>
                <xsl:map-entry key="'&#x690A;'" select="'zuo2'"/>
                <xsl:map-entry key="'&#x690B;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x690C;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x690D;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x690E;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x690F;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x6910;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x6911;'" select="'bei1	pi2'"/>
                <xsl:map-entry key="'&#x6912;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x6913;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x6914;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x6915;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x6916;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x6917;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x6918;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x6919;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x691A;'" select="''"/>
                <xsl:map-entry key="'&#x691B;'" select="''"/>
                <xsl:map-entry key="'&#x691C;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x691D;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x691E;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x691F;'" select="'du2'"/>
                <xsl:map-entry key="'&#x6920;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x6921;'" select="''"/>
                <xsl:map-entry key="'&#x6922;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x6923;'" select="''"/>
                <xsl:map-entry key="'&#x6924;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x6925;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x6926;'" select="''"/>
                <xsl:map-entry key="'&#x6927;'" select="''"/>
                <xsl:map-entry key="'&#x6928;'" select="''"/>
                <xsl:map-entry key="'&#x6929;'" select="''"/>
                <xsl:map-entry key="'&#x692A;'" select="'peng4'"/>
                <xsl:map-entry key="'&#x692B;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x692C;'" select="''"/>
                <xsl:map-entry key="'&#x692D;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x692E;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x692F;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x6930;'" select="'ye2	ye1'"/>
                <xsl:map-entry key="'&#x6931;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x6932;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x6933;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x6934;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x6935;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x6936;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x6937;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x6938;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6939;'" select="'shen4	zhen1'"/>
                <xsl:map-entry key="'&#x693A;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x693B;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x693C;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x693D;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x693E;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x693F;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x6940;'" select="'yu3	ju3'"/>
                <xsl:map-entry key="'&#x6941;'" select="'he2'"/>
                <xsl:map-entry key="'&#x6942;'" select="'zha1	cha2'"/>
                <xsl:map-entry key="'&#x6943;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x6944;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x6945;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6946;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x6947;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x6948;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x6949;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x694A;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x694B;'" select="'la4'"/>
                <xsl:map-entry key="'&#x694C;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x694D;'" select="'ben3'"/>
                <xsl:map-entry key="'&#x694E;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x694F;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x6950;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x6951;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x6952;'" select="'si1'"/>
                <xsl:map-entry key="'&#x6953;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x6954;'" select="'xie1'"/>
                <xsl:map-entry key="'&#x6955;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x6956;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6957;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x6958;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x6959;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x695A;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x695B;'" select="'hu4	ku3'"/>
                <xsl:map-entry key="'&#x695C;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x695D;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x695E;'" select="'leng2	leng4'"/>
                <xsl:map-entry key="'&#x695F;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x6960;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x6961;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6962;'" select="'you2'"/>
                <xsl:map-entry key="'&#x6963;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x6964;'" select="'song3'"/>
                <xsl:map-entry key="'&#x6965;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x6966;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x6967;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x6968;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x6969;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x696A;'" select="'die2'"/>
                <xsl:map-entry key="'&#x696B;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x696C;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x696D;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x696E;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x696F;'" select="'shun3'"/>
                <xsl:map-entry key="'&#x6970;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6971;'" select="'cou4'"/>
                <xsl:map-entry key="'&#x6972;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x6973;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x6974;'" select="'di4'"/>
                <xsl:map-entry key="'&#x6975;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6976;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6977;'" select="'kai3	jie1'"/>
                <xsl:map-entry key="'&#x6978;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x6979;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x697A;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x697B;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x697C;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x697D;'" select="'le4	yue4'"/>
                <xsl:map-entry key="'&#x697E;'" select="''"/>
                <xsl:map-entry key="'&#x697F;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x6980;'" select="'pin3'"/>
                <xsl:map-entry key="'&#x6981;'" select="''"/>
                <xsl:map-entry key="'&#x6982;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x6983;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x6984;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x6985;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x6986;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6987;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x6988;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x6989;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x698A;'" select="''"/>
                <xsl:map-entry key="'&#x698B;'" select="''"/>
                <xsl:map-entry key="'&#x698C;'" select="''"/>
                <xsl:map-entry key="'&#x698D;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x698E;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x698F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6990;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x6991;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x6992;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x6993;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6994;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x6995;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x6996;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x6997;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x6998;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x6999;'" select="'ta3'"/>
                <xsl:map-entry key="'&#x699A;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x699B;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x699C;'" select="'bang3'"/>
                <xsl:map-entry key="'&#x699D;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x699E;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x699F;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x69A0;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x69A1;'" select="'su4'"/>
                <xsl:map-entry key="'&#x69A2;'" select="'jia4'"/>
                <xsl:map-entry key="'&#x69A3;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x69A4;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x69A5;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x69A6;'" select="'gan4	han2'"/>
                <xsl:map-entry key="'&#x69A7;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x69A8;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x69A9;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x69AA;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x69AB;'" select="'sun3'"/>
                <xsl:map-entry key="'&#x69AC;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x69AD;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x69AE;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x69AF;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x69B0;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x69B1;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x69B2;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x69B3;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x69B4;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x69B5;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x69B6;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x69B7;'" select="'que4'"/>
                <xsl:map-entry key="'&#x69B8;'" select="'zhai1'"/>
                <xsl:map-entry key="'&#x69B9;'" select="'si1'"/>
                <xsl:map-entry key="'&#x69BA;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x69BB;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x69BC;'" select="'ke4	ke2'"/>
                <xsl:map-entry key="'&#x69BD;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x69BE;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x69BF;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x69C0;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x69C1;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x69C2;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x69C3;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x69C4;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x69C5;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x69C6;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x69C7;'" select="'dian1	zhen3'"/>
                <xsl:map-entry key="'&#x69C8;'" select="'nou4'"/>
                <xsl:map-entry key="'&#x69C9;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x69CA;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x69CB;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x69CC;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x69CD;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x69CE;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x69CF;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x69D0;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x69D1;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x69D2;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x69D3;'" select="'gang4'"/>
                <xsl:map-entry key="'&#x69D4;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x69D5;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x69D6;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x69D7;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x69D8;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x69D9;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x69DA;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x69DB;'" select="'jian4	kan3'"/>
                <xsl:map-entry key="'&#x69DC;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x69DD;'" select="''"/>
                <xsl:map-entry key="'&#x69DE;'" select="'long2'"/>
                <xsl:map-entry key="'&#x69DF;'" select="'bin1	bing1'"/>
                <xsl:map-entry key="'&#x69E0;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x69E1;'" select="''"/>
                <xsl:map-entry key="'&#x69E2;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x69E3;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x69E4;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x69E5;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x69E6;'" select="'yong2'"/>
                <xsl:map-entry key="'&#x69E7;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x69E8;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x69E9;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x69EA;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x69EB;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x69EC;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x69ED;'" select="'cu4	qi1'"/>
                <xsl:map-entry key="'&#x69EE;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x69EF;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x69F0;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x69F1;'" select="'you3'"/>
                <xsl:map-entry key="'&#x69F2;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x69F3;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x69F4;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x69F5;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x69F6;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x69F7;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x69F8;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x69F9;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x69FA;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x69FB;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x69FC;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x69FD;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x69FE;'" select="'man2	man4'"/>
                <xsl:map-entry key="'&#x69FF;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x6A00;'" select="'di4'"/>
                <xsl:map-entry key="'&#x6A01;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x6A02;'" select="'le4	yue4'"/>
                <xsl:map-entry key="'&#x6A03;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x6A04;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x6A05;'" select="'cong1	zong1'"/>
                <xsl:map-entry key="'&#x6A06;'" select="'li2'"/>
                <xsl:map-entry key="'&#x6A07;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x6A08;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x6A09;'" select="'shuang3'"/>
                <xsl:map-entry key="'&#x6A0A;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x6A0B;'" select="'tong1'"/>
                <xsl:map-entry key="'&#x6A0C;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x6A0D;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x6A0E;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x6A0F;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x6A10;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x6A11;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x6A12;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6A13;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x6A14;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x6A15;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6A16;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x6A17;'" select="'shu1	chu1'"/>
                <xsl:map-entry key="'&#x6A18;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x6A19;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x6A1A;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x6A1B;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x6A1C;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6A1D;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x6A1E;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6A1F;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x6A20;'" select="'men2'"/>
                <xsl:map-entry key="'&#x6A21;'" select="'mo2	mu2'"/>
                <xsl:map-entry key="'&#x6A22;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x6A23;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x6A24;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x6A25;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x6A26;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x6A27;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x6A28;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6A29;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x6A2A;'" select="'heng2	heng4'"/>
                <xsl:map-entry key="'&#x6A2B;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x6A2C;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x6A2D;'" select="''"/>
                <xsl:map-entry key="'&#x6A2E;'" select="''"/>
                <xsl:map-entry key="'&#x6A2F;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x6A30;'" select="''"/>
                <xsl:map-entry key="'&#x6A31;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x6A32;'" select="'er4'"/>
                <xsl:map-entry key="'&#x6A33;'" select="'xin2'"/>
                <xsl:map-entry key="'&#x6A34;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x6A35;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x6A36;'" select="'zui1'"/>
                <xsl:map-entry key="'&#x6A37;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x6A38;'" select="'pu2	po4	pu3'"/>
                <xsl:map-entry key="'&#x6A39;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6A3A;'" select="'hua4	hua2'"/>
                <xsl:map-entry key="'&#x6A3B;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x6A3C;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x6A3D;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x6A3E;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x6A3F;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x6A40;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6A41;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6A42;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x6A43;'" select="'fa1'"/>
                <xsl:map-entry key="'&#x6A44;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x6A45;'" select="'mo2	mu2'"/>
                <xsl:map-entry key="'&#x6A46;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x6A47;'" select="'qiao1	cui4'"/>
                <xsl:map-entry key="'&#x6A48;'" select="'rao2'"/>
                <xsl:map-entry key="'&#x6A49;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x6A4A;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x6A4B;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x6A4C;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x6A4D;'" select="'run4'"/>
                <xsl:map-entry key="'&#x6A4E;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x6A4F;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x6A50;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x6A51;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x6A52;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x6A53;'" select="'shun4'"/>
                <xsl:map-entry key="'&#x6A54;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x6A55;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x6A56;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x6A57;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x6A58;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x6A59;'" select="'cheng2	chen2'"/>
                <xsl:map-entry key="'&#x6A5A;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6A5B;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6A5C;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6A5D;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x6A5E;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6A5F;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x6A60;'" select="'nuo3'"/>
                <xsl:map-entry key="'&#x6A61;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x6A62;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x6A63;'" select="'ning3'"/>
                <xsl:map-entry key="'&#x6A64;'" select="'rui3'"/>
                <xsl:map-entry key="'&#x6A65;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x6A66;'" select="'chuang2	tong2'"/>
                <xsl:map-entry key="'&#x6A67;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x6A68;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x6A69;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x6A6A;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x6A6B;'" select="'heng2	heng4'"/>
                <xsl:map-entry key="'&#x6A6C;'" select="'cen2'"/>
                <xsl:map-entry key="'&#x6A6D;'" select="'gu1	ku1'"/>
                <xsl:map-entry key="'&#x6A6E;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x6A6F;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x6A70;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x6A71;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x6A72;'" select="''"/>
                <xsl:map-entry key="'&#x6A73;'" select="''"/>
                <xsl:map-entry key="'&#x6A74;'" select="''"/>
                <xsl:map-entry key="'&#x6A75;'" select="''"/>
                <xsl:map-entry key="'&#x6A76;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6A77;'" select="'dou1'"/>
                <xsl:map-entry key="'&#x6A78;'" select="''"/>
                <xsl:map-entry key="'&#x6A79;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x6A7A;'" select="''"/>
                <xsl:map-entry key="'&#x6A7B;'" select="''"/>
                <xsl:map-entry key="'&#x6A7C;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x6A7D;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x6A7E;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6A7F;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x6A80;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x6A81;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x6A82;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x6A83;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x6A84;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x6A85;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x6A86;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x6A87;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x6A88;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x6A89;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x6A8A;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x6A8B;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x6A8C;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x6A8D;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6A8E;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x6A8F;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x6A90;'" select="'yan2	yin2'"/>
                <xsl:map-entry key="'&#x6A91;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x6A92;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x6A93;'" select="'hui3'"/>
                <xsl:map-entry key="'&#x6A94;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x6A95;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6A96;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x6A97;'" select="'bo4	bo2'"/>
                <xsl:map-entry key="'&#x6A98;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6A99;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x6A9A;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x6A9B;'" select="'zhua1'"/>
                <xsl:map-entry key="'&#x6A9C;'" select="'gui4	hui4'"/>
                <xsl:map-entry key="'&#x6A9D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6A9E;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x6A9F;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x6AA0;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x6AA1;'" select="'zhe4'"/>
                <xsl:map-entry key="'&#x6AA2;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x6AA3;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x6AA4;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x6AA5;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x6AA6;'" select="'biao3'"/>
                <xsl:map-entry key="'&#x6AA7;'" select="'song1'"/>
                <xsl:map-entry key="'&#x6AA8;'" select="'she1'"/>
                <xsl:map-entry key="'&#x6AA9;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x6AAA;'" select="'li4	yue4'"/>
                <xsl:map-entry key="'&#x6AAB;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x6AAC;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x6AAD;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x6AAE;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x6AAF;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x6AB0;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x6AB1;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x6AB2;'" select="'toan2'"/>
                <xsl:map-entry key="'&#x6AB3;'" select="'bin1	bing1'"/>
                <xsl:map-entry key="'&#x6AB4;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x6AB5;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6AB6;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6AB7;'" select="'mi2	ni3'"/>
                <xsl:map-entry key="'&#x6AB8;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x6AB9;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x6ABA;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x6ABB;'" select="'jian4	kan3'"/>
                <xsl:map-entry key="'&#x6ABC;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x6ABD;'" select="'er2'"/>
                <xsl:map-entry key="'&#x6ABE;'" select="'qing3'"/>
                <xsl:map-entry key="'&#x6ABF;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x6AC0;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x6AC1;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6AC2;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x6AC3;'" select="'gui4	ju3'"/>
                <xsl:map-entry key="'&#x6AC4;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x6AC5;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x6AC6;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x6AC7;'" select="'po2'"/>
                <xsl:map-entry key="'&#x6AC8;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x6AC9;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x6ACA;'" select="''"/>
                <xsl:map-entry key="'&#x6ACB;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x6ACC;'" select="'you1'"/>
                <xsl:map-entry key="'&#x6ACD;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6ACE;'" select="'guang4'"/>
                <xsl:map-entry key="'&#x6ACF;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6AD0;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x6AD1;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x6AD2;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x6AD3;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x6AD4;'" select="'li44'"/>
                <xsl:map-entry key="'&#x6AD5;'" select="'cuan2'"/>
                <xsl:map-entry key="'&#x6AD6;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x6AD7;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x6AD8;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6AD9;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x6ADA;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x6ADB;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6ADC;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x6ADD;'" select="'du2'"/>
                <xsl:map-entry key="'&#x6ADE;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x6ADF;'" select="'li4	yue4'"/>
                <xsl:map-entry key="'&#x6AE0;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x6AE1;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x6AE2;'" select="'sou3'"/>
                <xsl:map-entry key="'&#x6AE3;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x6AE4;'" select="''"/>
                <xsl:map-entry key="'&#x6AE5;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x6AE6;'" select="''"/>
                <xsl:map-entry key="'&#x6AE7;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x6AE8;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x6AE9;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x6AEA;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6AEB;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x6AEC;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x6AED;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6AEE;'" select="'e4'"/>
                <xsl:map-entry key="'&#x6AEF;'" select="'su1'"/>
                <xsl:map-entry key="'&#x6AF0;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x6AF1;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x6AF2;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6AF3;'" select="'long2'"/>
                <xsl:map-entry key="'&#x6AF4;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x6AF5;'" select="''"/>
                <xsl:map-entry key="'&#x6AF6;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x6AF7;'" select="''"/>
                <xsl:map-entry key="'&#x6AF8;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x6AF9;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x6AFA;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x6AFB;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x6AFC;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x6AFD;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x6AFE;'" select="'you2'"/>
                <xsl:map-entry key="'&#x6AFF;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x6B00;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x6B01;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x6B02;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x6B03;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x6B04;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x6B05;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x6B06;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x6B07;'" select="'she4'"/>
                <xsl:map-entry key="'&#x6B08;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6B09;'" select="'cong4'"/>
                <xsl:map-entry key="'&#x6B0A;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x6B0B;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x6B0C;'" select="''"/>
                <xsl:map-entry key="'&#x6B0D;'" select="''"/>
                <xsl:map-entry key="'&#x6B0E;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6B0F;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x6B10;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6B11;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x6B12;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x6B13;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x6B14;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6B15;'" select="''"/>
                <xsl:map-entry key="'&#x6B16;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x6B17;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x6B18;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x6B19;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x6B1A;'" select="'li3	ji1'"/>
                <xsl:map-entry key="'&#x6B1B;'" select="'ba4	ba3'"/>
                <xsl:map-entry key="'&#x6B1C;'" select="'nang2'"/>
                <xsl:map-entry key="'&#x6B1D;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6B1E;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x6B1F;'" select="''"/>
                <xsl:map-entry key="'&#x6B20;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x6B21;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x6B22;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x6B23;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x6B24;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6B25;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6B26;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6B27;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x6B28;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x6B29;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x6B2A;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x6B2B;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6B2C;'" select="'kai4'"/>
                <xsl:map-entry key="'&#x6B2D;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6B2E;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6B2F;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x6B30;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x6B31;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x6B32;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6B33;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x6B34;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x6B35;'" select="'kuan3'"/>
                <xsl:map-entry key="'&#x6B36;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x6B37;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6B38;'" select="'e^1	e^2	e^3	e^4'"/>
                <xsl:map-entry key="'&#x6B39;'" select="'yi1	qi1'"/>
                <xsl:map-entry key="'&#x6B3A;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x6B3B;'" select="'hu1	chua1'"/>
                <xsl:map-entry key="'&#x6B3C;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x6B3D;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x6B3E;'" select="'kuan3'"/>
                <xsl:map-entry key="'&#x6B3F;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x6B40;'" select="'kuan3'"/>
                <xsl:map-entry key="'&#x6B41;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x6B42;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x6B43;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x6B44;'" select="'gua'"/>
                <xsl:map-entry key="'&#x6B45;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x6B46;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x6B47;'" select="'xie1'"/>
                <xsl:map-entry key="'&#x6B48;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6B49;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x6B4A;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x6B4B;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6B4C;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x6B4D;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x6B4E;'" select="'tan4'"/>
                <xsl:map-entry key="'&#x6B4F;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x6B50;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x6B51;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x6B52;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x6B53;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x6B54;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x6B55;'" select="'pen1'"/>
                <xsl:map-entry key="'&#x6B56;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6B57;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x6B58;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x6B59;'" select="'she4	xi1'"/>
                <xsl:map-entry key="'&#x6B5A;'" select="''"/>
                <xsl:map-entry key="'&#x6B5B;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x6B5C;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x6B5D;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6B5E;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x6B5F;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6B60;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x6B61;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x6B62;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x6B63;'" select="'zheng4	zheng1'"/>
                <xsl:map-entry key="'&#x6B64;'" select="'ci3'"/>
                <xsl:map-entry key="'&#x6B65;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x6B66;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x6B67;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x6B68;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x6B69;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x6B6A;'" select="'wai1'"/>
                <xsl:map-entry key="'&#x6B6B;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x6B6C;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x6B6D;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x6B6E;'" select="'se4'"/>
                <xsl:map-entry key="'&#x6B6F;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x6B70;'" select="'se4'"/>
                <xsl:map-entry key="'&#x6B71;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x6B72;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x6B73;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x6B74;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6B75;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x6B76;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6B77;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6B78;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x6B79;'" select="'dai3'"/>
                <xsl:map-entry key="'&#x6B7A;'" select="'dai3'"/>
                <xsl:map-entry key="'&#x6B7B;'" select="'si3'"/>
                <xsl:map-entry key="'&#x6B7C;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x6B7D;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x6B7E;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x6B7F;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x6B80;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x6B81;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x6B82;'" select="'cu2'"/>
                <xsl:map-entry key="'&#x6B83;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x6B84;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x6B85;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x6B86;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x6B87;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x6B88;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x6B89;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x6B8A;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6B8B;'" select="'can2'"/>
                <xsl:map-entry key="'&#x6B8C;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6B8D;'" select="'piao3'"/>
                <xsl:map-entry key="'&#x6B8E;'" select="'qia4'"/>
                <xsl:map-entry key="'&#x6B8F;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6B90;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6B91;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x6B92;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x6B93;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x6B94;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6B95;'" select="'fou3'"/>
                <xsl:map-entry key="'&#x6B96;'" select="'zhi2	shi5'"/>
                <xsl:map-entry key="'&#x6B97;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x6B98;'" select="'can2'"/>
                <xsl:map-entry key="'&#x6B99;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x6B9A;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x6B9B;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6B9C;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x6B9D;'" select="'zhen'"/>
                <xsl:map-entry key="'&#x6B9E;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x6B9F;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x6BA0;'" select="'chou4	xiu4'"/>
                <xsl:map-entry key="'&#x6BA1;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x6BA2;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x6BA3;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x6BA4;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x6BA5;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x6BA6;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x6BA7;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x6BA8;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6BA9;'" select="'cuan4'"/>
                <xsl:map-entry key="'&#x6BAA;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6BAB;'" select="'dan1	dan4'"/>
                <xsl:map-entry key="'&#x6BAC;'" select="'du4'"/>
                <xsl:map-entry key="'&#x6BAD;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x6BAE;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x6BAF;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x6BB0;'" select="'du2'"/>
                <xsl:map-entry key="'&#x6BB1;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x6BB2;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x6BB3;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x6BB4;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x6BB5;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x6BB6;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x6BB7;'" select="'yin1	yan1	yin3'"/>
                <xsl:map-entry key="'&#x6BB8;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x6BB9;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x6BBA;'" select="'sha1	shai4'"/>
                <xsl:map-entry key="'&#x6BBB;'" select="'ke2	qiao4'"/>
                <xsl:map-entry key="'&#x6BBC;'" select="'ke2	qiao4'"/>
                <xsl:map-entry key="'&#x6BBD;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x6BBE;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x6BBF;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x6BC0;'" select="'hui3'"/>
                <xsl:map-entry key="'&#x6BC1;'" select="'hui3'"/>
                <xsl:map-entry key="'&#x6BC2;'" select="'gu3	gu1'"/>
                <xsl:map-entry key="'&#x6BC3;'" select="'que4'"/>
                <xsl:map-entry key="'&#x6BC4;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x6BC5;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6BC6;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x6BC7;'" select="'hui3'"/>
                <xsl:map-entry key="'&#x6BC8;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x6BC9;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x6BCA;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x6BCB;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x6BCC;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x6BCD;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x6BCE;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x6BCF;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x6BD0;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x6BD1;'" select="'zuo3'"/>
                <xsl:map-entry key="'&#x6BD2;'" select="'du2'"/>
                <xsl:map-entry key="'&#x6BD3;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6BD4;'" select="'bi3	bi4'"/>
                <xsl:map-entry key="'&#x6BD5;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6BD6;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6BD7;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x6BD8;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x6BD9;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6BDA;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x6BDB;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x6BDC;'" select="''"/>
                <xsl:map-entry key="'&#x6BDD;'" select="''"/>
                <xsl:map-entry key="'&#x6BDE;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x6BDF;'" select="''"/>
                <xsl:map-entry key="'&#x6BE0;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x6BE1;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x6BE2;'" select="'sai1'"/>
                <xsl:map-entry key="'&#x6BE3;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x6BE4;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x6BE5;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6BE6;'" select="'er4'"/>
                <xsl:map-entry key="'&#x6BE7;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x6BE8;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x6BE9;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x6BEA;'" select="'mu2'"/>
                <xsl:map-entry key="'&#x6BEB;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x6BEC;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6BED;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x6BEE;'" select="''"/>
                <xsl:map-entry key="'&#x6BEF;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x6BF0;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x6BF1;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x6BF2;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x6BF3;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x6BF4;'" select="'bi1'"/>
                <xsl:map-entry key="'&#x6BF5;'" select="'san1'"/>
                <xsl:map-entry key="'&#x6BF6;'" select="''"/>
                <xsl:map-entry key="'&#x6BF7;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x6BF8;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x6BF9;'" select="'yu1	shu1'"/>
                <xsl:map-entry key="'&#x6BFA;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x6BFB;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x6BFC;'" select="'he2'"/>
                <xsl:map-entry key="'&#x6BFD;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x6BFE;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x6BFF;'" select="'san1'"/>
                <xsl:map-entry key="'&#x6C00;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x6C01;'" select="'mu2'"/>
                <xsl:map-entry key="'&#x6C02;'" select="'li2'"/>
                <xsl:map-entry key="'&#x6C03;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x6C04;'" select="'rong3'"/>
                <xsl:map-entry key="'&#x6C05;'" select="'chang3'"/>
                <xsl:map-entry key="'&#x6C06;'" select="'pu3	bang3'"/>
                <xsl:map-entry key="'&#x6C07;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x6C08;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x6C09;'" select="'sao4'"/>
                <xsl:map-entry key="'&#x6C0A;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x6C0B;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x6C0C;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x6C0D;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x6C0E;'" select="'die2'"/>
                <xsl:map-entry key="'&#x6C0F;'" select="'shi4	zhi1'"/>
                <xsl:map-entry key="'&#x6C10;'" select="'di3	di1'"/>
                <xsl:map-entry key="'&#x6C11;'" select="'min2'"/>
                <xsl:map-entry key="'&#x6C12;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6C13;'" select="'mang2	meng2'"/>
                <xsl:map-entry key="'&#x6C14;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6C15;'" select="'pie1'"/>
                <xsl:map-entry key="'&#x6C16;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x6C17;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6C18;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x6C19;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x6C1A;'" select="'chuan1'"/>
                <xsl:map-entry key="'&#x6C1B;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x6C1C;'" select="'ri4'"/>
                <xsl:map-entry key="'&#x6C1D;'" select="'nei4	nai3'"/>
                <xsl:map-entry key="'&#x6C1E;'" select="''"/>
                <xsl:map-entry key="'&#x6C1F;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x6C20;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x6C21;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x6C22;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x6C23;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6C24;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x6C25;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6C26;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x6C27;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x6C28;'" select="'an1'"/>
                <xsl:map-entry key="'&#x6C29;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x6C2A;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x6C2B;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x6C2C;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x6C2D;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x6C2E;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x6C2F;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x6C30;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x6C31;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x6C32;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x6C33;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x6C34;'" select="'shui3'"/>
                <xsl:map-entry key="'&#x6C35;'" select="'shui3'"/>
                <xsl:map-entry key="'&#x6C36;'" select="'zheng3'"/>
                <xsl:map-entry key="'&#x6C37;'" select="'bing1'"/>
                <xsl:map-entry key="'&#x6C38;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x6C39;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x6C3A;'" select="'shui3'"/>
                <xsl:map-entry key="'&#x6C3B;'" select="'le4'"/>
                <xsl:map-entry key="'&#x6C3C;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x6C3D;'" select="'tun3'"/>
                <xsl:map-entry key="'&#x6C3E;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x6C3F;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x6C40;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x6C41;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x6C42;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6C43;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x6C44;'" select="'ze4'"/>
                <xsl:map-entry key="'&#x6C45;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x6C46;'" select="'cuan1	tun3'"/>
                <xsl:map-entry key="'&#x6C47;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6C48;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x6C49;'" select="'han4'"/>
                <xsl:map-entry key="'&#x6C4A;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x6C4B;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x6C4C;'" select="'chuan4'"/>
                <xsl:map-entry key="'&#x6C4D;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x6C4E;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x6C4F;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x6C50;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6C51;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x6C52;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x6C53;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6C54;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6C55;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x6C56;'" select="'pai4'"/>
                <xsl:map-entry key="'&#x6C57;'" select="'han4	han2'"/>
                <xsl:map-entry key="'&#x6C58;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6C59;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x6C5A;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x6C5B;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x6C5C;'" select="'si4'"/>
                <xsl:map-entry key="'&#x6C5D;'" select="'ru3'"/>
                <xsl:map-entry key="'&#x6C5E;'" select="'gong3	hong4'"/>
                <xsl:map-entry key="'&#x6C5F;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x6C60;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x6C61;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x6C62;'" select="''"/>
                <xsl:map-entry key="'&#x6C63;'" select="''"/>
                <xsl:map-entry key="'&#x6C64;'" select="'tang1	shang1'"/>
                <xsl:map-entry key="'&#x6C65;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x6C66;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x6C67;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x6C68;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6C69;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x6C6A;'" select="'wang1'"/>
                <xsl:map-entry key="'&#x6C6B;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x6C6C;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x6C6D;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x6C6E;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x6C6F;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x6C70;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x6C71;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x6C72;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6C73;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x6C74;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x6C75;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x6C76;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x6C77;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x6C78;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x6C79;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x6C7A;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6C7B;'" select="'hu3'"/>
                <xsl:map-entry key="'&#x6C7C;'" select="'niou1'"/>
                <xsl:map-entry key="'&#x6C7D;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6C7E;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x6C7F;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6C80;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6C81;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x6C82;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6C83;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x6C84;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x6C85;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x6C86;'" select="'hang4'"/>
                <xsl:map-entry key="'&#x6C87;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x6C88;'" select="'chen2	shen3'"/>
                <xsl:map-entry key="'&#x6C89;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x6C8A;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x6C8B;'" select="'you2'"/>
                <xsl:map-entry key="'&#x6C8C;'" select="'dun4	zhuan4'"/>
                <xsl:map-entry key="'&#x6C8D;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6C8E;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x6C8F;'" select="'qie1	qi1'"/>
                <xsl:map-entry key="'&#x6C90;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x6C91;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x6C92;'" select="'mei2	mo4'"/>
                <xsl:map-entry key="'&#x6C93;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x6C94;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x6C95;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x6C96;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x6C97;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x6C98;'" select="'bi3	pi2'"/>
                <xsl:map-entry key="'&#x6C99;'" select="'sha1	sha4'"/>
                <xsl:map-entry key="'&#x6C9A;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x6C9B;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x6C9C;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x6C9D;'" select="'zhui3'"/>
                <xsl:map-entry key="'&#x6C9E;'" select="'za1'"/>
                <xsl:map-entry key="'&#x6C9F;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x6CA0;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x6CA1;'" select="'mei2	mo4'"/>
                <xsl:map-entry key="'&#x6CA2;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x6CA3;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x6CA4;'" select="'ou4	ou1'"/>
                <xsl:map-entry key="'&#x6CA5;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6CA6;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x6CA7;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x6CA8;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x6CA9;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6CAA;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6CAB;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x6CAC;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x6CAD;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6CAE;'" select="'ju1	ju3	ju4'"/>
                <xsl:map-entry key="'&#x6CAF;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x6CB0;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x6CB1;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x6CB2;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x6CB3;'" select="'he2'"/>
                <xsl:map-entry key="'&#x6CB4;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6CB5;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x6CB6;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6CB7;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x6CB8;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x6CB9;'" select="'you2'"/>
                <xsl:map-entry key="'&#x6CBA;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x6CBB;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6CBC;'" select="'zhao3'"/>
                <xsl:map-entry key="'&#x6CBD;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x6CBE;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x6CBF;'" select="'yan2	yan4'"/>
                <xsl:map-entry key="'&#x6CC0;'" select="'si1'"/>
                <xsl:map-entry key="'&#x6CC1;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x6CC2;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x6CC3;'" select="'ju4	gou1	ju1'"/>
                <xsl:map-entry key="'&#x6CC4;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x6CC5;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6CC6;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6CC7;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x6CC8;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x6CC9;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x6CCA;'" select="'bo2	po4'"/>
                <xsl:map-entry key="'&#x6CCB;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6CCC;'" select="'mi4	bi4'"/>
                <xsl:map-entry key="'&#x6CCD;'" select="'ben1'"/>
                <xsl:map-entry key="'&#x6CCE;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x6CCF;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x6CD0;'" select="'le4'"/>
                <xsl:map-entry key="'&#x6CD1;'" select="'you3'"/>
                <xsl:map-entry key="'&#x6CD2;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x6CD3;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x6CD4;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x6CD5;'" select="'fa3	fa4'"/>
                <xsl:map-entry key="'&#x6CD6;'" select="'mao3'"/>
                <xsl:map-entry key="'&#x6CD7;'" select="'si4'"/>
                <xsl:map-entry key="'&#x6CD8;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x6CD9;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x6CDA;'" select="'ci3'"/>
                <xsl:map-entry key="'&#x6CDB;'" select="'fan4	fan2'"/>
                <xsl:map-entry key="'&#x6CDC;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x6CDD;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6CDE;'" select="'ning4	zhu3'"/>
                <xsl:map-entry key="'&#x6CDF;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x6CE0;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x6CE1;'" select="'pao4	pao1'"/>
                <xsl:map-entry key="'&#x6CE2;'" select="'bo1	po1'"/>
                <xsl:map-entry key="'&#x6CE3;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6CE4;'" select="'si4'"/>
                <xsl:map-entry key="'&#x6CE5;'" select="'ni2	ni4'"/>
                <xsl:map-entry key="'&#x6CE6;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x6CE7;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x6CE8;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x6CE9;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x6CEA;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x6CEB;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x6CEC;'" select="'xue4'"/>
                <xsl:map-entry key="'&#x6CED;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x6CEE;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x6CEF;'" select="'min3'"/>
                <xsl:map-entry key="'&#x6CF0;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x6CF1;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x6CF2;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x6CF3;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x6CF4;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x6CF5;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x6CF6;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x6CF7;'" select="'long2	shuang1'"/>
                <xsl:map-entry key="'&#x6CF8;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x6CF9;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x6CFA;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x6CFB;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x6CFC;'" select="'po1'"/>
                <xsl:map-entry key="'&#x6CFD;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x6CFE;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x6CFF;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x6D00;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x6D01;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6D02;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6D03;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x6D04;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x6D05;'" select="'zui3'"/>
                <xsl:map-entry key="'&#x6D06;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x6D07;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x6D08;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6D09;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x6D0A;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x6D0B;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x6D0C;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x6D0D;'" select="'si4'"/>
                <xsl:map-entry key="'&#x6D0E;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6D0F;'" select="'er2'"/>
                <xsl:map-entry key="'&#x6D10;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x6D11;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x6D12;'" select="'sa3'"/>
                <xsl:map-entry key="'&#x6D13;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x6D14;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x6D15;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x6D16;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x6D17;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x6D18;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x6D19;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x6D1A;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x6D1B;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x6D1C;'" select="''"/>
                <xsl:map-entry key="'&#x6D1D;'" select="'an4'"/>
                <xsl:map-entry key="'&#x6D1E;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x6D1F;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x6D20;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x6D21;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x6D22;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x6D23;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x6D24;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x6D25;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x6D26;'" select="'po4'"/>
                <xsl:map-entry key="'&#x6D27;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x6D28;'" select="'xiao2'"/>
                <xsl:map-entry key="'&#x6D29;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x6D2A;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x6D2B;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6D2C;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6D2D;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x6D2E;'" select="'tao2	yao2'"/>
                <xsl:map-entry key="'&#x6D2F;'" select="'qie4	jie2'"/>
                <xsl:map-entry key="'&#x6D30;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x6D31;'" select="'er3'"/>
                <xsl:map-entry key="'&#x6D32;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x6D33;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x6D34;'" select="'ping2	beng4'"/>
                <xsl:map-entry key="'&#x6D35;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6D36;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x6D37;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6D38;'" select="'guang1	huang3'"/>
                <xsl:map-entry key="'&#x6D39;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x6D3A;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x6D3B;'" select="'huo2'"/>
                <xsl:map-entry key="'&#x6D3C;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x6D3D;'" select="'qia4	xia2'"/>
                <xsl:map-entry key="'&#x6D3E;'" select="'pai4	pa1'"/>
                <xsl:map-entry key="'&#x6D3F;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x6D40;'" select="'qu3'"/>
                <xsl:map-entry key="'&#x6D41;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x6D42;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6D43;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x6D44;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x6D45;'" select="'qian3	jian1'"/>
                <xsl:map-entry key="'&#x6D46;'" select="'jiang1	jiang4'"/>
                <xsl:map-entry key="'&#x6D47;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x6D48;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x6D49;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x6D4A;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x6D4B;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x6D4C;'" select="''"/>
                <xsl:map-entry key="'&#x6D4D;'" select="'kuai4	hui4'"/>
                <xsl:map-entry key="'&#x6D4E;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6D4F;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x6D50;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x6D51;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x6D52;'" select="'hu3	xu3'"/>
                <xsl:map-entry key="'&#x6D53;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x6D54;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6D55;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x6D56;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x6D57;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6D58;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x6D59;'" select="'zhe4'"/>
                <xsl:map-entry key="'&#x6D5A;'" select="'jun4	xun4'"/>
                <xsl:map-entry key="'&#x6D5B;'" select="'han2'"/>
                <xsl:map-entry key="'&#x6D5C;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x6D5D;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x6D5E;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x6D5F;'" select="'you2'"/>
                <xsl:map-entry key="'&#x6D60;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6D61;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x6D62;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x6D63;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x6D64;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x6D65;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6D66;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x6D67;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x6D68;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x6D69;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x6D6A;'" select="'lang4'"/>
                <xsl:map-entry key="'&#x6D6B;'" select="'han3'"/>
                <xsl:map-entry key="'&#x6D6C;'" select="'li3'"/>
                <xsl:map-entry key="'&#x6D6D;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x6D6E;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x6D6F;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x6D70;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6D71;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x6D72;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x6D73;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6D74;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6D75;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x6D76;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x6D77;'" select="'hai3'"/>
                <xsl:map-entry key="'&#x6D78;'" select="'jin4	jin1'"/>
                <xsl:map-entry key="'&#x6D79;'" select="'jia2	jia1'"/>
                <xsl:map-entry key="'&#x6D7A;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x6D7B;'" select="'weng3'"/>
                <xsl:map-entry key="'&#x6D7C;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x6D7D;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x6D7E;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x6D7F;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x6D80;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x6D81;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x6D82;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x6D83;'" select="'kun4'"/>
                <xsl:map-entry key="'&#x6D84;'" select="'pin1'"/>
                <xsl:map-entry key="'&#x6D85;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x6D86;'" select="'han4'"/>
                <xsl:map-entry key="'&#x6D87;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x6D88;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x6D89;'" select="'she4'"/>
                <xsl:map-entry key="'&#x6D8A;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x6D8B;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x6D8C;'" select="'yong3	chong1'"/>
                <xsl:map-entry key="'&#x6D8D;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x6D8E;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x6D8F;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x6D90;'" select="'e2'"/>
                <xsl:map-entry key="'&#x6D91;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6D92;'" select="'tun1'"/>
                <xsl:map-entry key="'&#x6D93;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x6D94;'" select="'cen2'"/>
                <xsl:map-entry key="'&#x6D95;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x6D96;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6D97;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x6D98;'" select="'si4'"/>
                <xsl:map-entry key="'&#x6D99;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x6D9A;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x6D9B;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x6D9C;'" select="'du2'"/>
                <xsl:map-entry key="'&#x6D9D;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x6D9E;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x6D9F;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x6DA0;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6DA1;'" select="'wo1	guo1'"/>
                <xsl:map-entry key="'&#x6DA2;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x6DA3;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x6DA4;'" select="'di2'"/>
                <xsl:map-entry key="'&#x6DA5;'" select="''"/>
                <xsl:map-entry key="'&#x6DA6;'" select="'run4'"/>
                <xsl:map-entry key="'&#x6DA7;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x6DA8;'" select="'zhang3	zhang4'"/>
                <xsl:map-entry key="'&#x6DA9;'" select="'se4'"/>
                <xsl:map-entry key="'&#x6DAA;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x6DAB;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x6DAC;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x6DAD;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x6DAE;'" select="'shuan4'"/>
                <xsl:map-entry key="'&#x6DAF;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x6DB0;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x6DB1;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x6DB2;'" select="'ye4	yi4'"/>
                <xsl:map-entry key="'&#x6DB3;'" select="'kong1'"/>
                <xsl:map-entry key="'&#x6DB4;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x6DB5;'" select="'han2'"/>
                <xsl:map-entry key="'&#x6DB6;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x6DB7;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x6DB8;'" select="'he2	hao4'"/>
                <xsl:map-entry key="'&#x6DB9;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x6DBA;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x6DBB;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x6DBC;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x6DBD;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x6DBE;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x6DBF;'" select="'zhuo1'"/>
                <xsl:map-entry key="'&#x6DC0;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x6DC1;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x6DC2;'" select="'de2'"/>
                <xsl:map-entry key="'&#x6DC3;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x6DC4;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x6DC5;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6DC6;'" select="'yao2	xiao2'"/>
                <xsl:map-entry key="'&#x6DC7;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x6DC8;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x6DC9;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x6DCA;'" select="'han4'"/>
                <xsl:map-entry key="'&#x6DCB;'" select="'lin2	lin4'"/>
                <xsl:map-entry key="'&#x6DCC;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x6DCD;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x6DCE;'" select="'peng3'"/>
                <xsl:map-entry key="'&#x6DCF;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x6DD0;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x6DD1;'" select="'shu2	shu1'"/>
                <xsl:map-entry key="'&#x6DD2;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x6DD3;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x6DD4;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x6DD5;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x6DD6;'" select="'nao4'"/>
                <xsl:map-entry key="'&#x6DD7;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x6DD8;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x6DD9;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x6DDA;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x6DDB;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6DDC;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x6DDD;'" select="'fei2'"/>
                <xsl:map-entry key="'&#x6DDE;'" select="'song1'"/>
                <xsl:map-entry key="'&#x6DDF;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x6DE0;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x6DE1;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x6DE2;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6DE3;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x6DE4;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x6DE5;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x6DE6;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x6DE7;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6DE8;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x6DE9;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x6DEA;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x6DEB;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x6DEC;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x6DED;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x6DEE;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x6DEF;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6DF0;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x6DF1;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x6DF2;'" select="'piao2	hu1'"/>
                <xsl:map-entry key="'&#x6DF3;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x6DF4;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x6DF5;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x6DF6;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x6DF7;'" select="'hun3	hun2	hun4'"/>
                <xsl:map-entry key="'&#x6DF8;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x6DF9;'" select="'yan1	yan4'"/>
                <xsl:map-entry key="'&#x6DFA;'" select="'qian3	jian1'"/>
                <xsl:map-entry key="'&#x6DFB;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x6DFC;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x6DFD;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x6DFE;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x6DFF;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6E00;'" select="'ben1'"/>
                <xsl:map-entry key="'&#x6E01;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x6E02;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x6E03;'" select="'re4'"/>
                <xsl:map-entry key="'&#x6E04;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x6E05;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x6E06;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x6E07;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x6E08;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6E09;'" select="'she4'"/>
                <xsl:map-entry key="'&#x6E0A;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x6E0B;'" select="'se4'"/>
                <xsl:map-entry key="'&#x6E0C;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x6E0D;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x6E0E;'" select="'du2'"/>
                <xsl:map-entry key="'&#x6E0F;'" select="''"/>
                <xsl:map-entry key="'&#x6E10;'" select="'jian4	jian1'"/>
                <xsl:map-entry key="'&#x6E11;'" select="'mian3	sheng2'"/>
                <xsl:map-entry key="'&#x6E12;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x6E13;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6E14;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6E15;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x6E16;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x6E17;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x6E18;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x6E19;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x6E1A;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x6E1B;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x6E1C;'" select="'nuan3'"/>
                <xsl:map-entry key="'&#x6E1D;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6E1E;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x6E1F;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x6E20;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x6E21;'" select="'du4'"/>
                <xsl:map-entry key="'&#x6E22;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x6E23;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x6E24;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x6E25;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x6E26;'" select="'wo1	guo1'"/>
                <xsl:map-entry key="'&#x6E27;'" select="'di4'"/>
                <xsl:map-entry key="'&#x6E28;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x6E29;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x6E2A;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x6E2B;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x6E2C;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x6E2D;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x6E2E;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x6E2F;'" select="'gang3'"/>
                <xsl:map-entry key="'&#x6E30;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x6E31;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x6E32;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x6E33;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x6E34;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x6E35;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x6E36;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x6E37;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x6E38;'" select="'you2'"/>
                <xsl:map-entry key="'&#x6E39;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x6E3A;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x6E3B;'" select="'xing3'"/>
                <xsl:map-entry key="'&#x6E3C;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x6E3D;'" select="'zai1'"/>
                <xsl:map-entry key="'&#x6E3E;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x6E3F;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x6E40;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x6E41;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x6E42;'" select="'e4'"/>
                <xsl:map-entry key="'&#x6E43;'" select="'pai4'"/>
                <xsl:map-entry key="'&#x6E44;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x6E45;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x6E46;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6E47;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x6E48;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x6E49;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x6E4A;'" select="'cou4'"/>
                <xsl:map-entry key="'&#x6E4B;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6E4C;'" select="'can1'"/>
                <xsl:map-entry key="'&#x6E4D;'" select="'tuan1'"/>
                <xsl:map-entry key="'&#x6E4E;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x6E4F;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x6E50;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x6E51;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x6E52;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6E53;'" select="'pen2'"/>
                <xsl:map-entry key="'&#x6E54;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x6E55;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x6E56;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x6E57;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x6E58;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x6E59;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6E5A;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x6E5B;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x6E5C;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x6E5D;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x6E5E;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x6E5F;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x6E60;'" select="'tan4'"/>
                <xsl:map-entry key="'&#x6E61;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6E62;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6E63;'" select="'min3'"/>
                <xsl:map-entry key="'&#x6E64;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x6E65;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x6E66;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x6E67;'" select="'yong3	chong1'"/>
                <xsl:map-entry key="'&#x6E68;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x6E69;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x6E6A;'" select="'suei4'"/>
                <xsl:map-entry key="'&#x6E6B;'" select="'jia3	jiao3	qiu1'"/>
                <xsl:map-entry key="'&#x6E6C;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x6E6D;'" select="'qiou2'"/>
                <xsl:map-entry key="'&#x6E6E;'" select="'yin1	yan1'"/>
                <xsl:map-entry key="'&#x6E6F;'" select="'tang1	shang1'"/>
                <xsl:map-entry key="'&#x6E70;'" select="'long2'"/>
                <xsl:map-entry key="'&#x6E71;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x6E72;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x6E73;'" select="'nan3'"/>
                <xsl:map-entry key="'&#x6E74;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x6E75;'" select="'you3'"/>
                <xsl:map-entry key="'&#x6E76;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x6E77;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x6E78;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x6E79;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x6E7A;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x6E7B;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x6E7C;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x6E7D;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x6E7E;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x6E7F;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x6E80;'" select="'man3'"/>
                <xsl:map-entry key="'&#x6E81;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x6E82;'" select="'la4'"/>
                <xsl:map-entry key="'&#x6E83;'" select="'kui4	hui4'"/>
                <xsl:map-entry key="'&#x6E84;'" select="''"/>
                <xsl:map-entry key="'&#x6E85;'" select="'jian4	jian1'"/>
                <xsl:map-entry key="'&#x6E86;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6E87;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x6E88;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x6E89;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x6E8A;'" select="''"/>
                <xsl:map-entry key="'&#x6E8B;'" select="''"/>
                <xsl:map-entry key="'&#x6E8C;'" select="'po1'"/>
                <xsl:map-entry key="'&#x6E8D;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x6E8E;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x6E8F;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x6E90;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x6E91;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x6E92;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x6E93;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x6E94;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x6E95;'" select="'meng4'"/>
                <xsl:map-entry key="'&#x6E96;'" select="'zhun3'"/>
                <xsl:map-entry key="'&#x6E97;'" select="'sheng2'"/>
                <xsl:map-entry key="'&#x6E98;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x6E99;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x6E9A;'" select="'ta3'"/>
                <xsl:map-entry key="'&#x6E9B;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x6E9C;'" select="'liu1	liu4'"/>
                <xsl:map-entry key="'&#x6E9D;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x6E9E;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x6E9F;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x6EA0;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x6EA1;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x6EA2;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6EA3;'" select="'lun4'"/>
                <xsl:map-entry key="'&#x6EA4;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x6EA5;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x6EA6;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x6EA7;'" select="'li4'"/>
                <xsl:map-entry key="'&#x6EA8;'" select="'cai2'"/>
                <xsl:map-entry key="'&#x6EA9;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x6EAA;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6EAB;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x6EAC;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x6EAD;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x6EAE;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x6EAF;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6EB0;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x6EB1;'" select="'zhen1	qin2'"/>
                <xsl:map-entry key="'&#x6EB2;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x6EB3;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x6EB4;'" select="'xiu4	chou4'"/>
                <xsl:map-entry key="'&#x6EB5;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x6EB6;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x6EB7;'" select="'hun4'"/>
                <xsl:map-entry key="'&#x6EB8;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6EB9;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6EBA;'" select="'ni4	niao4'"/>
                <xsl:map-entry key="'&#x6EBB;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x6EBC;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x6EBD;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x6EBE;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x6EBF;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x6EC0;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x6EC1;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x6EC2;'" select="'pang1'"/>
                <xsl:map-entry key="'&#x6EC3;'" select="'weng3'"/>
                <xsl:map-entry key="'&#x6EC4;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x6EC5;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x6EC6;'" select="'he2'"/>
                <xsl:map-entry key="'&#x6EC7;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x6EC8;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x6EC9;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x6ECA;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x6ECB;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x6ECC;'" select="'di2'"/>
                <xsl:map-entry key="'&#x6ECD;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x6ECE;'" select="'ying2	xing2'"/>
                <xsl:map-entry key="'&#x6ECF;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x6ED0;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6ED1;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x6ED2;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x6ED3;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x6ED4;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x6ED5;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x6ED6;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x6ED7;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6ED8;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x6ED9;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6EDA;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x6EDB;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x6EDC;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x6EDD;'" select="'long2	shuang1'"/>
                <xsl:map-entry key="'&#x6EDE;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6EDF;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x6EE0;'" select="'she4'"/>
                <xsl:map-entry key="'&#x6EE1;'" select="'man3'"/>
                <xsl:map-entry key="'&#x6EE2;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x6EE3;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x6EE4;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x6EE5;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x6EE6;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x6EE7;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x6EE8;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x6EE9;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x6EEA;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6EEB;'" select="'xiu1	xiu3'"/>
                <xsl:map-entry key="'&#x6EEC;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x6EED;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6EEE;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x6EEF;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6EF0;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x6EF1;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x6EF2;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x6EF3;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x6EF4;'" select="'di1'"/>
                <xsl:map-entry key="'&#x6EF5;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6EF6;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x6EF7;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x6EF8;'" select="'hu3	xu3'"/>
                <xsl:map-entry key="'&#x6EF9;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x6EFA;'" select="'you2'"/>
                <xsl:map-entry key="'&#x6EFB;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x6EFC;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x6EFD;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x6EFE;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x6EFF;'" select="'man3'"/>
                <xsl:map-entry key="'&#x6F00;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x6F01;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6F02;'" select="'piao1	piao3	piao4'"/>
                <xsl:map-entry key="'&#x6F03;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6F04;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x6F05;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x6F06;'" select="'qi1	qu4'"/>
                <xsl:map-entry key="'&#x6F07;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x6F08;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6F09;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x6F0A;'" select="'lu:3	lou2'"/>
                <xsl:map-entry key="'&#x6F0B;'" select="'long2'"/>
                <xsl:map-entry key="'&#x6F0C;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x6F0D;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x6F0E;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x6F0F;'" select="'lou4'"/>
                <xsl:map-entry key="'&#x6F10;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x6F11;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x6F12;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x6F13;'" select="'li2'"/>
                <xsl:map-entry key="'&#x6F14;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x6F15;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x6F16;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x6F17;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x6F18;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x6F19;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x6F1A;'" select="'ou4	ou1'"/>
                <xsl:map-entry key="'&#x6F1B;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x6F1C;'" select="'ye3'"/>
                <xsl:map-entry key="'&#x6F1D;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x6F1E;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6F1F;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x6F20;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x6F21;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x6F22;'" select="'han4'"/>
                <xsl:map-entry key="'&#x6F23;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x6F24;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x6F25;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x6F26;'" select="'li2'"/>
                <xsl:map-entry key="'&#x6F27;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x6F28;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x6F29;'" select="'xuan2	xuan4'"/>
                <xsl:map-entry key="'&#x6F2A;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x6F2B;'" select="'man4	man2'"/>
                <xsl:map-entry key="'&#x6F2C;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x6F2D;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x6F2E;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x6F2F;'" select="'lei3	luo4	ta4'"/>
                <xsl:map-entry key="'&#x6F30;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x6F31;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6F32;'" select="'zhang3	zhang4'"/>
                <xsl:map-entry key="'&#x6F33;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x6F34;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x6F35;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6F36;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x6F37;'" select="'kuo4	huo3'"/>
                <xsl:map-entry key="'&#x6F38;'" select="'jian4	jian1'"/>
                <xsl:map-entry key="'&#x6F39;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x6F3A;'" select="'chuang3	shuang3'"/>
                <xsl:map-entry key="'&#x6F3B;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x6F3C;'" select="'cui3'"/>
                <xsl:map-entry key="'&#x6F3D;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x6F3E;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x6F3F;'" select="'jiang1	jiang4'"/>
                <xsl:map-entry key="'&#x6F40;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x6F41;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x6F42;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x6F43;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x6F44;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6F45;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x6F46;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x6F47;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x6F48;'" select="''"/>
                <xsl:map-entry key="'&#x6F49;'" select="''"/>
                <xsl:map-entry key="'&#x6F4A;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x6F4B;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x6F4C;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x6F4D;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6F4E;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x6F4F;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x6F50;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x6F51;'" select="'po1'"/>
                <xsl:map-entry key="'&#x6F52;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x6F53;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6F54;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x6F55;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x6F56;'" select="'pa2'"/>
                <xsl:map-entry key="'&#x6F57;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6F58;'" select="'pan1'"/>
                <xsl:map-entry key="'&#x6F59;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6F5A;'" select="'xiao1	su4'"/>
                <xsl:map-entry key="'&#x6F5B;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x6F5C;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x6F5D;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x6F5E;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x6F5F;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x6F60;'" select="'sun4'"/>
                <xsl:map-entry key="'&#x6F61;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x6F62;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x6F63;'" select="'min3'"/>
                <xsl:map-entry key="'&#x6F64;'" select="'run4'"/>
                <xsl:map-entry key="'&#x6F65;'" select="'su4'"/>
                <xsl:map-entry key="'&#x6F66;'" select="'liao2	lao3	liao3'"/>
                <xsl:map-entry key="'&#x6F67;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x6F68;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x6F69;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6F6A;'" select="'di2'"/>
                <xsl:map-entry key="'&#x6F6B;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x6F6C;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x6F6D;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x6F6E;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x6F6F;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x6F70;'" select="'kui4	hui4'"/>
                <xsl:map-entry key="'&#x6F71;'" select="'yie'"/>
                <xsl:map-entry key="'&#x6F72;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x6F73;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x6F74;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x6F75;'" select="'sa3'"/>
                <xsl:map-entry key="'&#x6F76;'" select="'hei1'"/>
                <xsl:map-entry key="'&#x6F77;'" select="'bi3	bi4'"/>
                <xsl:map-entry key="'&#x6F78;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x6F79;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x6F7A;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x6F7B;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x6F7C;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x6F7D;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x6F7E;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x6F7F;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6F80;'" select="'se4'"/>
                <xsl:map-entry key="'&#x6F81;'" select="'se4'"/>
                <xsl:map-entry key="'&#x6F82;'" select="'cheng2	deng4'"/>
                <xsl:map-entry key="'&#x6F83;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x6F84;'" select="'cheng2	deng4'"/>
                <xsl:map-entry key="'&#x6F85;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x6F86;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x6F87;'" select="'lao4	lao2'"/>
                <xsl:map-entry key="'&#x6F88;'" select="'che4'"/>
                <xsl:map-entry key="'&#x6F89;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x6F8A;'" select="'cun1'"/>
                <xsl:map-entry key="'&#x6F8B;'" select="'heng4'"/>
                <xsl:map-entry key="'&#x6F8C;'" select="'si1'"/>
                <xsl:map-entry key="'&#x6F8D;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6F8E;'" select="'peng2	peng1'"/>
                <xsl:map-entry key="'&#x6F8F;'" select="'han4'"/>
                <xsl:map-entry key="'&#x6F90;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x6F91;'" select="'liu4	liu1'"/>
                <xsl:map-entry key="'&#x6F92;'" select="'hong4'"/>
                <xsl:map-entry key="'&#x6F93;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x6F94;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x6F95;'" select="'he2'"/>
                <xsl:map-entry key="'&#x6F96;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x6F97;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x6F98;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x6F99;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x6F9A;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x6F9B;'" select="''"/>
                <xsl:map-entry key="'&#x6F9C;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x6F9D;'" select="''"/>
                <xsl:map-entry key="'&#x6F9E;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x6F9F;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x6FA0;'" select="'min3	mian3	sheng2'"/>
                <xsl:map-entry key="'&#x6FA1;'" select="'zao3'"/>
                <xsl:map-entry key="'&#x6FA2;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x6FA3;'" select="'huan3'"/>
                <xsl:map-entry key="'&#x6FA4;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x6FA5;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x6FA6;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x6FA7;'" select="'li3'"/>
                <xsl:map-entry key="'&#x6FA8;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x6FA9;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x6FAA;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x6FAB;'" select="'man4'"/>
                <xsl:map-entry key="'&#x6FAC;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x6FAD;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x6FAE;'" select="'kuai4	hui4'"/>
                <xsl:map-entry key="'&#x6FAF;'" select="'can4'"/>
                <xsl:map-entry key="'&#x6FB0;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x6FB1;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x6FB2;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x6FB3;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x6FB4;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x6FB5;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x6FB6;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x6FB7;'" select="'man4'"/>
                <xsl:map-entry key="'&#x6FB8;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x6FB9;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x6FBA;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x6FBB;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x6FBC;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x6FBD;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x6FBE;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x6FBF;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x6FC0;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x6FC1;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x6FC2;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x6FC3;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x6FC4;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x6FC5;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x6FC6;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x6FC7;'" select="'se4'"/>
                <xsl:map-entry key="'&#x6FC8;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x6FC9;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x6FCA;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x6FCB;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x6FCC;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x6FCD;'" select="'song1'"/>
                <xsl:map-entry key="'&#x6FCE;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x6FCF;'" select="'se4'"/>
                <xsl:map-entry key="'&#x6FD0;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x6FD1;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x6FD2;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x6FD3;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x6FD4;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x6FD5;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x6FD6;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x6FD7;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x6FD8;'" select="'ning4	neng4'"/>
                <xsl:map-entry key="'&#x6FD9;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x6FDA;'" select="'ying2	xing2'"/>
                <xsl:map-entry key="'&#x6FDB;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x6FDC;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x6FDD;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x6FDE;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x6FDF;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x6FE0;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x6FE1;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x6FE2;'" select="'zui3	cui4'"/>
                <xsl:map-entry key="'&#x6FE3;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x6FE4;'" select="'tao1	tao2'"/>
                <xsl:map-entry key="'&#x6FE5;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x6FE6;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x6FE7;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x6FE8;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x6FE9;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x6FEA;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x6FEB;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x6FEC;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x6FED;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x6FEE;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x6FEF;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x6FF0;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x6FF1;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x6FF2;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x6FF3;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x6FF4;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x6FF5;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x6FF6;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x6FF7;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x6FF8;'" select="''"/>
                <xsl:map-entry key="'&#x6FF9;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x6FFA;'" select="'jian4	jian1'"/>
                <xsl:map-entry key="'&#x6FFB;'" select="'dui4	wei2'"/>
                <xsl:map-entry key="'&#x6FFC;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x6FFD;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x6FFE;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x6FFF;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7000;'" select="'you1'"/>
                <xsl:map-entry key="'&#x7001;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x7002;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x7003;'" select="'si4'"/>
                <xsl:map-entry key="'&#x7004;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x7005;'" select="'ying4	ying2'"/>
                <xsl:map-entry key="'&#x7006;'" select="'du2'"/>
                <xsl:map-entry key="'&#x7007;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x7008;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x7009;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x700A;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x700B;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x700C;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x700D;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x700E;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x700F;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x7010;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7011;'" select="'pu4	bao4'"/>
                <xsl:map-entry key="'&#x7012;'" select="'se4'"/>
                <xsl:map-entry key="'&#x7013;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x7014;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x7015;'" select="'bin1	pin2'"/>
                <xsl:map-entry key="'&#x7016;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x7017;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x7018;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x7019;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x701A;'" select="'han4'"/>
                <xsl:map-entry key="'&#x701B;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x701C;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x701D;'" select="'li4'"/>
                <xsl:map-entry key="'&#x701E;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x701F;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7020;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x7021;'" select="'sui3'"/>
                <xsl:map-entry key="'&#x7022;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x7023;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7024;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x7025;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x7026;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x7027;'" select="'long2	shuang1'"/>
                <xsl:map-entry key="'&#x7028;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x7029;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x702A;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x702B;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x702C;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x702D;'" select="''"/>
                <xsl:map-entry key="'&#x702E;'" select="''"/>
                <xsl:map-entry key="'&#x702F;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x7030;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x7031;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7032;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x7033;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x7034;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x7035;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x7036;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7037;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7038;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7039;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x703A;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x703B;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x703C;'" select="'rang2	rang4'"/>
                <xsl:map-entry key="'&#x703D;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x703E;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x703F;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x7040;'" select="'shuang4'"/>
                <xsl:map-entry key="'&#x7041;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x7042;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x7043;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x7044;'" select="'she4'"/>
                <xsl:map-entry key="'&#x7045;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x7046;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x7047;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x7048;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x7049;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x704A;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x704B;'" select="'fa3'"/>
                <xsl:map-entry key="'&#x704C;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x704D;'" select="'que4'"/>
                <xsl:map-entry key="'&#x704E;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x704F;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x7050;'" select="''"/>
                <xsl:map-entry key="'&#x7051;'" select="'sa3'"/>
                <xsl:map-entry key="'&#x7052;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x7053;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x7054;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7055;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7056;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x7057;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x7058;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x7059;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x705A;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x705B;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x705C;'" select="''"/>
                <xsl:map-entry key="'&#x705D;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x705E;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x705F;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x7060;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x7061;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x7062;'" select="'nang3'"/>
                <xsl:map-entry key="'&#x7063;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x7064;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x7065;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x7066;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x7067;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7068;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x7069;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x706A;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x706B;'" select="'huo3'"/>
                <xsl:map-entry key="'&#x706C;'" select="'huo3'"/>
                <xsl:map-entry key="'&#x706D;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x706E;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x706F;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x7070;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x7071;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7072;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7073;'" select="'hu11'"/>
                <xsl:map-entry key="'&#x7074;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7075;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7076;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x7077;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x7078;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x7079;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x707A;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x707B;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x707C;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x707D;'" select="'zai1'"/>
                <xsl:map-entry key="'&#x707E;'" select="'zai1'"/>
                <xsl:map-entry key="'&#x707F;'" select="'can4'"/>
                <xsl:map-entry key="'&#x7080;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x7081;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x7082;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x7083;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x7084;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x7085;'" select="'jiong3	gui4'"/>
                <xsl:map-entry key="'&#x7086;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x7087;'" select="'po4'"/>
                <xsl:map-entry key="'&#x7088;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7089;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x708A;'" select="'chui1	chui4'"/>
                <xsl:map-entry key="'&#x708B;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x708C;'" select="'kai4'"/>
                <xsl:map-entry key="'&#x708D;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x708E;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x708F;'" select="'kai4'"/>
                <xsl:map-entry key="'&#x7090;'" select="'pang4'"/>
                <xsl:map-entry key="'&#x7091;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x7092;'" select="'chao3'"/>
                <xsl:map-entry key="'&#x7093;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x7094;'" select="'gui4	que1'"/>
                <xsl:map-entry key="'&#x7095;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x7096;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x7097;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x7098;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x7099;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x709A;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x709B;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x709C;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x709D;'" select="'qiang4'"/>
                <xsl:map-entry key="'&#x709E;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x709F;'" select="'da2'"/>
                <xsl:map-entry key="'&#x70A0;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x70A1;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x70A2;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x70A3;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x70A4;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x70A5;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x70A6;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x70A7;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x70A8;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x70A9;'" select="'ling4'"/>
                <xsl:map-entry key="'&#x70AA;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x70AB;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x70AC;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x70AD;'" select="'tan4'"/>
                <xsl:map-entry key="'&#x70AE;'" select="'pao4	bao1	pao2'"/>
                <xsl:map-entry key="'&#x70AF;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x70B0;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x70B1;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x70B2;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x70B3;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x70B4;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x70B5;'" select="'tong1	dong1'"/>
                <xsl:map-entry key="'&#x70B6;'" select="'han1'"/>
                <xsl:map-entry key="'&#x70B7;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x70B8;'" select="'zha4	zha2'"/>
                <xsl:map-entry key="'&#x70B9;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x70BA;'" select="'wei4	wei2'"/>
                <xsl:map-entry key="'&#x70BB;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x70BC;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x70BD;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x70BE;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x70BF;'" select="''"/>
                <xsl:map-entry key="'&#x70C0;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x70C1;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x70C2;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x70C3;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x70C4;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x70C5;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x70C6;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x70C7;'" select="'quan4'"/>
                <xsl:map-entry key="'&#x70C8;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x70C9;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x70CA;'" select="'yang2	yang4'"/>
                <xsl:map-entry key="'&#x70CB;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x70CC;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x70CD;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x70CE;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x70CF;'" select="'wu1	wu4'"/>
                <xsl:map-entry key="'&#x70D0;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x70D1;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x70D2;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x70D3;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x70D4;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x70D5;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x70D6;'" select="'zai1'"/>
                <xsl:map-entry key="'&#x70D7;'" select="'kai4'"/>
                <xsl:map-entry key="'&#x70D8;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x70D9;'" select="'luo4	lao4'"/>
                <xsl:map-entry key="'&#x70DA;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x70DB;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x70DC;'" select="'xuan3'"/>
                <xsl:map-entry key="'&#x70DD;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x70DE;'" select="'po4'"/>
                <xsl:map-entry key="'&#x70DF;'" select="'yan1	yin1'"/>
                <xsl:map-entry key="'&#x70E0;'" select="'hui3'"/>
                <xsl:map-entry key="'&#x70E1;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x70E2;'" select="'zhe4'"/>
                <xsl:map-entry key="'&#x70E3;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x70E4;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x70E5;'" select="''"/>
                <xsl:map-entry key="'&#x70E6;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x70E7;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x70E8;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x70E9;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x70EA;'" select="''"/>
                <xsl:map-entry key="'&#x70EB;'" select="'tang4'"/>
                <xsl:map-entry key="'&#x70EC;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x70ED;'" select="'re4'"/>
                <xsl:map-entry key="'&#x70EE;'" select="''"/>
                <xsl:map-entry key="'&#x70EF;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x70F0;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x70F1;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x70F2;'" select="'che4'"/>
                <xsl:map-entry key="'&#x70F3;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x70F4;'" select="'jing3	ting1'"/>
                <xsl:map-entry key="'&#x70F5;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x70F6;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x70F7;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x70F8;'" select="'hai3'"/>
                <xsl:map-entry key="'&#x70F9;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x70FA;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x70FB;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x70FC;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x70FD;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x70FE;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x70FF;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7100;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x7101;'" select="'xi'"/>
                <xsl:map-entry key="'&#x7102;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x7103;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x7104;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x7105;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x7106;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7107;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7108;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7109;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x710A;'" select="'han4'"/>
                <xsl:map-entry key="'&#x710B;'" select="'zhuang4'"/>
                <xsl:map-entry key="'&#x710C;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x710D;'" select="'di4	ti1'"/>
                <xsl:map-entry key="'&#x710E;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x710F;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x7110;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x7111;'" select="''"/>
                <xsl:map-entry key="'&#x7112;'" select="''"/>
                <xsl:map-entry key="'&#x7113;'" select="'han2'"/>
                <xsl:map-entry key="'&#x7114;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7115;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x7116;'" select="'men4'"/>
                <xsl:map-entry key="'&#x7117;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x7118;'" select="'dao4	tao1'"/>
                <xsl:map-entry key="'&#x7119;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x711A;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x711B;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x711C;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x711D;'" select="'hun4'"/>
                <xsl:map-entry key="'&#x711E;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x711F;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x7120;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x7121;'" select="'wu2	mo2'"/>
                <xsl:map-entry key="'&#x7122;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x7123;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x7124;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x7125;'" select="'yue1'"/>
                <xsl:map-entry key="'&#x7126;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x7127;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x7128;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x7129;'" select="'ping1'"/>
                <xsl:map-entry key="'&#x712A;'" select="'qiong1'"/>
                <xsl:map-entry key="'&#x712B;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x712C;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x712D;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x712E;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x712F;'" select="'zhuo1'"/>
                <xsl:map-entry key="'&#x7130;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7131;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7132;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7133;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7134;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7135;'" select="'gang4'"/>
                <xsl:map-entry key="'&#x7136;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x7137;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x7138;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7139;'" select="''"/>
                <xsl:map-entry key="'&#x713A;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x713B;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x713C;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x713D;'" select="''"/>
                <xsl:map-entry key="'&#x713E;'" select="''"/>
                <xsl:map-entry key="'&#x713F;'" select="''"/>
                <xsl:map-entry key="'&#x7140;'" select="''"/>
                <xsl:map-entry key="'&#x7141;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x7142;'" select="'he4'"/>
                <xsl:map-entry key="'&#x7143;'" select="'kui3'"/>
                <xsl:map-entry key="'&#x7144;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x7145;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x7146;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x7147;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x7148;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x7149;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x714A;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x714B;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x714C;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x714D;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x714E;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x714F;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7150;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7151;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x7152;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x7153;'" select="'tuan1'"/>
                <xsl:map-entry key="'&#x7154;'" select="'tian4'"/>
                <xsl:map-entry key="'&#x7155;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7156;'" select="'nuan3	xuan1'"/>
                <xsl:map-entry key="'&#x7157;'" select="'nuan3'"/>
                <xsl:map-entry key="'&#x7158;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x7159;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x715A;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x715B;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x715C;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x715D;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x715E;'" select="'sha4	sha1'"/>
                <xsl:map-entry key="'&#x715F;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x7160;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x7161;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x7162;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x7163;'" select="'rou3'"/>
                <xsl:map-entry key="'&#x7164;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x7165;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x7166;'" select="'xu3	xu4'"/>
                <xsl:map-entry key="'&#x7167;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x7168;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x7169;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x716A;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x716B;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x716C;'" select="'yang2	yang4'"/>
                <xsl:map-entry key="'&#x716D;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x716E;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x716F;'" select="'jie'"/>
                <xsl:map-entry key="'&#x7170;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x7171;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x7172;'" select="'bao4	bao1'"/>
                <xsl:map-entry key="'&#x7173;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x7174;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x7175;'" select="'xia1'"/>
                <xsl:map-entry key="'&#x7176;'" select="''"/>
                <xsl:map-entry key="'&#x7177;'" select="''"/>
                <xsl:map-entry key="'&#x7178;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7179;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x717A;'" select="'tui4'"/>
                <xsl:map-entry key="'&#x717B;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x717C;'" select="'chao3'"/>
                <xsl:map-entry key="'&#x717D;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x717E;'" select="'n'"/>
                <xsl:map-entry key="'&#x717F;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x7180;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x7181;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x7182;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x7183;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x7184;'" select="'xi2	xi1'"/>
                <xsl:map-entry key="'&#x7185;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x7186;'" select="'he2'"/>
                <xsl:map-entry key="'&#x7187;'" select="'he4'"/>
                <xsl:map-entry key="'&#x7188;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7189;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x718A;'" select="'xiong2'"/>
                <xsl:map-entry key="'&#x718B;'" select="'nai2'"/>
                <xsl:map-entry key="'&#x718C;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x718D;'" select="'qiong'"/>
                <xsl:map-entry key="'&#x718E;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x718F;'" select="'xun1	xun4'"/>
                <xsl:map-entry key="'&#x7190;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x7191;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x7192;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x7193;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x7194;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7195;'" select="''"/>
                <xsl:map-entry key="'&#x7196;'" select="''"/>
                <xsl:map-entry key="'&#x7197;'" select="'qiang4'"/>
                <xsl:map-entry key="'&#x7198;'" select="'liu1'"/>
                <xsl:map-entry key="'&#x7199;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x719A;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x719B;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x719C;'" select="'zong3	cong1'"/>
                <xsl:map-entry key="'&#x719D;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x719E;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x719F;'" select="'shou2	shu2'"/>
                <xsl:map-entry key="'&#x71A0;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x71A1;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x71A2;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x71A3;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x71A4;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x71A5;'" select="'tong1'"/>
                <xsl:map-entry key="'&#x71A6;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x71A7;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x71A8;'" select="'yun4	yu4'"/>
                <xsl:map-entry key="'&#x71A9;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x71AA;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x71AB;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x71AC;'" select="'ao2	ao1'"/>
                <xsl:map-entry key="'&#x71AD;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x71AE;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x71AF;'" select="'han4'"/>
                <xsl:map-entry key="'&#x71B0;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x71B1;'" select="'re4'"/>
                <xsl:map-entry key="'&#x71B2;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x71B3;'" select="'man4'"/>
                <xsl:map-entry key="'&#x71B4;'" select="''"/>
                <xsl:map-entry key="'&#x71B5;'" select="'shang1	di1'"/>
                <xsl:map-entry key="'&#x71B6;'" select="'cuan4'"/>
                <xsl:map-entry key="'&#x71B7;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x71B8;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x71B9;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x71BA;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x71BB;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x71BC;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x71BD;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x71BE;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x71BF;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x71C0;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x71C1;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x71C2;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x71C3;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x71C4;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x71C5;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x71C6;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x71C7;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x71C8;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x71C9;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x71CA;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x71CB;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x71CC;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x71CD;'" select="'si1'"/>
                <xsl:map-entry key="'&#x71CE;'" select="'liao4	liao2	liao3'"/>
                <xsl:map-entry key="'&#x71CF;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x71D0;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x71D1;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x71D2;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x71D3;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x71D4;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x71D5;'" select="'yan4	yan1'"/>
                <xsl:map-entry key="'&#x71D6;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x71D7;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x71D8;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x71D9;'" select="'tang4'"/>
                <xsl:map-entry key="'&#x71DA;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x71DB;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x71DC;'" select="'men4'"/>
                <xsl:map-entry key="'&#x71DD;'" select="''"/>
                <xsl:map-entry key="'&#x71DE;'" select="''"/>
                <xsl:map-entry key="'&#x71DF;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x71E0;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x71E1;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x71E2;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x71E3;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x71E4;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x71E5;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x71E6;'" select="'can4'"/>
                <xsl:map-entry key="'&#x71E7;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x71E8;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x71E9;'" select="'que4'"/>
                <xsl:map-entry key="'&#x71EA;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x71EB;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x71EC;'" select="'hui3'"/>
                <xsl:map-entry key="'&#x71ED;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x71EE;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x71EF;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x71F0;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x71F1;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x71F2;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x71F3;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x71F4;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x71F5;'" select="''"/>
                <xsl:map-entry key="'&#x71F6;'" select="''"/>
                <xsl:map-entry key="'&#x71F7;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x71F8;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x71F9;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x71FA;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x71FB;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x71FC;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x71FD;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x71FE;'" select="'dao4	tao1	tao2'"/>
                <xsl:map-entry key="'&#x71FF;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x7200;'" select="'he4'"/>
                <xsl:map-entry key="'&#x7201;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x7202;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x7203;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7204;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7205;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x7206;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x7207;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x7208;'" select="'di4'"/>
                <xsl:map-entry key="'&#x7209;'" select="'ly4'"/>
                <xsl:map-entry key="'&#x720A;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x720B;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x720C;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x720D;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x720E;'" select="''"/>
                <xsl:map-entry key="'&#x720F;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7210;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x7211;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7212;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x7213;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7214;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7215;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7216;'" select="'long2'"/>
                <xsl:map-entry key="'&#x7217;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7218;'" select="''"/>
                <xsl:map-entry key="'&#x7219;'" select="'rang3	shang4'"/>
                <xsl:map-entry key="'&#x721A;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x721B;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x721C;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x721D;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x721E;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x721F;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x7220;'" select="''"/>
                <xsl:map-entry key="'&#x7221;'" select="'che4'"/>
                <xsl:map-entry key="'&#x7222;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x7223;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x7224;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x7225;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x7226;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x7227;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7228;'" select="'cuan4'"/>
                <xsl:map-entry key="'&#x7229;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x722A;'" select="'zhua3	zhao3'"/>
                <xsl:map-entry key="'&#x722B;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x722C;'" select="'pa2'"/>
                <xsl:map-entry key="'&#x722D;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x722E;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x722F;'" select="'zhao3'"/>
                <xsl:map-entry key="'&#x7230;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7231;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x7232;'" select="'wei4	wei2'"/>
                <xsl:map-entry key="'&#x7233;'" select="''"/>
                <xsl:map-entry key="'&#x7234;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7235;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7236;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x7237;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x7238;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x7239;'" select="'die1'"/>
                <xsl:map-entry key="'&#x723A;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x723B;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x723C;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x723D;'" select="'shuang3'"/>
                <xsl:map-entry key="'&#x723E;'" select="'er3'"/>
                <xsl:map-entry key="'&#x723F;'" select="'qiang2	pan2'"/>
                <xsl:map-entry key="'&#x7240;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x7241;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x7242;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x7243;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x7244;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x7245;'" select="'die2'"/>
                <xsl:map-entry key="'&#x7246;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x7247;'" select="'pian4	pian1'"/>
                <xsl:map-entry key="'&#x7248;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x7249;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x724A;'" select="'shao2'"/>
                <xsl:map-entry key="'&#x724B;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x724C;'" select="'pai2'"/>
                <xsl:map-entry key="'&#x724D;'" select="'du2'"/>
                <xsl:map-entry key="'&#x724E;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x724F;'" select="'tou2'"/>
                <xsl:map-entry key="'&#x7250;'" select="'tou2'"/>
                <xsl:map-entry key="'&#x7251;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7252;'" select="'die2'"/>
                <xsl:map-entry key="'&#x7253;'" select="'bang3'"/>
                <xsl:map-entry key="'&#x7254;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x7255;'" select="'bang3'"/>
                <xsl:map-entry key="'&#x7256;'" select="'you3'"/>
                <xsl:map-entry key="'&#x7257;'" select="''"/>
                <xsl:map-entry key="'&#x7258;'" select="'du2'"/>
                <xsl:map-entry key="'&#x7259;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x725A;'" select="'cheng4	cheng1'"/>
                <xsl:map-entry key="'&#x725B;'" select="'niu2'"/>
                <xsl:map-entry key="'&#x725C;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x725D;'" select="'pin4'"/>
                <xsl:map-entry key="'&#x725E;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x725F;'" select="'mou2	mu4'"/>
                <xsl:map-entry key="'&#x7260;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x7261;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x7262;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x7263;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x7264;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x7265;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x7266;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x7267;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x7268;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x7269;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x726A;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x726B;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x726C;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x726D;'" select="'si4'"/>
                <xsl:map-entry key="'&#x726E;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x726F;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x7270;'" select="'you4'"/>
                <xsl:map-entry key="'&#x7271;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x7272;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x7273;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x7274;'" select="'di3'"/>
                <xsl:map-entry key="'&#x7275;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x7276;'" select="'quan4'"/>
                <xsl:map-entry key="'&#x7277;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x7278;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x7279;'" select="'te4'"/>
                <xsl:map-entry key="'&#x727A;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x727B;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x727C;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x727D;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x727E;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x727F;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x7280;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7281;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7282;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7283;'" select="'pou3'"/>
                <xsl:map-entry key="'&#x7284;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7285;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x7286;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x7287;'" select="'ben1	ben4'"/>
                <xsl:map-entry key="'&#x7288;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x7289;'" select="'run1'"/>
                <xsl:map-entry key="'&#x728A;'" select="'du2'"/>
                <xsl:map-entry key="'&#x728B;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x728C;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x728D;'" select="'jian1	qian2'"/>
                <xsl:map-entry key="'&#x728E;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x728F;'" select="'pian1'"/>
                <xsl:map-entry key="'&#x7290;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x7291;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x7292;'" select="'kao4	di2'"/>
                <xsl:map-entry key="'&#x7293;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x7294;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x7295;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x7296;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x7297;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x7298;'" select="'ma2'"/>
                <xsl:map-entry key="'&#x7299;'" select="'san1'"/>
                <xsl:map-entry key="'&#x729A;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x729B;'" select="'li2'"/>
                <xsl:map-entry key="'&#x729C;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x729D;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x729E;'" select="'se4'"/>
                <xsl:map-entry key="'&#x729F;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x72A0;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x72A1;'" select="'li4'"/>
                <xsl:map-entry key="'&#x72A2;'" select="'du2'"/>
                <xsl:map-entry key="'&#x72A3;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x72A4;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x72A5;'" select="'piao3'"/>
                <xsl:map-entry key="'&#x72A6;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x72A7;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x72A8;'" select="'chou1'"/>
                <xsl:map-entry key="'&#x72A9;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x72AA;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x72AB;'" select="'chou1'"/>
                <xsl:map-entry key="'&#x72AC;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x72AD;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x72AE;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x72AF;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x72B0;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x72B1;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x72B2;'" select="'chai2'"/>
                <xsl:map-entry key="'&#x72B3;'" select="'chuo2'"/>
                <xsl:map-entry key="'&#x72B4;'" select="'an4	han4'"/>
                <xsl:map-entry key="'&#x72B5;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x72B6;'" select="'zhuang4'"/>
                <xsl:map-entry key="'&#x72B7;'" select="'guang3'"/>
                <xsl:map-entry key="'&#x72B8;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x72B9;'" select="'you2'"/>
                <xsl:map-entry key="'&#x72BA;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x72BB;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x72BC;'" select="'hou3'"/>
                <xsl:map-entry key="'&#x72BD;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x72BE;'" select="'han4'"/>
                <xsl:map-entry key="'&#x72BF;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x72C0;'" select="'zhuang4'"/>
                <xsl:map-entry key="'&#x72C1;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x72C2;'" select="'kuang2'"/>
                <xsl:map-entry key="'&#x72C3;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x72C4;'" select="'di2'"/>
                <xsl:map-entry key="'&#x72C5;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x72C6;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x72C7;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x72C8;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x72C9;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x72CA;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x72CB;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x72CC;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x72CD;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x72CE;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x72CF;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x72D0;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x72D1;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x72D2;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x72D3;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x72D4;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x72D5;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x72D6;'" select="'you4'"/>
                <xsl:map-entry key="'&#x72D7;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x72D8;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x72D9;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x72DA;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x72DB;'" select="'bo4'"/>
                <xsl:map-entry key="'&#x72DC;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x72DD;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x72DE;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x72DF;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x72E0;'" select="'hen3'"/>
                <xsl:map-entry key="'&#x72E1;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x72E2;'" select="'he2	hao2	mo4'"/>
                <xsl:map-entry key="'&#x72E3;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x72E4;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x72E5;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x72E6;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x72E7;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x72E8;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x72E9;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x72EA;'" select="'tong1'"/>
                <xsl:map-entry key="'&#x72EB;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x72EC;'" select="'du2'"/>
                <xsl:map-entry key="'&#x72ED;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x72EE;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x72EF;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x72F0;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x72F1;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x72F2;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x72F3;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x72F4;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x72F5;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x72F6;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x72F7;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x72F8;'" select="'li2'"/>
                <xsl:map-entry key="'&#x72F9;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x72FA;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x72FB;'" select="'suan1'"/>
                <xsl:map-entry key="'&#x72FC;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x72FD;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x72FE;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x72FF;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x7300;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x7301;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7302;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7303;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x7304;'" select="'han4'"/>
                <xsl:map-entry key="'&#x7305;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x7306;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x7307;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x7308;'" select="'ba4	pi2'"/>
                <xsl:map-entry key="'&#x7309;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x730A;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x730B;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x730C;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x730D;'" select="'li2'"/>
                <xsl:map-entry key="'&#x730E;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x730F;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7310;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x7311;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x7312;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x7313;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x7314;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x7315;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x7316;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x7317;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x7318;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7319;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x731A;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x731B;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x731C;'" select="'cai1'"/>
                <xsl:map-entry key="'&#x731D;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x731E;'" select="'she4	she1'"/>
                <xsl:map-entry key="'&#x731F;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x7320;'" select="''"/>
                <xsl:map-entry key="'&#x7321;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x7322;'" select="'hu2	hu1'"/>
                <xsl:map-entry key="'&#x7323;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x7324;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x7325;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x7326;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x7327;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x7328;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7329;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x732A;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x732B;'" select="'mao1	mao2'"/>
                <xsl:map-entry key="'&#x732C;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x732D;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x732E;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x732F;'" select="'tuan1'"/>
                <xsl:map-entry key="'&#x7330;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x7331;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x7332;'" select="'xie1	he4'"/>
                <xsl:map-entry key="'&#x7333;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x7334;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x7335;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7336;'" select="'you2'"/>
                <xsl:map-entry key="'&#x7337;'" select="'you2'"/>
                <xsl:map-entry key="'&#x7338;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x7339;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x733A;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x733B;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x733C;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x733D;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x733E;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x733F;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7340;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x7341;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x7342;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7343;'" select="'dai1'"/>
                <xsl:map-entry key="'&#x7344;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7345;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x7346;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x7347;'" select="''"/>
                <xsl:map-entry key="'&#x7348;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7349;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x734A;'" select="'chuang4'"/>
                <xsl:map-entry key="'&#x734B;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x734C;'" select="'man4'"/>
                <xsl:map-entry key="'&#x734D;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x734E;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x734F;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x7350;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x7351;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x7352;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x7353;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x7354;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x7355;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x7356;'" select="'ben4'"/>
                <xsl:map-entry key="'&#x7357;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7358;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7359;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x735A;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x735B;'" select="'bu3'"/>
                <xsl:map-entry key="'&#x735C;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x735D;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x735E;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x735F;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x7360;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x7361;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x7362;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7363;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x7364;'" select="''"/>
                <xsl:map-entry key="'&#x7365;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x7366;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x7367;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x7368;'" select="'du2'"/>
                <xsl:map-entry key="'&#x7369;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x736A;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x736B;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x736C;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x736D;'" select="'ta3'"/>
                <xsl:map-entry key="'&#x736E;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x736F;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x7370;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x7371;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7372;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x7373;'" select="'nou2'"/>
                <xsl:map-entry key="'&#x7374;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x7375;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x7376;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x7377;'" select="'guang3'"/>
                <xsl:map-entry key="'&#x7378;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x7379;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x737A;'" select="'ta4	ta3'"/>
                <xsl:map-entry key="'&#x737B;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x737C;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x737D;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x737E;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x737F;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x7380;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x7381;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x7382;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7383;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x7384;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x7385;'" select="'miao4'"/>
                <xsl:map-entry key="'&#x7386;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x7387;'" select="'lu:4	shuai4	shuo4'"/>
                <xsl:map-entry key="'&#x7388;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x7389;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x738A;'" select="'su4'"/>
                <xsl:map-entry key="'&#x738B;'" select="'wang2	wang4'"/>
                <xsl:map-entry key="'&#x738C;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x738D;'" select="'ga3'"/>
                <xsl:map-entry key="'&#x738E;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x738F;'" select="'le4'"/>
                <xsl:map-entry key="'&#x7390;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x7391;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7392;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7393;'" select="'di4'"/>
                <xsl:map-entry key="'&#x7394;'" select="'chuan4'"/>
                <xsl:map-entry key="'&#x7395;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x7396;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x7397;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x7398;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x7399;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x739A;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x739B;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x739C;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x739D;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x739E;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x739F;'" select="'wen2	min2'"/>
                <xsl:map-entry key="'&#x73A0;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x73A1;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x73A2;'" select="'bin1	fen1'"/>
                <xsl:map-entry key="'&#x73A3;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x73A4;'" select="'beng3'"/>
                <xsl:map-entry key="'&#x73A5;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x73A6;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x73A7;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x73A8;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x73A9;'" select="'wan2	wan4'"/>
                <xsl:map-entry key="'&#x73AA;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x73AB;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x73AC;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x73AD;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x73AE;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x73AF;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x73B0;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x73B1;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x73B2;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x73B3;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x73B4;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x73B5;'" select="'an2'"/>
                <xsl:map-entry key="'&#x73B6;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x73B7;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x73B8;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x73B9;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x73BA;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x73BB;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x73BC;'" select="'ci3'"/>
                <xsl:map-entry key="'&#x73BD;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x73BE;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x73BF;'" select="'shao2'"/>
                <xsl:map-entry key="'&#x73C0;'" select="'po4'"/>
                <xsl:map-entry key="'&#x73C1;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x73C2;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x73C3;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x73C4;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x73C5;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x73C6;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x73C7;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x73C8;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x73C9;'" select="'min2'"/>
                <xsl:map-entry key="'&#x73CA;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x73CB;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x73CC;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x73CD;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x73CE;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x73CF;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x73D0;'" select="'fa4'"/>
                <xsl:map-entry key="'&#x73D1;'" select="'long2'"/>
                <xsl:map-entry key="'&#x73D2;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x73D3;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x73D4;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x73D5;'" select="'li4'"/>
                <xsl:map-entry key="'&#x73D6;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x73D7;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x73D8;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x73D9;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x73DA;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x73DB;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x73DC;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x73DD;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x73DE;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x73DF;'" select="'su4'"/>
                <xsl:map-entry key="'&#x73E0;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x73E1;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x73E2;'" select="'ken4'"/>
                <xsl:map-entry key="'&#x73E3;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x73E4;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x73E5;'" select="'er3'"/>
                <xsl:map-entry key="'&#x73E6;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x73E7;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x73E8;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x73E9;'" select="'heng2	hang2'"/>
                <xsl:map-entry key="'&#x73EA;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x73EB;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x73EC;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x73ED;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x73EE;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x73EF;'" select="''"/>
                <xsl:map-entry key="'&#x73F0;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x73F1;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x73F2;'" select="'hun2	hui1'"/>
                <xsl:map-entry key="'&#x73F3;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x73F4;'" select="'e2'"/>
                <xsl:map-entry key="'&#x73F5;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x73F6;'" select="'ti2	di4'"/>
                <xsl:map-entry key="'&#x73F7;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x73F8;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x73F9;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x73FA;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x73FB;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x73FC;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x73FD;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x73FE;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x73FF;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x7400;'" select="'han2'"/>
                <xsl:map-entry key="'&#x7401;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x7402;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x7403;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x7404;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x7405;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x7406;'" select="'li3'"/>
                <xsl:map-entry key="'&#x7407;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x7408;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7409;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x740A;'" select="'ye2	ya2'"/>
                <xsl:map-entry key="'&#x740B;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x740C;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x740D;'" select="'li4'"/>
                <xsl:map-entry key="'&#x740E;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x740F;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x7410;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x7411;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x7412;'" select="''"/>
                <xsl:map-entry key="'&#x7413;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x7414;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x7415;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x7416;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x7417;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x7418;'" select="'min2'"/>
                <xsl:map-entry key="'&#x7419;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x741A;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x741B;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x741C;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x741D;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x741E;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x741F;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x7420;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x7421;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x7422;'" select="'zhuo2	zuo2'"/>
                <xsl:map-entry key="'&#x7423;'" select="'pei3'"/>
                <xsl:map-entry key="'&#x7424;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x7425;'" select="'hu3'"/>
                <xsl:map-entry key="'&#x7426;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7427;'" select="'e4'"/>
                <xsl:map-entry key="'&#x7428;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x7429;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x742A;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x742B;'" select="'beng3'"/>
                <xsl:map-entry key="'&#x742C;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x742D;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x742E;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x742F;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x7430;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x7431;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x7432;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x7433;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7434;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x7435;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x7436;'" select="'pa2	ba5'"/>
                <xsl:map-entry key="'&#x7437;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x7438;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x7439;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x743A;'" select="'fa4'"/>
                <xsl:map-entry key="'&#x743B;'" select="''"/>
                <xsl:map-entry key="'&#x743C;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x743D;'" select="'du3'"/>
                <xsl:map-entry key="'&#x743E;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x743F;'" select="'hun2	hui1'"/>
                <xsl:map-entry key="'&#x7440;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x7441;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x7442;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x7443;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x7444;'" select="'xuan1	xuan3'"/>
                <xsl:map-entry key="'&#x7445;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x7446;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x7447;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x7448;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x7449;'" select="'min2'"/>
                <xsl:map-entry key="'&#x744A;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x744B;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x744C;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x744D;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x744E;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x744F;'" select="'chuan1'"/>
                <xsl:map-entry key="'&#x7450;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7451;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x7452;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x7453;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x7454;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x7455;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x7456;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x7457;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x7458;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x7459;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x745A;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x745B;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x745C;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x745D;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x745E;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x745F;'" select="'se4'"/>
                <xsl:map-entry key="'&#x7460;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x7461;'" select="'shi'"/>
                <xsl:map-entry key="'&#x7462;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7463;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x7464;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x7465;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x7466;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x7467;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x7468;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x7469;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x746A;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x746B;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x746C;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x746D;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x746E;'" select="'li4'"/>
                <xsl:map-entry key="'&#x746F;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x7470;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x7471;'" select="'tian4'"/>
                <xsl:map-entry key="'&#x7472;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x7473;'" select="'cuo3'"/>
                <xsl:map-entry key="'&#x7474;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7475;'" select="'zhao3'"/>
                <xsl:map-entry key="'&#x7476;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x7477;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x7478;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x7479;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x747A;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x747B;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x747C;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x747D;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x747E;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x747F;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x7480;'" select="'cui3'"/>
                <xsl:map-entry key="'&#x7481;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x7482;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7483;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7484;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x7485;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x7486;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x7487;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x7488;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x7489;'" select="'lian2	lian3'"/>
                <xsl:map-entry key="'&#x748A;'" select="'man2'"/>
                <xsl:map-entry key="'&#x748B;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x748C;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x748D;'" select="''"/>
                <xsl:map-entry key="'&#x748E;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x748F;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x7490;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7491;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x7492;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x7493;'" select="'xiou4'"/>
                <xsl:map-entry key="'&#x7494;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x7495;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x7496;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x7497;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x7498;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7499;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x749A;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x749B;'" select="'su4'"/>
                <xsl:map-entry key="'&#x749C;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x749D;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x749E;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x749F;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x74A0;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x74A1;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x74A2;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x74A3;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x74A4;'" select="''"/>
                <xsl:map-entry key="'&#x74A5;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x74A6;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x74A7;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x74A8;'" select="'can4'"/>
                <xsl:map-entry key="'&#x74A9;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x74AA;'" select="'zao3'"/>
                <xsl:map-entry key="'&#x74AB;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x74AC;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x74AD;'" select="'gun4'"/>
                <xsl:map-entry key="'&#x74AE;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x74AF;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x74B0;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x74B1;'" select="'se4'"/>
                <xsl:map-entry key="'&#x74B2;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x74B3;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x74B4;'" select="''"/>
                <xsl:map-entry key="'&#x74B5;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x74B6;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x74B7;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x74B8;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x74B9;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x74BA;'" select="'wen4	wen2'"/>
                <xsl:map-entry key="'&#x74BB;'" select="'zui3'"/>
                <xsl:map-entry key="'&#x74BC;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x74BD;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x74BE;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x74BF;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x74C0;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x74C1;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x74C2;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x74C3;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x74C4;'" select="'du2'"/>
                <xsl:map-entry key="'&#x74C5;'" select="'li4'"/>
                <xsl:map-entry key="'&#x74C6;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x74C7;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x74C8;'" select="'li2'"/>
                <xsl:map-entry key="'&#x74C9;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x74CA;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x74CB;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x74CC;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x74CD;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x74CE;'" select="'la4'"/>
                <xsl:map-entry key="'&#x74CF;'" select="'long2'"/>
                <xsl:map-entry key="'&#x74D0;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x74D1;'" select="'li4'"/>
                <xsl:map-entry key="'&#x74D2;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x74D3;'" select="'lan4'"/>
                <xsl:map-entry key="'&#x74D4;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x74D5;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x74D6;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x74D7;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x74D8;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x74D9;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x74DA;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x74DB;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x74DC;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x74DD;'" select="'bao2'"/>
                <xsl:map-entry key="'&#x74DE;'" select="'die2'"/>
                <xsl:map-entry key="'&#x74DF;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x74E0;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x74E1;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x74E2;'" select="'piao2'"/>
                <xsl:map-entry key="'&#x74E3;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x74E4;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x74E5;'" select="'li4'"/>
                <xsl:map-entry key="'&#x74E6;'" select="'wa3	wa4'"/>
                <xsl:map-entry key="'&#x74E7;'" select="''"/>
                <xsl:map-entry key="'&#x74E8;'" select="'jiang1	hong2'"/>
                <xsl:map-entry key="'&#x74E9;'" select="'qian2wa3'"/>
                <xsl:map-entry key="'&#x74EA;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x74EB;'" select="'pen2'"/>
                <xsl:map-entry key="'&#x74EC;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x74ED;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x74EE;'" select="'weng4'"/>
                <xsl:map-entry key="'&#x74EF;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x74F0;'" select="''"/>
                <xsl:map-entry key="'&#x74F1;'" select="''"/>
                <xsl:map-entry key="'&#x74F2;'" select="''"/>
                <xsl:map-entry key="'&#x74F3;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x74F4;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x74F5;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x74F6;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x74F7;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x74F8;'" select="''"/>
                <xsl:map-entry key="'&#x74F9;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x74FA;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x74FB;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x74FC;'" select="''"/>
                <xsl:map-entry key="'&#x74FD;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x74FE;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x74FF;'" select="'pou3	bu4'"/>
                <xsl:map-entry key="'&#x7500;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x7501;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x7502;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7503;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7504;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x7505;'" select="''"/>
                <xsl:map-entry key="'&#x7506;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x7507;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7508;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x7509;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x750A;'" select="'lou3'"/>
                <xsl:map-entry key="'&#x750B;'" select="'di4'"/>
                <xsl:map-entry key="'&#x750C;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x750D;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x750E;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x750F;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x7510;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7511;'" select="'zeng4'"/>
                <xsl:map-entry key="'&#x7512;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x7513;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x7514;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x7515;'" select="'weng4'"/>
                <xsl:map-entry key="'&#x7516;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7517;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x7518;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x7519;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x751A;'" select="'shen2	she2	shen4'"/>
                <xsl:map-entry key="'&#x751B;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x751C;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x751D;'" select="'han1'"/>
                <xsl:map-entry key="'&#x751E;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x751F;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x7520;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x7521;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7522;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x7523;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x7524;'" select="'rui2'"/>
                <xsl:map-entry key="'&#x7525;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x7526;'" select="'su1'"/>
                <xsl:map-entry key="'&#x7527;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7528;'" select="'yong4'"/>
                <xsl:map-entry key="'&#x7529;'" select="'shuai3'"/>
                <xsl:map-entry key="'&#x752A;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x752B;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x752C;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x752D;'" select="'beng2'"/>
                <xsl:map-entry key="'&#x752E;'" select="'fong4'"/>
                <xsl:map-entry key="'&#x752F;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x7530;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x7531;'" select="'you2'"/>
                <xsl:map-entry key="'&#x7532;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x7533;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7534;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x7535;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x7536;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7537;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x7538;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x7539;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x753A;'" select="'ting3	ding1'"/>
                <xsl:map-entry key="'&#x753B;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x753C;'" select="'ting3	ding1'"/>
                <xsl:map-entry key="'&#x753D;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x753E;'" select="'zi1	zai1'"/>
                <xsl:map-entry key="'&#x753F;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x7540;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7541;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7542;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x7543;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x7544;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x7545;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x7546;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x7547;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x7548;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x7549;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x754A;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x754B;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x754C;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x754D;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x754E;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x754F;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x7550;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x7551;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x7552;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x7553;'" select="''"/>
                <xsl:map-entry key="'&#x7554;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x7555;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x7556;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x7557;'" select="'da2'"/>
                <xsl:map-entry key="'&#x7558;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x7559;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x755A;'" select="'ben3'"/>
                <xsl:map-entry key="'&#x755B;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x755C;'" select="'chu4	xu4'"/>
                <xsl:map-entry key="'&#x755D;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x755E;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x755F;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x7560;'" select="''"/>
                <xsl:map-entry key="'&#x7561;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x7562;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7563;'" select="'da2'"/>
                <xsl:map-entry key="'&#x7564;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7565;'" select="'lu:e4'"/>
                <xsl:map-entry key="'&#x7566;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7567;'" select="'lu:e4'"/>
                <xsl:map-entry key="'&#x7568;'" select="'pan1'"/>
                <xsl:map-entry key="'&#x7569;'" select="''"/>
                <xsl:map-entry key="'&#x756A;'" select="'fan1	pan1'"/>
                <xsl:map-entry key="'&#x756B;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x756C;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x756D;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x756E;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x756F;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x7570;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7571;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x7572;'" select="'she1'"/>
                <xsl:map-entry key="'&#x7573;'" select="'die2'"/>
                <xsl:map-entry key="'&#x7574;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7575;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x7576;'" select="'dang1	dang4'"/>
                <xsl:map-entry key="'&#x7577;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x7578;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7579;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x757A;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x757B;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x757C;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x757D;'" select="'tun3'"/>
                <xsl:map-entry key="'&#x757E;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x757F;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7580;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x7581;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x7582;'" select="'die2'"/>
                <xsl:map-entry key="'&#x7583;'" select="'tuan3'"/>
                <xsl:map-entry key="'&#x7584;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7585;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x7586;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x7587;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7588;'" select="'bo4'"/>
                <xsl:map-entry key="'&#x7589;'" select="'die2'"/>
                <xsl:map-entry key="'&#x758A;'" select="'die2'"/>
                <xsl:map-entry key="'&#x758B;'" select="'pi3	shu1	ya3'"/>
                <xsl:map-entry key="'&#x758C;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x758D;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x758E;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x758F;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x7590;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7591;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x7592;'" select="'chuang2'"/>
                <xsl:map-entry key="'&#x7593;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x7594;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x7595;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x7596;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x7597;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x7598;'" select="'gong1	gang1'"/>
                <xsl:map-entry key="'&#x7599;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x759A;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x759B;'" select="'zhou3'"/>
                <xsl:map-entry key="'&#x759C;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x759D;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x759E;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x759F;'" select="'nu:e4	yao4'"/>
                <xsl:map-entry key="'&#x75A0;'" select="'li4'"/>
                <xsl:map-entry key="'&#x75A1;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x75A2;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x75A3;'" select="'you2'"/>
                <xsl:map-entry key="'&#x75A4;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x75A5;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x75A6;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x75A7;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x75A8;'" select="'xia1'"/>
                <xsl:map-entry key="'&#x75A9;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x75AA;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x75AB;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x75AC;'" select="'li4'"/>
                <xsl:map-entry key="'&#x75AD;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x75AE;'" select="'chuang1'"/>
                <xsl:map-entry key="'&#x75AF;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x75B0;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x75B1;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x75B2;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x75B3;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x75B4;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x75B5;'" select="'ci1	ci2'"/>
                <xsl:map-entry key="'&#x75B6;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x75B7;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x75B8;'" select="'dan3	da5'"/>
                <xsl:map-entry key="'&#x75B9;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x75BA;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x75BB;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x75BC;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x75BD;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x75BE;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x75BF;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x75C0;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x75C1;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x75C2;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x75C3;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x75C4;'" select="'chi2	zha4'"/>
                <xsl:map-entry key="'&#x75C5;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x75C6;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x75C7;'" select="'zheng4	zheng1'"/>
                <xsl:map-entry key="'&#x75C8;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x75C9;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x75CA;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x75CB;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x75CC;'" select="'tong1'"/>
                <xsl:map-entry key="'&#x75CD;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x75CE;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x75CF;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x75D0;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x75D1;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x75D2;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x75D3;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x75D4;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x75D5;'" select="'hen2'"/>
                <xsl:map-entry key="'&#x75D6;'" select="'ya3'"/>
                <xsl:map-entry key="'&#x75D7;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x75D8;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x75D9;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x75DA;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x75DB;'" select="'tong4'"/>
                <xsl:map-entry key="'&#x75DC;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x75DD;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x75DE;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x75DF;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x75E0;'" select="'suan1'"/>
                <xsl:map-entry key="'&#x75E1;'" select="'pu1'"/>
                <xsl:map-entry key="'&#x75E2;'" select="'li4'"/>
                <xsl:map-entry key="'&#x75E3;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x75E4;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x75E5;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x75E6;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x75E7;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x75E8;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x75E9;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x75EA;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x75EB;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x75EC;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x75ED;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x75EE;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x75EF;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x75F0;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x75F1;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x75F2;'" select="'ma2'"/>
                <xsl:map-entry key="'&#x75F3;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x75F4;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x75F5;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x75F6;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x75F7;'" select="'an1'"/>
                <xsl:map-entry key="'&#x75F8;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x75F9;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x75FA;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x75FB;'" select="'min2'"/>
                <xsl:map-entry key="'&#x75FC;'" select="'gu1	gu4'"/>
                <xsl:map-entry key="'&#x75FD;'" select="'dui1'"/>
                <xsl:map-entry key="'&#x75FE;'" select="'e1'"/>
                <xsl:map-entry key="'&#x75FF;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x7600;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x7601;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x7602;'" select="'ya3'"/>
                <xsl:map-entry key="'&#x7603;'" select="'zhu3	zhu2'"/>
                <xsl:map-entry key="'&#x7604;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7605;'" select="'dan4	dan1	dan3'"/>
                <xsl:map-entry key="'&#x7606;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x7607;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x7608;'" select="'ji4	zhi4'"/>
                <xsl:map-entry key="'&#x7609;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x760A;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x760B;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x760C;'" select="'la4'"/>
                <xsl:map-entry key="'&#x760D;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x760E;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x760F;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x7610;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x7611;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x7612;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x7613;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x7614;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x7615;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x7616;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x7617;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7618;'" select="'lou4'"/>
                <xsl:map-entry key="'&#x7619;'" select="'sao4'"/>
                <xsl:map-entry key="'&#x761A;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x761B;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x761C;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x761D;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x761E;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x761F;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x7620;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x7621;'" select="'chuang1'"/>
                <xsl:map-entry key="'&#x7622;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x7623;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x7624;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x7625;'" select="'chai4	cuo2'"/>
                <xsl:map-entry key="'&#x7626;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x7627;'" select="'nu:e4	yao4'"/>
                <xsl:map-entry key="'&#x7628;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x7629;'" select="'da2'"/>
                <xsl:map-entry key="'&#x762A;'" select="'bie1	bie3'"/>
                <xsl:map-entry key="'&#x762B;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x762C;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x762D;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x762E;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x762F;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x7630;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x7631;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7632;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x7633;'" select="'chou1'"/>
                <xsl:map-entry key="'&#x7634;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x7635;'" select="'zhai4'"/>
                <xsl:map-entry key="'&#x7636;'" select="'sou4'"/>
                <xsl:map-entry key="'&#x7637;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x7638;'" select="'que2'"/>
                <xsl:map-entry key="'&#x7639;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x763A;'" select="'lou4'"/>
                <xsl:map-entry key="'&#x763B;'" select="'lou4'"/>
                <xsl:map-entry key="'&#x763C;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x763D;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x763E;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x763F;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x7640;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x7641;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7642;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x7643;'" select="'long2'"/>
                <xsl:map-entry key="'&#x7644;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x7645;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x7646;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x7647;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x7648;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x7649;'" select="'dan4	dan1	dan3'"/>
                <xsl:map-entry key="'&#x764A;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x764B;'" select="'he4'"/>
                <xsl:map-entry key="'&#x764C;'" select="'yan2	ai2'"/>
                <xsl:map-entry key="'&#x764D;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x764E;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x764F;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x7650;'" select="'guai4'"/>
                <xsl:map-entry key="'&#x7651;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x7652;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7653;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x7654;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7655;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x7656;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x7657;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x7658;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7659;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x765A;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x765B;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x765C;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x765D;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x765E;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x765F;'" select="'bie1	bie3'"/>
                <xsl:map-entry key="'&#x7660;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7661;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x7662;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x7663;'" select="'xuan3'"/>
                <xsl:map-entry key="'&#x7664;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x7665;'" select="'zheng1	zheng4'"/>
                <xsl:map-entry key="'&#x7666;'" select="''"/>
                <xsl:map-entry key="'&#x7667;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7668;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x7669;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x766A;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x766B;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x766C;'" select="'xian3	xuan3'"/>
                <xsl:map-entry key="'&#x766D;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x766E;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x766F;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x7670;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x7671;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x7672;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x7673;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x7674;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x7675;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x7676;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x7677;'" select="''"/>
                <xsl:map-entry key="'&#x7678;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x7679;'" select="'po1'"/>
                <xsl:map-entry key="'&#x767A;'" select="'fa1'"/>
                <xsl:map-entry key="'&#x767B;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x767C;'" select="'fa1'"/>
                <xsl:map-entry key="'&#x767D;'" select="'bai2'"/>
                <xsl:map-entry key="'&#x767E;'" select="'bai3	bo2'"/>
                <xsl:map-entry key="'&#x767F;'" select="'qie2'"/>
                <xsl:map-entry key="'&#x7680;'" select="'bi1'"/>
                <xsl:map-entry key="'&#x7681;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x7682;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x7683;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x7684;'" select="'de5	di2	di4'"/>
                <xsl:map-entry key="'&#x7685;'" select="'pa1'"/>
                <xsl:map-entry key="'&#x7686;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x7687;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x7688;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x7689;'" select="'ci3'"/>
                <xsl:map-entry key="'&#x768A;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x768B;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x768C;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x768D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x768E;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x768F;'" select="'peng3'"/>
                <xsl:map-entry key="'&#x7690;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x7691;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x7692;'" select="'e2'"/>
                <xsl:map-entry key="'&#x7693;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x7694;'" select="'han4'"/>
                <xsl:map-entry key="'&#x7695;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7696;'" select="'wan3	huan3'"/>
                <xsl:map-entry key="'&#x7697;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7698;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x7699;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x769A;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x769B;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x769C;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x769D;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x769E;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x769F;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x76A0;'" select="'cui2'"/>
                <xsl:map-entry key="'&#x76A1;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x76A2;'" select="'xiao3'"/>
                <xsl:map-entry key="'&#x76A3;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x76A4;'" select="'po2'"/>
                <xsl:map-entry key="'&#x76A5;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x76A6;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x76A7;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x76A8;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x76A9;'" select="'huang4'"/>
                <xsl:map-entry key="'&#x76AA;'" select="'li4'"/>
                <xsl:map-entry key="'&#x76AB;'" select="'piao3'"/>
                <xsl:map-entry key="'&#x76AC;'" select="'he4'"/>
                <xsl:map-entry key="'&#x76AD;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x76AE;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x76AF;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x76B0;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x76B1;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x76B2;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x76B3;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x76B4;'" select="'cun1'"/>
                <xsl:map-entry key="'&#x76B5;'" select="'que4'"/>
                <xsl:map-entry key="'&#x76B6;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x76B7;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x76B8;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x76B9;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x76BA;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x76BB;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x76BC;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x76BD;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x76BE;'" select="'du2'"/>
                <xsl:map-entry key="'&#x76BF;'" select="'min3'"/>
                <xsl:map-entry key="'&#x76C0;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x76C1;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x76C2;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x76C3;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x76C4;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x76C5;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x76C6;'" select="'pen2'"/>
                <xsl:map-entry key="'&#x76C7;'" select="'he2'"/>
                <xsl:map-entry key="'&#x76C8;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x76C9;'" select="'he2'"/>
                <xsl:map-entry key="'&#x76CA;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x76CB;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x76CC;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x76CD;'" select="'he2'"/>
                <xsl:map-entry key="'&#x76CE;'" select="'ang4'"/>
                <xsl:map-entry key="'&#x76CF;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x76D0;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x76D1;'" select="'jian1	jian4'"/>
                <xsl:map-entry key="'&#x76D2;'" select="'he2'"/>
                <xsl:map-entry key="'&#x76D3;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x76D4;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x76D5;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x76D6;'" select="'ge3'"/>
                <xsl:map-entry key="'&#x76D7;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x76D8;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x76D9;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x76DA;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x76DB;'" select="'sheng4	cheng2'"/>
                <xsl:map-entry key="'&#x76DC;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x76DD;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x76DE;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x76DF;'" select="'meng2	ming2'"/>
                <xsl:map-entry key="'&#x76E0;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x76E1;'" select="'jin4	jin3'"/>
                <xsl:map-entry key="'&#x76E2;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x76E3;'" select="'jian1	jian4'"/>
                <xsl:map-entry key="'&#x76E4;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x76E5;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x76E6;'" select="'an1'"/>
                <xsl:map-entry key="'&#x76E7;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x76E8;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x76E9;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x76EA;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x76EB;'" select="'an1'"/>
                <xsl:map-entry key="'&#x76EC;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x76ED;'" select="'li4'"/>
                <xsl:map-entry key="'&#x76EE;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x76EF;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x76F0;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x76F1;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x76F2;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x76F3;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x76F4;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x76F5;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x76F6;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x76F7;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x76F8;'" select="'xiang1	xiang4'"/>
                <xsl:map-entry key="'&#x76F9;'" select="'dun4	dun3'"/>
                <xsl:map-entry key="'&#x76FA;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x76FB;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x76FC;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x76FD;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x76FE;'" select="'dun4	shun3'"/>
                <xsl:map-entry key="'&#x76FF;'" select="'min2'"/>
                <xsl:map-entry key="'&#x7700;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x7701;'" select="'sheng3	xing3'"/>
                <xsl:map-entry key="'&#x7702;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x7703;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x7704;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x7705;'" select="'pan1'"/>
                <xsl:map-entry key="'&#x7706;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x7707;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x7708;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x7709;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x770A;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x770B;'" select="'kan4	kan1'"/>
                <xsl:map-entry key="'&#x770C;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x770D;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x770E;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x770F;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x7710;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x7711;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x7712;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7713;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x7714;'" select="'da4'"/>
                <xsl:map-entry key="'&#x7715;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7716;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x7717;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x7718;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x7719;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x771A;'" select="'sheng3'"/>
                <xsl:map-entry key="'&#x771B;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x771C;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x771D;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x771E;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x771F;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x7720;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x7721;'" select="'di1'"/>
                <xsl:map-entry key="'&#x7722;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x7723;'" select="'die2'"/>
                <xsl:map-entry key="'&#x7724;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x7725;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x7726;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x7727;'" select="'chao3'"/>
                <xsl:map-entry key="'&#x7728;'" select="'zha3'"/>
                <xsl:map-entry key="'&#x7729;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x772A;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x772B;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x772C;'" select="'long2'"/>
                <xsl:map-entry key="'&#x772D;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x772E;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x772F;'" select="'mi3	mi1'"/>
                <xsl:map-entry key="'&#x7730;'" select="'die2'"/>
                <xsl:map-entry key="'&#x7731;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x7732;'" select="'er4'"/>
                <xsl:map-entry key="'&#x7733;'" select="'ming3'"/>
                <xsl:map-entry key="'&#x7734;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x7735;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x7736;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x7737;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x7738;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x7739;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x773A;'" select="'tiao4'"/>
                <xsl:map-entry key="'&#x773B;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x773C;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x773D;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x773E;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x773F;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x7740;'" select="'zhao2	zhao1	zhe5	zhuo2'"/>
                <xsl:map-entry key="'&#x7741;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x7742;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x7743;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x7744;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x7745;'" select="'han4'"/>
                <xsl:map-entry key="'&#x7746;'" select="'huan3'"/>
                <xsl:map-entry key="'&#x7747;'" select="'di4'"/>
                <xsl:map-entry key="'&#x7748;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x7749;'" select="'cuo1'"/>
                <xsl:map-entry key="'&#x774A;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x774B;'" select="'e2'"/>
                <xsl:map-entry key="'&#x774C;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x774D;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x774E;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x774F;'" select="'kun4'"/>
                <xsl:map-entry key="'&#x7750;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x7751;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7752;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x7753;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x7754;'" select="'hun3'"/>
                <xsl:map-entry key="'&#x7755;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x7756;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7757;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x7758;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x7759;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x775A;'" select="'ai2	ya2'"/>
                <xsl:map-entry key="'&#x775B;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x775C;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x775D;'" select="'li2'"/>
                <xsl:map-entry key="'&#x775E;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x775F;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x7760;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x7761;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x7762;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x7763;'" select="'du1'"/>
                <xsl:map-entry key="'&#x7764;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x7765;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x7766;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x7767;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x7768;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x7769;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x776A;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x776B;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x776C;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x776D;'" select="'zhou3'"/>
                <xsl:map-entry key="'&#x776E;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x776F;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x7770;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x7771;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x7772;'" select="'xing3'"/>
                <xsl:map-entry key="'&#x7773;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x7774;'" select="'gun4'"/>
                <xsl:map-entry key="'&#x7775;'" select="'cai'"/>
                <xsl:map-entry key="'&#x7776;'" select="'chun3'"/>
                <xsl:map-entry key="'&#x7777;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7778;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x7779;'" select="'du3'"/>
                <xsl:map-entry key="'&#x777A;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x777B;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x777C;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x777D;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x777E;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x777F;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x7780;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x7781;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7782;'" select="'fa1'"/>
                <xsl:map-entry key="'&#x7783;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x7784;'" select="'miao2'"/>
                <xsl:map-entry key="'&#x7785;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x7786;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x7787;'" select="'mi1'"/>
                <xsl:map-entry key="'&#x7788;'" select="'weng3'"/>
                <xsl:map-entry key="'&#x7789;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x778A;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x778B;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x778C;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x778D;'" select="'sou3'"/>
                <xsl:map-entry key="'&#x778E;'" select="'xia1'"/>
                <xsl:map-entry key="'&#x778F;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x7790;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x7791;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x7792;'" select="'man2'"/>
                <xsl:map-entry key="'&#x7793;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x7794;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x7795;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x7796;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7797;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x7798;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x7799;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x779A;'" select="'shun4'"/>
                <xsl:map-entry key="'&#x779B;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x779C;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x779D;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x779E;'" select="'man2'"/>
                <xsl:map-entry key="'&#x779F;'" select="'piao3'"/>
                <xsl:map-entry key="'&#x77A0;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x77A1;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x77A2;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x77A3;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x77A4;'" select="'run2'"/>
                <xsl:map-entry key="'&#x77A5;'" select="'pie1'"/>
                <xsl:map-entry key="'&#x77A6;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x77A7;'" select="'qiao2	ya3'"/>
                <xsl:map-entry key="'&#x77A8;'" select="'pu1'"/>
                <xsl:map-entry key="'&#x77A9;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x77AA;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x77AB;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x77AC;'" select="'shun4'"/>
                <xsl:map-entry key="'&#x77AD;'" select="'liao3	liao4'"/>
                <xsl:map-entry key="'&#x77AE;'" select="'che4'"/>
                <xsl:map-entry key="'&#x77AF;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x77B0;'" select="'kan4'"/>
                <xsl:map-entry key="'&#x77B1;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x77B2;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x77B3;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x77B4;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x77B5;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x77B6;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x77B7;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x77B8;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x77B9;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x77BA;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x77BB;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x77BC;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x77BD;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x77BE;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x77BF;'" select="'qu1	ju4	qu2'"/>
                <xsl:map-entry key="'&#x77C0;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x77C1;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x77C2;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x77C3;'" select="'ning3'"/>
                <xsl:map-entry key="'&#x77C4;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x77C5;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x77C6;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x77C7;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x77C8;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x77C9;'" select="'bin1	pin2'"/>
                <xsl:map-entry key="'&#x77CA;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x77CB;'" select="'li4'"/>
                <xsl:map-entry key="'&#x77CC;'" select="'guang4'"/>
                <xsl:map-entry key="'&#x77CD;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x77CE;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x77CF;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x77D0;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x77D1;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x77D2;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x77D3;'" select="'long2'"/>
                <xsl:map-entry key="'&#x77D4;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x77D5;'" select="'man3'"/>
                <xsl:map-entry key="'&#x77D6;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x77D7;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x77D8;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x77D9;'" select="'kan4'"/>
                <xsl:map-entry key="'&#x77DA;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x77DB;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x77DC;'" select="'jin1	guan1	qin2'"/>
                <xsl:map-entry key="'&#x77DD;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x77DE;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x77DF;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x77E0;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x77E1;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x77E2;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x77E3;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x77E4;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x77E5;'" select="'zhi1	zhi4'"/>
                <xsl:map-entry key="'&#x77E6;'" select="'hou2	hou4'"/>
                <xsl:map-entry key="'&#x77E7;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x77E8;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x77E9;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x77EA;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x77EB;'" select="'jiao3	jia3	jiao2'"/>
                <xsl:map-entry key="'&#x77EC;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x77ED;'" select="'duan3'"/>
                <xsl:map-entry key="'&#x77EE;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x77EF;'" select="'jiao3	jia3	jiao2'"/>
                <xsl:map-entry key="'&#x77F0;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x77F1;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x77F2;'" select="'bai3	bai4	pai3'"/>
                <xsl:map-entry key="'&#x77F3;'" select="'shi2	dan4'"/>
                <xsl:map-entry key="'&#x77F4;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x77F5;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x77F6;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x77F7;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x77F8;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x77F9;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x77FA;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x77FB;'" select="'ku4	ku1'"/>
                <xsl:map-entry key="'&#x77FC;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x77FD;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x77FE;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x77FF;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x7800;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x7801;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x7802;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x7803;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x7804;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7805;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7806;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7807;'" select="'min2'"/>
                <xsl:map-entry key="'&#x7808;'" select="'nuo3'"/>
                <xsl:map-entry key="'&#x7809;'" select="'hua1	xu1'"/>
                <xsl:map-entry key="'&#x780A;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x780B;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x780C;'" select="'qi4	qie4'"/>
                <xsl:map-entry key="'&#x780D;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x780E;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x780F;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x7810;'" select="'e4'"/>
                <xsl:map-entry key="'&#x7811;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x7812;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x7813;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x7814;'" select="'yan2	yan4'"/>
                <xsl:map-entry key="'&#x7815;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x7816;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x7817;'" select="'che1'"/>
                <xsl:map-entry key="'&#x7818;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x7819;'" select="'pan1'"/>
                <xsl:map-entry key="'&#x781A;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x781B;'" select="''"/>
                <xsl:map-entry key="'&#x781C;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x781D;'" select="'fa2	fa3'"/>
                <xsl:map-entry key="'&#x781E;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x781F;'" select="'zha4	zha3	zuo2'"/>
                <xsl:map-entry key="'&#x7820;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x7821;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7822;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x7823;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x7824;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x7825;'" select="'di3'"/>
                <xsl:map-entry key="'&#x7826;'" select="'zhai4'"/>
                <xsl:map-entry key="'&#x7827;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x7828;'" select="'e4'"/>
                <xsl:map-entry key="'&#x7829;'" select="'fei4	fu2'"/>
                <xsl:map-entry key="'&#x782A;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x782B;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x782C;'" select="'li4	la2'"/>
                <xsl:map-entry key="'&#x782D;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x782E;'" select="'nu3'"/>
                <xsl:map-entry key="'&#x782F;'" select="'ping1'"/>
                <xsl:map-entry key="'&#x7830;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x7831;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7832;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x7833;'" select="'le4'"/>
                <xsl:map-entry key="'&#x7834;'" select="'po4'"/>
                <xsl:map-entry key="'&#x7835;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x7836;'" select="'po4'"/>
                <xsl:map-entry key="'&#x7837;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7838;'" select="'za2'"/>
                <xsl:map-entry key="'&#x7839;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x783A;'" select="'li4'"/>
                <xsl:map-entry key="'&#x783B;'" select="'long2'"/>
                <xsl:map-entry key="'&#x783C;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x783D;'" select="''"/>
                <xsl:map-entry key="'&#x783E;'" select="'li4'"/>
                <xsl:map-entry key="'&#x783F;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x7840;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x7841;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x7842;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x7843;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x7844;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x7845;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x7846;'" select="'e4'"/>
                <xsl:map-entry key="'&#x7847;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x7848;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x7849;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x784A;'" select="'wei3	kui4'"/>
                <xsl:map-entry key="'&#x784B;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x784C;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x784D;'" select="'ken4'"/>
                <xsl:map-entry key="'&#x784E;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x784F;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x7850;'" select="'tong2	dong3'"/>
                <xsl:map-entry key="'&#x7851;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x7852;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7853;'" select="''"/>
                <xsl:map-entry key="'&#x7854;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7855;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x7856;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x7857;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x7858;'" select="''"/>
                <xsl:map-entry key="'&#x7859;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x785A;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x785B;'" select="''"/>
                <xsl:map-entry key="'&#x785C;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x785D;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x785E;'" select="'que4'"/>
                <xsl:map-entry key="'&#x785F;'" select="'chan4'"/>
                <xsl:map-entry key="'&#x7860;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x7861;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x7862;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x7863;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7864;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x7865;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x7866;'" select="'long4'"/>
                <xsl:map-entry key="'&#x7867;'" select="'iong3'"/>
                <xsl:map-entry key="'&#x7868;'" select="'che1'"/>
                <xsl:map-entry key="'&#x7869;'" select="'che4'"/>
                <xsl:map-entry key="'&#x786A;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x786B;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x786C;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x786D;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x786E;'" select="'que4'"/>
                <xsl:map-entry key="'&#x786F;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x7870;'" select="'cuo3'"/>
                <xsl:map-entry key="'&#x7871;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x7872;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7873;'" select="''"/>
                <xsl:map-entry key="'&#x7874;'" select="''"/>
                <xsl:map-entry key="'&#x7875;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x7876;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x7877;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7878;'" select="'nue4'"/>
                <xsl:map-entry key="'&#x7879;'" select="'song1'"/>
                <xsl:map-entry key="'&#x787A;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x787B;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x787C;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x787D;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x787E;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x787F;'" select="'kong1'"/>
                <xsl:map-entry key="'&#x7880;'" select="'ceng2'"/>
                <xsl:map-entry key="'&#x7881;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7882;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x7883;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x7884;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7885;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x7886;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x7887;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x7888;'" select="'min2'"/>
                <xsl:map-entry key="'&#x7889;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x788A;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x788B;'" select="'he4'"/>
                <xsl:map-entry key="'&#x788C;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x788D;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x788E;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x788F;'" select="'que4'"/>
                <xsl:map-entry key="'&#x7890;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7891;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x7892;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x7893;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x7894;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x7895;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7896;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x7897;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x7898;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x7899;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x789A;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x789B;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x789C;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x789D;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x789E;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x789F;'" select="'die2'"/>
                <xsl:map-entry key="'&#x78A0;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x78A1;'" select="'du2	zhou2'"/>
                <xsl:map-entry key="'&#x78A2;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x78A3;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x78A4;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x78A5;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x78A6;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x78A7;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x78A8;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x78A9;'" select="'shuo4	shi2'"/>
                <xsl:map-entry key="'&#x78AA;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x78AB;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x78AC;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x78AD;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x78AE;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x78AF;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x78B0;'" select="'peng4'"/>
                <xsl:map-entry key="'&#x78B1;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x78B2;'" select="'di4'"/>
                <xsl:map-entry key="'&#x78B3;'" select="'tan4'"/>
                <xsl:map-entry key="'&#x78B4;'" select="'cha2	cha1'"/>
                <xsl:map-entry key="'&#x78B5;'" select="''"/>
                <xsl:map-entry key="'&#x78B6;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x78B7;'" select="''"/>
                <xsl:map-entry key="'&#x78B8;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x78B9;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x78BA;'" select="'que4'"/>
                <xsl:map-entry key="'&#x78BB;'" select="'que4'"/>
                <xsl:map-entry key="'&#x78BC;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x78BD;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x78BE;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x78BF;'" select="'su4'"/>
                <xsl:map-entry key="'&#x78C0;'" select="'e2'"/>
                <xsl:map-entry key="'&#x78C1;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x78C2;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x78C3;'" select="'si1'"/>
                <xsl:map-entry key="'&#x78C4;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x78C5;'" select="'bang4	pang1	pang2'"/>
                <xsl:map-entry key="'&#x78C6;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x78C7;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x78C8;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x78C9;'" select="'sang3'"/>
                <xsl:map-entry key="'&#x78CA;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x78CB;'" select="'cuo1'"/>
                <xsl:map-entry key="'&#x78CC;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x78CD;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x78CE;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x78CF;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x78D0;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x78D1;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x78D2;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x78D3;'" select="'dui1'"/>
                <xsl:map-entry key="'&#x78D4;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x78D5;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x78D6;'" select="'la2'"/>
                <xsl:map-entry key="'&#x78D7;'" select="''"/>
                <xsl:map-entry key="'&#x78D8;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x78D9;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x78DA;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x78DB;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x78DC;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x78DD;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x78DE;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x78DF;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x78E0;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x78E1;'" select="'kan4'"/>
                <xsl:map-entry key="'&#x78E2;'" select="'qiang3'"/>
                <xsl:map-entry key="'&#x78E3;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x78E4;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x78E5;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x78E6;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x78E7;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x78E8;'" select="'mo2	mo4'"/>
                <xsl:map-entry key="'&#x78E9;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x78EA;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x78EB;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x78EC;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x78ED;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x78EE;'" select="''"/>
                <xsl:map-entry key="'&#x78EF;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x78F0;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x78F1;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x78F2;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x78F3;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x78F4;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x78F5;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x78F6;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x78F7;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x78F8;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x78F9;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x78FA;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x78FB;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x78FC;'" select="'za2'"/>
                <xsl:map-entry key="'&#x78FD;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x78FE;'" select="'di1'"/>
                <xsl:map-entry key="'&#x78FF;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7900;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x7901;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x7902;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7903;'" select="'zhang3'"/>
                <xsl:map-entry key="'&#x7904;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x7905;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x7906;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7907;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7908;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x7909;'" select="'he2'"/>
                <xsl:map-entry key="'&#x790A;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x790B;'" select="'zhai2'"/>
                <xsl:map-entry key="'&#x790C;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x790D;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x790E;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x790F;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x7910;'" select="'que4'"/>
                <xsl:map-entry key="'&#x7911;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x7912;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x7913;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x7914;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x7915;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x7916;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7917;'" select="'pin1'"/>
                <xsl:map-entry key="'&#x7918;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x7919;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x791A;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x791B;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x791C;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x791D;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x791E;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x791F;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x7920;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x7921;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x7922;'" select="''"/>
                <xsl:map-entry key="'&#x7923;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x7924;'" select="'ca3'"/>
                <xsl:map-entry key="'&#x7925;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x7926;'" select="'kuang4	gong3'"/>
                <xsl:map-entry key="'&#x7927;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x7928;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x7929;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x792A;'" select="'li4'"/>
                <xsl:map-entry key="'&#x792B;'" select="'li4'"/>
                <xsl:map-entry key="'&#x792C;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x792D;'" select="'que4'"/>
                <xsl:map-entry key="'&#x792E;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x792F;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7930;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7931;'" select="'long2'"/>
                <xsl:map-entry key="'&#x7932;'" select="'long2'"/>
                <xsl:map-entry key="'&#x7933;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x7934;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x7935;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x7936;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x7937;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x7938;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x7939;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x793A;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x793B;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x793C;'" select="'li3'"/>
                <xsl:map-entry key="'&#x793D;'" select="'reng2'"/>
                <xsl:map-entry key="'&#x793E;'" select="'she4'"/>
                <xsl:map-entry key="'&#x793F;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x7940;'" select="'si4'"/>
                <xsl:map-entry key="'&#x7941;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7942;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x7943;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x7944;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7945;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x7946;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x7947;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x7948;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7949;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x794A;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x794B;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x794C;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x794D;'" select="''"/>
                <xsl:map-entry key="'&#x794E;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x794F;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x7950;'" select="'you4'"/>
                <xsl:map-entry key="'&#x7951;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7952;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x7953;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7954;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x7955;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x7956;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x7957;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x7958;'" select="'suan4'"/>
                <xsl:map-entry key="'&#x7959;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x795A;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x795B;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x795C;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x795D;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x795E;'" select="'shen2'"/>
                <xsl:map-entry key="'&#x795F;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x7960;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x7961;'" select="'chai2'"/>
                <xsl:map-entry key="'&#x7962;'" select="'mi2	ni3'"/>
                <xsl:map-entry key="'&#x7963;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x7964;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x7965;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x7966;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x7967;'" select="'tiao1'"/>
                <xsl:map-entry key="'&#x7968;'" select="'piao4'"/>
                <xsl:map-entry key="'&#x7969;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x796A;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x796B;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x796C;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x796D;'" select="'ji4	zhai4'"/>
                <xsl:map-entry key="'&#x796E;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x796F;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x7970;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x7971;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x7972;'" select="'jin1	jin4'"/>
                <xsl:map-entry key="'&#x7973;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7974;'" select="'gai1	jie4'"/>
                <xsl:map-entry key="'&#x7975;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x7976;'" select="'di4'"/>
                <xsl:map-entry key="'&#x7977;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x7978;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x7979;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x797A;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x797B;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x797C;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x797D;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x797E;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x797F;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7980;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x7981;'" select="'jin4	jin1'"/>
                <xsl:map-entry key="'&#x7982;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x7983;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x7984;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7985;'" select="'shan4	chan2'"/>
                <xsl:map-entry key="'&#x7986;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x7987;'" select="'zhe3'"/>
                <xsl:map-entry key="'&#x7988;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x7989;'" select="'you3'"/>
                <xsl:map-entry key="'&#x798A;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x798B;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x798C;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x798D;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x798E;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x798F;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7990;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x7991;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x7992;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x7993;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x7994;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x7995;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x7996;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x7997;'" select="'si1'"/>
                <xsl:map-entry key="'&#x7998;'" select="'di4'"/>
                <xsl:map-entry key="'&#x7999;'" select="''"/>
                <xsl:map-entry key="'&#x799A;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x799B;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x799C;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x799D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x799E;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x799F;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x79A0;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x79A1;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x79A2;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x79A3;'" select="''"/>
                <xsl:map-entry key="'&#x79A4;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x79A5;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x79A6;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x79A7;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x79A8;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x79A9;'" select="'si4'"/>
                <xsl:map-entry key="'&#x79AA;'" select="'chan2	shan4'"/>
                <xsl:map-entry key="'&#x79AB;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x79AC;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x79AD;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x79AE;'" select="'li3'"/>
                <xsl:map-entry key="'&#x79AF;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x79B0;'" select="'ni3	mi2'"/>
                <xsl:map-entry key="'&#x79B1;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x79B2;'" select="'li4'"/>
                <xsl:map-entry key="'&#x79B3;'" select="'rang2	rang3'"/>
                <xsl:map-entry key="'&#x79B4;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x79B5;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x79B6;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x79B7;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x79B8;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x79B9;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x79BA;'" select="'yu2	yu4'"/>
                <xsl:map-entry key="'&#x79BB;'" select="'chi1	li2'"/>
                <xsl:map-entry key="'&#x79BC;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x79BD;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x79BE;'" select="'he2'"/>
                <xsl:map-entry key="'&#x79BF;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x79C0;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x79C1;'" select="'si1'"/>
                <xsl:map-entry key="'&#x79C2;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x79C3;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x79C4;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x79C5;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x79C6;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x79C7;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x79C8;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x79C9;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x79CA;'" select="'nian2'"/>
                <xsl:map-entry key="'&#x79CB;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x79CC;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x79CD;'" select="'chong2	zhong3	zhong4'"/>
                <xsl:map-entry key="'&#x79CE;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x79CF;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x79D0;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x79D1;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x79D2;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x79D3;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x79D4;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x79D5;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x79D6;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x79D7;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x79D8;'" select="'mi4	bi4	lin2'"/>
                <xsl:map-entry key="'&#x79D9;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x79DA;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x79DB;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x79DC;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x79DD;'" select="'li4'"/>
                <xsl:map-entry key="'&#x79DE;'" select="'you2'"/>
                <xsl:map-entry key="'&#x79DF;'" select="'zu1'"/>
                <xsl:map-entry key="'&#x79E0;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x79E1;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x79E2;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x79E3;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x79E4;'" select="'cheng4	chen4	cheng1'"/>
                <xsl:map-entry key="'&#x79E5;'" select="'nian2'"/>
                <xsl:map-entry key="'&#x79E6;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x79E7;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x79E8;'" select="'zuo2'"/>
                <xsl:map-entry key="'&#x79E9;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x79EA;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x79EB;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x79EC;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x79ED;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x79EE;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x79EF;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x79F0;'" select="'cheng1	chen4	cheng4'"/>
                <xsl:map-entry key="'&#x79F1;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x79F2;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x79F3;'" select="'huo2'"/>
                <xsl:map-entry key="'&#x79F4;'" select="'he2'"/>
                <xsl:map-entry key="'&#x79F5;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x79F6;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x79F7;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x79F8;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x79F9;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x79FA;'" select="'du4'"/>
                <xsl:map-entry key="'&#x79FB;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x79FC;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x79FD;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x79FE;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x79FF;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x7A00;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7A01;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x7A02;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x7A03;'" select="'fu1	fu2'"/>
                <xsl:map-entry key="'&#x7A04;'" select="'ze4'"/>
                <xsl:map-entry key="'&#x7A05;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x7A06;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x7A07;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x7A08;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x7A09;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x7A0A;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x7A0B;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x7A0C;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x7A0D;'" select="'shao1	shao4'"/>
                <xsl:map-entry key="'&#x7A0E;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x7A0F;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x7A10;'" select="'lun3'"/>
                <xsl:map-entry key="'&#x7A11;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7A12;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x7A13;'" select="'zuo2'"/>
                <xsl:map-entry key="'&#x7A14;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x7A15;'" select="'zhun4'"/>
                <xsl:map-entry key="'&#x7A16;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x7A17;'" select="'bai4	bi4'"/>
                <xsl:map-entry key="'&#x7A18;'" select="'ji1	qi2'"/>
                <xsl:map-entry key="'&#x7A19;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x7A1A;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7A1B;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x7A1C;'" select="'leng2'"/>
                <xsl:map-entry key="'&#x7A1D;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x7A1E;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x7A1F;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x7A20;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7A21;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x7A22;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7A23;'" select="'su1'"/>
                <xsl:map-entry key="'&#x7A24;'" select="'l��4'"/>
                <xsl:map-entry key="'&#x7A25;'" select="''"/>
                <xsl:map-entry key="'&#x7A26;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x7A27;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x7A28;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7A29;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7A2A;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x7A2B;'" select="'bi1'"/>
                <xsl:map-entry key="'&#x7A2C;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x7A2D;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x7A2E;'" select="'zhong3	chong2	zhong4'"/>
                <xsl:map-entry key="'&#x7A2F;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x7A30;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x7A31;'" select="'cheng1	chen4	cheng4'"/>
                <xsl:map-entry key="'&#x7A32;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x7A33;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x7A34;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x7A35;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x7A36;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7A37;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7A38;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7A39;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7A3A;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7A3B;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x7A3C;'" select="'jia4'"/>
                <xsl:map-entry key="'&#x7A3D;'" select="'ji1	qi3'"/>
                <xsl:map-entry key="'&#x7A3E;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x7A3F;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x7A40;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x7A41;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7A42;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x7A43;'" select="''"/>
                <xsl:map-entry key="'&#x7A44;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7A45;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x7A46;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x7A47;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x7A48;'" select="'men2'"/>
                <xsl:map-entry key="'&#x7A49;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7A4A;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7A4B;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7A4C;'" select="'su1	wei4'"/>
                <xsl:map-entry key="'&#x7A4D;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7A4E;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x7A4F;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x7A50;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x7A51;'" select="'se4'"/>
                <xsl:map-entry key="'&#x7A52;'" select="''"/>
                <xsl:map-entry key="'&#x7A53;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7A54;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x7A55;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x7A56;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x7A57;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x7A58;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7A59;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x7A5A;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x7A5B;'" select="'zhuo1'"/>
                <xsl:map-entry key="'&#x7A5C;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x7A5D;'" select="''"/>
                <xsl:map-entry key="'&#x7A5E;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x7A5F;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x7A60;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x7A61;'" select="'se4'"/>
                <xsl:map-entry key="'&#x7A62;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7A63;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x7A64;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x7A65;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7A66;'" select="'bin'"/>
                <xsl:map-entry key="'&#x7A67;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7A68;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x7A69;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x7A6A;'" select="'cheng1	chen4	cheng4'"/>
                <xsl:map-entry key="'&#x7A6B;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x7A6C;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x7A6D;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x7A6E;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x7A6F;'" select="''"/>
                <xsl:map-entry key="'&#x7A70;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x7A71;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7A72;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7A73;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x7A74;'" select="'xue4	xue2'"/>
                <xsl:map-entry key="'&#x7A75;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x7A76;'" select="'jiu4	jiu1'"/>
                <xsl:map-entry key="'&#x7A77;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x7A78;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7A79;'" select="'qiong1	qiong2'"/>
                <xsl:map-entry key="'&#x7A7A;'" select="'kong1	kong4'"/>
                <xsl:map-entry key="'&#x7A7B;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x7A7C;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x7A7D;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x7A7E;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x7A7F;'" select="'chuan1'"/>
                <xsl:map-entry key="'&#x7A80;'" select="'zhun1'"/>
                <xsl:map-entry key="'&#x7A81;'" select="'tu2	tu1'"/>
                <xsl:map-entry key="'&#x7A82;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x7A83;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x7A84;'" select="'zhai3	ze2'"/>
                <xsl:map-entry key="'&#x7A85;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x7A86;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x7A87;'" select="'bao2'"/>
                <xsl:map-entry key="'&#x7A88;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x7A89;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x7A8A;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x7A8B;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x7A8C;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x7A8D;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x7A8E;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x7A8F;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x7A90;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x7A91;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x7A92;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7A93;'" select="'chuan1'"/>
                <xsl:map-entry key="'&#x7A94;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x7A95;'" select="'tiao3'"/>
                <xsl:map-entry key="'&#x7A96;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x7A97;'" select="'chuang1'"/>
                <xsl:map-entry key="'&#x7A98;'" select="'jiong3	jun3'"/>
                <xsl:map-entry key="'&#x7A99;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7A9A;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x7A9B;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x7A9C;'" select="'cuan4'"/>
                <xsl:map-entry key="'&#x7A9D;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x7A9E;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x7A9F;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x7AA0;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x7AA1;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x7AA2;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7AA3;'" select="'cu4	su1'"/>
                <xsl:map-entry key="'&#x7AA4;'" select="'guan'"/>
                <xsl:map-entry key="'&#x7AA5;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x7AA6;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x7AA7;'" select="''"/>
                <xsl:map-entry key="'&#x7AA8;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x7AA9;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x7AAA;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x7AAB;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x7AAC;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x7AAD;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x7AAE;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x7AAF;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x7AB0;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x7AB1;'" select="'tiao4'"/>
                <xsl:map-entry key="'&#x7AB2;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x7AB3;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x7AB4;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x7AB5;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x7AB6;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x7AB7;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x7AB8;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7AB9;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x7ABA;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x7ABB;'" select="'chuang1'"/>
                <xsl:map-entry key="'&#x7ABC;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x7ABD;'" select="''"/>
                <xsl:map-entry key="'&#x7ABE;'" select="'kuan3'"/>
                <xsl:map-entry key="'&#x7ABF;'" select="'long2'"/>
                <xsl:map-entry key="'&#x7AC0;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x7AC1;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x7AC2;'" select="'piao2'"/>
                <xsl:map-entry key="'&#x7AC3;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x7AC4;'" select="'cuan4'"/>
                <xsl:map-entry key="'&#x7AC5;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x7AC6;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x7AC7;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x7AC8;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x7AC9;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x7ACA;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x7ACB;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7ACC;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x7ACD;'" select="'shi2gong1sheng1'"/>
                <xsl:map-entry key="'&#x7ACE;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x7ACF;'" select="'qian1gong1sheng1'"/>
                <xsl:map-entry key="'&#x7AD0;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x7AD1;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7AD2;'" select="'qi2	ji1'"/>
                <xsl:map-entry key="'&#x7AD3;'" select="'qian1fen1zhi1yi1gong1sheng1'"/>
                <xsl:map-entry key="'&#x7AD4;'" select="'gong1sheng1'"/>
                <xsl:map-entry key="'&#x7AD5;'" select="'shi2fen1zhi1yi1gong1sheng1'"/>
                <xsl:map-entry key="'&#x7AD6;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x7AD7;'" select="'miao4'"/>
                <xsl:map-entry key="'&#x7AD8;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x7AD9;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x7ADA;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x7ADB;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7ADC;'" select="'long2'"/>
                <xsl:map-entry key="'&#x7ADD;'" select="'bing4	bing1'"/>
                <xsl:map-entry key="'&#x7ADE;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x7ADF;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x7AE0;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x7AE1;'" select="'yi1gong1sheng1deyi1bai3bei4'"/>
                <xsl:map-entry key="'&#x7AE2;'" select="'si4	qi2'"/>
                <xsl:map-entry key="'&#x7AE3;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x7AE4;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7AE5;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x7AE6;'" select="'song3'"/>
                <xsl:map-entry key="'&#x7AE7;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x7AE8;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x7AE9;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7AEA;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x7AEB;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x7AEC;'" select="'qu3'"/>
                <xsl:map-entry key="'&#x7AED;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x7AEE;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x7AEF;'" select="'duan1'"/>
                <xsl:map-entry key="'&#x7AF0;'" select="'shao2'"/>
                <xsl:map-entry key="'&#x7AF1;'" select="'zhuan3'"/>
                <xsl:map-entry key="'&#x7AF2;'" select="'ceng2'"/>
                <xsl:map-entry key="'&#x7AF3;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x7AF4;'" select="'cun1'"/>
                <xsl:map-entry key="'&#x7AF5;'" select="'huai1'"/>
                <xsl:map-entry key="'&#x7AF6;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x7AF7;'" select="'kan4'"/>
                <xsl:map-entry key="'&#x7AF8;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x7AF9;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x7AFA;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x7AFB;'" select="'le4'"/>
                <xsl:map-entry key="'&#x7AFC;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x7AFD;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x7AFE;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x7AFF;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x7B00;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x7B01;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x7B02;'" select="''"/>
                <xsl:map-entry key="'&#x7B03;'" select="'du3'"/>
                <xsl:map-entry key="'&#x7B04;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7B05;'" select="'xiao2'"/>
                <xsl:map-entry key="'&#x7B06;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x7B07;'" select="'suan4'"/>
                <xsl:map-entry key="'&#x7B08;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x7B09;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7B0A;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x7B0B;'" select="'sun3'"/>
                <xsl:map-entry key="'&#x7B0C;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x7B0D;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x7B0E;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7B0F;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x7B10;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x7B11;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x7B12;'" select="'cen2'"/>
                <xsl:map-entry key="'&#x7B13;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x7B14;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x7B15;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7B16;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x7B17;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x7B18;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x7B19;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x7B1A;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x7B1B;'" select="'di2'"/>
                <xsl:map-entry key="'&#x7B1C;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x7B1D;'" select="'na4'"/>
                <xsl:map-entry key="'&#x7B1E;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x7B1F;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x7B20;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7B21;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x7B22;'" select="'min3'"/>
                <xsl:map-entry key="'&#x7B23;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x7B24;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x7B25;'" select="'si4'"/>
                <xsl:map-entry key="'&#x7B26;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7B27;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x7B28;'" select="'ben4'"/>
                <xsl:map-entry key="'&#x7B29;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x7B2A;'" select="'da2'"/>
                <xsl:map-entry key="'&#x7B2B;'" select="'zi3	yi2'"/>
                <xsl:map-entry key="'&#x7B2C;'" select="'di4'"/>
                <xsl:map-entry key="'&#x7B2D;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7B2E;'" select="'ze2	zuo2'"/>
                <xsl:map-entry key="'&#x7B2F;'" select="'nu2'"/>
                <xsl:map-entry key="'&#x7B30;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7B31;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x7B32;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x7B33;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x7B34;'" select="'ge3'"/>
                <xsl:map-entry key="'&#x7B35;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x7B36;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x7B37;'" select="'mao3'"/>
                <xsl:map-entry key="'&#x7B38;'" select="'po3'"/>
                <xsl:map-entry key="'&#x7B39;'" select="''"/>
                <xsl:map-entry key="'&#x7B3A;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7B3B;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x7B3C;'" select="'long2	long3'"/>
                <xsl:map-entry key="'&#x7B3D;'" select="''"/>
                <xsl:map-entry key="'&#x7B3E;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7B3F;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x7B40;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x7B41;'" select="'qu3	qu1'"/>
                <xsl:map-entry key="'&#x7B42;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x7B43;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x7B44;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x7B45;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x7B46;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x7B47;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x7B48;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x7B49;'" select="'deng3'"/>
                <xsl:map-entry key="'&#x7B4A;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x7B4B;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x7B4C;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x7B4D;'" select="'sun3'"/>
                <xsl:map-entry key="'&#x7B4E;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x7B4F;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x7B50;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x7B51;'" select="'zhu2	zhu4'"/>
                <xsl:map-entry key="'&#x7B52;'" select="'tong3'"/>
                <xsl:map-entry key="'&#x7B53;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7B54;'" select="'da2	da1'"/>
                <xsl:map-entry key="'&#x7B55;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x7B56;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x7B57;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x7B58;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x7B59;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x7B5A;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7B5B;'" select="'shai1'"/>
                <xsl:map-entry key="'&#x7B5C;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x7B5D;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x7B5E;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x7B5F;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7B60;'" select="'yun2	jun1'"/>
                <xsl:map-entry key="'&#x7B61;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x7B62;'" select="'pa2'"/>
                <xsl:map-entry key="'&#x7B63;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7B64;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x7B65;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x7B66;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x7B67;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7B68;'" select="'han2'"/>
                <xsl:map-entry key="'&#x7B69;'" select="'tong3'"/>
                <xsl:map-entry key="'&#x7B6A;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x7B6B;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7B6C;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x7B6D;'" select="'suan4'"/>
                <xsl:map-entry key="'&#x7B6E;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x7B6F;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x7B70;'" select="'zuo2'"/>
                <xsl:map-entry key="'&#x7B71;'" select="'xiao3'"/>
                <xsl:map-entry key="'&#x7B72;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x7B73;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x7B74;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x7B75;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x7B76;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x7B77;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x7B78;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x7B79;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7B7A;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x7B7B;'" select="'gang4'"/>
                <xsl:map-entry key="'&#x7B7C;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x7B7D;'" select="''"/>
                <xsl:map-entry key="'&#x7B7E;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x7B7F;'" select="'xiao3'"/>
                <xsl:map-entry key="'&#x7B80;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7B81;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x7B82;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x7B83;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x7B84;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7B85;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7B86;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7B87;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x7B88;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x7B89;'" select="'guai3'"/>
                <xsl:map-entry key="'&#x7B8A;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x7B8B;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7B8C;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x7B8D;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x7B8E;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x7B8F;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x7B90;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x7B91;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x7B92;'" select="'zhou3'"/>
                <xsl:map-entry key="'&#x7B93;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7B94;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x7B95;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7B96;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7B97;'" select="'suan4'"/>
                <xsl:map-entry key="'&#x7B98;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x7B99;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7B9A;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x7B9B;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x7B9C;'" select="'kong1'"/>
                <xsl:map-entry key="'&#x7B9D;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x7B9E;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x7B9F;'" select="'jun4	jun1'"/>
                <xsl:map-entry key="'&#x7BA0;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x7BA1;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x7BA2;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x7BA3;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x7BA4;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x7BA5;'" select="'bo3'"/>
                <xsl:map-entry key="'&#x7BA6;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x7BA7;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x7BA8;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x7BA9;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x7BAA;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x7BAB;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7BAC;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x7BAD;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x7BAE;'" select="'xuan'"/>
                <xsl:map-entry key="'&#x7BAF;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7BB0;'" select="'sun3'"/>
                <xsl:map-entry key="'&#x7BB1;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x7BB2;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x7BB3;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x7BB4;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x7BB5;'" select="'sheng3'"/>
                <xsl:map-entry key="'&#x7BB6;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x7BB7;'" select="'shi1	yi2'"/>
                <xsl:map-entry key="'&#x7BB8;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x7BB9;'" select="'yue1'"/>
                <xsl:map-entry key="'&#x7BBA;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x7BBB;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7BBC;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x7BBD;'" select="'dong3'"/>
                <xsl:map-entry key="'&#x7BBE;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x7BBF;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x7BC0;'" select="'jie2	jie1'"/>
                <xsl:map-entry key="'&#x7BC1;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x7BC2;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x7BC3;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x7BC4;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x7BC5;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x7BC6;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x7BC7;'" select="'pian1'"/>
                <xsl:map-entry key="'&#x7BC8;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x7BC9;'" select="'zhu2	zhu4'"/>
                <xsl:map-entry key="'&#x7BCA;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7BCB;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x7BCC;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x7BCD;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x7BCE;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x7BCF;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x7BD0;'" select="''"/>
                <xsl:map-entry key="'&#x7BD1;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x7BD2;'" select="''"/>
                <xsl:map-entry key="'&#x7BD3;'" select="'lou3'"/>
                <xsl:map-entry key="'&#x7BD4;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x7BD5;'" select="'he2'"/>
                <xsl:map-entry key="'&#x7BD6;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x7BD7;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x7BD8;'" select="'chou1'"/>
                <xsl:map-entry key="'&#x7BD9;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x7BDA;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x7BDB;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x7BDC;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x7BDD;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x7BDE;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x7BDF;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x7BE0;'" select="'xiao3'"/>
                <xsl:map-entry key="'&#x7BE1;'" select="'cuan4'"/>
                <xsl:map-entry key="'&#x7BE2;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x7BE3;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x7BE4;'" select="'du3'"/>
                <xsl:map-entry key="'&#x7BE5;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7BE6;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7BE7;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x7BE8;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x7BE9;'" select="'shai1'"/>
                <xsl:map-entry key="'&#x7BEA;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x7BEB;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x7BEC;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x7BED;'" select="'long2	long3'"/>
                <xsl:map-entry key="'&#x7BEE;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x7BEF;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7BF0;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x7BF1;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7BF2;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7BF3;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7BF4;'" select="'di2'"/>
                <xsl:map-entry key="'&#x7BF5;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x7BF6;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x7BF7;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x7BF8;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x7BF9;'" select="'cuan4'"/>
                <xsl:map-entry key="'&#x7BFA;'" select="'pai2'"/>
                <xsl:map-entry key="'&#x7BFB;'" select="'piao4'"/>
                <xsl:map-entry key="'&#x7BFC;'" select="'dou1'"/>
                <xsl:map-entry key="'&#x7BFD;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x7BFE;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x7BFF;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x7C00;'" select="'ze2	kui4'"/>
                <xsl:map-entry key="'&#x7C01;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x7C02;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x7C03;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x7C04;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x7C05;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x7C06;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x7C07;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x7C08;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x7C09;'" select="'chou4	zao4'"/>
                <xsl:map-entry key="'&#x7C0A;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7C0B;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x7C0C;'" select="'su4'"/>
                <xsl:map-entry key="'&#x7C0D;'" select="'lou3'"/>
                <xsl:map-entry key="'&#x7C0E;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x7C0F;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7C10;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x7C11;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x7C12;'" select="'cuan4'"/>
                <xsl:map-entry key="'&#x7C13;'" select="''"/>
                <xsl:map-entry key="'&#x7C14;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x7C15;'" select="'le4'"/>
                <xsl:map-entry key="'&#x7C16;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x7C17;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x7C18;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7C19;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x7C1A;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x7C1B;'" select="'shai1'"/>
                <xsl:map-entry key="'&#x7C1C;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x7C1D;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x7C1E;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x7C1F;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x7C20;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x7C21;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7C22;'" select="'min3'"/>
                <xsl:map-entry key="'&#x7C23;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x7C24;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x7C25;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x7C26;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x7C27;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x7C28;'" select="'sun3'"/>
                <xsl:map-entry key="'&#x7C29;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x7C2A;'" select="'zan1'"/>
                <xsl:map-entry key="'&#x7C2B;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7C2C;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7C2D;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x7C2E;'" select="'zan1'"/>
                <xsl:map-entry key="'&#x7C2F;'" select="''"/>
                <xsl:map-entry key="'&#x7C30;'" select="'pai2'"/>
                <xsl:map-entry key="'&#x7C31;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7C32;'" select="'pai2'"/>
                <xsl:map-entry key="'&#x7C33;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x7C34;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x7C35;'" select="'du4'"/>
                <xsl:map-entry key="'&#x7C36;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7C37;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x7C38;'" select="'bo4	bo3'"/>
                <xsl:map-entry key="'&#x7C39;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x7C3A;'" select="'sai4'"/>
                <xsl:map-entry key="'&#x7C3B;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x7C3C;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x7C3D;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x7C3E;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x7C3F;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x7C40;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7C41;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x7C42;'" select="''"/>
                <xsl:map-entry key="'&#x7C43;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x7C44;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x7C45;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x7C46;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x7C47;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x7C48;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x7C49;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x7C4A;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x7C4B;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x7C4C;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7C4D;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x7C4E;'" select="''"/>
                <xsl:map-entry key="'&#x7C4F;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7C50;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x7C51;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x7C52;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7C53;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x7C54;'" select="'sou3'"/>
                <xsl:map-entry key="'&#x7C55;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7C56;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x7C57;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x7C58;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x7C59;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7C5A;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x7C5B;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7C5C;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x7C5D;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x7C5E;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7C5F;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x7C60;'" select="'long2	long3'"/>
                <xsl:map-entry key="'&#x7C61;'" select="''"/>
                <xsl:map-entry key="'&#x7C62;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x7C63;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x7C64;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x7C65;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x7C66;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x7C67;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x7C68;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x7C69;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7C6A;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x7C6B;'" select="'zuan3'"/>
                <xsl:map-entry key="'&#x7C6C;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7C6D;'" select="'shai1'"/>
                <xsl:map-entry key="'&#x7C6E;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x7C6F;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x7C70;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x7C71;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x7C72;'" select="'xu1	yu1	yu4'"/>
                <xsl:map-entry key="'&#x7C73;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x7C74;'" select="'di2'"/>
                <xsl:map-entry key="'&#x7C75;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x7C76;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7C77;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x7C78;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7C79;'" select="'nu:3'"/>
                <xsl:map-entry key="'&#x7C7A;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x7C7B;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x7C7C;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x7C7D;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x7C7E;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x7C7F;'" select="'cun4'"/>
                <xsl:map-entry key="'&#x7C80;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x7C81;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x7C82;'" select="''"/>
                <xsl:map-entry key="'&#x7C83;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x7C84;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x7C85;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x7C86;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x7C87;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x7C88;'" select="'rou3'"/>
                <xsl:map-entry key="'&#x7C89;'" select="'fen3'"/>
                <xsl:map-entry key="'&#x7C8A;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7C8B;'" select="'cui4	sui4'"/>
                <xsl:map-entry key="'&#x7C8C;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x7C8D;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7C8E;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x7C8F;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x7C90;'" select="''"/>
                <xsl:map-entry key="'&#x7C91;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x7C92;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7C93;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x7C94;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x7C95;'" select="'po4'"/>
                <xsl:map-entry key="'&#x7C96;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x7C97;'" select="'cu1'"/>
                <xsl:map-entry key="'&#x7C98;'" select="'nian2	zhan1'"/>
                <xsl:map-entry key="'&#x7C99;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7C9A;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7C9B;'" select="'su4'"/>
                <xsl:map-entry key="'&#x7C9C;'" select="'tiao4'"/>
                <xsl:map-entry key="'&#x7C9D;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7C9E;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7C9F;'" select="'su4'"/>
                <xsl:map-entry key="'&#x7CA0;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7CA1;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x7CA2;'" select="'zi1	ci2'"/>
                <xsl:map-entry key="'&#x7CA3;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x7CA4;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x7CA5;'" select="'zhou1	yu4'"/>
                <xsl:map-entry key="'&#x7CA6;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7CA7;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x7CA8;'" select="'bai3'"/>
                <xsl:map-entry key="'&#x7CA9;'" select="''"/>
                <xsl:map-entry key="'&#x7CAA;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x7CAB;'" select="'mian4'"/>
                <xsl:map-entry key="'&#x7CAC;'" select="''"/>
                <xsl:map-entry key="'&#x7CAD;'" select="''"/>
                <xsl:map-entry key="'&#x7CAE;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x7CAF;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x7CB0;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7CB1;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x7CB2;'" select="'can4'"/>
                <xsl:map-entry key="'&#x7CB3;'" select="'geng1	jing1'"/>
                <xsl:map-entry key="'&#x7CB4;'" select="'li3'"/>
                <xsl:map-entry key="'&#x7CB5;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x7CB6;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x7CB7;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x7CB8;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7CB9;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x7CBA;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x7CBB;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x7CBC;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7CBD;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x7CBE;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x7CBF;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x7CC0;'" select="''"/>
                <xsl:map-entry key="'&#x7CC1;'" select="'san3	shen1'"/>
                <xsl:map-entry key="'&#x7CC2;'" select="'san3	shen1'"/>
                <xsl:map-entry key="'&#x7CC3;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x7CC4;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x7CC5;'" select="'rou3	rou2'"/>
                <xsl:map-entry key="'&#x7CC6;'" select="'mian4'"/>
                <xsl:map-entry key="'&#x7CC7;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x7CC8;'" select="'xu3	xu1'"/>
                <xsl:map-entry key="'&#x7CC9;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x7CCA;'" select="'hu2	hu1	hu4'"/>
                <xsl:map-entry key="'&#x7CCB;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x7CCC;'" select="'zan2	zan1'"/>
                <xsl:map-entry key="'&#x7CCD;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x7CCE;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7CCF;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7CD0;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7CD1;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x7CD2;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x7CD3;'" select="'gu3	yu4'"/>
                <xsl:map-entry key="'&#x7CD4;'" select="'xiu3'"/>
                <xsl:map-entry key="'&#x7CD5;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x7CD6;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x7CD7;'" select="'qiu3'"/>
                <xsl:map-entry key="'&#x7CD8;'" select="''"/>
                <xsl:map-entry key="'&#x7CD9;'" select="'cao1	cao4'"/>
                <xsl:map-entry key="'&#x7CDA;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x7CDB;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x7CDC;'" select="'mi2	mei2'"/>
                <xsl:map-entry key="'&#x7CDD;'" select="'san3	shen1'"/>
                <xsl:map-entry key="'&#x7CDE;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x7CDF;'" select="'zao1'"/>
                <xsl:map-entry key="'&#x7CE0;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x7CE1;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x7CE2;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x7CE3;'" select="'san3'"/>
                <xsl:map-entry key="'&#x7CE4;'" select="'san3'"/>
                <xsl:map-entry key="'&#x7CE5;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x7CE6;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x7CE7;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x7CE8;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x7CE9;'" select="'kuai1'"/>
                <xsl:map-entry key="'&#x7CEA;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x7CEB;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x7CEC;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x7CED;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x7CEE;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x7CEF;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x7CF0;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x7CF1;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x7CF2;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7CF3;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x7CF4;'" select="'di2'"/>
                <xsl:map-entry key="'&#x7CF5;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x7CF6;'" select="'tiao4'"/>
                <xsl:map-entry key="'&#x7CF7;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x7CF8;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x7CF9;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x7CFA;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x7CFB;'" select="'xi4	ji4'"/>
                <xsl:map-entry key="'&#x7CFC;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x7CFD;'" select="'zheng3'"/>
                <xsl:map-entry key="'&#x7CFE;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x7CFF;'" select="'you4'"/>
                <xsl:map-entry key="'&#x7D00;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7D01;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x7D02;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7D03;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x7D04;'" select="'yue1	yao1'"/>
                <xsl:map-entry key="'&#x7D05;'" select="'hong2	gong1'"/>
                <xsl:map-entry key="'&#x7D06;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x7D07;'" select="'he2	ge1'"/>
                <xsl:map-entry key="'&#x7D08;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x7D09;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x7D0A;'" select="'wen4	wen3'"/>
                <xsl:map-entry key="'&#x7D0B;'" select="'wen2	wen4'"/>
                <xsl:map-entry key="'&#x7D0C;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x7D0D;'" select="'na4'"/>
                <xsl:map-entry key="'&#x7D0E;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x7D0F;'" select="'tou3'"/>
                <xsl:map-entry key="'&#x7D10;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x7D11;'" select="'fou2'"/>
                <xsl:map-entry key="'&#x7D12;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x7D13;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x7D14;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x7D15;'" select="'pi2	pi1'"/>
                <xsl:map-entry key="'&#x7D16;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x7D17;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x7D18;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7D19;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x7D1A;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x7D1B;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x7D1C;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x7D1D;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x7D1E;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x7D1F;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x7D20;'" select="'su4'"/>
                <xsl:map-entry key="'&#x7D21;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x7D22;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x7D23;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x7D24;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x7D25;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x7D26;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x7D27;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x7D28;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7D29;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7D2A;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x7D2B;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x7D2C;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7D2D;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7D2E;'" select="'zha2	za1'"/>
                <xsl:map-entry key="'&#x7D2F;'" select="'lei4	lei2	lei3'"/>
                <xsl:map-entry key="'&#x7D30;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x7D31;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7D32;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7D33;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7D34;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x7D35;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x7D36;'" select="'qu3	qu1'"/>
                <xsl:map-entry key="'&#x7D37;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7D38;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x7D39;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x7D3A;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x7D3B;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x7D3C;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7D3D;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x7D3E;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7D3F;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x7D40;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x7D41;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x7D42;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x7D43;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x7D44;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x7D45;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x7D46;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x7D47;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x7D48;'" select="'pa4'"/>
                <xsl:map-entry key="'&#x7D49;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x7D4A;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x7D4B;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x7D4C;'" select="'jing1	jing4'"/>
                <xsl:map-entry key="'&#x7D4D;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x7D4E;'" select="'heng4	hang2'"/>
                <xsl:map-entry key="'&#x7D4F;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7D50;'" select="'jie2	jie1'"/>
                <xsl:map-entry key="'&#x7D51;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x7D52;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7D53;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x7D54;'" select="'bai3'"/>
                <xsl:map-entry key="'&#x7D55;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7D56;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x7D57;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x7D58;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x7D59;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x7D5A;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x7D5B;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x7D5C;'" select="'xie2	jie2'"/>
                <xsl:map-entry key="'&#x7D5D;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x7D5E;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x7D5F;'" select="'quan1'"/>
                <xsl:map-entry key="'&#x7D60;'" select="'gai3'"/>
                <xsl:map-entry key="'&#x7D61;'" select="'luo4	lao4'"/>
                <xsl:map-entry key="'&#x7D62;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x7D63;'" select="'beng1	ping3'"/>
                <xsl:map-entry key="'&#x7D64;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x7D65;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7D66;'" select="'gei3	ji3'"/>
                <xsl:map-entry key="'&#x7D67;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x7D68;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7D69;'" select="'tiao4'"/>
                <xsl:map-entry key="'&#x7D6A;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x7D6B;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x7D6C;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7D6D;'" select="'quan4'"/>
                <xsl:map-entry key="'&#x7D6E;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7D6F;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x7D70;'" select="'die2'"/>
                <xsl:map-entry key="'&#x7D71;'" select="'tong3'"/>
                <xsl:map-entry key="'&#x7D72;'" select="'si1'"/>
                <xsl:map-entry key="'&#x7D73;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x7D74;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x7D75;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7D76;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7D77;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x7D78;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7D79;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x7D7A;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x7D7B;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x7D7C;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7D7D;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x7D7E;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x7D7F;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x7D80;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x7D81;'" select="'bang3'"/>
                <xsl:map-entry key="'&#x7D82;'" select="'tong3'"/>
                <xsl:map-entry key="'&#x7D83;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7D84;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x7D85;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x7D86;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x7D87;'" select="'xiu3'"/>
                <xsl:map-entry key="'&#x7D88;'" select="'ti2	di4	ti4'"/>
                <xsl:map-entry key="'&#x7D89;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x7D8A;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x7D8B;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7D8C;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x7D8D;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7D8E;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x7D8F;'" select="'sui1	sui2'"/>
                <xsl:map-entry key="'&#x7D90;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x7D91;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x7D92;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7D93;'" select="'jing1	jing4'"/>
                <xsl:map-entry key="'&#x7D94;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x7D95;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x7D96;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x7D97;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x7D98;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x7D99;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7D9A;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7D9B;'" select="''"/>
                <xsl:map-entry key="'&#x7D9C;'" select="'zong4	zeng4	zong1'"/>
                <xsl:map-entry key="'&#x7D9D;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x7D9E;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x7D9F;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7DA0;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x7DA1;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x7DA2;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7DA3;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x7DA4;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x7DA5;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x7DA6;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7DA7;'" select="'zhun3'"/>
                <xsl:map-entry key="'&#x7DA8;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7DA9;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x7DAA;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x7DAB;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x7DAC;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x7DAD;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x7DAE;'" select="'qi3	qing4'"/>
                <xsl:map-entry key="'&#x7DAF;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x7DB0;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x7DB1;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x7DB2;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x7DB3;'" select="'beng1	beng3	beng4'"/>
                <xsl:map-entry key="'&#x7DB4;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x7DB5;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x7DB6;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x7DB7;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x7DB8;'" select="'lun2	guan1'"/>
                <xsl:map-entry key="'&#x7DB9;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x7DBA;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x7DBB;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x7DBC;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x7DBD;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x7DBE;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7DBF;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x7DC0;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x7DC1;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x7DC2;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x7DC3;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x7DC4;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x7DC5;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x7DC6;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7DC7;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x7DC8;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x7DC9;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x7DCA;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x7DCB;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x7DCC;'" select="'rui2'"/>
                <xsl:map-entry key="'&#x7DCD;'" select="'min2'"/>
                <xsl:map-entry key="'&#x7DCE;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7DCF;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x7DD0;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x7DD1;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x7DD2;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7DD3;'" select="'yingl'"/>
                <xsl:map-entry key="'&#x7DD4;'" select="'shang4'"/>
                <xsl:map-entry key="'&#x7DD5;'" select="''"/>
                <xsl:map-entry key="'&#x7DD6;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7DD7;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x7DD8;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7DD9;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x7DDA;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x7DDB;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x7DDC;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x7DDD;'" select="'ji1	qi1'"/>
                <xsl:map-entry key="'&#x7DDE;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x7DDF;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x7DE0;'" select="'di4'"/>
                <xsl:map-entry key="'&#x7DE1;'" select="'min2'"/>
                <xsl:map-entry key="'&#x7DE2;'" select="'miao2'"/>
                <xsl:map-entry key="'&#x7DE3;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7DE4;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7DE5;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x7DE6;'" select="'si1'"/>
                <xsl:map-entry key="'&#x7DE7;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x7DE8;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7DE9;'" select="'huan3'"/>
                <xsl:map-entry key="'&#x7DEA;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x7DEB;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x7DEC;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x7DED;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x7DEE;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x7DEF;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x7DF0;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x7DF1;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x7DF2;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x7DF3;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x7DF4;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x7DF5;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x7DF6;'" select="'bian4	pian2'"/>
                <xsl:map-entry key="'&#x7DF7;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x7DF8;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x7DF9;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x7DFA;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x7DFB;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7DFC;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x7DFD;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x7DFE;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x7DFF;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x7E00;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x7E01;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7E02;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x7E03;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x7E04;'" select="'sheng2'"/>
                <xsl:map-entry key="'&#x7E05;'" select="''"/>
                <xsl:map-entry key="'&#x7E06;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x7E07;'" select="''"/>
                <xsl:map-entry key="'&#x7E08;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x7E09;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x7E0A;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7E0B;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x7E0C;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x7E0D;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x7E0E;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x7E0F;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x7E10;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7E11;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7E12;'" select="'cuo3'"/>
                <xsl:map-entry key="'&#x7E13;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x7E14;'" select="'shuang3'"/>
                <xsl:map-entry key="'&#x7E15;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x7E16;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x7E17;'" select="'shuai1'"/>
                <xsl:map-entry key="'&#x7E18;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7E19;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7E1A;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x7E1B;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7E1C;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x7E1D;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7E1E;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x7E1F;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x7E20;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x7E21;'" select="'zai3'"/>
                <xsl:map-entry key="'&#x7E22;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x7E23;'" select="'xian4	xuan2'"/>
                <xsl:map-entry key="'&#x7E24;'" select="'su4'"/>
                <xsl:map-entry key="'&#x7E25;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7E26;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x7E27;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x7E28;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x7E29;'" select="'cai4'"/>
                <xsl:map-entry key="'&#x7E2A;'" select="'bi4	bie4'"/>
                <xsl:map-entry key="'&#x7E2B;'" select="'feng2	feng4'"/>
                <xsl:map-entry key="'&#x7E2C;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x7E2D;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7E2E;'" select="'suo1	su4'"/>
                <xsl:map-entry key="'&#x7E2F;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x7E30;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x7E31;'" select="'zong4	zong1'"/>
                <xsl:map-entry key="'&#x7E32;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x7E33;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x7E34;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x7E35;'" select="'man4'"/>
                <xsl:map-entry key="'&#x7E36;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x7E37;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x7E38;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x7E39;'" select="'piao3	piao1'"/>
                <xsl:map-entry key="'&#x7E3A;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x7E3B;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x7E3C;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x7E3D;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x7E3E;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7E3F;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x7E40;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x7E41;'" select="'fan2	po2'"/>
                <xsl:map-entry key="'&#x7E42;'" select="'shuai4'"/>
                <xsl:map-entry key="'&#x7E43;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x7E44;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x7E45;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x7E46;'" select="'mou2	miao4	miu4'"/>
                <xsl:map-entry key="'&#x7E47;'" select="'zhou4	yao2	you2'"/>
                <xsl:map-entry key="'&#x7E48;'" select="'qiang3'"/>
                <xsl:map-entry key="'&#x7E49;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x7E4A;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x7E4B;'" select="'xi4	ji4'"/>
                <xsl:map-entry key="'&#x7E4C;'" select="''"/>
                <xsl:map-entry key="'&#x7E4D;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x7E4E;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x7E4F;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x7E50;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7E51;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x7E52;'" select="'zeng1	zeng4'"/>
                <xsl:map-entry key="'&#x7E53;'" select="'zuo3'"/>
                <xsl:map-entry key="'&#x7E54;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x7E55;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x7E56;'" select="'san3'"/>
                <xsl:map-entry key="'&#x7E57;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7E58;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7E59;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x7E5A;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x7E5B;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x7E5C;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x7E5D;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x7E5E;'" select="'rao4	rao3'"/>
                <xsl:map-entry key="'&#x7E5F;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x7E60;'" select="'rui3'"/>
                <xsl:map-entry key="'&#x7E61;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x7E62;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7E63;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x7E64;'" select="'zuan3'"/>
                <xsl:map-entry key="'&#x7E65;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7E66;'" select="'qiang3'"/>
                <xsl:map-entry key="'&#x7E67;'" select="''"/>
                <xsl:map-entry key="'&#x7E68;'" select="'da2'"/>
                <xsl:map-entry key="'&#x7E69;'" select="'sheng2'"/>
                <xsl:map-entry key="'&#x7E6A;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7E6B;'" select="'xi4	ji4'"/>
                <xsl:map-entry key="'&#x7E6C;'" select="'se4'"/>
                <xsl:map-entry key="'&#x7E6D;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7E6E;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x7E6F;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x7E70;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x7E71;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x7E72;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x7E73;'" select="'jiao3	jia3	zhuo2'"/>
                <xsl:map-entry key="'&#x7E74;'" select="'bo4'"/>
                <xsl:map-entry key="'&#x7E75;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x7E76;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7E77;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x7E78;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x7E79;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7E7A;'" select="'shai3'"/>
                <xsl:map-entry key="'&#x7E7B;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x7E7C;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7E7D;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x7E7E;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x7E7F;'" select="'jian4	kan3'"/>
                <xsl:map-entry key="'&#x7E80;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x7E81;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x7E82;'" select="'zuan3'"/>
                <xsl:map-entry key="'&#x7E83;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7E84;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x7E85;'" select="'li4'"/>
                <xsl:map-entry key="'&#x7E86;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x7E87;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x7E88;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x7E89;'" select="'zuan3'"/>
                <xsl:map-entry key="'&#x7E8A;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x7E8B;'" select="'you1'"/>
                <xsl:map-entry key="'&#x7E8C;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7E8D;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x7E8E;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x7E8F;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x7E90;'" select="''"/>
                <xsl:map-entry key="'&#x7E91;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x7E92;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x7E93;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7E94;'" select="'cai2'"/>
                <xsl:map-entry key="'&#x7E95;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x7E96;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x7E97;'" select="'zui1'"/>
                <xsl:map-entry key="'&#x7E98;'" select="'zuan3'"/>
                <xsl:map-entry key="'&#x7E99;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x7E9A;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x7E9B;'" select="'dao4	du2'"/>
                <xsl:map-entry key="'&#x7E9C;'" select="'lan4	lan3'"/>
                <xsl:map-entry key="'&#x7E9D;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x7E9E;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x7E9F;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x7EA0;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x7EA1;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x7EA2;'" select="'hong2	gong1'"/>
                <xsl:map-entry key="'&#x7EA3;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7EA4;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x7EA5;'" select="'he2	ge1'"/>
                <xsl:map-entry key="'&#x7EA6;'" select="'yue1	yao1'"/>
                <xsl:map-entry key="'&#x7EA7;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x7EA8;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x7EA9;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x7EAA;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7EAB;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x7EAC;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x7EAD;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x7EAE;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7EAF;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x7EB0;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x7EB1;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x7EB2;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x7EB3;'" select="'na4'"/>
                <xsl:map-entry key="'&#x7EB4;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x7EB5;'" select="'zong4'"/>
                <xsl:map-entry key="'&#x7EB6;'" select="'lun2	guan1'"/>
                <xsl:map-entry key="'&#x7EB7;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x7EB8;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x7EB9;'" select="'wen2	wen4'"/>
                <xsl:map-entry key="'&#x7EBA;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x7EBB;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x7EBC;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x7EBD;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x7EBE;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x7EBF;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x7EC0;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x7EC1;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7EC2;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7EC3;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x7EC4;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x7EC5;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x7EC6;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x7EC7;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x7EC8;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x7EC9;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x7ECA;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x7ECB;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x7ECC;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x7ECD;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x7ECE;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7ECF;'" select="'jing1	jing4'"/>
                <xsl:map-entry key="'&#x7ED0;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x7ED1;'" select="'bang3'"/>
                <xsl:map-entry key="'&#x7ED2;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7ED3;'" select="'jie2	jie1'"/>
                <xsl:map-entry key="'&#x7ED4;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x7ED5;'" select="'rao4	rao3'"/>
                <xsl:map-entry key="'&#x7ED6;'" select="'die2'"/>
                <xsl:map-entry key="'&#x7ED7;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x7ED8;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7ED9;'" select="'gei3'"/>
                <xsl:map-entry key="'&#x7EDA;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x7EDB;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x7EDC;'" select="'luo4	lao4'"/>
                <xsl:map-entry key="'&#x7EDD;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x7EDE;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x7EDF;'" select="'tong3'"/>
                <xsl:map-entry key="'&#x7EE0;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x7EE1;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x7EE2;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x7EE3;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x7EE4;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x7EE5;'" select="'sui2'"/>
                <xsl:map-entry key="'&#x7EE6;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x7EE7;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7EE8;'" select="'ti2	di4	ti4'"/>
                <xsl:map-entry key="'&#x7EE9;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7EEA;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7EEB;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7EEC;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x7EED;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x7EEE;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x7EEF;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x7EF0;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x7EF1;'" select="'shang4'"/>
                <xsl:map-entry key="'&#x7EF2;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x7EF3;'" select="'sheng2'"/>
                <xsl:map-entry key="'&#x7EF4;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x7EF5;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x7EF6;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x7EF7;'" select="'beng1	beng3	beng4'"/>
                <xsl:map-entry key="'&#x7EF8;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x7EF9;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x7EFA;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x7EFB;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x7EFC;'" select="'zeng4	zong1'"/>
                <xsl:map-entry key="'&#x7EFD;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x7EFE;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x7EFF;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x7F00;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x7F01;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x7F02;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x7F03;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x7F04;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7F05;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x7F06;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x7F07;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x7F08;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x7F09;'" select="'ji1	qi1'"/>
                <xsl:map-entry key="'&#x7F0A;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x7F0B;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7F0C;'" select="'si1'"/>
                <xsl:map-entry key="'&#x7F0D;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x7F0E;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x7F0F;'" select="'bian4	pian2'"/>
                <xsl:map-entry key="'&#x7F10;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x7F11;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x7F12;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x7F13;'" select="'huan3'"/>
                <xsl:map-entry key="'&#x7F14;'" select="'di4'"/>
                <xsl:map-entry key="'&#x7F15;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x7F16;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x7F17;'" select="'min2'"/>
                <xsl:map-entry key="'&#x7F18;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7F19;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x7F1A;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x7F1B;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x7F1C;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x7F1D;'" select="'feng2	feng4'"/>
                <xsl:map-entry key="'&#x7F1E;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x7F1F;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x7F20;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x7F21;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7F22;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7F23;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x7F24;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x7F25;'" select="'piao3	piao1'"/>
                <xsl:map-entry key="'&#x7F26;'" select="'man4'"/>
                <xsl:map-entry key="'&#x7F27;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x7F28;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7F29;'" select="'suo1	su4'"/>
                <xsl:map-entry key="'&#x7F2A;'" select="'mou2	miao4	miu4'"/>
                <xsl:map-entry key="'&#x7F2B;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x7F2C;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x7F2D;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x7F2E;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x7F2F;'" select="'zeng1	zeng4'"/>
                <xsl:map-entry key="'&#x7F30;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x7F31;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x7F32;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x7F33;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x7F34;'" select="'jiao3	jia3	zhuo2'"/>
                <xsl:map-entry key="'&#x7F35;'" select="'zuan3'"/>
                <xsl:map-entry key="'&#x7F36;'" select="'fou3'"/>
                <xsl:map-entry key="'&#x7F37;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x7F38;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x7F39;'" select="'fou3'"/>
                <xsl:map-entry key="'&#x7F3A;'" select="'que1'"/>
                <xsl:map-entry key="'&#x7F3B;'" select="'fou3'"/>
                <xsl:map-entry key="'&#x7F3C;'" select="'que1'"/>
                <xsl:map-entry key="'&#x7F3D;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x7F3E;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x7F3F;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x7F40;'" select="''"/>
                <xsl:map-entry key="'&#x7F41;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x7F42;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7F43;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7F44;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x7F45;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x7F46;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x7F47;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x7F48;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x7F49;'" select="''"/>
                <xsl:map-entry key="'&#x7F4A;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x7F4B;'" select="'weng4'"/>
                <xsl:map-entry key="'&#x7F4C;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x7F4D;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x7F4E;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x7F4F;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x7F50;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x7F51;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x7F52;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x7F53;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x7F54;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x7F55;'" select="'han3'"/>
                <xsl:map-entry key="'&#x7F56;'" select="''"/>
                <xsl:map-entry key="'&#x7F57;'" select="'luo1	luo2'"/>
                <xsl:map-entry key="'&#x7F58;'" select="'fu2	fou2'"/>
                <xsl:map-entry key="'&#x7F59;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x7F5A;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x7F5B;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x7F5C;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x7F5D;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x7F5E;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x7F5F;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x7F60;'" select="'min2'"/>
                <xsl:map-entry key="'&#x7F61;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x7F62;'" select="'ba4	ba5'"/>
                <xsl:map-entry key="'&#x7F63;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x7F64;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x7F65;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x7F66;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x7F67;'" select="'lin2	sen1'"/>
                <xsl:map-entry key="'&#x7F68;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x7F69;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x7F6A;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x7F6B;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x7F6C;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x7F6D;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x7F6E;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7F6F;'" select="'an3'"/>
                <xsl:map-entry key="'&#x7F70;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x7F71;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x7F72;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x7F73;'" select="'si1'"/>
                <xsl:map-entry key="'&#x7F74;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x7F75;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x7F76;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x7F77;'" select="'ba4	ba5	pi2'"/>
                <xsl:map-entry key="'&#x7F78;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x7F79;'" select="'li2'"/>
                <xsl:map-entry key="'&#x7F7A;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x7F7B;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x7F7C;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x7F7D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x7F7E;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x7F7F;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x7F80;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x7F81;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7F82;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x7F83;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x7F84;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x7F85;'" select="'luo2	luo1'"/>
                <xsl:map-entry key="'&#x7F86;'" select="'pi2	biao1'"/>
                <xsl:map-entry key="'&#x7F87;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7F88;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x7F89;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x7F8A;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x7F8B;'" select="'mie1'"/>
                <xsl:map-entry key="'&#x7F8C;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x7F8D;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x7F8E;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x7F8F;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x7F90;'" select="'you3'"/>
                <xsl:map-entry key="'&#x7F91;'" select="'you3'"/>
                <xsl:map-entry key="'&#x7F92;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x7F93;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x7F94;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x7F95;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x7F96;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x7F97;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x7F98;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x7F99;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x7F9A;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7F9B;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7F9C;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x7F9D;'" select="'di1'"/>
                <xsl:map-entry key="'&#x7F9E;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x7F9F;'" select="'qiang3'"/>
                <xsl:map-entry key="'&#x7FA0;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x7FA1;'" select="'xian4	yi2'"/>
                <xsl:map-entry key="'&#x7FA2;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x7FA3;'" select="'qun2'"/>
                <xsl:map-entry key="'&#x7FA4;'" select="'qun2'"/>
                <xsl:map-entry key="'&#x7FA5;'" select="'qian1	qiang3'"/>
                <xsl:map-entry key="'&#x7FA6;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x7FA7;'" select="'zui1	suo1'"/>
                <xsl:map-entry key="'&#x7FA8;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x7FA9;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7FAA;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x7FAB;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x7FAC;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x7FAD;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x7FAE;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x7FAF;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x7FB0;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x7FB1;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x7FB2;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x7FB3;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x7FB4;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x7FB5;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x7FB6;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x7FB7;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x7FB8;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x7FB9;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x7FBA;'" select="'nou2'"/>
                <xsl:map-entry key="'&#x7FBB;'" select="'qiang4'"/>
                <xsl:map-entry key="'&#x7FBC;'" select="'chan4'"/>
                <xsl:map-entry key="'&#x7FBD;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x7FBE;'" select="'gong4'"/>
                <xsl:map-entry key="'&#x7FBF;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7FC0;'" select="'chong2	chong1'"/>
                <xsl:map-entry key="'&#x7FC1;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x7FC2;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x7FC3;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7FC4;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x7FC5;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x7FC6;'" select="'cui2'"/>
                <xsl:map-entry key="'&#x7FC7;'" select="'fu2	pei4'"/>
                <xsl:map-entry key="'&#x7FC8;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x7FC9;'" select="'pen3'"/>
                <xsl:map-entry key="'&#x7FCA;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7FCB;'" select="'la1'"/>
                <xsl:map-entry key="'&#x7FCC;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7FCD;'" select="'pi1	po1'"/>
                <xsl:map-entry key="'&#x7FCE;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x7FCF;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x7FD0;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x7FD1;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x7FD2;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x7FD3;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x7FD4;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x7FD5;'" select="'xi4	xi1'"/>
                <xsl:map-entry key="'&#x7FD6;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x7FD7;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x7FD8;'" select="'qiao2	qiao4'"/>
                <xsl:map-entry key="'&#x7FD9;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7FDA;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x7FDB;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x7FDC;'" select="'se4'"/>
                <xsl:map-entry key="'&#x7FDD;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x7FDE;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x7FDF;'" select="'di2	zhai2'"/>
                <xsl:map-entry key="'&#x7FE0;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x7FE1;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x7FE2;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x7FE3;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x7FE4;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x7FE5;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x7FE6;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x7FE7;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x7FE8;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x7FE9;'" select="'pian1'"/>
                <xsl:map-entry key="'&#x7FEA;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x7FEB;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x7FEC;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x7FED;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x7FEE;'" select="'he2'"/>
                <xsl:map-entry key="'&#x7FEF;'" select="'he4'"/>
                <xsl:map-entry key="'&#x7FF0;'" select="'han4'"/>
                <xsl:map-entry key="'&#x7FF1;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x7FF2;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x7FF3;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7FF4;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x7FF5;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x7FF6;'" select="''"/>
                <xsl:map-entry key="'&#x7FF7;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x7FF8;'" select="'pen3'"/>
                <xsl:map-entry key="'&#x7FF9;'" select="'qiao2	qiao4'"/>
                <xsl:map-entry key="'&#x7FFA;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x7FFB;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x7FFC;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x7FFD;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x7FFE;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x7FFF;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x8000;'" select="'yao4	yue4'"/>
                <xsl:map-entry key="'&#x8001;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x8002;'" select="''"/>
                <xsl:map-entry key="'&#x8003;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x8004;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x8005;'" select="'zhe3'"/>
                <xsl:map-entry key="'&#x8006;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8007;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x8008;'" select="'gou2'"/>
                <xsl:map-entry key="'&#x8009;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x800A;'" select="'die2'"/>
                <xsl:map-entry key="'&#x800B;'" select="'die2'"/>
                <xsl:map-entry key="'&#x800C;'" select="'er2'"/>
                <xsl:map-entry key="'&#x800D;'" select="'shua3'"/>
                <xsl:map-entry key="'&#x800E;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x800F;'" select="'er2'"/>
                <xsl:map-entry key="'&#x8010;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x8011;'" select="'zhuan1	duan1'"/>
                <xsl:map-entry key="'&#x8012;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x8013;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x8014;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x8015;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x8016;'" select="'chao4'"/>
                <xsl:map-entry key="'&#x8017;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x8018;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x8019;'" select="'pa2	ba4'"/>
                <xsl:map-entry key="'&#x801A;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x801B;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x801C;'" select="'si4'"/>
                <xsl:map-entry key="'&#x801D;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x801E;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x801F;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x8020;'" select="'huo1'"/>
                <xsl:map-entry key="'&#x8021;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x8022;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x8023;'" select="'lun3'"/>
                <xsl:map-entry key="'&#x8024;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x8025;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x8026;'" select="'ou3'"/>
                <xsl:map-entry key="'&#x8027;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x8028;'" select="'nou4'"/>
                <xsl:map-entry key="'&#x8029;'" select="'gou1	jiang3'"/>
                <xsl:map-entry key="'&#x802A;'" select="'pang3'"/>
                <xsl:map-entry key="'&#x802B;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x802C;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x802D;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x802E;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x802F;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x8030;'" select="'you1'"/>
                <xsl:map-entry key="'&#x8031;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x8032;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x8033;'" select="'er3'"/>
                <xsl:map-entry key="'&#x8034;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8035;'" select="'ting1	ding1'"/>
                <xsl:map-entry key="'&#x8036;'" select="'ye2	ye1'"/>
                <xsl:map-entry key="'&#x8037;'" select="'da1'"/>
                <xsl:map-entry key="'&#x8038;'" select="'song3'"/>
                <xsl:map-entry key="'&#x8039;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x803A;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x803B;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x803C;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x803D;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x803E;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x803F;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x8040;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x8041;'" select="''"/>
                <xsl:map-entry key="'&#x8042;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x8043;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x8044;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x8045;'" select="'che4'"/>
                <xsl:map-entry key="'&#x8046;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8047;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x8048;'" select="'you3'"/>
                <xsl:map-entry key="'&#x8049;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x804A;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x804B;'" select="'long2'"/>
                <xsl:map-entry key="'&#x804C;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x804D;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x804E;'" select="'tiao1'"/>
                <xsl:map-entry key="'&#x804F;'" select="'er2	er4'"/>
                <xsl:map-entry key="'&#x8050;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x8051;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8052;'" select="'gua1	guo1'"/>
                <xsl:map-entry key="'&#x8053;'" select="''"/>
                <xsl:map-entry key="'&#x8054;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8055;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x8056;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x8057;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x8058;'" select="'pin4'"/>
                <xsl:map-entry key="'&#x8059;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x805A;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x805B;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x805C;'" select="'di3'"/>
                <xsl:map-entry key="'&#x805D;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x805E;'" select="'wen2	wen4'"/>
                <xsl:map-entry key="'&#x805F;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x8060;'" select="'ping1'"/>
                <xsl:map-entry key="'&#x8061;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x8062;'" select="''"/>
                <xsl:map-entry key="'&#x8063;'" select="''"/>
                <xsl:map-entry key="'&#x8064;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x8065;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x8066;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x8067;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x8068;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8069;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x806A;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x806B;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x806C;'" select="'weng3'"/>
                <xsl:map-entry key="'&#x806D;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x806E;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x806F;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8070;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x8071;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x8072;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x8073;'" select="'song3'"/>
                <xsl:map-entry key="'&#x8074;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x8075;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x8076;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x8077;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x8078;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x8079;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x807A;'" select="'qie'"/>
                <xsl:map-entry key="'&#x807B;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x807C;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x807D;'" select="'ting1	ting4'"/>
                <xsl:map-entry key="'&#x807E;'" select="'long2'"/>
                <xsl:map-entry key="'&#x807F;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8080;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8081;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x8082;'" select="'si4'"/>
                <xsl:map-entry key="'&#x8083;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8084;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8085;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8086;'" select="'si4'"/>
                <xsl:map-entry key="'&#x8087;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x8088;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x8089;'" select="'rou4'"/>
                <xsl:map-entry key="'&#x808A;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x808B;'" select="'le4	le1	lei4'"/>
                <xsl:map-entry key="'&#x808C;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x808D;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x808E;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x808F;'" select="'cao4'"/>
                <xsl:map-entry key="'&#x8090;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x8091;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8092;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x8093;'" select="'huang1'"/>
                <xsl:map-entry key="'&#x8094;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x8095;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x8096;'" select="'xiao4	xiao1'"/>
                <xsl:map-entry key="'&#x8097;'" select="'ru3'"/>
                <xsl:map-entry key="'&#x8098;'" select="'zhou3'"/>
                <xsl:map-entry key="'&#x8099;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x809A;'" select="'du4	du3'"/>
                <xsl:map-entry key="'&#x809B;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x809C;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x809D;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x809E;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x809F;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x80A0;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x80A1;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x80A2;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x80A3;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x80A4;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x80A5;'" select="'fei2'"/>
                <xsl:map-entry key="'&#x80A6;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x80A7;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x80A8;'" select="'pang4'"/>
                <xsl:map-entry key="'&#x80A9;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x80AA;'" select="'fang2'"/>
                <xsl:map-entry key="'&#x80AB;'" select="'zhun1'"/>
                <xsl:map-entry key="'&#x80AC;'" select="'you2'"/>
                <xsl:map-entry key="'&#x80AD;'" select="'na4'"/>
                <xsl:map-entry key="'&#x80AE;'" select="'hang2	ang1'"/>
                <xsl:map-entry key="'&#x80AF;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x80B0;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x80B1;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x80B2;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x80B3;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x80B4;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x80B5;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x80B6;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x80B7;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x80B8;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x80B9;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x80BA;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x80BB;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x80BC;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x80BD;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x80BE;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x80BF;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x80C0;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x80C1;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x80C2;'" select="'shen1	shen4'"/>
                <xsl:map-entry key="'&#x80C3;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x80C4;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x80C5;'" select="'die2'"/>
                <xsl:map-entry key="'&#x80C6;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x80C7;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x80C8;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x80C9;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x80CA;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x80CB;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x80CC;'" select="'bei4	bei1'"/>
                <xsl:map-entry key="'&#x80CD;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x80CE;'" select="'tai1'"/>
                <xsl:map-entry key="'&#x80CF;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x80D0;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x80D1;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x80D2;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x80D3;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x80D4;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x80D5;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x80D6;'" select="'pang4	pan2'"/>
                <xsl:map-entry key="'&#x80D7;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x80D8;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x80D9;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x80DA;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x80DB;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x80DC;'" select="'sheng4	sheng1'"/>
                <xsl:map-entry key="'&#x80DD;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x80DE;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x80DF;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x80E0;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x80E1;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x80E2;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x80E3;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x80E4;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x80E5;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x80E6;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x80E7;'" select="'long2'"/>
                <xsl:map-entry key="'&#x80E8;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x80E9;'" select="'ka3'"/>
                <xsl:map-entry key="'&#x80EA;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x80EB;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x80EC;'" select="'nu3'"/>
                <xsl:map-entry key="'&#x80ED;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x80EE;'" select="'pang1'"/>
                <xsl:map-entry key="'&#x80EF;'" select="'kua4'"/>
                <xsl:map-entry key="'&#x80F0;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x80F1;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x80F2;'" select="'hai3'"/>
                <xsl:map-entry key="'&#x80F3;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x80F4;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x80F5;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x80F6;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x80F7;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x80F8;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x80F9;'" select="'er2'"/>
                <xsl:map-entry key="'&#x80FA;'" select="'e4'"/>
                <xsl:map-entry key="'&#x80FB;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x80FC;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x80FD;'" select="'neng2'"/>
                <xsl:map-entry key="'&#x80FE;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x80FF;'" select="'gui'"/>
                <xsl:map-entry key="'&#x8100;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x8101;'" select="'tiao4'"/>
                <xsl:map-entry key="'&#x8102;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x8103;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x8104;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x8105;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x8106;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x8107;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x8108;'" select="'mo4	mai4'"/>
                <xsl:map-entry key="'&#x8109;'" select="'mai4	mo4'"/>
                <xsl:map-entry key="'&#x810A;'" select="'ji2	ji3'"/>
                <xsl:map-entry key="'&#x810B;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x810C;'" select="''"/>
                <xsl:map-entry key="'&#x810D;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x810E;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x810F;'" select="'zang4'"/>
                <xsl:map-entry key="'&#x8110;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8111;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x8112;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x8113;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x8114;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x8115;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x8116;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x8117;'" select="'wen3'"/>
                <xsl:map-entry key="'&#x8118;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x8119;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x811A;'" select="'jiao3	jia3	jue2'"/>
                <xsl:map-entry key="'&#x811B;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x811C;'" select="'you3'"/>
                <xsl:map-entry key="'&#x811D;'" select="'heng1'"/>
                <xsl:map-entry key="'&#x811E;'" select="'cuo3'"/>
                <xsl:map-entry key="'&#x811F;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x8120;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x8121;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x8122;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x8123;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x8124;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x8125;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x8126;'" select="'de'"/>
                <xsl:map-entry key="'&#x8127;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x8128;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x8129;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x812A;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x812B;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x812C;'" select="'pao1'"/>
                <xsl:map-entry key="'&#x812D;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x812E;'" select="'nei3'"/>
                <xsl:map-entry key="'&#x812F;'" select="'fu3	pu2'"/>
                <xsl:map-entry key="'&#x8130;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x8131;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x8132;'" select="'niao4'"/>
                <xsl:map-entry key="'&#x8133;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x8134;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x8135;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x8136;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x8137;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8138;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x8139;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x813A;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x813B;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x813C;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x813D;'" select="'shui2'"/>
                <xsl:map-entry key="'&#x813E;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x813F;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x8140;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x8141;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x8142;'" select="'guo4'"/>
                <xsl:map-entry key="'&#x8143;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x8144;'" select="'chui2	zhui1'"/>
                <xsl:map-entry key="'&#x8145;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x8146;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x8147;'" select="'nei3'"/>
                <xsl:map-entry key="'&#x8148;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x8149;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x814A;'" select="'la4	xi1	xi2'"/>
                <xsl:map-entry key="'&#x814B;'" select="'yi4	ye4'"/>
                <xsl:map-entry key="'&#x814C;'" select="'a1	ang1	yan1'"/>
                <xsl:map-entry key="'&#x814D;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x814E;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x814F;'" select="'chuo4	duo2'"/>
                <xsl:map-entry key="'&#x8150;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x8151;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x8152;'" select="'ju1	qu2'"/>
                <xsl:map-entry key="'&#x8153;'" select="'fei2'"/>
                <xsl:map-entry key="'&#x8154;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x8155;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x8156;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x8157;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x8158;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x8159;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x815A;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x815B;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x815C;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x815D;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x815E;'" select="'zhuan4	dun4'"/>
                <xsl:map-entry key="'&#x815F;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8160;'" select="'cou4'"/>
                <xsl:map-entry key="'&#x8161;'" select="'gua1	luo2'"/>
                <xsl:map-entry key="'&#x8162;'" select="'ou3'"/>
                <xsl:map-entry key="'&#x8163;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8164;'" select="'an1'"/>
                <xsl:map-entry key="'&#x8165;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x8166;'" select="'nao3'"/>
                <xsl:map-entry key="'&#x8167;'" select="'yu2	shu4'"/>
                <xsl:map-entry key="'&#x8168;'" select="'shuan4'"/>
                <xsl:map-entry key="'&#x8169;'" select="'nan3'"/>
                <xsl:map-entry key="'&#x816A;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x816B;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x816C;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x816D;'" select="'e4'"/>
                <xsl:map-entry key="'&#x816E;'" select="'sai1'"/>
                <xsl:map-entry key="'&#x816F;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x8170;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x8171;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8172;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8173;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x8174;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8175;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x8176;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x8177;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8178;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x8179;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x817A;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x817B;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x817C;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x817D;'" select="'wa4'"/>
                <xsl:map-entry key="'&#x817E;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x817F;'" select="'tui3'"/>
                <xsl:map-entry key="'&#x8180;'" select="'bang3	pang1	pang2'"/>
                <xsl:map-entry key="'&#x8181;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x8182;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x8183;'" select="'wa4'"/>
                <xsl:map-entry key="'&#x8184;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x8185;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x8186;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8187;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x8188;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x8189;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x818A;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x818B;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x818C;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x818D;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x818E;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x818F;'" select="'gao1	gao4'"/>
                <xsl:map-entry key="'&#x8190;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x8191;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x8192;'" select="'ou'"/>
                <xsl:map-entry key="'&#x8193;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x8194;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8195;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x8196;'" select="'pang1'"/>
                <xsl:map-entry key="'&#x8197;'" select="'chuai2'"/>
                <xsl:map-entry key="'&#x8198;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x8199;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x819A;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x819B;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x819C;'" select="'mo4	mo2'"/>
                <xsl:map-entry key="'&#x819D;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x819E;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x819F;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x81A0;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x81A1;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x81A2;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x81A3;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x81A4;'" select="'xue3'"/>
                <xsl:map-entry key="'&#x81A5;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x81A6;'" select="'lian3	lin4'"/>
                <xsl:map-entry key="'&#x81A7;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x81A8;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x81A9;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x81AA;'" select="'chuai4'"/>
                <xsl:map-entry key="'&#x81AB;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x81AC;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x81AD;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x81AE;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x81AF;'" select="'teng1'"/>
                <xsl:map-entry key="'&#x81B0;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x81B1;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x81B2;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x81B3;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x81B4;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x81B5;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x81B6;'" select="'run4'"/>
                <xsl:map-entry key="'&#x81B7;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x81B8;'" select="'sui3'"/>
                <xsl:map-entry key="'&#x81B9;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x81BA;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x81BB;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x81BC;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x81BD;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x81BE;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x81BF;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x81C0;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x81C1;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x81C2;'" select="'bi4	bei4	bei5'"/>
                <xsl:map-entry key="'&#x81C3;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x81C4;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x81C5;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x81C6;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x81C7;'" select="'juan3'"/>
                <xsl:map-entry key="'&#x81C8;'" select="'la4	xi1'"/>
                <xsl:map-entry key="'&#x81C9;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x81CA;'" select="'sao1	sao4'"/>
                <xsl:map-entry key="'&#x81CB;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x81CC;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x81CD;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x81CE;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x81CF;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x81D0;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x81D1;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x81D2;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x81D3;'" select="'zang4'"/>
                <xsl:map-entry key="'&#x81D4;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x81D5;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x81D6;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x81D7;'" select="'kuan1'"/>
                <xsl:map-entry key="'&#x81D8;'" select="'la4	xi1'"/>
                <xsl:map-entry key="'&#x81D9;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x81DA;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x81DB;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x81DC;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x81DD;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x81DE;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x81DF;'" select="'zang4'"/>
                <xsl:map-entry key="'&#x81E0;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x81E1;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x81E2;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x81E3;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x81E4;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x81E5;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x81E6;'" select="'guang4	wang3'"/>
                <xsl:map-entry key="'&#x81E7;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x81E8;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x81E9;'" select="'guang4'"/>
                <xsl:map-entry key="'&#x81EA;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x81EB;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x81EC;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x81ED;'" select="'chou4	xiu4'"/>
                <xsl:map-entry key="'&#x81EE;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x81EF;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x81F0;'" select="'chou4	xiu4'"/>
                <xsl:map-entry key="'&#x81F1;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x81F2;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x81F3;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x81F4;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x81F5;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x81F6;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x81F7;'" select="'die2'"/>
                <xsl:map-entry key="'&#x81F8;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x81F9;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x81FA;'" select="'tai2	tai1'"/>
                <xsl:map-entry key="'&#x81FB;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x81FC;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x81FD;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x81FE;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x81FF;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x8200;'" select="'yao3	wai3'"/>
                <xsl:map-entry key="'&#x8201;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8202;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x8203;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x8204;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x8205;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x8206;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8207;'" select="'yu3	yu4'"/>
                <xsl:map-entry key="'&#x8208;'" select="'xing1	xing4'"/>
                <xsl:map-entry key="'&#x8209;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x820A;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x820B;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x820C;'" select="'she2'"/>
                <xsl:map-entry key="'&#x820D;'" select="'she4	she3'"/>
                <xsl:map-entry key="'&#x820E;'" select="'she3	she4'"/>
                <xsl:map-entry key="'&#x820F;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x8210;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8211;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x8212;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x8213;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8214;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x8215;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x8216;'" select="'pu4'"/>
                <xsl:map-entry key="'&#x8217;'" select="'pu4	pu1'"/>
                <xsl:map-entry key="'&#x8218;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x8219;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x821A;'" select="'tian4'"/>
                <xsl:map-entry key="'&#x821B;'" select="'chuan3'"/>
                <xsl:map-entry key="'&#x821C;'" select="'shun4'"/>
                <xsl:map-entry key="'&#x821D;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x821E;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x821F;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8220;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x8221;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x8222;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x8223;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x8224;'" select="''"/>
                <xsl:map-entry key="'&#x8225;'" select="'pa1'"/>
                <xsl:map-entry key="'&#x8226;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x8227;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x8228;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x8229;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x822A;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x822B;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x822C;'" select="'ban1	bo1	pan2'"/>
                <xsl:map-entry key="'&#x822D;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x822E;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x822F;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x8230;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8231;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x8232;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8233;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x8234;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8235;'" select="'duo4	tuo2'"/>
                <xsl:map-entry key="'&#x8236;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x8237;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8238;'" select="'ge3'"/>
                <xsl:map-entry key="'&#x8239;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x823A;'" select="'jia2	xia2'"/>
                <xsl:map-entry key="'&#x823B;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x823C;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x823D;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x823E;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x823F;'" select="''"/>
                <xsl:map-entry key="'&#x8240;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8241;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x8242;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x8243;'" select="'li2'"/>
                <xsl:map-entry key="'&#x8244;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x8245;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8246;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x8247;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x8248;'" select="''"/>
                <xsl:map-entry key="'&#x8249;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x824A;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x824B;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x824C;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x824D;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x824E;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x824F;'" select="'shou3'"/>
                <xsl:map-entry key="'&#x8250;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x8251;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8252;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x8253;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8254;'" select="''"/>
                <xsl:map-entry key="'&#x8255;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x8256;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x8257;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8258;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x8259;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x825A;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x825B;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x825C;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x825D;'" select="''"/>
                <xsl:map-entry key="'&#x825E;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x825F;'" select="'tong2	chong1'"/>
                <xsl:map-entry key="'&#x8260;'" select="''"/>
                <xsl:map-entry key="'&#x8261;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x8262;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x8263;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x8264;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x8265;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x8266;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8267;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x8268;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x8269;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x826A;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x826B;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x826C;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x826D;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x826E;'" select="'gen4	gen3'"/>
                <xsl:map-entry key="'&#x826F;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x8270;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x8271;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x8272;'" select="'se4	shai3'"/>
                <xsl:map-entry key="'&#x8273;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8274;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8275;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x8276;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8277;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8278;'" select="'cao3'"/>
                <xsl:map-entry key="'&#x8279;'" select="'cao3'"/>
                <xsl:map-entry key="'&#x827A;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x827B;'" select="'le4'"/>
                <xsl:map-entry key="'&#x827C;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x827D;'" select="'qiu2	jiao1'"/>
                <xsl:map-entry key="'&#x827E;'" select="'ai4	yi4'"/>
                <xsl:map-entry key="'&#x827F;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x8280;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x8281;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x8282;'" select="'jie2	jie1'"/>
                <xsl:map-entry key="'&#x8283;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x8284;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x8285;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8286;'" select="'chai1'"/>
                <xsl:map-entry key="'&#x8287;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x8288;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x8289;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x828A;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x828B;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x828C;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x828D;'" select="'shao2'"/>
                <xsl:map-entry key="'&#x828E;'" select="'qiong1	qiong2	xiong1'"/>
                <xsl:map-entry key="'&#x828F;'" select="'tu3	du4'"/>
                <xsl:map-entry key="'&#x8290;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x8291;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x8292;'" select="'mang2	wang2'"/>
                <xsl:map-entry key="'&#x8293;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x8294;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8295;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x8296;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8297;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x8298;'" select="'bi4	pi2'"/>
                <xsl:map-entry key="'&#x8299;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x829A;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x829B;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x829C;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x829D;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x829E;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x829F;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x82A0;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x82A1;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x82A2;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x82A3;'" select="'fou2'"/>
                <xsl:map-entry key="'&#x82A4;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x82A5;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x82A6;'" select="'lu2	lu3'"/>
                <xsl:map-entry key="'&#x82A7;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x82A8;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x82A9;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x82AA;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x82AB;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x82AC;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x82AD;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x82AE;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x82AF;'" select="'xin1	xin4'"/>
                <xsl:map-entry key="'&#x82B0;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x82B1;'" select="'hua1'"/>
                <xsl:map-entry key="'&#x82B2;'" select="'hua1'"/>
                <xsl:map-entry key="'&#x82B3;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x82B4;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x82B5;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x82B6;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x82B7;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x82B8;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x82B9;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x82BA;'" select="'ao3'"/>
                <xsl:map-entry key="'&#x82BB;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x82BC;'" select="'mao4	mao2'"/>
                <xsl:map-entry key="'&#x82BD;'" select="'ya2	di2'"/>
                <xsl:map-entry key="'&#x82BE;'" select="'fei4	fu2'"/>
                <xsl:map-entry key="'&#x82BF;'" select="'reng4'"/>
                <xsl:map-entry key="'&#x82C0;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x82C1;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x82C2;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x82C3;'" select="'you3'"/>
                <xsl:map-entry key="'&#x82C4;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x82C5;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x82C6;'" select="''"/>
                <xsl:map-entry key="'&#x82C7;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x82C8;'" select="'li4'"/>
                <xsl:map-entry key="'&#x82C9;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x82CA;'" select="'e4'"/>
                <xsl:map-entry key="'&#x82CB;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x82CC;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x82CD;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x82CE;'" select="'ning2	zhu4'"/>
                <xsl:map-entry key="'&#x82CF;'" select="'su1'"/>
                <xsl:map-entry key="'&#x82D0;'" select="'yi2	ti2'"/>
                <xsl:map-entry key="'&#x82D1;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x82D2;'" select="'ran3'"/>
                <xsl:map-entry key="'&#x82D3;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x82D4;'" select="'tai2	tai1'"/>
                <xsl:map-entry key="'&#x82D5;'" select="'tiao2	shao2'"/>
                <xsl:map-entry key="'&#x82D6;'" select="'di2'"/>
                <xsl:map-entry key="'&#x82D7;'" select="'miao2'"/>
                <xsl:map-entry key="'&#x82D8;'" select="'qing3'"/>
                <xsl:map-entry key="'&#x82D9;'" select="'li4'"/>
                <xsl:map-entry key="'&#x82DA;'" select="'rao2'"/>
                <xsl:map-entry key="'&#x82DB;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x82DC;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x82DD;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x82DE;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x82DF;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x82E0;'" select="'min2'"/>
                <xsl:map-entry key="'&#x82E1;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x82E2;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x82E3;'" select="'ju4	qu3'"/>
                <xsl:map-entry key="'&#x82E4;'" select="'pi3	pie3'"/>
                <xsl:map-entry key="'&#x82E5;'" select="'ruo4	re3'"/>
                <xsl:map-entry key="'&#x82E6;'" select="'ku3'"/>
                <xsl:map-entry key="'&#x82E7;'" select="'zhu4	ning2'"/>
                <xsl:map-entry key="'&#x82E8;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x82E9;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x82EA;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x82EB;'" select="'shan1	shan4'"/>
                <xsl:map-entry key="'&#x82EC;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x82ED;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x82EE;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x82EF;'" select="'ben3'"/>
                <xsl:map-entry key="'&#x82F0;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x82F1;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x82F2;'" select="'zha3'"/>
                <xsl:map-entry key="'&#x82F3;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x82F4;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x82F5;'" select="'die2'"/>
                <xsl:map-entry key="'&#x82F6;'" select="'nie2'"/>
                <xsl:map-entry key="'&#x82F7;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x82F8;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x82F9;'" select="'ping2	pin2'"/>
                <xsl:map-entry key="'&#x82FA;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x82FB;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x82FC;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x82FD;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x82FE;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x82FF;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8300;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8301;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x8302;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x8303;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x8304;'" select="'qie2	jia1'"/>
                <xsl:map-entry key="'&#x8305;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x8306;'" select="'mao3	mao2'"/>
                <xsl:map-entry key="'&#x8307;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x8308;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x8309;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x830A;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x830B;'" select="'di3'"/>
                <xsl:map-entry key="'&#x830C;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x830D;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x830E;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x830F;'" select="'long2'"/>
                <xsl:map-entry key="'&#x8310;'" select="''"/>
                <xsl:map-entry key="'&#x8311;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x8312;'" select="''"/>
                <xsl:map-entry key="'&#x8313;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x8314;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8315;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x8316;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x8317;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x8318;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8319;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x831A;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x831B;'" select="'gen4'"/>
                <xsl:map-entry key="'&#x831C;'" select="'qian4	xi1'"/>
                <xsl:map-entry key="'&#x831D;'" select="'chai3'"/>
                <xsl:map-entry key="'&#x831E;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x831F;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8320;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x8321;'" select="'zi4'"/>
                <xsl:map-entry key="'&#x8322;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x8323;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x8324;'" select="'duo1'"/>
                <xsl:map-entry key="'&#x8325;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x8326;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x8327;'" select="'chong2	jian3'"/>
                <xsl:map-entry key="'&#x8328;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x8329;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x832A;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x832B;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x832C;'" select="'chi2	cha2'"/>
                <xsl:map-entry key="'&#x832D;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x832E;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x832F;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8330;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8331;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x8332;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8333;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x8334;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x8335;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x8336;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x8337;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x8338;'" select="'rong2	rong3'"/>
                <xsl:map-entry key="'&#x8339;'" select="'ru2	ru4'"/>
                <xsl:map-entry key="'&#x833A;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x833B;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x833C;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x833D;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x833E;'" select="''"/>
                <xsl:map-entry key="'&#x833F;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x8340;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x8341;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x8342;'" select="'kua1'"/>
                <xsl:map-entry key="'&#x8343;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8344;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x8345;'" select="'da2'"/>
                <xsl:map-entry key="'&#x8346;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x8347;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x8348;'" select="'chuan3'"/>
                <xsl:map-entry key="'&#x8349;'" select="'cao3'"/>
                <xsl:map-entry key="'&#x834A;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x834B;'" select="'er2'"/>
                <xsl:map-entry key="'&#x834C;'" select="'an4'"/>
                <xsl:map-entry key="'&#x834D;'" select="'shou1'"/>
                <xsl:map-entry key="'&#x834E;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x834F;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x8350;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8351;'" select="'ti2	yi2'"/>
                <xsl:map-entry key="'&#x8352;'" select="'huang1'"/>
                <xsl:map-entry key="'&#x8353;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x8354;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8355;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x8356;'" select="'lao3	pei2'"/>
                <xsl:map-entry key="'&#x8357;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x8358;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x8359;'" select="'da2'"/>
                <xsl:map-entry key="'&#x835A;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x835B;'" select="'rao2'"/>
                <xsl:map-entry key="'&#x835C;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x835D;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x835E;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x835F;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8360;'" select="'qi2	ji4'"/>
                <xsl:map-entry key="'&#x8361;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x8362;'" select="''"/>
                <xsl:map-entry key="'&#x8363;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x8364;'" select="'hun1	xun1'"/>
                <xsl:map-entry key="'&#x8365;'" select="'ying2	xing2'"/>
                <xsl:map-entry key="'&#x8366;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x8367;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8368;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x8369;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x836A;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x836B;'" select="'yin4	yin1'"/>
                <xsl:map-entry key="'&#x836C;'" select="'mai3'"/>
                <xsl:map-entry key="'&#x836D;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x836E;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x836F;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x8370;'" select="'du4'"/>
                <xsl:map-entry key="'&#x8371;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8372;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x8373;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x8374;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x8375;'" select="'ren3'"/>
                <xsl:map-entry key="'&#x8376;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x8377;'" select="'he2	he4'"/>
                <xsl:map-entry key="'&#x8378;'" select="'bi2'"/>
                <xsl:map-entry key="'&#x8379;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x837A;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x837B;'" select="'di2'"/>
                <xsl:map-entry key="'&#x837C;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x837D;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x837E;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x837F;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x8380;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8381;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x8382;'" select="'bie2'"/>
                <xsl:map-entry key="'&#x8383;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8384;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x8385;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8386;'" select="'fu3	pu2'"/>
                <xsl:map-entry key="'&#x8387;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x8388;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x8389;'" select="'li4'"/>
                <xsl:map-entry key="'&#x838A;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x838B;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x838C;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x838D;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x838E;'" select="'sha1	suo1'"/>
                <xsl:map-entry key="'&#x838F;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x8390;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8391;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x8392;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x8393;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x8394;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x8395;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x8396;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x8397;'" select="'che1'"/>
                <xsl:map-entry key="'&#x8398;'" select="'xin1	shen1'"/>
                <xsl:map-entry key="'&#x8399;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x839A;'" select="'yan2	yan4'"/>
                <xsl:map-entry key="'&#x839B;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x839C;'" select="'you2'"/>
                <xsl:map-entry key="'&#x839D;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x839E;'" select="'guan1	guan3	wan1'"/>
                <xsl:map-entry key="'&#x839F;'" select="'han4'"/>
                <xsl:map-entry key="'&#x83A0;'" select="'you3'"/>
                <xsl:map-entry key="'&#x83A1;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x83A2;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x83A3;'" select="'wang4'"/>
                <xsl:map-entry key="'&#x83A4;'" select="'you2'"/>
                <xsl:map-entry key="'&#x83A5;'" select="'niu3	chou3'"/>
                <xsl:map-entry key="'&#x83A6;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x83A7;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x83A8;'" select="'lang4	liang2'"/>
                <xsl:map-entry key="'&#x83A9;'" select="'piao3'"/>
                <xsl:map-entry key="'&#x83AA;'" select="'e2'"/>
                <xsl:map-entry key="'&#x83AB;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x83AC;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x83AD;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x83AE;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x83AF;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x83B0;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x83B1;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x83B2;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x83B3;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x83B4;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x83B5;'" select="'tu2	tu4'"/>
                <xsl:map-entry key="'&#x83B6;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x83B7;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x83B8;'" select="'you2'"/>
                <xsl:map-entry key="'&#x83B9;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x83BA;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x83BB;'" select="''"/>
                <xsl:map-entry key="'&#x83BC;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x83BD;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x83BE;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x83BF;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x83C0;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x83C1;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x83C2;'" select="'di1'"/>
                <xsl:map-entry key="'&#x83C3;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x83C4;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x83C5;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x83C6;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x83C7;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x83C8;'" select="'la1'"/>
                <xsl:map-entry key="'&#x83C9;'" select="'lu4	lu:4'"/>
                <xsl:map-entry key="'&#x83CA;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x83CB;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x83CC;'" select="'jun4	jun1'"/>
                <xsl:map-entry key="'&#x83CD;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x83CE;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x83CF;'" select="'he2	ge1'"/>
                <xsl:map-entry key="'&#x83D0;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x83D1;'" select="'zai1'"/>
                <xsl:map-entry key="'&#x83D2;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x83D3;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x83D4;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x83D5;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x83D6;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x83D7;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x83D8;'" select="'song1'"/>
                <xsl:map-entry key="'&#x83D9;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x83DA;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x83DB;'" select="'men2'"/>
                <xsl:map-entry key="'&#x83DC;'" select="'cai4'"/>
                <xsl:map-entry key="'&#x83DD;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x83DE;'" select="'li2'"/>
                <xsl:map-entry key="'&#x83DF;'" select="'tu4	tu2'"/>
                <xsl:map-entry key="'&#x83E0;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x83E1;'" select="'han4'"/>
                <xsl:map-entry key="'&#x83E2;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x83E3;'" select="'qin4'"/>
                <xsl:map-entry key="'&#x83E4;'" select="'juan3'"/>
                <xsl:map-entry key="'&#x83E5;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x83E6;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x83E7;'" select="'di3'"/>
                <xsl:map-entry key="'&#x83E8;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x83E9;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x83EA;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x83EB;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x83EC;'" select="'zhao3'"/>
                <xsl:map-entry key="'&#x83ED;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x83EE;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x83EF;'" select="'hua2	hua4'"/>
                <xsl:map-entry key="'&#x83F0;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x83F1;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x83F2;'" select="'fei1	fei3'"/>
                <xsl:map-entry key="'&#x83F3;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x83F4;'" select="'an4'"/>
                <xsl:map-entry key="'&#x83F5;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x83F6;'" select="'beng3'"/>
                <xsl:map-entry key="'&#x83F7;'" select="'zhou3'"/>
                <xsl:map-entry key="'&#x83F8;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x83F9;'" select="'ju1	zu1'"/>
                <xsl:map-entry key="'&#x83FA;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x83FB;'" select="'lin3'"/>
                <xsl:map-entry key="'&#x83FC;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x83FD;'" select="'shu2	shu1'"/>
                <xsl:map-entry key="'&#x83FE;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x83FF;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x8400;'" select="'hu3'"/>
                <xsl:map-entry key="'&#x8401;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8402;'" select="'he2'"/>
                <xsl:map-entry key="'&#x8403;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x8404;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x8405;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x8406;'" select="'bei1	bi4'"/>
                <xsl:map-entry key="'&#x8407;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x8408;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x8409;'" select="'fei2'"/>
                <xsl:map-entry key="'&#x840A;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x840B;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x840C;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x840D;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x840E;'" select="'wei1	wei3'"/>
                <xsl:map-entry key="'&#x840F;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x8410;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x8411;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x8412;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x8413;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8414;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x8415;'" select="'qi2	ji4'"/>
                <xsl:map-entry key="'&#x8416;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x8417;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x8418;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x8419;'" select="''"/>
                <xsl:map-entry key="'&#x841A;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x841B;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x841C;'" select="'tie1'"/>
                <xsl:map-entry key="'&#x841D;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x841E;'" select="''"/>
                <xsl:map-entry key="'&#x841F;'" select="''"/>
                <xsl:map-entry key="'&#x8420;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x8421;'" select="''"/>
                <xsl:map-entry key="'&#x8422;'" select="''"/>
                <xsl:map-entry key="'&#x8423;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x8424;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8425;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8426;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8427;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x8428;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x8429;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x842A;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x842B;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x842C;'" select="'wan4	mo4'"/>
                <xsl:map-entry key="'&#x842D;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x842E;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x842F;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8430;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x8431;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8432;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8433;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x8434;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8435;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x8436;'" select="'chun3'"/>
                <xsl:map-entry key="'&#x8437;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x8438;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8439;'" select="'pian1	bian3'"/>
                <xsl:map-entry key="'&#x843A;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x843B;'" select="'an1'"/>
                <xsl:map-entry key="'&#x843C;'" select="'e4'"/>
                <xsl:map-entry key="'&#x843D;'" select="'luo4	la4	lao4'"/>
                <xsl:map-entry key="'&#x843E;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x843F;'" select="'huo2'"/>
                <xsl:map-entry key="'&#x8440;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x8441;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x8442;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x8443;'" select="'zuo2'"/>
                <xsl:map-entry key="'&#x8444;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x8445;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x8446;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x8447;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x8448;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x8449;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x844A;'" select="'an1'"/>
                <xsl:map-entry key="'&#x844B;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x844C;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x844D;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x844E;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x844F;'" select="'ly4'"/>
                <xsl:map-entry key="'&#x8450;'" select="'pen2'"/>
                <xsl:map-entry key="'&#x8451;'" select="'feng1	feng4'"/>
                <xsl:map-entry key="'&#x8452;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x8453;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x8454;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x8455;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x8456;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x8457;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x8458;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8459;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x845A;'" select="'shen4	ren4'"/>
                <xsl:map-entry key="'&#x845B;'" select="'ge3'"/>
                <xsl:map-entry key="'&#x845C;'" select="'qia1'"/>
                <xsl:map-entry key="'&#x845D;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x845E;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x845F;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x8460;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x8461;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x8462;'" select="'ge3'"/>
                <xsl:map-entry key="'&#x8463;'" select="'dong3'"/>
                <xsl:map-entry key="'&#x8464;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x8465;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x8466;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8467;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x8468;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x8469;'" select="'pa1'"/>
                <xsl:map-entry key="'&#x846A;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x846B;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x846C;'" select="'zang4'"/>
                <xsl:map-entry key="'&#x846D;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x846E;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x846F;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x8470;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x8471;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x8472;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8473;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x8474;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8475;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x8476;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x8477;'" select="'hun1	xun1'"/>
                <xsl:map-entry key="'&#x8478;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x8479;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x847A;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x847B;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x847C;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x847D;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x847E;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x847F;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x8480;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x8481;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x8482;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8483;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x8484;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x8485;'" select="''"/>
                <xsl:map-entry key="'&#x8486;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x8487;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x8488;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x8489;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x848A;'" select="''"/>
                <xsl:map-entry key="'&#x848B;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x848C;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x848D;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x848E;'" select="'pai4'"/>
                <xsl:map-entry key="'&#x848F;'" select="''"/>
                <xsl:map-entry key="'&#x8490;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x8491;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x8492;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x8493;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x8494;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8495;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x8496;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x8497;'" select="'lang4'"/>
                <xsl:map-entry key="'&#x8498;'" select="'nu2'"/>
                <xsl:map-entry key="'&#x8499;'" select="'meng2	meng1	meng3'"/>
                <xsl:map-entry key="'&#x849A;'" select="'he2'"/>
                <xsl:map-entry key="'&#x849B;'" select="'que1'"/>
                <xsl:map-entry key="'&#x849C;'" select="'suan4'"/>
                <xsl:map-entry key="'&#x849D;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x849E;'" select="'li4'"/>
                <xsl:map-entry key="'&#x849F;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x84A0;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x84A1;'" select="'pang2	bang4'"/>
                <xsl:map-entry key="'&#x84A2;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x84A3;'" select="'xu2'"/>
                <xsl:map-entry key="'&#x84A4;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x84A5;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x84A6;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x84A7;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x84A8;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x84A9;'" select="'zu1'"/>
                <xsl:map-entry key="'&#x84AA;'" select="'po4'"/>
                <xsl:map-entry key="'&#x84AB;'" select="'cuo1'"/>
                <xsl:map-entry key="'&#x84AC;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x84AD;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x84AE;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x84AF;'" select="'kuai3'"/>
                <xsl:map-entry key="'&#x84B0;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x84B1;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x84B2;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x84B3;'" select="'na4'"/>
                <xsl:map-entry key="'&#x84B4;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x84B5;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x84B6;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x84B7;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x84B8;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x84B9;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x84BA;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x84BB;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x84BC;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x84BD;'" select="'en1'"/>
                <xsl:map-entry key="'&#x84BE;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x84BF;'" select="'hao1'"/>
                <xsl:map-entry key="'&#x84C0;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x84C1;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x84C2;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x84C3;'" select="'sou'"/>
                <xsl:map-entry key="'&#x84C4;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x84C5;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x84C6;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x84C7;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x84C8;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x84C9;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x84CA;'" select="'weng3'"/>
                <xsl:map-entry key="'&#x84CB;'" select="'gai4	ge3	he2'"/>
                <xsl:map-entry key="'&#x84CC;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x84CD;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x84CE;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x84CF;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x84D0;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x84D1;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x84D2;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x84D3;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x84D4;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x84D5;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x84D6;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x84D7;'" select="'zong3'"/>
                <xsl:map-entry key="'&#x84D8;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x84D9;'" select="''"/>
                <xsl:map-entry key="'&#x84DA;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x84DB;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x84DC;'" select="''"/>
                <xsl:map-entry key="'&#x84DD;'" select="'lan2	la5'"/>
                <xsl:map-entry key="'&#x84DE;'" select="''"/>
                <xsl:map-entry key="'&#x84DF;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x84E0;'" select="'li2'"/>
                <xsl:map-entry key="'&#x84E1;'" select="'can1'"/>
                <xsl:map-entry key="'&#x84E2;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x84E3;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x84E4;'" select="''"/>
                <xsl:map-entry key="'&#x84E5;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x84E6;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x84E7;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x84E8;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x84E9;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x84EA;'" select="'tong1'"/>
                <xsl:map-entry key="'&#x84EB;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x84EC;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x84ED;'" select="'an1'"/>
                <xsl:map-entry key="'&#x84EE;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x84EF;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x84F0;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x84F1;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x84F2;'" select="'qiu1	ou1'"/>
                <xsl:map-entry key="'&#x84F3;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x84F4;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x84F5;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x84F6;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x84F7;'" select="'tui1'"/>
                <xsl:map-entry key="'&#x84F8;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x84F9;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x84FA;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x84FB;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x84FC;'" select="'liao3'"/>
                <xsl:map-entry key="'&#x84FD;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x84FE;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x84FF;'" select="'xu5'"/>
                <xsl:map-entry key="'&#x8500;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x8501;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x8502;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x8503;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x8504;'" select="'man4'"/>
                <xsl:map-entry key="'&#x8505;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x8506;'" select="'leng2'"/>
                <xsl:map-entry key="'&#x8507;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x8508;'" select="'biao1	piao4'"/>
                <xsl:map-entry key="'&#x8509;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x850A;'" select="'han3'"/>
                <xsl:map-entry key="'&#x850B;'" select="'di2'"/>
                <xsl:map-entry key="'&#x850C;'" select="'su4'"/>
                <xsl:map-entry key="'&#x850D;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x850E;'" select="'she4'"/>
                <xsl:map-entry key="'&#x850F;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x8510;'" select="'di2'"/>
                <xsl:map-entry key="'&#x8511;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x8512;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x8513;'" select="'man4	man2	wan4'"/>
                <xsl:map-entry key="'&#x8514;'" select="'bo5	bu3'"/>
                <xsl:map-entry key="'&#x8515;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8516;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x8517;'" select="'zhe4'"/>
                <xsl:map-entry key="'&#x8518;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x8519;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x851A;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x851B;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x851C;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x851D;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x851E;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x851F;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x8520;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x8521;'" select="'cai4'"/>
                <xsl:map-entry key="'&#x8522;'" select="'po2'"/>
                <xsl:map-entry key="'&#x8523;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x8524;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x8525;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x8526;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x8527;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8528;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x8529;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x852A;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x852B;'" select="'nian1'"/>
                <xsl:map-entry key="'&#x852C;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x852D;'" select="'yin4	yin1'"/>
                <xsl:map-entry key="'&#x852E;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x852F;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8530;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x8531;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x8532;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x8533;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x8534;'" select="'ma2	ma1'"/>
                <xsl:map-entry key="'&#x8535;'" select="'cang2	zang4'"/>
                <xsl:map-entry key="'&#x8536;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8537;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x8538;'" select="'dou1'"/>
                <xsl:map-entry key="'&#x8539;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x853A;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x853B;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x853C;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x853D;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x853E;'" select="'li2'"/>
                <xsl:map-entry key="'&#x853F;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8540;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x8541;'" select="'xun2	qian2'"/>
                <xsl:map-entry key="'&#x8542;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x8543;'" select="'fan2	fan1'"/>
                <xsl:map-entry key="'&#x8544;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x8545;'" select="'ou3'"/>
                <xsl:map-entry key="'&#x8546;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x8547;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x8548;'" select="'xun4	jun4'"/>
                <xsl:map-entry key="'&#x8549;'" select="'jiao1	qiao2'"/>
                <xsl:map-entry key="'&#x854A;'" select="'rui3'"/>
                <xsl:map-entry key="'&#x854B;'" select="'rui3'"/>
                <xsl:map-entry key="'&#x854C;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x854D;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x854E;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x854F;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x8550;'" select="'hua2	hua1	hua4'"/>
                <xsl:map-entry key="'&#x8551;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x8552;'" select="'mai3'"/>
                <xsl:map-entry key="'&#x8553;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x8554;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x8555;'" select="'you2'"/>
                <xsl:map-entry key="'&#x8556;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x8557;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8558;'" select="'rao2'"/>
                <xsl:map-entry key="'&#x8559;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x855A;'" select="'e4'"/>
                <xsl:map-entry key="'&#x855B;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x855C;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x855D;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x855E;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x855F;'" select="'fa4'"/>
                <xsl:map-entry key="'&#x8560;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x8561;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x8562;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x8563;'" select="'shun4'"/>
                <xsl:map-entry key="'&#x8564;'" select="'rui2'"/>
                <xsl:map-entry key="'&#x8565;'" select="'ya3'"/>
                <xsl:map-entry key="'&#x8566;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x8567;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8568;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8569;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x856A;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x856B;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x856C;'" select="'si1'"/>
                <xsl:map-entry key="'&#x856D;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x856E;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x856F;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x8570;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x8571;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x8572;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8573;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x8574;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x8575;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x8576;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8577;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8578;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x8579;'" select="'yong1	weng4'"/>
                <xsl:map-entry key="'&#x857A;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x857B;'" select="'hong4	hong2'"/>
                <xsl:map-entry key="'&#x857C;'" select="'si4'"/>
                <xsl:map-entry key="'&#x857D;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x857E;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x857F;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8580;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x8581;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8582;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x8583;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x8584;'" select="'bo2	bao2	bo4'"/>
                <xsl:map-entry key="'&#x8585;'" select="'hao1'"/>
                <xsl:map-entry key="'&#x8586;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x8587;'" select="'wei2	wei1'"/>
                <xsl:map-entry key="'&#x8588;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8589;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x858A;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x858B;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x858C;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x858D;'" select="'luan4'"/>
                <xsl:map-entry key="'&#x858E;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x858F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8590;'" select="'leng2'"/>
                <xsl:map-entry key="'&#x8591;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x8592;'" select="'can4'"/>
                <xsl:map-entry key="'&#x8593;'" select="'shen1	can1	cen1'"/>
                <xsl:map-entry key="'&#x8594;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x8595;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8596;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x8597;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x8598;'" select="'da2'"/>
                <xsl:map-entry key="'&#x8599;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x859A;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x859B;'" select="'xue1'"/>
                <xsl:map-entry key="'&#x859C;'" select="'bi4	bo4'"/>
                <xsl:map-entry key="'&#x859D;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x859E;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x859F;'" select="'lian4	xian1'"/>
                <xsl:map-entry key="'&#x85A0;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x85A1;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x85A2;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x85A3;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x85A4;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x85A5;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x85A6;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x85A7;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x85A8;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x85A9;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x85AA;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x85AB;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x85AC;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x85AD;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x85AE;'" select="'sou3'"/>
                <xsl:map-entry key="'&#x85AF;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x85B0;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x85B1;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x85B2;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x85B3;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x85B4;'" select="'neng2'"/>
                <xsl:map-entry key="'&#x85B5;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x85B6;'" select="'mai2'"/>
                <xsl:map-entry key="'&#x85B7;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x85B8;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x85B9;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x85BA;'" select="'qi2	ji4'"/>
                <xsl:map-entry key="'&#x85BB;'" select="'zao3'"/>
                <xsl:map-entry key="'&#x85BC;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x85BD;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x85BE;'" select="'er3'"/>
                <xsl:map-entry key="'&#x85BF;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x85C0;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x85C1;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x85C2;'" select="'cong4'"/>
                <xsl:map-entry key="'&#x85C3;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x85C4;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x85C5;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x85C6;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x85C7;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x85C8;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x85C9;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x85CA;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x85CB;'" select="'di2'"/>
                <xsl:map-entry key="'&#x85CC;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x85CD;'" select="'lan2	la5'"/>
                <xsl:map-entry key="'&#x85CE;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x85CF;'" select="'cang2	zang4'"/>
                <xsl:map-entry key="'&#x85D0;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x85D1;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x85D2;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x85D3;'" select="'li4'"/>
                <xsl:map-entry key="'&#x85D4;'" select="''"/>
                <xsl:map-entry key="'&#x85D5;'" select="'ou3'"/>
                <xsl:map-entry key="'&#x85D6;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x85D7;'" select="'su4'"/>
                <xsl:map-entry key="'&#x85D8;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x85D9;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x85DA;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x85DB;'" select="'xie3'"/>
                <xsl:map-entry key="'&#x85DC;'" select="'li2'"/>
                <xsl:map-entry key="'&#x85DD;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x85DE;'" select="'la3'"/>
                <xsl:map-entry key="'&#x85DF;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x85E0;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x85E1;'" select="'di2'"/>
                <xsl:map-entry key="'&#x85E2;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x85E3;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x85E4;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x85E5;'" select="'yao4	yue4'"/>
                <xsl:map-entry key="'&#x85E6;'" select="'mo4	mo2'"/>
                <xsl:map-entry key="'&#x85E7;'" select="'huan3'"/>
                <xsl:map-entry key="'&#x85E8;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x85E9;'" select="'fan2	fan1'"/>
                <xsl:map-entry key="'&#x85EA;'" select="'sou3'"/>
                <xsl:map-entry key="'&#x85EB;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x85EC;'" select="'tui1'"/>
                <xsl:map-entry key="'&#x85ED;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x85EE;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x85EF;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x85F0;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x85F1;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x85F2;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x85F3;'" select="'gao3'"/>
                <xsl:map-entry key="'&#x85F4;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x85F5;'" select="''"/>
                <xsl:map-entry key="'&#x85F6;'" select="'li4'"/>
                <xsl:map-entry key="'&#x85F7;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x85F8;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x85F9;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x85FA;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x85FB;'" select="'zao3'"/>
                <xsl:map-entry key="'&#x85FC;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x85FD;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x85FE;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x85FF;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x8600;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x8601;'" select="'wu4	e4'"/>
                <xsl:map-entry key="'&#x8602;'" select="'rui3'"/>
                <xsl:map-entry key="'&#x8603;'" select="'rui3'"/>
                <xsl:map-entry key="'&#x8604;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8605;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x8606;'" select="'lu2	lu3'"/>
                <xsl:map-entry key="'&#x8607;'" select="'su1'"/>
                <xsl:map-entry key="'&#x8608;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x8609;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x860A;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x860B;'" select="'pin2	ping2'"/>
                <xsl:map-entry key="'&#x860C;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x860D;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x860E;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x860F;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x8610;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8611;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x8612;'" select="''"/>
                <xsl:map-entry key="'&#x8613;'" select="'su1'"/>
                <xsl:map-entry key="'&#x8614;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x8615;'" select="''"/>
                <xsl:map-entry key="'&#x8616;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x8617;'" select="'bo4	nie4'"/>
                <xsl:map-entry key="'&#x8618;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x8619;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x861A;'" select="'xian3	li4'"/>
                <xsl:map-entry key="'&#x861B;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x861C;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x861D;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x861E;'" select="'lian4	lian3'"/>
                <xsl:map-entry key="'&#x861F;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x8620;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x8621;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x8622;'" select="'long2'"/>
                <xsl:map-entry key="'&#x8623;'" select="'tou3'"/>
                <xsl:map-entry key="'&#x8624;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8625;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x8626;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8627;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x8628;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x8629;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x862A;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x862B;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x862C;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x862D;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x862E;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x862F;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x8630;'" select="''"/>
                <xsl:map-entry key="'&#x8631;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x8632;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x8633;'" select="'tong4'"/>
                <xsl:map-entry key="'&#x8634;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x8635;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x8636;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8637;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x8638;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x8639;'" select="'huai4'"/>
                <xsl:map-entry key="'&#x863A;'" select="'li2'"/>
                <xsl:map-entry key="'&#x863B;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x863C;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x863D;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x863E;'" select="'huai4'"/>
                <xsl:map-entry key="'&#x863F;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x8640;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8641;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x8642;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8643;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x8644;'" select="''"/>
                <xsl:map-entry key="'&#x8645;'" select="''"/>
                <xsl:map-entry key="'&#x8646;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x8647;'" select="'quan3'"/>
                <xsl:map-entry key="'&#x8648;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x8649;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x864A;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x864B;'" select="'men2'"/>
                <xsl:map-entry key="'&#x864C;'" select="'bie1'"/>
                <xsl:map-entry key="'&#x864D;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x864E;'" select="'hu3'"/>
                <xsl:map-entry key="'&#x864F;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x8650;'" select="'nu:e4'"/>
                <xsl:map-entry key="'&#x8651;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x8652;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8653;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x8654;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x8655;'" select="'chu4	chu3'"/>
                <xsl:map-entry key="'&#x8656;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x8657;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x8658;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x8659;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x865A;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x865B;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x865C;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x865D;'" select="'hu3	hu4'"/>
                <xsl:map-entry key="'&#x865E;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x865F;'" select="'hao4	hao2'"/>
                <xsl:map-entry key="'&#x8660;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x8661;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x8662;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x8663;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x8664;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x8665;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x8666;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x8667;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x8668;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x8669;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x866A;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x866B;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x866C;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x866D;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x866E;'" select="'ji1	ji3'"/>
                <xsl:map-entry key="'&#x866F;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x8670;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x8671;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x8672;'" select="''"/>
                <xsl:map-entry key="'&#x8673;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8674;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8675;'" select="'she2	yi2'"/>
                <xsl:map-entry key="'&#x8676;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x8677;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x8678;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x8679;'" select="'hong2	jiang4'"/>
                <xsl:map-entry key="'&#x867A;'" select="'hui3	hui1'"/>
                <xsl:map-entry key="'&#x867B;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x867C;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x867D;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x867E;'" select="'xia1	ha2'"/>
                <xsl:map-entry key="'&#x867F;'" select="'chai4'"/>
                <xsl:map-entry key="'&#x8680;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x8681;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x8682;'" select="'ma3	ma1	ma4'"/>
                <xsl:map-entry key="'&#x8683;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x8684;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x8685;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8686;'" select="'pa1'"/>
                <xsl:map-entry key="'&#x8687;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x8688;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x8689;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x868A;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x868B;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x868C;'" select="'bang4	beng4'"/>
                <xsl:map-entry key="'&#x868D;'" select="'bi3	pi2'"/>
                <xsl:map-entry key="'&#x868E;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x868F;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x8690;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x8691;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8692;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x8693;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x8694;'" select="'qi2	chi2'"/>
                <xsl:map-entry key="'&#x8695;'" select="'tian3	can2'"/>
                <xsl:map-entry key="'&#x8696;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x8697;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8698;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x8699;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x869A;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x869B;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x869C;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x869D;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x869E;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x869F;'" select="'wang2'"/>
                <xsl:map-entry key="'&#x86A0;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x86A1;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x86A2;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x86A3;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x86A4;'" select="'zao3'"/>
                <xsl:map-entry key="'&#x86A5;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x86A6;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x86A7;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x86A8;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x86A9;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x86AA;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x86AB;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x86AC;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x86AD;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x86AE;'" select="'te4'"/>
                <xsl:map-entry key="'&#x86AF;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x86B0;'" select="'you2'"/>
                <xsl:map-entry key="'&#x86B1;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x86B2;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x86B3;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x86B4;'" select="'you1	you4'"/>
                <xsl:map-entry key="'&#x86B5;'" select="'he2'"/>
                <xsl:map-entry key="'&#x86B6;'" select="'han1'"/>
                <xsl:map-entry key="'&#x86B7;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x86B8;'" select="'li4'"/>
                <xsl:map-entry key="'&#x86B9;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x86BA;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x86BB;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x86BC;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x86BD;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x86BE;'" select="'bo3'"/>
                <xsl:map-entry key="'&#x86BF;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x86C0;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x86C1;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x86C2;'" select="'bie3'"/>
                <xsl:map-entry key="'&#x86C3;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x86C4;'" select="'gu1	gu3'"/>
                <xsl:map-entry key="'&#x86C5;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x86C6;'" select="'qu1	ju1'"/>
                <xsl:map-entry key="'&#x86C7;'" select="'she2	yi2'"/>
                <xsl:map-entry key="'&#x86C8;'" select="'tie4'"/>
                <xsl:map-entry key="'&#x86C9;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x86CA;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x86CB;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x86CC;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x86CD;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x86CE;'" select="'li4'"/>
                <xsl:map-entry key="'&#x86CF;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x86D0;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x86D1;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x86D2;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x86D3;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x86D4;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x86D5;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x86D6;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x86D7;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x86D8;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x86D9;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x86DA;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x86DB;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x86DC;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x86DD;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x86DE;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x86DF;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x86E0;'" select="'li4'"/>
                <xsl:map-entry key="'&#x86E1;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x86E2;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x86E3;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x86E4;'" select="'ha2'"/>
                <xsl:map-entry key="'&#x86E5;'" select="'she2'"/>
                <xsl:map-entry key="'&#x86E6;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x86E7;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x86E8;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x86E9;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x86EA;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x86EB;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x86EC;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x86ED;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x86EE;'" select="'man2'"/>
                <xsl:map-entry key="'&#x86EF;'" select="''"/>
                <xsl:map-entry key="'&#x86F0;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x86F1;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x86F2;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x86F3;'" select="'si1'"/>
                <xsl:map-entry key="'&#x86F4;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x86F5;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x86F6;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x86F7;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x86F8;'" select="'shao1	xiao1'"/>
                <xsl:map-entry key="'&#x86F9;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x86FA;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x86FB;'" select="'tui4'"/>
                <xsl:map-entry key="'&#x86FC;'" select="'che1'"/>
                <xsl:map-entry key="'&#x86FD;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x86FE;'" select="'e2	yi3'"/>
                <xsl:map-entry key="'&#x86FF;'" select="'han4'"/>
                <xsl:map-entry key="'&#x8700;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x8701;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x8702;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x8703;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x8704;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x8705;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x8706;'" select="'xian4	xian3'"/>
                <xsl:map-entry key="'&#x8707;'" select="'zhe2	zhe1'"/>
                <xsl:map-entry key="'&#x8708;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x8709;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x870A;'" select="'li2'"/>
                <xsl:map-entry key="'&#x870B;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x870C;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x870D;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x870E;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x870F;'" select="'you3'"/>
                <xsl:map-entry key="'&#x8710;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8711;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x8712;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x8713;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x8714;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x8715;'" select="'tui4'"/>
                <xsl:map-entry key="'&#x8716;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x8717;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x8718;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x8719;'" select="'song1'"/>
                <xsl:map-entry key="'&#x871A;'" select="'fei1	fei3'"/>
                <xsl:map-entry key="'&#x871B;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x871C;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x871D;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x871E;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x871F;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8720;'" select="'jun3'"/>
                <xsl:map-entry key="'&#x8721;'" select="'zha4	cha4	la4'"/>
                <xsl:map-entry key="'&#x8722;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x8723;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x8724;'" select="'si1'"/>
                <xsl:map-entry key="'&#x8725;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8726;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x8727;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8728;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8729;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x872A;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x872B;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x872C;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x872D;'" select="'han4'"/>
                <xsl:map-entry key="'&#x872E;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x872F;'" select="'bang4	beng4'"/>
                <xsl:map-entry key="'&#x8730;'" select="'fei2'"/>
                <xsl:map-entry key="'&#x8731;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x8732;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8733;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x8734;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8735;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x8736;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8737;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8738;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x8739;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x873A;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x873B;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x873C;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x873D;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x873E;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x873F;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x8740;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x8741;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8742;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x8743;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x8744;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x8745;'" select="'can2'"/>
                <xsl:map-entry key="'&#x8746;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x8747;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8748;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x8749;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x874A;'" select="''"/>
                <xsl:map-entry key="'&#x874B;'" select="'la4	zha4'"/>
                <xsl:map-entry key="'&#x874C;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x874D;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x874E;'" select="'xie1'"/>
                <xsl:map-entry key="'&#x874F;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x8750;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x8751;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x8752;'" select="'mian2'"/>
                <xsl:map-entry key="'&#x8753;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8754;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x8755;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x8756;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8757;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x8758;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x8759;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x875A;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x875B;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x875C;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x875D;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x875E;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x875F;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8760;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8761;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x8762;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x8763;'" select="'you2'"/>
                <xsl:map-entry key="'&#x8764;'" select="'you2'"/>
                <xsl:map-entry key="'&#x8765;'" select="'mao2	mou2'"/>
                <xsl:map-entry key="'&#x8766;'" select="'xia1	ha2'"/>
                <xsl:map-entry key="'&#x8767;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x8768;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x8769;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x876A;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x876B;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x876C;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x876D;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x876E;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x876F;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x8770;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x8771;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x8772;'" select="'la4'"/>
                <xsl:map-entry key="'&#x8773;'" select="'du2'"/>
                <xsl:map-entry key="'&#x8774;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x8775;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x8776;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8777;'" select="'li4	xi2'"/>
                <xsl:map-entry key="'&#x8778;'" select="'gua1	wo1'"/>
                <xsl:map-entry key="'&#x8779;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x877A;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x877B;'" select="'nan3'"/>
                <xsl:map-entry key="'&#x877C;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x877D;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x877E;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x877F;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8780;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x8781;'" select="'tun4'"/>
                <xsl:map-entry key="'&#x8782;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x8783;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x8784;'" select="'si1	shi1'"/>
                <xsl:map-entry key="'&#x8785;'" select="'xi1	xi2'"/>
                <xsl:map-entry key="'&#x8786;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8787;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8788;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x8789;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x878A;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x878B;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x878C;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x878D;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x878E;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x878F;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x8790;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x8791;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x8792;'" select="'han4'"/>
                <xsl:map-entry key="'&#x8793;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x8794;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8795;'" select="'bi1'"/>
                <xsl:map-entry key="'&#x8796;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x8797;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x8798;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x8799;'" select="'du4'"/>
                <xsl:map-entry key="'&#x879A;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x879B;'" select="'he2'"/>
                <xsl:map-entry key="'&#x879C;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x879D;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x879E;'" select="'ma3	ma1	ma4'"/>
                <xsl:map-entry key="'&#x879F;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x87A0;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x87A1;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x87A2;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x87A3;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x87A4;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x87A5;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x87A6;'" select="''"/>
                <xsl:map-entry key="'&#x87A7;'" select="''"/>
                <xsl:map-entry key="'&#x87A8;'" select="'man3'"/>
                <xsl:map-entry key="'&#x87A9;'" select="''"/>
                <xsl:map-entry key="'&#x87AA;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x87AB;'" select="'zhe1	shi4'"/>
                <xsl:map-entry key="'&#x87AC;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x87AD;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x87AE;'" select="'di4'"/>
                <xsl:map-entry key="'&#x87AF;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x87B0;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x87B1;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x87B2;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x87B3;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x87B4;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x87B5;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x87B6;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x87B7;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x87B8;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x87B9;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x87BA;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x87BB;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x87BC;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x87BD;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x87BE;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x87BF;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x87C0;'" select="'shuai4	shuo4'"/>
                <xsl:map-entry key="'&#x87C1;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x87C2;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x87C3;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x87C4;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x87C5;'" select="'zhe4'"/>
                <xsl:map-entry key="'&#x87C6;'" select="'ma2	ma5'"/>
                <xsl:map-entry key="'&#x87C7;'" select="'ma2'"/>
                <xsl:map-entry key="'&#x87C8;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x87C9;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x87CA;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x87CB;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x87CC;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x87CD;'" select="'li2'"/>
                <xsl:map-entry key="'&#x87CE;'" select="'man3'"/>
                <xsl:map-entry key="'&#x87CF;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x87D0;'" select="''"/>
                <xsl:map-entry key="'&#x87D1;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x87D2;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x87D3;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x87D4;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x87D5;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x87D6;'" select="'si1'"/>
                <xsl:map-entry key="'&#x87D7;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x87D8;'" select="'te4'"/>
                <xsl:map-entry key="'&#x87D9;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x87DA;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x87DB;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x87DC;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x87DD;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x87DE;'" select="'bie2'"/>
                <xsl:map-entry key="'&#x87DF;'" select="'liao3'"/>
                <xsl:map-entry key="'&#x87E0;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x87E1;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x87E2;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x87E3;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x87E4;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x87E5;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x87E6;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x87E7;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x87E8;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x87E9;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x87EA;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x87EB;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x87EC;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x87ED;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x87EE;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x87EF;'" select="'rao2	nao2'"/>
                <xsl:map-entry key="'&#x87F0;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x87F1;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x87F2;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x87F3;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x87F4;'" select="'si1'"/>
                <xsl:map-entry key="'&#x87F5;'" select="''"/>
                <xsl:map-entry key="'&#x87F6;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x87F7;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x87F8;'" select="'li3	li2'"/>
                <xsl:map-entry key="'&#x87F9;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x87FA;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x87FB;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x87FC;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x87FD;'" select="'da2'"/>
                <xsl:map-entry key="'&#x87FE;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x87FF;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x8800;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8801;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x8802;'" select="'she4'"/>
                <xsl:map-entry key="'&#x8803;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x8804;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x8805;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8806;'" select="'chai4'"/>
                <xsl:map-entry key="'&#x8807;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8808;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8809;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x880A;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x880B;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x880C;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x880D;'" select="'xie1'"/>
                <xsl:map-entry key="'&#x880E;'" select="'mang3'"/>
                <xsl:map-entry key="'&#x880F;'" select="'xie4	xie3'"/>
                <xsl:map-entry key="'&#x8810;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8811;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x8812;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8813;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x8814;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x8815;'" select="'ruan3	ru2'"/>
                <xsl:map-entry key="'&#x8816;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x8817;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x8818;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8819;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x881A;'" select="'he4'"/>
                <xsl:map-entry key="'&#x881B;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x881C;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x881D;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x881E;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x881F;'" select="'la4	zha4'"/>
                <xsl:map-entry key="'&#x8820;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x8821;'" select="'li3	li2'"/>
                <xsl:map-entry key="'&#x8822;'" select="'chun3'"/>
                <xsl:map-entry key="'&#x8823;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8824;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x8825;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x8826;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x8827;'" select="'du4'"/>
                <xsl:map-entry key="'&#x8828;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x8829;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x882A;'" select="'long2'"/>
                <xsl:map-entry key="'&#x882B;'" select="'li4'"/>
                <xsl:map-entry key="'&#x882C;'" select="'long2'"/>
                <xsl:map-entry key="'&#x882D;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x882E;'" select="'ye1'"/>
                <xsl:map-entry key="'&#x882F;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x8830;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x8831;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x8832;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x8833;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x8834;'" select="''"/>
                <xsl:map-entry key="'&#x8835;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8836;'" select="'can2'"/>
                <xsl:map-entry key="'&#x8837;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x8838;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8839;'" select="'du4'"/>
                <xsl:map-entry key="'&#x883A;'" select="'can2'"/>
                <xsl:map-entry key="'&#x883B;'" select="'man2'"/>
                <xsl:map-entry key="'&#x883C;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x883D;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x883E;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x883F;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x8840;'" select="'xie3	xue4'"/>
                <xsl:map-entry key="'&#x8841;'" select="'huang1'"/>
                <xsl:map-entry key="'&#x8842;'" select="'nu:4'"/>
                <xsl:map-entry key="'&#x8843;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x8844;'" select="'nu:4'"/>
                <xsl:map-entry key="'&#x8845;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x8846;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x8847;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x8848;'" select="'er4'"/>
                <xsl:map-entry key="'&#x8849;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x884A;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x884B;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x884C;'" select="'xing2	hang2'"/>
                <xsl:map-entry key="'&#x884D;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x884E;'" select="'kan4'"/>
                <xsl:map-entry key="'&#x884F;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x8850;'" select="''"/>
                <xsl:map-entry key="'&#x8851;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8852;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x8853;'" select="'shu4	zhu2'"/>
                <xsl:map-entry key="'&#x8854;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8855;'" select="'tong4'"/>
                <xsl:map-entry key="'&#x8856;'" select="'long4'"/>
                <xsl:map-entry key="'&#x8857;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x8858;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8859;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x885A;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x885B;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x885C;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x885D;'" select="'chong1	chong4'"/>
                <xsl:map-entry key="'&#x885E;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x885F;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x8860;'" select="'zhun1'"/>
                <xsl:map-entry key="'&#x8861;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x8862;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x8863;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x8864;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x8865;'" select="'bu3'"/>
                <xsl:map-entry key="'&#x8866;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x8867;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8868;'" select="'biao3'"/>
                <xsl:map-entry key="'&#x8869;'" select="'cha4	cha3'"/>
                <xsl:map-entry key="'&#x886A;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x886B;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x886C;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x886D;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x886E;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x886F;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x8870;'" select="'shuai1	cui1'"/>
                <xsl:map-entry key="'&#x8871;'" select="'jie2	ji2'"/>
                <xsl:map-entry key="'&#x8872;'" select="'na4'"/>
                <xsl:map-entry key="'&#x8873;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x8874;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x8875;'" select="'ri4'"/>
                <xsl:map-entry key="'&#x8876;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x8877;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x8878;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x8879;'" select="'qi2	zhi1'"/>
                <xsl:map-entry key="'&#x887A;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x887B;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x887C;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x887D;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x887E;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x887F;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x8880;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x8881;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x8882;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x8883;'" select="'chai4'"/>
                <xsl:map-entry key="'&#x8884;'" select="'ao3'"/>
                <xsl:map-entry key="'&#x8885;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x8886;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x8887;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x8888;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x8889;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x888A;'" select="'ling3'"/>
                <xsl:map-entry key="'&#x888B;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x888C;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x888D;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x888E;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x888F;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x8890;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8891;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x8892;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x8893;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x8894;'" select="'he4'"/>
                <xsl:map-entry key="'&#x8895;'" select="'xue4'"/>
                <xsl:map-entry key="'&#x8896;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x8897;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x8898;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8899;'" select="'pa4'"/>
                <xsl:map-entry key="'&#x889A;'" select="'bo1	fu2'"/>
                <xsl:map-entry key="'&#x889B;'" select="'di1'"/>
                <xsl:map-entry key="'&#x889C;'" select="'wa4'"/>
                <xsl:map-entry key="'&#x889D;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x889E;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x889F;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x88A0;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x88A1;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x88A2;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x88A3;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x88A4;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x88A5;'" select="'tuo'"/>
                <xsl:map-entry key="'&#x88A6;'" select="'na4'"/>
                <xsl:map-entry key="'&#x88A7;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x88A8;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x88A9;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x88AA;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x88AB;'" select="'bei4	pi1'"/>
                <xsl:map-entry key="'&#x88AC;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x88AD;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x88AE;'" select="''"/>
                <xsl:map-entry key="'&#x88AF;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x88B0;'" select="''"/>
                <xsl:map-entry key="'&#x88B1;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x88B2;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x88B3;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x88B4;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x88B5;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x88B6;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x88B7;'" select="'jia2	qia1'"/>
                <xsl:map-entry key="'&#x88B8;'" select="'cun2'"/>
                <xsl:map-entry key="'&#x88B9;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x88BA;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x88BB;'" select="'er2'"/>
                <xsl:map-entry key="'&#x88BC;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x88BD;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x88BE;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x88BF;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x88C0;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x88C1;'" select="'cai2'"/>
                <xsl:map-entry key="'&#x88C2;'" select="'lie4	lie3'"/>
                <xsl:map-entry key="'&#x88C3;'" select="''"/>
                <xsl:map-entry key="'&#x88C4;'" select="''"/>
                <xsl:map-entry key="'&#x88C5;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x88C6;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x88C7;'" select="''"/>
                <xsl:map-entry key="'&#x88C8;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x88C9;'" select="'ken4'"/>
                <xsl:map-entry key="'&#x88CA;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x88CB;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x88CC;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x88CD;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x88CE;'" select="'cheng2	cheng3'"/>
                <xsl:map-entry key="'&#x88CF;'" select="'li3'"/>
                <xsl:map-entry key="'&#x88D0;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x88D1;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x88D2;'" select="'pou2'"/>
                <xsl:map-entry key="'&#x88D3;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x88D4;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x88D5;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x88D6;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x88D7;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x88D8;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x88D9;'" select="'qun2'"/>
                <xsl:map-entry key="'&#x88DA;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x88DB;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x88DC;'" select="'bu3'"/>
                <xsl:map-entry key="'&#x88DD;'" select="'zhuang1'"/>
                <xsl:map-entry key="'&#x88DE;'" select="'shui4'"/>
                <xsl:map-entry key="'&#x88DF;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x88E0;'" select="'qun2'"/>
                <xsl:map-entry key="'&#x88E1;'" select="'li3'"/>
                <xsl:map-entry key="'&#x88E2;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x88E3;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x88E4;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x88E5;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x88E6;'" select="'fou2'"/>
                <xsl:map-entry key="'&#x88E7;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x88E8;'" select="'bi4	pi2'"/>
                <xsl:map-entry key="'&#x88E9;'" select="'gun1'"/>
                <xsl:map-entry key="'&#x88EA;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x88EB;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x88EC;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x88ED;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x88EE;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x88EF;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x88F0;'" select="'duo2	duo1'"/>
                <xsl:map-entry key="'&#x88F1;'" select="'biao3'"/>
                <xsl:map-entry key="'&#x88F2;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x88F3;'" select="'chang2	shang5'"/>
                <xsl:map-entry key="'&#x88F4;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x88F5;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x88F6;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x88F7;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x88F8;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x88F9;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x88FA;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x88FB;'" select="'du3'"/>
                <xsl:map-entry key="'&#x88FC;'" select="'xi2	ti4'"/>
                <xsl:map-entry key="'&#x88FD;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x88FE;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x88FF;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x8900;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x8901;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x8902;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x8903;'" select="'ken4'"/>
                <xsl:map-entry key="'&#x8904;'" select="''"/>
                <xsl:map-entry key="'&#x8905;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x8906;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8907;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8908;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x8909;'" select="'xie1'"/>
                <xsl:map-entry key="'&#x890A;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x890B;'" select="'die2'"/>
                <xsl:map-entry key="'&#x890C;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x890D;'" select="'duan1'"/>
                <xsl:map-entry key="'&#x890E;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x890F;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x8910;'" select="'he2	he4'"/>
                <xsl:map-entry key="'&#x8911;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x8912;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x8913;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x8914;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8915;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8916;'" select="'tuan4'"/>
                <xsl:map-entry key="'&#x8917;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x8918;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x8919;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x891A;'" select="'chu3	zhu3'"/>
                <xsl:map-entry key="'&#x891B;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x891C;'" select="''"/>
                <xsl:map-entry key="'&#x891D;'" select="''"/>
                <xsl:map-entry key="'&#x891E;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x891F;'" select="'ta1'"/>
                <xsl:map-entry key="'&#x8920;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x8921;'" select="'da1'"/>
                <xsl:map-entry key="'&#x8922;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x8923;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x8924;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x8925;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x8926;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x8927;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x8928;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x8929;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x892A;'" select="'tun4	tui4'"/>
                <xsl:map-entry key="'&#x892B;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x892C;'" select="'sang3'"/>
                <xsl:map-entry key="'&#x892D;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x892E;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x892F;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x8930;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x8931;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x8932;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x8933;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8934;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x8935;'" select="'li2'"/>
                <xsl:map-entry key="'&#x8936;'" select="'zhe2	xi2	zhe3'"/>
                <xsl:map-entry key="'&#x8937;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x8938;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x8939;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x893A;'" select="'die1'"/>
                <xsl:map-entry key="'&#x893B;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x893C;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x893D;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x893E;'" select="'biao3'"/>
                <xsl:map-entry key="'&#x893F;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x8940;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8941;'" select="'qiang3'"/>
                <xsl:map-entry key="'&#x8942;'" select="'sen1'"/>
                <xsl:map-entry key="'&#x8943;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x8944;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x8945;'" select="''"/>
                <xsl:map-entry key="'&#x8946;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x8947;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8948;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x8949;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x894A;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x894B;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x894C;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x894D;'" select="'za2'"/>
                <xsl:map-entry key="'&#x894E;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x894F;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x8950;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x8951;'" select="'xin2'"/>
                <xsl:map-entry key="'&#x8952;'" select="'bie2	bi4'"/>
                <xsl:map-entry key="'&#x8953;'" select="'rao2'"/>
                <xsl:map-entry key="'&#x8954;'" select="'man3'"/>
                <xsl:map-entry key="'&#x8955;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x8956;'" select="'ao3'"/>
                <xsl:map-entry key="'&#x8957;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x8958;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8959;'" select="'cao4'"/>
                <xsl:map-entry key="'&#x895A;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x895B;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x895C;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x895D;'" select="'lian4	lian3'"/>
                <xsl:map-entry key="'&#x895E;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x895F;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x8960;'" select="'dang1'"/>
                <xsl:map-entry key="'&#x8961;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x8962;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x8963;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8964;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x8965;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x8966;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x8967;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x8968;'" select="''"/>
                <xsl:map-entry key="'&#x8969;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x896A;'" select="'wa4'"/>
                <xsl:map-entry key="'&#x896B;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x896C;'" select="'bai3'"/>
                <xsl:map-entry key="'&#x896D;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x896E;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x896F;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x8970;'" select="'lai3'"/>
                <xsl:map-entry key="'&#x8971;'" select="'long2'"/>
                <xsl:map-entry key="'&#x8972;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x8973;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x8974;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x8975;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8976;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x8977;'" select="''"/>
                <xsl:map-entry key="'&#x8978;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x8979;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x897A;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x897B;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x897C;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x897D;'" select="''"/>
                <xsl:map-entry key="'&#x897E;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x897F;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8980;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8981;'" select="'yao4	yao1'"/>
                <xsl:map-entry key="'&#x8982;'" select="'feng3'"/>
                <xsl:map-entry key="'&#x8983;'" select="'tan2	qin2'"/>
                <xsl:map-entry key="'&#x8984;'" select="''"/>
                <xsl:map-entry key="'&#x8985;'" select="'fiao4'"/>
                <xsl:map-entry key="'&#x8986;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8987;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x8988;'" select="'he2'"/>
                <xsl:map-entry key="'&#x8989;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x898A;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x898B;'" select="'jian4	xian4'"/>
                <xsl:map-entry key="'&#x898C;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x898D;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x898E;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x898F;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x8990;'" select="'jue2	jiao4'"/>
                <xsl:map-entry key="'&#x8991;'" select="'pian3'"/>
                <xsl:map-entry key="'&#x8992;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x8993;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x8994;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x8995;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x8996;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8997;'" select="'si1'"/>
                <xsl:map-entry key="'&#x8998;'" select="'zhan1	chan1'"/>
                <xsl:map-entry key="'&#x8999;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x899A;'" select="'jue2	jiao4'"/>
                <xsl:map-entry key="'&#x899B;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x899C;'" select="'tiao4'"/>
                <xsl:map-entry key="'&#x899D;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x899E;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x899F;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x89A0;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x89A1;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x89A2;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x89A3;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x89A4;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x89A5;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x89A6;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x89A7;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x89A8;'" select="'e4'"/>
                <xsl:map-entry key="'&#x89A9;'" select="'du3'"/>
                <xsl:map-entry key="'&#x89AA;'" select="'qin1	qing4'"/>
                <xsl:map-entry key="'&#x89AB;'" select="'pang3'"/>
                <xsl:map-entry key="'&#x89AC;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x89AD;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x89AE;'" select="'ping1'"/>
                <xsl:map-entry key="'&#x89AF;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x89B0;'" select="'qu4	qu1'"/>
                <xsl:map-entry key="'&#x89B1;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x89B2;'" select="'jin3	jin4'"/>
                <xsl:map-entry key="'&#x89B3;'" select="'guan1	guan4'"/>
                <xsl:map-entry key="'&#x89B4;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x89B5;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x89B6;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x89B7;'" select="'qu4	qu1'"/>
                <xsl:map-entry key="'&#x89B8;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x89B9;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x89BA;'" select="'jue2	jiao4'"/>
                <xsl:map-entry key="'&#x89BB;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x89BC;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x89BD;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x89BE;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x89BF;'" select="'di2'"/>
                <xsl:map-entry key="'&#x89C0;'" select="'guan1	guan4'"/>
                <xsl:map-entry key="'&#x89C1;'" select="'jian4	xian4'"/>
                <xsl:map-entry key="'&#x89C2;'" select="'guan1	guan4'"/>
                <xsl:map-entry key="'&#x89C3;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x89C4;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x89C5;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x89C6;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x89C7;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x89C8;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x89C9;'" select="'jue2	jiao4'"/>
                <xsl:map-entry key="'&#x89CA;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x89CB;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x89CC;'" select="'di2'"/>
                <xsl:map-entry key="'&#x89CD;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x89CE;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x89CF;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x89D0;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x89D1;'" select="'qu4	qu1'"/>
                <xsl:map-entry key="'&#x89D2;'" select="'jiao3	jia3	jue2'"/>
                <xsl:map-entry key="'&#x89D3;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x89D4;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x89D5;'" select="'cu1'"/>
                <xsl:map-entry key="'&#x89D6;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x89D7;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x89D8;'" select="'chao4'"/>
                <xsl:map-entry key="'&#x89D9;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x89DA;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x89DB;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x89DC;'" select="'zui3	zi1'"/>
                <xsl:map-entry key="'&#x89DD;'" select="'di3'"/>
                <xsl:map-entry key="'&#x89DE;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x89DF;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x89E0;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x89E1;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x89E2;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x89E3;'" select="'jie3	jie4	xie4'"/>
                <xsl:map-entry key="'&#x89E4;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x89E5;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x89E6;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x89E7;'" select="'jie3	jie4	xie4'"/>
                <xsl:map-entry key="'&#x89E8;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x89E9;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x89EA;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x89EB;'" select="'su4'"/>
                <xsl:map-entry key="'&#x89EC;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x89ED;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x89EE;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x89EF;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x89F0;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x89F1;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x89F2;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x89F3;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x89F4;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x89F5;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x89F6;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x89F7;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x89F8;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x89F9;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x89FA;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x89FB;'" select="'li4'"/>
                <xsl:map-entry key="'&#x89FC;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x89FD;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x89FE;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x89FF;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8A00;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x8A01;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x8A02;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x8A03;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8A04;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x8A05;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x8A06;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x8A07;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x8A08;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x8A09;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x8A0A;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x8A0B;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x8A0C;'" select="'hong2	hong4'"/>
                <xsl:map-entry key="'&#x8A0D;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x8A0E;'" select="'tao3'"/>
                <xsl:map-entry key="'&#x8A0F;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x8A10;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8A11;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8A12;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x8A13;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x8A14;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x8A15;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x8A16;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x8A17;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x8A18;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x8A19;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x8A1A;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x8A1B;'" select="'e2'"/>
                <xsl:map-entry key="'&#x8A1C;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x8A1D;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x8A1E;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x8A1F;'" select="'song4'"/>
                <xsl:map-entry key="'&#x8A20;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x8A21;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x8A22;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x8A23;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8A24;'" select="'xiao2'"/>
                <xsl:map-entry key="'&#x8A25;'" select="'ne4	na4'"/>
                <xsl:map-entry key="'&#x8A26;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8A27;'" select="'you2'"/>
                <xsl:map-entry key="'&#x8A28;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x8A29;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x8A2A;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x8A2B;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x8A2C;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x8A2D;'" select="'she4'"/>
                <xsl:map-entry key="'&#x8A2E;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x8A2F;'" select="'sa3'"/>
                <xsl:map-entry key="'&#x8A30;'" select="'zhun1'"/>
                <xsl:map-entry key="'&#x8A31;'" select="'xu3	hu3'"/>
                <xsl:map-entry key="'&#x8A32;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8A33;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8A34;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8A35;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x8A36;'" select="'he1'"/>
                <xsl:map-entry key="'&#x8A37;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x8A38;'" select="'he2'"/>
                <xsl:map-entry key="'&#x8A39;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x8A3A;'" select="'zhen3	zhen1'"/>
                <xsl:map-entry key="'&#x8A3B;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x8A3C;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x8A3D;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x8A3E;'" select="'zi3	zi1'"/>
                <xsl:map-entry key="'&#x8A3F;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x8A40;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x8A41;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x8A42;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8A43;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8A44;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8A45;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8A46;'" select="'di3'"/>
                <xsl:map-entry key="'&#x8A47;'" select="'yang4'"/>
                <xsl:map-entry key="'&#x8A48;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8A49;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x8A4A;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x8A4B;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x8A4C;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x8A4D;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8A4E;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x8A4F;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x8A50;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x8A51;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x8A52;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8A53;'" select="'qu3'"/>
                <xsl:map-entry key="'&#x8A54;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x8A55;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x8A56;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8A57;'" select="'xiong4'"/>
                <xsl:map-entry key="'&#x8A58;'" select="'chu4	qu1'"/>
                <xsl:map-entry key="'&#x8A59;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x8A5A;'" select="'da2'"/>
                <xsl:map-entry key="'&#x8A5B;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x8A5C;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x8A5D;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x8A5E;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x8A5F;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8A60;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x8A61;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x8A62;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x8A63;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8A64;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x8A65;'" select="'he2'"/>
                <xsl:map-entry key="'&#x8A66;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8A67;'" select="'cha2'"/>
                <xsl:map-entry key="'&#x8A68;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x8A69;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x8A6A;'" select="'hen3'"/>
                <xsl:map-entry key="'&#x8A6B;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x8A6C;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x8A6D;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x8A6E;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8A6F;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8A70;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8A71;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x8A72;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x8A73;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x8A74;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8A75;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x8A76;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x8A77;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x8A78;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x8A79;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x8A7A;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x8A7B;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8A7C;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x8A7D;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x8A7E;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x8A7F;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x8A80;'" select="'er4'"/>
                <xsl:map-entry key="'&#x8A81;'" select="'beng3'"/>
                <xsl:map-entry key="'&#x8A82;'" select="'tiao3'"/>
                <xsl:map-entry key="'&#x8A83;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x8A84;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x8A85;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x8A86;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x8A87;'" select="'kua1'"/>
                <xsl:map-entry key="'&#x8A88;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x8A89;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8A8A;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x8A8B;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x8A8C;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8A8D;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x8A8E;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8A8F;'" select="'lang3'"/>
                <xsl:map-entry key="'&#x8A90;'" select="'e2'"/>
                <xsl:map-entry key="'&#x8A91;'" select="'kuang2'"/>
                <xsl:map-entry key="'&#x8A92;'" select="'e^1	e^2	e^3	e^4'"/>
                <xsl:map-entry key="'&#x8A93;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8A94;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x8A95;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x8A96;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x8A97;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x8A98;'" select="'you4'"/>
                <xsl:map-entry key="'&#x8A99;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x8A9A;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x8A9B;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x8A9C;'" select="'shua4'"/>
                <xsl:map-entry key="'&#x8A9D;'" select="'an1'"/>
                <xsl:map-entry key="'&#x8A9E;'" select="'yu3	yu4'"/>
                <xsl:map-entry key="'&#x8A9F;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x8AA0;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x8AA1;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x8AA2;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x8AA3;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x8AA4;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x8AA5;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x8AA6;'" select="'song4'"/>
                <xsl:map-entry key="'&#x8AA7;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x8AA8;'" select="'hui4	hui3'"/>
                <xsl:map-entry key="'&#x8AA9;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x8AAA;'" select="'shuo1	shui4	yue4'"/>
                <xsl:map-entry key="'&#x8AAB;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x8AAC;'" select="'shuo1	shui4	yue4'"/>
                <xsl:map-entry key="'&#x8AAD;'" select="'du2	dou4'"/>
                <xsl:map-entry key="'&#x8AAE;'" select="''"/>
                <xsl:map-entry key="'&#x8AAF;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x8AB0;'" select="'shui2	shei2'"/>
                <xsl:map-entry key="'&#x8AB1;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8AB2;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x8AB3;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x8AB4;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x8AB5;'" select="'xiao2'"/>
                <xsl:map-entry key="'&#x8AB6;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x8AB7;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x8AB8;'" select="'xuan2'"/>
                <xsl:map-entry key="'&#x8AB9;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x8ABA;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x8ABB;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x8ABC;'" select="'yi2	yi4'"/>
                <xsl:map-entry key="'&#x8ABD;'" select="'na2'"/>
                <xsl:map-entry key="'&#x8ABE;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x8ABF;'" select="'diao4	tiao2'"/>
                <xsl:map-entry key="'&#x8AC0;'" select="'pi3'"/>
                <xsl:map-entry key="'&#x8AC1;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x8AC2;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x8AC3;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x8AC4;'" select="'zhun1'"/>
                <xsl:map-entry key="'&#x8AC5;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8AC6;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x8AC7;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x8AC8;'" select="'chui4'"/>
                <xsl:map-entry key="'&#x8AC9;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8ACA;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x8ACB;'" select="'qing3'"/>
                <xsl:map-entry key="'&#x8ACC;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8ACD;'" select="'zheng1	zheng4'"/>
                <xsl:map-entry key="'&#x8ACE;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8ACF;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x8AD0;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x8AD1;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x8AD2;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x8AD3;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8AD4;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x8AD5;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x8AD6;'" select="'lun4	lun2'"/>
                <xsl:map-entry key="'&#x8AD7;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x8AD8;'" select="'biao3'"/>
                <xsl:map-entry key="'&#x8AD9;'" select="'huai4'"/>
                <xsl:map-entry key="'&#x8ADA;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x8ADB;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8ADC;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8ADD;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x8ADE;'" select="'pian2	pian3'"/>
                <xsl:map-entry key="'&#x8ADF;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8AE0;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8AE1;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8AE2;'" select="'hun4'"/>
                <xsl:map-entry key="'&#x8AE3;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x8AE4;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8AE5;'" select="'zhong4'"/>
                <xsl:map-entry key="'&#x8AE6;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8AE7;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x8AE8;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8AE9;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x8AEA;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x8AEB;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8AEC;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x8AED;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8AEE;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8AEF;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x8AF0;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x8AF1;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8AF2;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x8AF3;'" select="'an1'"/>
                <xsl:map-entry key="'&#x8AF4;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8AF5;'" select="'nan2'"/>
                <xsl:map-entry key="'&#x8AF6;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8AF7;'" select="'feng1	feng3	feng4'"/>
                <xsl:map-entry key="'&#x8AF8;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x8AF9;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x8AFA;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8AFB;'" select="'heng1	heng2'"/>
                <xsl:map-entry key="'&#x8AFC;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8AFD;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x8AFE;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x8AFF;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x8B00;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x8B01;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x8B02;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8B03;'" select="''"/>
                <xsl:map-entry key="'&#x8B04;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x8B05;'" select="'zou1	zhou1'"/>
                <xsl:map-entry key="'&#x8B06;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x8B07;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8B08;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x8B09;'" select="'ku14'"/>
                <xsl:map-entry key="'&#x8B0A;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x8B0B;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x8B0C;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x8B0D;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8B0E;'" select="'mi2	mei4'"/>
                <xsl:map-entry key="'&#x8B0F;'" select="'xiao3	sou3'"/>
                <xsl:map-entry key="'&#x8B10;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x8B11;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x8B12;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x8B13;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x8B14;'" select="'nu:e4	xue4'"/>
                <xsl:map-entry key="'&#x8B15;'" select="'si1'"/>
                <xsl:map-entry key="'&#x8B16;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8B17;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x8B18;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x8B19;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x8B1A;'" select="'shi4	yi4'"/>
                <xsl:map-entry key="'&#x8B1B;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x8B1C;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x8B1D;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x8B1E;'" select="'xue4'"/>
                <xsl:map-entry key="'&#x8B1F;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x8B20;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x8B21;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x8B22;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x8B23;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8B24;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x8B25;'" select="'cong4'"/>
                <xsl:map-entry key="'&#x8B26;'" select="'qing4	qing3'"/>
                <xsl:map-entry key="'&#x8B27;'" select="'li2'"/>
                <xsl:map-entry key="'&#x8B28;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x8B29;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x8B2A;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x8B2B;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8B2C;'" select="'miu4'"/>
                <xsl:map-entry key="'&#x8B2D;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8B2E;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8B2F;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x8B30;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8B31;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x8B32;'" select="'can1'"/>
                <xsl:map-entry key="'&#x8B33;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x8B34;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x8B35;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x8B36;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x8B37;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x8B38;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x8B39;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x8B3A;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8B3B;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8B3C;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x8B3D;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x8B3E;'" select="'man2	man4'"/>
                <xsl:map-entry key="'&#x8B3F;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x8B40;'" select="'han4'"/>
                <xsl:map-entry key="'&#x8B41;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x8B42;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x8B43;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x8B44;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x8B45;'" select="'se4'"/>
                <xsl:map-entry key="'&#x8B46;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8B47;'" select="'she1'"/>
                <xsl:map-entry key="'&#x8B48;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x8B49;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x8B4A;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x8B4B;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x8B4C;'" select="'e2'"/>
                <xsl:map-entry key="'&#x8B4D;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x8B4E;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8B4F;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8B50;'" select="'zun3'"/>
                <xsl:map-entry key="'&#x8B51;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x8B52;'" select="'bo4'"/>
                <xsl:map-entry key="'&#x8B53;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8B54;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x8B55;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x8B56;'" select="'zen4'"/>
                <xsl:map-entry key="'&#x8B57;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x8B58;'" select="'shi4	zhi4'"/>
                <xsl:map-entry key="'&#x8B59;'" select="'qiao2	qiao4'"/>
                <xsl:map-entry key="'&#x8B5A;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x8B5B;'" select="'zen4'"/>
                <xsl:map-entry key="'&#x8B5C;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x8B5D;'" select="'sheng2'"/>
                <xsl:map-entry key="'&#x8B5E;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8B5F;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x8B60;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x8B61;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x8B62;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x8B63;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x8B64;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8B65;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x8B66;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x8B67;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8B68;'" select="'nou4'"/>
                <xsl:map-entry key="'&#x8B69;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x8B6A;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x8B6B;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x8B6C;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x8B6D;'" select="'hui3'"/>
                <xsl:map-entry key="'&#x8B6E;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x8B6F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8B70;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8B71;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x8B72;'" select="'rang4'"/>
                <xsl:map-entry key="'&#x8B73;'" select="'nou4'"/>
                <xsl:map-entry key="'&#x8B74;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x8B75;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x8B76;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x8B77;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x8B78;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8B79;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x8B7A;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x8B7B;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x8B7C;'" select="'jian4	jian1'"/>
                <xsl:map-entry key="'&#x8B7D;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8B7E;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8B7F;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8B80;'" select="'du2	dou4'"/>
                <xsl:map-entry key="'&#x8B81;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8B82;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x8B83;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x8B84;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x8B85;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x8B86;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8B87;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x8B88;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8B89;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8B8A;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8B8B;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8B8C;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8B8D;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8B8E;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x8B8F;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8B90;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x8B91;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x8B92;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x8B93;'" select="'rang4'"/>
                <xsl:map-entry key="'&#x8B94;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x8B95;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x8B96;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x8B97;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x8B98;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8B99;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x8B9A;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x8B9B;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8B9C;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x8B9D;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x8B9E;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8B9F;'" select="'du2'"/>
                <xsl:map-entry key="'&#x8BA0;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x8BA1;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x8BA2;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x8BA3;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8BA4;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x8BA5;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8BA6;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8BA7;'" select="'hong4'"/>
                <xsl:map-entry key="'&#x8BA8;'" select="'tao3'"/>
                <xsl:map-entry key="'&#x8BA9;'" select="'rang4'"/>
                <xsl:map-entry key="'&#x8BAA;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x8BAB;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x8BAC;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x8BAD;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x8BAE;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8BAF;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x8BB0;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x8BB1;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x8BB2;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x8BB3;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8BB4;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x8BB5;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x8BB6;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x8BB7;'" select="'ne4'"/>
                <xsl:map-entry key="'&#x8BB8;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x8BB9;'" select="'e2'"/>
                <xsl:map-entry key="'&#x8BBA;'" select="'lun4	lun2'"/>
                <xsl:map-entry key="'&#x8BBB;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x8BBC;'" select="'song4'"/>
                <xsl:map-entry key="'&#x8BBD;'" select="'feng3'"/>
                <xsl:map-entry key="'&#x8BBE;'" select="'she4'"/>
                <xsl:map-entry key="'&#x8BBF;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x8BC0;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8BC1;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x8BC2;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x8BC3;'" select="'he1'"/>
                <xsl:map-entry key="'&#x8BC4;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x8BC5;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x8BC6;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8BC7;'" select="'xiong4'"/>
                <xsl:map-entry key="'&#x8BC8;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x8BC9;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8BCA;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x8BCB;'" select="'di3'"/>
                <xsl:map-entry key="'&#x8BCC;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8BCD;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x8BCE;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x8BCF;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x8BD0;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8BD1;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8BD2;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8BD3;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x8BD4;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x8BD5;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8BD6;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x8BD7;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x8BD8;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8BD9;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x8BDA;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x8BDB;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x8BDC;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x8BDD;'" select="'hua4'"/>
                <xsl:map-entry key="'&#x8BDE;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x8BDF;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x8BE0;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8BE1;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x8BE2;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x8BE3;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8BE4;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x8BE5;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x8BE6;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x8BE7;'" select="'cha4'"/>
                <xsl:map-entry key="'&#x8BE8;'" select="'hun4'"/>
                <xsl:map-entry key="'&#x8BE9;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x8BEA;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8BEB;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x8BEC;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x8BED;'" select="'yu3	yu4'"/>
                <xsl:map-entry key="'&#x8BEE;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x8BEF;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x8BF0;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x8BF1;'" select="'you4'"/>
                <xsl:map-entry key="'&#x8BF2;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8BF3;'" select="'kuang2'"/>
                <xsl:map-entry key="'&#x8BF4;'" select="'shuo1	shui4	yue4'"/>
                <xsl:map-entry key="'&#x8BF5;'" select="'song4'"/>
                <xsl:map-entry key="'&#x8BF6;'" select="'e^1	e^2	e^3	e^4'"/>
                <xsl:map-entry key="'&#x8BF7;'" select="'qing3'"/>
                <xsl:map-entry key="'&#x8BF8;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x8BF9;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x8BFA;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x8BFB;'" select="'du2	dou4'"/>
                <xsl:map-entry key="'&#x8BFC;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x8BFD;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x8BFE;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x8BFF;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8C00;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8C01;'" select="'shui2	shei2'"/>
                <xsl:map-entry key="'&#x8C02;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x8C03;'" select="'diao4	tiao2'"/>
                <xsl:map-entry key="'&#x8C04;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x8C05;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x8C06;'" select="'zhun1'"/>
                <xsl:map-entry key="'&#x8C07;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x8C08;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x8C09;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x8C0A;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8C0B;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x8C0C;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8C0D;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8C0E;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x8C0F;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8C10;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x8C11;'" select="'xue4'"/>
                <xsl:map-entry key="'&#x8C12;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x8C13;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8C14;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8C15;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8C16;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8C17;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x8C18;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8C19;'" select="'an1'"/>
                <xsl:map-entry key="'&#x8C1A;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8C1B;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8C1C;'" select="'mi2	mei4'"/>
                <xsl:map-entry key="'&#x8C1D;'" select="'pian3'"/>
                <xsl:map-entry key="'&#x8C1E;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x8C1F;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x8C20;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x8C21;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8C22;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x8C23;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x8C24;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x8C25;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8C26;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x8C27;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x8C28;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x8C29;'" select="'man2	man4'"/>
                <xsl:map-entry key="'&#x8C2A;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8C2B;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8C2C;'" select="'miu4'"/>
                <xsl:map-entry key="'&#x8C2D;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x8C2E;'" select="'zen4'"/>
                <xsl:map-entry key="'&#x8C2F;'" select="'qiao2	qiao4'"/>
                <xsl:map-entry key="'&#x8C30;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x8C31;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x8C32;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8C33;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8C34;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x8C35;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x8C36;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x8C37;'" select="'gu3	yu4'"/>
                <xsl:map-entry key="'&#x8C38;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x8C39;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x8C3A;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x8C3B;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8C3C;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x8C3D;'" select="'han1'"/>
                <xsl:map-entry key="'&#x8C3E;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x8C3F;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x8C40;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8C41;'" select="'huo4	hua2	huo1'"/>
                <xsl:map-entry key="'&#x8C42;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x8C43;'" select="'han3'"/>
                <xsl:map-entry key="'&#x8C44;'" select="'du2'"/>
                <xsl:map-entry key="'&#x8C45;'" select="'long2'"/>
                <xsl:map-entry key="'&#x8C46;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x8C47;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x8C48;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x8C49;'" select="'shi4	chi3'"/>
                <xsl:map-entry key="'&#x8C4A;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x8C4B;'" select="'deng1'"/>
                <xsl:map-entry key="'&#x8C4C;'" select="'wan1'"/>
                <xsl:map-entry key="'&#x8C4D;'" select="'bi1'"/>
                <xsl:map-entry key="'&#x8C4E;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x8C4F;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x8C50;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x8C51;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8C52;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8C53;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8C54;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8C55;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x8C56;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x8C57;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x8C58;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x8C59;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8C5A;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x8C5B;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8C5C;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x8C5D;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x8C5E;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x8C5F;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8C60;'" select="'cu2'"/>
                <xsl:map-entry key="'&#x8C61;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x8C62;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x8C63;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x8C64;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x8C65;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x8C66;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x8C67;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x8C68;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8C69;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x8C6A;'" select="'hao2'"/>
                <xsl:map-entry key="'&#x8C6B;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8C6C;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x8C6D;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x8C6E;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x8C6F;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8C70;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x8C71;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x8C72;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x8C73;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x8C74;'" select="'di2'"/>
                <xsl:map-entry key="'&#x8C75;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x8C76;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x8C77;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8C78;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8C79;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x8C7A;'" select="'chai2'"/>
                <xsl:map-entry key="'&#x8C7B;'" select="'han4	an4'"/>
                <xsl:map-entry key="'&#x8C7C;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x8C7D;'" select="'na4'"/>
                <xsl:map-entry key="'&#x8C7E;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x8C7F;'" select="'gou3'"/>
                <xsl:map-entry key="'&#x8C80;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x8C81;'" select="'you4'"/>
                <xsl:map-entry key="'&#x8C82;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x8C83;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x8C84;'" select="'si4'"/>
                <xsl:map-entry key="'&#x8C85;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x8C86;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x8C87;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x8C88;'" select="'he2'"/>
                <xsl:map-entry key="'&#x8C89;'" select="'he2	hao2	mo4'"/>
                <xsl:map-entry key="'&#x8C8A;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x8C8B;'" select="'an4'"/>
                <xsl:map-entry key="'&#x8C8C;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x8C8D;'" select="'li2'"/>
                <xsl:map-entry key="'&#x8C8E;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x8C8F;'" select="'bi3'"/>
                <xsl:map-entry key="'&#x8C90;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x8C91;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x8C92;'" select="'tuan1'"/>
                <xsl:map-entry key="'&#x8C93;'" select="'mao1'"/>
                <xsl:map-entry key="'&#x8C94;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x8C95;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8C96;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8C97;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x8C98;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x8C99;'" select="'chu1'"/>
                <xsl:map-entry key="'&#x8C9A;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x8C9B;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x8C9C;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x8C9D;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x8C9E;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x8C9F;'" select="'yuan2	yun2	yun4'"/>
                <xsl:map-entry key="'&#x8CA0;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8CA1;'" select="'cai2'"/>
                <xsl:map-entry key="'&#x8CA2;'" select="'gong4'"/>
                <xsl:map-entry key="'&#x8CA3;'" select="'te4'"/>
                <xsl:map-entry key="'&#x8CA4;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8CA5;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x8CA6;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x8CA7;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x8CA8;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x8CA9;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x8CAA;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x8CAB;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x8CAC;'" select="'ze2	zhai4'"/>
                <xsl:map-entry key="'&#x8CAD;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8CAE;'" select="'er4'"/>
                <xsl:map-entry key="'&#x8CAF;'" select="'zhu3	zhu4'"/>
                <xsl:map-entry key="'&#x8CB0;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8CB1;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8CB2;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8CB3;'" select="'er4'"/>
                <xsl:map-entry key="'&#x8CB4;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x8CB5;'" select="'pian1'"/>
                <xsl:map-entry key="'&#x8CB6;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x8CB7;'" select="'mai3'"/>
                <xsl:map-entry key="'&#x8CB8;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x8CB9;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x8CBA;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x8CBB;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x8CBC;'" select="'tie1'"/>
                <xsl:map-entry key="'&#x8CBD;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8CBE;'" select="'chi2	chi1'"/>
                <xsl:map-entry key="'&#x8CBF;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x8CC0;'" select="'he4'"/>
                <xsl:map-entry key="'&#x8CC1;'" select="'bi4	ben1'"/>
                <xsl:map-entry key="'&#x8CC2;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8CC3;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x8CC4;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8CC5;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x8CC6;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x8CC7;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8CC8;'" select="'jia3	gu3'"/>
                <xsl:map-entry key="'&#x8CC9;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x8CCA;'" select="'zei2	ze2'"/>
                <xsl:map-entry key="'&#x8CCB;'" select="'jiao3'"/>
                <xsl:map-entry key="'&#x8CCC;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x8CCD;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x8CCE;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8CCF;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x8CD0;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x8CD1;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x8CD2;'" select="'she1'"/>
                <xsl:map-entry key="'&#x8CD3;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x8CD4;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x8CD5;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x8CD6;'" select="'she1'"/>
                <xsl:map-entry key="'&#x8CD7;'" select="'chuan4'"/>
                <xsl:map-entry key="'&#x8CD8;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x8CD9;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8CDA;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x8CDB;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x8CDC;'" select="'si4'"/>
                <xsl:map-entry key="'&#x8CDD;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x8CDE;'" select="'shang3'"/>
                <xsl:map-entry key="'&#x8CDF;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x8CE0;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x8CE1;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x8CE2;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8CE3;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x8CE4;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8CE5;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x8CE6;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8CE7;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x8CE8;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x8CE9;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x8CEA;'" select="'zhi2	zhi4'"/>
                <xsl:map-entry key="'&#x8CEB;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8CEC;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x8CED;'" select="'du3'"/>
                <xsl:map-entry key="'&#x8CEE;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x8CEF;'" select="'xiong1'"/>
                <xsl:map-entry key="'&#x8CF0;'" select="'shun3'"/>
                <xsl:map-entry key="'&#x8CF1;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x8CF2;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x8CF3;'" select="'zai1'"/>
                <xsl:map-entry key="'&#x8CF4;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x8CF5;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x8CF6;'" select="'cang4'"/>
                <xsl:map-entry key="'&#x8CF7;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8CF8;'" select="'sheng4'"/>
                <xsl:map-entry key="'&#x8CF9;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x8CFA;'" select="'zhuan4	zuan4'"/>
                <xsl:map-entry key="'&#x8CFB;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8CFC;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x8CFD;'" select="'sai4'"/>
                <xsl:map-entry key="'&#x8CFE;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8CFF;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x8D00;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8D01;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x8D02;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x8D03;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x8D04;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8D05;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x8D06;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x8D07;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x8D08;'" select="'zeng4'"/>
                <xsl:map-entry key="'&#x8D09;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x8D0A;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x8D0B;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8D0C;'" select="''"/>
                <xsl:map-entry key="'&#x8D0D;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x8D0E;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x8D0F;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8D10;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x8D11;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x8D12;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8D13;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x8D14;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8D15;'" select="'du2'"/>
                <xsl:map-entry key="'&#x8D16;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x8D17;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8D18;'" select="''"/>
                <xsl:map-entry key="'&#x8D19;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x8D1A;'" select="'long4'"/>
                <xsl:map-entry key="'&#x8D1B;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x8D1C;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x8D1D;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x8D1E;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x8D1F;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8D20;'" select="'yuan2	yun2	yun4'"/>
                <xsl:map-entry key="'&#x8D21;'" select="'gong4'"/>
                <xsl:map-entry key="'&#x8D22;'" select="'cai2'"/>
                <xsl:map-entry key="'&#x8D23;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8D24;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8D25;'" select="'bai4'"/>
                <xsl:map-entry key="'&#x8D26;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x8D27;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x8D28;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8D29;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x8D2A;'" select="'tan1'"/>
                <xsl:map-entry key="'&#x8D2B;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x8D2C;'" select="'bian3'"/>
                <xsl:map-entry key="'&#x8D2D;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x8D2E;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x8D2F;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x8D30;'" select="'er4'"/>
                <xsl:map-entry key="'&#x8D31;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8D32;'" select="'bi4	ben1'"/>
                <xsl:map-entry key="'&#x8D33;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8D34;'" select="'tie1'"/>
                <xsl:map-entry key="'&#x8D35;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x8D36;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x8D37;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x8D38;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x8D39;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x8D3A;'" select="'he4'"/>
                <xsl:map-entry key="'&#x8D3B;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8D3C;'" select="'zei2'"/>
                <xsl:map-entry key="'&#x8D3D;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8D3E;'" select="'jia3	gu3'"/>
                <xsl:map-entry key="'&#x8D3F;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x8D40;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8D41;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x8D42;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8D43;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x8D44;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8D45;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x8D46;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x8D47;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x8D48;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x8D49;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x8D4A;'" select="'she1'"/>
                <xsl:map-entry key="'&#x8D4B;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8D4C;'" select="'du3'"/>
                <xsl:map-entry key="'&#x8D4D;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8D4E;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x8D4F;'" select="'shang3'"/>
                <xsl:map-entry key="'&#x8D50;'" select="'ci4'"/>
                <xsl:map-entry key="'&#x8D51;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8D52;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8D53;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x8D54;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x8D55;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x8D56;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x8D57;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x8D58;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x8D59;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8D5A;'" select="'zhuan4	zuan4'"/>
                <xsl:map-entry key="'&#x8D5B;'" select="'sai4'"/>
                <xsl:map-entry key="'&#x8D5C;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8D5D;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x8D5E;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x8D5F;'" select="'yun1'"/>
                <xsl:map-entry key="'&#x8D60;'" select="'zeng4'"/>
                <xsl:map-entry key="'&#x8D61;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x8D62;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8D63;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x8D64;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x8D65;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x8D66;'" select="'she4'"/>
                <xsl:map-entry key="'&#x8D67;'" select="'nan3'"/>
                <xsl:map-entry key="'&#x8D68;'" select="'xiong2'"/>
                <xsl:map-entry key="'&#x8D69;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x8D6A;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x8D6B;'" select="'he4'"/>
                <xsl:map-entry key="'&#x8D6C;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x8D6D;'" select="'zhe3'"/>
                <xsl:map-entry key="'&#x8D6E;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x8D6F;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x8D70;'" select="'zou3'"/>
                <xsl:map-entry key="'&#x8D71;'" select="'zou3'"/>
                <xsl:map-entry key="'&#x8D72;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8D73;'" select="'jiu3	jiu1'"/>
                <xsl:map-entry key="'&#x8D74;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8D75;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x8D76;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x8D77;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x8D78;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x8D79;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x8D7A;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x8D7B;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x8D7C;'" select="'ci1'"/>
                <xsl:map-entry key="'&#x8D7D;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8D7E;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x8D7F;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x8D80;'" select="'ci1'"/>
                <xsl:map-entry key="'&#x8D81;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x8D82;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x8D83;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8D84;'" select="'ju1	ju4	qie4'"/>
                <xsl:map-entry key="'&#x8D85;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x8D86;'" select="'di1'"/>
                <xsl:map-entry key="'&#x8D87;'" select="'se4'"/>
                <xsl:map-entry key="'&#x8D88;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x8D89;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x8D8A;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x8D8B;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x8D8C;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8D8D;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x8D8E;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x8D8F;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x8D90;'" select="'xue4'"/>
                <xsl:map-entry key="'&#x8D91;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8D92;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x8D93;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x8D94;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x8D95;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x8D96;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x8D97;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x8D98;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x8D99;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x8D9A;'" select="'su4'"/>
                <xsl:map-entry key="'&#x8D9B;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x8D9C;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x8D9D;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8D9E;'" select="'que4'"/>
                <xsl:map-entry key="'&#x8D9F;'" select="'tang4	tang1'"/>
                <xsl:map-entry key="'&#x8DA0;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x8DA1;'" select="'cui3'"/>
                <xsl:map-entry key="'&#x8DA2;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8DA3;'" select="'qu4	cu4'"/>
                <xsl:map-entry key="'&#x8DA4;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x8DA5;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x8DA6;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8DA7;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x8DA8;'" select="'qu1	cu4'"/>
                <xsl:map-entry key="'&#x8DA9;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x8DAA;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x8DAB;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x8DAC;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x8DAD;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x8DAE;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x8DAF;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x8DB0;'" select="''"/>
                <xsl:map-entry key="'&#x8DB1;'" select="'zan3'"/>
                <xsl:map-entry key="'&#x8DB2;'" select="'zan3	zan4'"/>
                <xsl:map-entry key="'&#x8DB3;'" select="'zu2	ju4'"/>
                <xsl:map-entry key="'&#x8DB4;'" select="'pa1'"/>
                <xsl:map-entry key="'&#x8DB5;'" select="'bao4	bo1'"/>
                <xsl:map-entry key="'&#x8DB6;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x8DB7;'" select="'he2'"/>
                <xsl:map-entry key="'&#x8DB8;'" select="'dun3'"/>
                <xsl:map-entry key="'&#x8DB9;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8DBA;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x8DBB;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x8DBC;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8DBD;'" select="'fang4'"/>
                <xsl:map-entry key="'&#x8DBE;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x8DBF;'" select="'sa4	ta1'"/>
                <xsl:map-entry key="'&#x8DC0;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x8DC1;'" select="'pa2'"/>
                <xsl:map-entry key="'&#x8DC2;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8DC3;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x8DC4;'" select="'qiang1	qiang4'"/>
                <xsl:map-entry key="'&#x8DC5;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x8DC6;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x8DC7;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8DC8;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x8DC9;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8DCA;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x8DCB;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x8DCC;'" select="'die1	die2'"/>
                <xsl:map-entry key="'&#x8DCD;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x8DCE;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x8DCF;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x8DD0;'" select="'ci3'"/>
                <xsl:map-entry key="'&#x8DD1;'" select="'pao3	pao2'"/>
                <xsl:map-entry key="'&#x8DD2;'" select="'qia3'"/>
                <xsl:map-entry key="'&#x8DD3;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x8DD4;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x8DD5;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8DD6;'" select="'zhi1	zhi2'"/>
                <xsl:map-entry key="'&#x8DD7;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x8DD8;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x8DD9;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x8DDA;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x8DDB;'" select="'bo3'"/>
                <xsl:map-entry key="'&#x8DDC;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x8DDD;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x8DDE;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8DDF;'" select="'gen1'"/>
                <xsl:map-entry key="'&#x8DE0;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8DE1;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8DE2;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x8DE3;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x8DE4;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x8DE5;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x8DE6;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x8DE7;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8DE8;'" select="'kua4'"/>
                <xsl:map-entry key="'&#x8DE9;'" select="'zhuai3	shi4'"/>
                <xsl:map-entry key="'&#x8DEA;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x8DEB;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x8DEC;'" select="'kui3'"/>
                <xsl:map-entry key="'&#x8DED;'" select="'xiang2'"/>
                <xsl:map-entry key="'&#x8DEE;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x8DEF;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8DF0;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x8DF1;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8DF2;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x8DF3;'" select="'tiao4'"/>
                <xsl:map-entry key="'&#x8DF4;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x8DF5;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8DF6;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x8DF7;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x8DF8;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8DF9;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x8DFA;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x8DFB;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8DFC;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x8DFD;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x8DFE;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x8DFF;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x8E00;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x8E01;'" select="'xing4'"/>
                <xsl:map-entry key="'&#x8E02;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x8E03;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x8E04;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x8E05;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x8E06;'" select="'qun1'"/>
                <xsl:map-entry key="'&#x8E07;'" select="'mou3'"/>
                <xsl:map-entry key="'&#x8E08;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x8E09;'" select="'liang2	liang4'"/>
                <xsl:map-entry key="'&#x8E0A;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x8E0B;'" select="'jiao3	jia3	jue2'"/>
                <xsl:map-entry key="'&#x8E0C;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x8E0D;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x8E0E;'" select="''"/>
                <xsl:map-entry key="'&#x8E0F;'" select="'ta4	ta1'"/>
                <xsl:map-entry key="'&#x8E10;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8E11;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8E12;'" select="'wo1'"/>
                <xsl:map-entry key="'&#x8E13;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x8E14;'" select="'chuo1'"/>
                <xsl:map-entry key="'&#x8E15;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8E16;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x8E17;'" select="'nie1'"/>
                <xsl:map-entry key="'&#x8E18;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x8E19;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x8E1A;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x8E1B;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8E1C;'" select="'leng4'"/>
                <xsl:map-entry key="'&#x8E1D;'" select="'huai2'"/>
                <xsl:map-entry key="'&#x8E1E;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x8E1F;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x8E20;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x8E21;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8E22;'" select="'ti1'"/>
                <xsl:map-entry key="'&#x8E23;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x8E24;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x8E25;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x8E26;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x8E27;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x8E28;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x8E29;'" select="'cai3'"/>
                <xsl:map-entry key="'&#x8E2A;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x8E2B;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x8E2C;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8E2D;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x8E2E;'" select="'dian3	dian4	die1'"/>
                <xsl:map-entry key="'&#x8E2F;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x8E30;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8E31;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x8E32;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x8E33;'" select="'chun3'"/>
                <xsl:map-entry key="'&#x8E34;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x8E35;'" select="'zhong3'"/>
                <xsl:map-entry key="'&#x8E36;'" select="'di3'"/>
                <xsl:map-entry key="'&#x8E37;'" select="'zha3'"/>
                <xsl:map-entry key="'&#x8E38;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x8E39;'" select="'chuai4'"/>
                <xsl:map-entry key="'&#x8E3A;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x8E3B;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x8E3C;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x8E3D;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x8E3E;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8E3F;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x8E40;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8E41;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x8E42;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x8E43;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x8E44;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x8E45;'" select="'cha3'"/>
                <xsl:map-entry key="'&#x8E46;'" select="'tui3'"/>
                <xsl:map-entry key="'&#x8E47;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x8E48;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x8E49;'" select="'cuo1'"/>
                <xsl:map-entry key="'&#x8E4A;'" select="'xi1	qi1'"/>
                <xsl:map-entry key="'&#x8E4B;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x8E4C;'" select="'qiang1	qiang4'"/>
                <xsl:map-entry key="'&#x8E4D;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x8E4E;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x8E4F;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x8E50;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x8E51;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x8E52;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x8E53;'" select="'liu1	liu4'"/>
                <xsl:map-entry key="'&#x8E54;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x8E55;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8E56;'" select="'chong1'"/>
                <xsl:map-entry key="'&#x8E57;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8E58;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x8E59;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x8E5A;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x8E5B;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x8E5C;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x8E5D;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x8E5E;'" select="'kui3'"/>
                <xsl:map-entry key="'&#x8E5F;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8E60;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x8E61;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x8E62;'" select="'di2	zhi2'"/>
                <xsl:map-entry key="'&#x8E63;'" select="'man2	pan2'"/>
                <xsl:map-entry key="'&#x8E64;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x8E65;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8E66;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x8E67;'" select="'zao1'"/>
                <xsl:map-entry key="'&#x8E68;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x8E69;'" select="'bie2'"/>
                <xsl:map-entry key="'&#x8E6A;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x8E6B;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x8E6C;'" select="'deng4	deng1'"/>
                <xsl:map-entry key="'&#x8E6D;'" select="'ceng4'"/>
                <xsl:map-entry key="'&#x8E6E;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x8E6F;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x8E70;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x8E71;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x8E72;'" select="'dun1	cun2'"/>
                <xsl:map-entry key="'&#x8E73;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x8E74;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x8E75;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x8E76;'" select="'jue2	jue3'"/>
                <xsl:map-entry key="'&#x8E77;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8E78;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x8E79;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x8E7A;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x8E7B;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x8E7C;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x8E7D;'" select="'liao1'"/>
                <xsl:map-entry key="'&#x8E7E;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x8E7F;'" select="'cuan1'"/>
                <xsl:map-entry key="'&#x8E80;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x8E81;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x8E82;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x8E83;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8E84;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x8E85;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x8E86;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x8E87;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x8E88;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x8E89;'" select="'dun3'"/>
                <xsl:map-entry key="'&#x8E8A;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x8E8B;'" select="'ji1	ji4'"/>
                <xsl:map-entry key="'&#x8E8C;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x8E8D;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x8E8E;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x8E8F;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x8E90;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x8E91;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x8E92;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8E93;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8E94;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x8E95;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x8E96;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x8E97;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8E98;'" select="'long2'"/>
                <xsl:map-entry key="'&#x8E99;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x8E9A;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x8E9B;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8E9C;'" select="'zuan1'"/>
                <xsl:map-entry key="'&#x8E9D;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x8E9E;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x8E9F;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x8EA0;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x8EA1;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x8EA2;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x8EA3;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x8EA4;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x8EA5;'" select="'cuan1'"/>
                <xsl:map-entry key="'&#x8EA6;'" select="'zuan1'"/>
                <xsl:map-entry key="'&#x8EA7;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x8EA8;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x8EA9;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x8EAA;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x8EAB;'" select="'shen1	juan1'"/>
                <xsl:map-entry key="'&#x8EAC;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x8EAD;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x8EAE;'" select="''"/>
                <xsl:map-entry key="'&#x8EAF;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x8EB0;'" select="'ti3'"/>
                <xsl:map-entry key="'&#x8EB1;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x8EB2;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x8EB3;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x8EB4;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x8EB5;'" select="''"/>
                <xsl:map-entry key="'&#x8EB6;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x8EB7;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x8EB8;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8EB9;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x8EBA;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x8EBB;'" select="''"/>
                <xsl:map-entry key="'&#x8EBC;'" select="''"/>
                <xsl:map-entry key="'&#x8EBD;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x8EBE;'" select="''"/>
                <xsl:map-entry key="'&#x8EBF;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x8EC0;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x8EC1;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x8EC2;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x8EC3;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x8EC4;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x8EC5;'" select="''"/>
                <xsl:map-entry key="'&#x8EC6;'" select="'ti3'"/>
                <xsl:map-entry key="'&#x8EC7;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x8EC8;'" select="''"/>
                <xsl:map-entry key="'&#x8EC9;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8ECA;'" select="'che1	ju1'"/>
                <xsl:map-entry key="'&#x8ECB;'" select="'ya4	zha2'"/>
                <xsl:map-entry key="'&#x8ECC;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x8ECD;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x8ECE;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8ECF;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x8ED0;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x8ED1;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8ED2;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8ED3;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x8ED4;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x8ED5;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x8ED6;'" select="'qiang2'"/>
                <xsl:map-entry key="'&#x8ED7;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x8ED8;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x8ED9;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8EDA;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x8EDB;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8EDC;'" select="'na4'"/>
                <xsl:map-entry key="'&#x8EDD;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8EDE;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x8EDF;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x8EE0;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x8EE1;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x8EE2;'" select="'zhuan3	zhuai3	zhuan4'"/>
                <xsl:map-entry key="'&#x8EE3;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x8EE4;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x8EE5;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x8EE6;'" select="'huang4'"/>
                <xsl:map-entry key="'&#x8EE7;'" select="'di3'"/>
                <xsl:map-entry key="'&#x8EE8;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8EE9;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x8EEA;'" select="'ao1'"/>
                <xsl:map-entry key="'&#x8EEB;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x8EEC;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x8EED;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x8EEE;'" select="'ang3'"/>
                <xsl:map-entry key="'&#x8EEF;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x8EF0;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x8EF1;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x8EF2;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x8EF3;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x8EF4;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x8EF5;'" select="'rong3	fu3'"/>
                <xsl:map-entry key="'&#x8EF6;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8EF7;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x8EF8;'" select="'zhou2	zhou4	zhu2'"/>
                <xsl:map-entry key="'&#x8EF9;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x8EFA;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x8EFB;'" select="'ke1	ke3'"/>
                <xsl:map-entry key="'&#x8EFC;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8EFD;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x8EFE;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8EFF;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x8F00;'" select="'er2'"/>
                <xsl:map-entry key="'&#x8F01;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x8F02;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x8F03;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x8F04;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x8F05;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8F06;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x8F07;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8F08;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8F09;'" select="'zai4	zai3'"/>
                <xsl:map-entry key="'&#x8F0A;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8F0B;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x8F0C;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x8F0D;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x8F0E;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x8F0F;'" select="'you2'"/>
                <xsl:map-entry key="'&#x8F10;'" select="'huan3'"/>
                <xsl:map-entry key="'&#x8F11;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x8F12;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8F13;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x8F14;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x8F15;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x8F16;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8F17;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x8F18;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x8F19;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8F1A;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x8F1B;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x8F1C;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8F1D;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x8F1E;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x8F1F;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x8F20;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x8F21;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x8F22;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x8F23;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x8F24;'" select="'qian4'"/>
                <xsl:map-entry key="'&#x8F25;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x8F26;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x8F27;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x8F28;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x8F29;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x8F2A;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x8F2B;'" select="'pai2'"/>
                <xsl:map-entry key="'&#x8F2C;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x8F2D;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x8F2E;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x8F2F;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x8F30;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x8F31;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x8F32;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x8F33;'" select="'cou4'"/>
                <xsl:map-entry key="'&#x8F34;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x8F35;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x8F36;'" select="'you2'"/>
                <xsl:map-entry key="'&#x8F37;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x8F38;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x8F39;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x8F3A;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8F3B;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8F3C;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x8F3D;'" select="'ben4'"/>
                <xsl:map-entry key="'&#x8F3E;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x8F3F;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8F40;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x8F41;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x8F42;'" select="'gu3	gu1'"/>
                <xsl:map-entry key="'&#x8F43;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x8F44;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x8F45;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x8F46;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8F47;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x8F48;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x8F49;'" select="'zhuan3	zhuai3	zhuan4'"/>
                <xsl:map-entry key="'&#x8F4A;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x8F4B;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x8F4C;'" select="''"/>
                <xsl:map-entry key="'&#x8F4D;'" select="'che4	zhe2'"/>
                <xsl:map-entry key="'&#x8F4E;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x8F4F;'" select="'zhan4'"/>
                <xsl:map-entry key="'&#x8F50;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x8F51;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x8F52;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x8F53;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x8F54;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x8F55;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x8F56;'" select="'se4'"/>
                <xsl:map-entry key="'&#x8F57;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x8F58;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x8F59;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x8F5A;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x8F5B;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x8F5C;'" select="'er2'"/>
                <xsl:map-entry key="'&#x8F5D;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x8F5E;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x8F5F;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x8F60;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x8F61;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x8F62;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8F63;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8F64;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x8F65;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x8F66;'" select="'che1	ju1'"/>
                <xsl:map-entry key="'&#x8F67;'" select="'ya4	zha2'"/>
                <xsl:map-entry key="'&#x8F68;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x8F69;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x8F6A;'" select="'di4'"/>
                <xsl:map-entry key="'&#x8F6B;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x8F6C;'" select="'zhuan3	zhuai3	zhuan4'"/>
                <xsl:map-entry key="'&#x8F6D;'" select="'e4'"/>
                <xsl:map-entry key="'&#x8F6E;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x8F6F;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x8F70;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x8F71;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x8F72;'" select="'ke1	ke3'"/>
                <xsl:map-entry key="'&#x8F73;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x8F74;'" select="'zhou2	zhou4'"/>
                <xsl:map-entry key="'&#x8F75;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x8F76;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x8F77;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x8F78;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x8F79;'" select="'li4'"/>
                <xsl:map-entry key="'&#x8F7A;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x8F7B;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x8F7C;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x8F7D;'" select="'zai4	zai3'"/>
                <xsl:map-entry key="'&#x8F7E;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8F7F;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x8F80;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x8F81;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x8F82;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8F83;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x8F84;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8F85;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x8F86;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x8F87;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x8F88;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x8F89;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x8F8A;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x8F8B;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x8F8C;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x8F8D;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x8F8E;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x8F8F;'" select="'cou4'"/>
                <xsl:map-entry key="'&#x8F90;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x8F91;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x8F92;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x8F93;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x8F94;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x8F95;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x8F96;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x8F97;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x8F98;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x8F99;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x8F9A;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x8F9B;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x8F9C;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x8F9D;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x8F9E;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x8F9F;'" select="'pi4	bi4'"/>
                <xsl:map-entry key="'&#x8FA0;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x8FA1;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8FA2;'" select="'la4'"/>
                <xsl:map-entry key="'&#x8FA3;'" select="'la4'"/>
                <xsl:map-entry key="'&#x8FA4;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x8FA5;'" select="'xue1'"/>
                <xsl:map-entry key="'&#x8FA6;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x8FA7;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8FA8;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8FA9;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8FAA;'" select="''"/>
                <xsl:map-entry key="'&#x8FAB;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8FAC;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x8FAD;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x8FAE;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8FAF;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x8FB0;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8FB1;'" select="'ru4	ru3'"/>
                <xsl:map-entry key="'&#x8FB2;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x8FB3;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x8FB4;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x8FB5;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x8FB6;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x8FB7;'" select="''"/>
                <xsl:map-entry key="'&#x8FB8;'" select="'reng2'"/>
                <xsl:map-entry key="'&#x8FB9;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x8FBA;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x8FBB;'" select="''"/>
                <xsl:map-entry key="'&#x8FBC;'" select="''"/>
                <xsl:map-entry key="'&#x8FBD;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x8FBE;'" select="'da2'"/>
                <xsl:map-entry key="'&#x8FBF;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x8FC0;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x8FC1;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x8FC2;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x8FC3;'" select="'yu1'"/>
                <xsl:map-entry key="'&#x8FC4;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x8FC5;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x8FC6;'" select="'yi3	yi2'"/>
                <xsl:map-entry key="'&#x8FC7;'" select="'guo4	guo1'"/>
                <xsl:map-entry key="'&#x8FC8;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x8FC9;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x8FCA;'" select="'za1'"/>
                <xsl:map-entry key="'&#x8FCB;'" select="'wang4'"/>
                <xsl:map-entry key="'&#x8FCC;'" select="'jia'"/>
                <xsl:map-entry key="'&#x8FCD;'" select="'zhun1'"/>
                <xsl:map-entry key="'&#x8FCE;'" select="'ying2'"/>
                <xsl:map-entry key="'&#x8FCF;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x8FD0;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x8FD1;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x8FD2;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x8FD3;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x8FD4;'" select="'fan3'"/>
                <xsl:map-entry key="'&#x8FD5;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x8FD6;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x8FD7;'" select="'e2'"/>
                <xsl:map-entry key="'&#x8FD8;'" select="'hai2'"/>
                <xsl:map-entry key="'&#x8FD9;'" select="'zhe4	zhei4'"/>
                <xsl:map-entry key="'&#x8FDA;'" select="''"/>
                <xsl:map-entry key="'&#x8FDB;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x8FDC;'" select="'yuan3'"/>
                <xsl:map-entry key="'&#x8FDD;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x8FDE;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x8FDF;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x8FE0;'" select="'che4'"/>
                <xsl:map-entry key="'&#x8FE1;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x8FE2;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x8FE3;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x8FE4;'" select="'yi3	yi2'"/>
                <xsl:map-entry key="'&#x8FE5;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x8FE6;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x8FE7;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x8FE8;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x8FE9;'" select="'er3'"/>
                <xsl:map-entry key="'&#x8FEA;'" select="'di2'"/>
                <xsl:map-entry key="'&#x8FEB;'" select="'po4	pai3'"/>
                <xsl:map-entry key="'&#x8FEC;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x8FED;'" select="'die2'"/>
                <xsl:map-entry key="'&#x8FEE;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x8FEF;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x8FF0;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x8FF1;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x8FF2;'" select="''"/>
                <xsl:map-entry key="'&#x8FF3;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x8FF4;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x8FF5;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x8FF6;'" select="'you4'"/>
                <xsl:map-entry key="'&#x8FF7;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x8FF8;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x8FF9;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x8FFA;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x8FFB;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x8FFC;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x8FFD;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x8FFE;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x8FFF;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x9000;'" select="'tui4'"/>
                <xsl:map-entry key="'&#x9001;'" select="'song4'"/>
                <xsl:map-entry key="'&#x9002;'" select="'gua1	kuo4	shi4'"/>
                <xsl:map-entry key="'&#x9003;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x9004;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x9005;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x9006;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x9007;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x9008;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x9009;'" select="'xuan3'"/>
                <xsl:map-entry key="'&#x900A;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x900B;'" select="'bu1'"/>
                <xsl:map-entry key="'&#x900C;'" select="'you2'"/>
                <xsl:map-entry key="'&#x900D;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x900E;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x900F;'" select="'tou4'"/>
                <xsl:map-entry key="'&#x9010;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x9011;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x9012;'" select="'di4'"/>
                <xsl:map-entry key="'&#x9013;'" select="'di4'"/>
                <xsl:map-entry key="'&#x9014;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x9015;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x9016;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x9017;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x9018;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x9019;'" select="'zhe4	zhei4'"/>
                <xsl:map-entry key="'&#x901A;'" select="'tong1	tong4'"/>
                <xsl:map-entry key="'&#x901B;'" select="'guang4'"/>
                <xsl:map-entry key="'&#x901C;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x901D;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x901E;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x901F;'" select="'su4'"/>
                <xsl:map-entry key="'&#x9020;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x9021;'" select="'qun1'"/>
                <xsl:map-entry key="'&#x9022;'" select="'feng2'"/>
                <xsl:map-entry key="'&#x9023;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x9024;'" select="'suo4'"/>
                <xsl:map-entry key="'&#x9025;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x9026;'" select="'li3'"/>
                <xsl:map-entry key="'&#x9027;'" select="''"/>
                <xsl:map-entry key="'&#x9028;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x9029;'" select="'ben1'"/>
                <xsl:map-entry key="'&#x902A;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x902B;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x902C;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x902D;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x902E;'" select="'dai4	dai3'"/>
                <xsl:map-entry key="'&#x902F;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9030;'" select="'you2'"/>
                <xsl:map-entry key="'&#x9031;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x9032;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x9033;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9034;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x9035;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x9036;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x9037;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x9038;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9039;'" select="'da2'"/>
                <xsl:map-entry key="'&#x903A;'" select="'yuan3'"/>
                <xsl:map-entry key="'&#x903B;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x903C;'" select="'bi1'"/>
                <xsl:map-entry key="'&#x903D;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x903E;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x903F;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x9040;'" select="'sui2'"/>
                <xsl:map-entry key="'&#x9041;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x9042;'" select="'sui4	sui2'"/>
                <xsl:map-entry key="'&#x9043;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9044;'" select="'chuan2'"/>
                <xsl:map-entry key="'&#x9045;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x9046;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9047;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9048;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x9049;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x904A;'" select="'you2'"/>
                <xsl:map-entry key="'&#x904B;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x904C;'" select="'e4'"/>
                <xsl:map-entry key="'&#x904D;'" select="'bian4	pian4'"/>
                <xsl:map-entry key="'&#x904E;'" select="'guo4	guo1'"/>
                <xsl:map-entry key="'&#x904F;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9050;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x9051;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9052;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x9053;'" select="'dao4'"/>
                <xsl:map-entry key="'&#x9054;'" select="'da2'"/>
                <xsl:map-entry key="'&#x9055;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x9056;'" select="''"/>
                <xsl:map-entry key="'&#x9057;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9058;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x9059;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x905A;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x905B;'" select="'liu2	liu4'"/>
                <xsl:map-entry key="'&#x905C;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x905D;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x905E;'" select="'di4'"/>
                <xsl:map-entry key="'&#x905F;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x9060;'" select="'yuan3	yuan4'"/>
                <xsl:map-entry key="'&#x9061;'" select="'su4'"/>
                <xsl:map-entry key="'&#x9062;'" select="'ta4	ta1'"/>
                <xsl:map-entry key="'&#x9063;'" select="'qian3'"/>
                <xsl:map-entry key="'&#x9064;'" select="''"/>
                <xsl:map-entry key="'&#x9065;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x9066;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x9067;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x9068;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x9069;'" select="'shi4	kuo4'"/>
                <xsl:map-entry key="'&#x906A;'" select="'ce4'"/>
                <xsl:map-entry key="'&#x906B;'" select="'su4'"/>
                <xsl:map-entry key="'&#x906C;'" select="'su4'"/>
                <xsl:map-entry key="'&#x906D;'" select="'zao1'"/>
                <xsl:map-entry key="'&#x906E;'" select="'zhe1'"/>
                <xsl:map-entry key="'&#x906F;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x9070;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9071;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x9072;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x9073;'" select="'cuo1'"/>
                <xsl:map-entry key="'&#x9074;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x9075;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x9076;'" select="'rao4'"/>
                <xsl:map-entry key="'&#x9077;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9078;'" select="'xuan3'"/>
                <xsl:map-entry key="'&#x9079;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x907A;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x907B;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x907C;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x907D;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x907E;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x907F;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9080;'" select="'yao1'"/>
                <xsl:map-entry key="'&#x9081;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x9082;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x9083;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x9084;'" select="'huan2	hai2	xuan2'"/>
                <xsl:map-entry key="'&#x9085;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x9086;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x9087;'" select="'er3'"/>
                <xsl:map-entry key="'&#x9088;'" select="'miao3'"/>
                <xsl:map-entry key="'&#x9089;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x908A;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x908B;'" select="'la2	la1'"/>
                <xsl:map-entry key="'&#x908C;'" select="'li2'"/>
                <xsl:map-entry key="'&#x908D;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x908E;'" select="'you2'"/>
                <xsl:map-entry key="'&#x908F;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x9090;'" select="'li3'"/>
                <xsl:map-entry key="'&#x9091;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9092;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x9093;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x9094;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x9095;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x9096;'" select="'shan1'"/>
                <xsl:map-entry key="'&#x9097;'" select="'han2'"/>
                <xsl:map-entry key="'&#x9098;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9099;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x909A;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x909B;'" select="'qiong2'"/>
                <xsl:map-entry key="'&#x909C;'" select="''"/>
                <xsl:map-entry key="'&#x909D;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x909E;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x909F;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x90A0;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x90A1;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x90A2;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x90A3;'" select="'na4	na1'"/>
                <xsl:map-entry key="'&#x90A4;'" select="'xin'"/>
                <xsl:map-entry key="'&#x90A5;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x90A6;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x90A7;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x90A8;'" select="'cun1'"/>
                <xsl:map-entry key="'&#x90A9;'" select="'huo3'"/>
                <xsl:map-entry key="'&#x90AA;'" select="'xie2	ye2'"/>
                <xsl:map-entry key="'&#x90AB;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x90AC;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x90AD;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x90AE;'" select="'you2'"/>
                <xsl:map-entry key="'&#x90AF;'" select="'han2'"/>
                <xsl:map-entry key="'&#x90B0;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x90B1;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x90B2;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x90B3;'" select="'pei2	pi1'"/>
                <xsl:map-entry key="'&#x90B4;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x90B5;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x90B6;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x90B7;'" select="'wa3'"/>
                <xsl:map-entry key="'&#x90B8;'" select="'di3'"/>
                <xsl:map-entry key="'&#x90B9;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x90BA;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x90BB;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x90BC;'" select="'kuang1'"/>
                <xsl:map-entry key="'&#x90BD;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x90BE;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x90BF;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x90C0;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x90C1;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x90C2;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x90C3;'" select="'he2'"/>
                <xsl:map-entry key="'&#x90C4;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x90C5;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x90C6;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x90C7;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x90C8;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x90C9;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x90CA;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x90CB;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x90CC;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x90CD;'" select="'nuo2'"/>
                <xsl:map-entry key="'&#x90CE;'" select="'lang2	lang4'"/>
                <xsl:map-entry key="'&#x90CF;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x90D0;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x90D1;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x90D2;'" select="'lang2	lang4'"/>
                <xsl:map-entry key="'&#x90D3;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x90D4;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x90D5;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x90D6;'" select="'dou1'"/>
                <xsl:map-entry key="'&#x90D7;'" select="'chi1	xi1'"/>
                <xsl:map-entry key="'&#x90D8;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x90D9;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x90DA;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x90DB;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x90DC;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x90DD;'" select="'hao3	he4'"/>
                <xsl:map-entry key="'&#x90DE;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x90DF;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x90E0;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x90E1;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x90E2;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x90E3;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x90E4;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x90E5;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x90E6;'" select="'li4'"/>
                <xsl:map-entry key="'&#x90E7;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x90E8;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x90E9;'" select="'xiao2'"/>
                <xsl:map-entry key="'&#x90EA;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x90EB;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x90EC;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x90ED;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x90EE;'" select="'zhou'"/>
                <xsl:map-entry key="'&#x90EF;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x90F0;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x90F1;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x90F2;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x90F3;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x90F4;'" select="'chen1'"/>
                <xsl:map-entry key="'&#x90F5;'" select="'you2'"/>
                <xsl:map-entry key="'&#x90F6;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x90F7;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x90F8;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x90F9;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x90FA;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x90FB;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x90FC;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x90FD;'" select="'du1	dou1'"/>
                <xsl:map-entry key="'&#x90FE;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x90FF;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x9100;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x9101;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x9102;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9103;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9104;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x9105;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x9106;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x9107;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x9108;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x9109;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x910A;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x910B;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x910C;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x910D;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x910E;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x910F;'" select="'ru4'"/>
                <xsl:map-entry key="'&#x9110;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x9111;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9112;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x9113;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x9114;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x9115;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x9116;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x9117;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x9118;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x9119;'" select="'bi3	bi4'"/>
                <xsl:map-entry key="'&#x911A;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x911B;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x911C;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x911D;'" select="'liao3'"/>
                <xsl:map-entry key="'&#x911E;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x911F;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x9120;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x9121;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x9122;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x9123;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x9124;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x9125;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x9126;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x9127;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x9128;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9129;'" select="'xin2'"/>
                <xsl:map-entry key="'&#x912A;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x912B;'" select="'ceng2'"/>
                <xsl:map-entry key="'&#x912C;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x912D;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x912E;'" select="'mao4'"/>
                <xsl:map-entry key="'&#x912F;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9130;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x9131;'" select="'po2'"/>
                <xsl:map-entry key="'&#x9132;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x9133;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x9134;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9135;'" select="'cao1'"/>
                <xsl:map-entry key="'&#x9136;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x9137;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x9138;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x9139;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x913A;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x913B;'" select="'lian3'"/>
                <xsl:map-entry key="'&#x913C;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x913D;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x913E;'" select="'you1'"/>
                <xsl:map-entry key="'&#x913F;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9140;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x9141;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x9142;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x9143;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9144;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x9145;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x9146;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x9147;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x9148;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9149;'" select="'you3'"/>
                <xsl:map-entry key="'&#x914A;'" select="'ding3	ding1'"/>
                <xsl:map-entry key="'&#x914B;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x914C;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x914D;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x914E;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x914F;'" select="'yi2	yi3'"/>
                <xsl:map-entry key="'&#x9150;'" select="'hang3	gan1'"/>
                <xsl:map-entry key="'&#x9151;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x9152;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x9153;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9154;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x9155;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x9156;'" select="'dan1	zhen4'"/>
                <xsl:map-entry key="'&#x9157;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x9158;'" select="'tou2'"/>
                <xsl:map-entry key="'&#x9159;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x915A;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x915B;'" select="''"/>
                <xsl:map-entry key="'&#x915C;'" select="''"/>
                <xsl:map-entry key="'&#x915D;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x915E;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x915F;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x9160;'" select="'qia3'"/>
                <xsl:map-entry key="'&#x9161;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9162;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x9163;'" select="'han1'"/>
                <xsl:map-entry key="'&#x9164;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x9165;'" select="'su1'"/>
                <xsl:map-entry key="'&#x9166;'" select="'fa1	po1'"/>
                <xsl:map-entry key="'&#x9167;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x9168;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x9169;'" select="'ming2	ming3'"/>
                <xsl:map-entry key="'&#x916A;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x916B;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x916C;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x916D;'" select="'you4'"/>
                <xsl:map-entry key="'&#x916E;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x916F;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x9170;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x9171;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x9172;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x9173;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x9174;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x9175;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x9176;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x9177;'" select="'ku4'"/>
                <xsl:map-entry key="'&#x9178;'" select="'suan1'"/>
                <xsl:map-entry key="'&#x9179;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x917A;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x917B;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x917C;'" select="'hai3'"/>
                <xsl:map-entry key="'&#x917D;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x917E;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x917F;'" select="'niang4'"/>
                <xsl:map-entry key="'&#x9180;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x9181;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9182;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x9183;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x9184;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x9185;'" select="'pei1'"/>
                <xsl:map-entry key="'&#x9186;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x9187;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x9188;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x9189;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x918A;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x918B;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x918C;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x918D;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x918E;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x918F;'" select="'du1'"/>
                <xsl:map-entry key="'&#x9190;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9191;'" select="'xu3'"/>
                <xsl:map-entry key="'&#x9192;'" select="'xing3'"/>
                <xsl:map-entry key="'&#x9193;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x9194;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x9195;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x9196;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x9197;'" select="'fa1	po1'"/>
                <xsl:map-entry key="'&#x9198;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x9199;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x919A;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x919B;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x919C;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x919D;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x919E;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x919F;'" select="'yong4'"/>
                <xsl:map-entry key="'&#x91A0;'" select="'ang4'"/>
                <xsl:map-entry key="'&#x91A1;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x91A2;'" select="'hai3'"/>
                <xsl:map-entry key="'&#x91A3;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x91A4;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x91A5;'" select="'piao3'"/>
                <xsl:map-entry key="'&#x91A6;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x91A7;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x91A8;'" select="'li2'"/>
                <xsl:map-entry key="'&#x91A9;'" select="'zao2'"/>
                <xsl:map-entry key="'&#x91AA;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x91AB;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x91AC;'" select="'jiang4'"/>
                <xsl:map-entry key="'&#x91AD;'" select="'pu1	bu2'"/>
                <xsl:map-entry key="'&#x91AE;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x91AF;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x91B0;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x91B1;'" select="'fa1	po4'"/>
                <xsl:map-entry key="'&#x91B2;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x91B3;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x91B4;'" select="'li3'"/>
                <xsl:map-entry key="'&#x91B5;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x91B6;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x91B7;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x91B8;'" select="'niang4'"/>
                <xsl:map-entry key="'&#x91B9;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x91BA;'" select="'xun1'"/>
                <xsl:map-entry key="'&#x91BB;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x91BC;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x91BD;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x91BE;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x91BF;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x91C0;'" select="'niang4'"/>
                <xsl:map-entry key="'&#x91C1;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x91C2;'" select="'jiao4'"/>
                <xsl:map-entry key="'&#x91C3;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x91C4;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x91C5;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x91C6;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x91C7;'" select="'cai3	cai4'"/>
                <xsl:map-entry key="'&#x91C8;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x91C9;'" select="'you4'"/>
                <xsl:map-entry key="'&#x91CA;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x91CB;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x91CC;'" select="'li3'"/>
                <xsl:map-entry key="'&#x91CD;'" select="'zhong4	chong2'"/>
                <xsl:map-entry key="'&#x91CE;'" select="'ye3'"/>
                <xsl:map-entry key="'&#x91CF;'" select="'liang4	liang2'"/>
                <xsl:map-entry key="'&#x91D0;'" select="'li2'"/>
                <xsl:map-entry key="'&#x91D1;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x91D2;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x91D3;'" select="'ga2'"/>
                <xsl:map-entry key="'&#x91D4;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x91D5;'" select="'liao3	liao4'"/>
                <xsl:map-entry key="'&#x91D6;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x91D7;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x91D8;'" select="'ding1	ding4'"/>
                <xsl:map-entry key="'&#x91D9;'" select="'li4'"/>
                <xsl:map-entry key="'&#x91DA;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x91DB;'" select="'he2'"/>
                <xsl:map-entry key="'&#x91DC;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x91DD;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x91DE;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x91DF;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x91E0;'" select="'luan4'"/>
                <xsl:map-entry key="'&#x91E1;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x91E2;'" select="'nai3'"/>
                <xsl:map-entry key="'&#x91E3;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x91E4;'" select="'shan4	shan1'"/>
                <xsl:map-entry key="'&#x91E5;'" select="'qiao3'"/>
                <xsl:map-entry key="'&#x91E6;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x91E7;'" select="'chuan4'"/>
                <xsl:map-entry key="'&#x91E8;'" select="'zi3'"/>
                <xsl:map-entry key="'&#x91E9;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x91EA;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x91EB;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x91EC;'" select="'han4'"/>
                <xsl:map-entry key="'&#x91ED;'" select="'gong1	gang1'"/>
                <xsl:map-entry key="'&#x91EE;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x91EF;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x91F0;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x91F1;'" select="'di4'"/>
                <xsl:map-entry key="'&#x91F2;'" select="'si4'"/>
                <xsl:map-entry key="'&#x91F3;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x91F4;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x91F5;'" select="'chai1'"/>
                <xsl:map-entry key="'&#x91F6;'" select="'ta1	tuo2'"/>
                <xsl:map-entry key="'&#x91F7;'" select="'tu3'"/>
                <xsl:map-entry key="'&#x91F8;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x91F9;'" select="'nu:3'"/>
                <xsl:map-entry key="'&#x91FA;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x91FB;'" select="''"/>
                <xsl:map-entry key="'&#x91FC;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x91FD;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x91FE;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x91FF;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x9200;'" select="'ba3	pa2'"/>
                <xsl:map-entry key="'&#x9201;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x9202;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x9203;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9204;'" select="'tou3'"/>
                <xsl:map-entry key="'&#x9205;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x9206;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x9207;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x9208;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x9209;'" select="'na4'"/>
                <xsl:map-entry key="'&#x920A;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x920B;'" select="'e2'"/>
                <xsl:map-entry key="'&#x920C;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x920D;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x920E;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x920F;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x9210;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9211;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x9212;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x9213;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x9214;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x9215;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x9216;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x9217;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x9218;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9219;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x921A;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x921B;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x921C;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x921D;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x921E;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x921F;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x9220;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9221;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x9222;'" select="'nie1'"/>
                <xsl:map-entry key="'&#x9223;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x9224;'" select="'ri4'"/>
                <xsl:map-entry key="'&#x9225;'" select="'huo2	huo3'"/>
                <xsl:map-entry key="'&#x9226;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x9227;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x9228;'" select="''"/>
                <xsl:map-entry key="'&#x9229;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x922A;'" select="''"/>
                <xsl:map-entry key="'&#x922B;'" select="''"/>
                <xsl:map-entry key="'&#x922C;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x922D;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x922E;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x922F;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x9230;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x9231;'" select="'min2'"/>
                <xsl:map-entry key="'&#x9232;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x9233;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x9234;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9235;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x9236;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9237;'" select="'gu1	gu3'"/>
                <xsl:map-entry key="'&#x9238;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x9239;'" select="'pi1	pi2'"/>
                <xsl:map-entry key="'&#x923A;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x923B;'" select="'si4'"/>
                <xsl:map-entry key="'&#x923C;'" select="'zuo2'"/>
                <xsl:map-entry key="'&#x923D;'" select="'bu4	bu1'"/>
                <xsl:map-entry key="'&#x923E;'" select="'you2'"/>
                <xsl:map-entry key="'&#x923F;'" select="'dian4	tian2'"/>
                <xsl:map-entry key="'&#x9240;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x9241;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x9242;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x9243;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x9244;'" select="'tie3'"/>
                <xsl:map-entry key="'&#x9245;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x9246;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x9247;'" select="'ta1	tuo2'"/>
                <xsl:map-entry key="'&#x9248;'" select="'she2	ta1	tuo2'"/>
                <xsl:map-entry key="'&#x9249;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x924A;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x924B;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x924C;'" select="'he2'"/>
                <xsl:map-entry key="'&#x924D;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x924E;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x924F;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x9250;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x9251;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x9252;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x9253;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x9254;'" select="'za1'"/>
                <xsl:map-entry key="'&#x9255;'" select="'po3'"/>
                <xsl:map-entry key="'&#x9256;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x9257;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9258;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9259;'" select="'zhai3'"/>
                <xsl:map-entry key="'&#x925A;'" select="'liu3	mao3'"/>
                <xsl:map-entry key="'&#x925B;'" select="'qian1	yan2'"/>
                <xsl:map-entry key="'&#x925C;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x925D;'" select="'li4'"/>
                <xsl:map-entry key="'&#x925E;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x925F;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x9260;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x9261;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x9262;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x9263;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x9264;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x9265;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x9266;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x9267;'" select="'mu3'"/>
                <xsl:map-entry key="'&#x9268;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x9269;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x926A;'" select="'di4'"/>
                <xsl:map-entry key="'&#x926B;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x926C;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x926D;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x926E;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x926F;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x9270;'" select="'si1'"/>
                <xsl:map-entry key="'&#x9271;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x9272;'" select="'ka1'"/>
                <xsl:map-entry key="'&#x9273;'" select="'bei3'"/>
                <xsl:map-entry key="'&#x9274;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x9275;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x9276;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x9277;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x9278;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x9279;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x927A;'" select="'er3'"/>
                <xsl:map-entry key="'&#x927B;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x927C;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x927D;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x927E;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x927F;'" select="'jia2	ha1'"/>
                <xsl:map-entry key="'&#x9280;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x9281;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x9282;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x9283;'" select="'chong4'"/>
                <xsl:map-entry key="'&#x9284;'" select="'shang4'"/>
                <xsl:map-entry key="'&#x9285;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x9286;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9287;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x9288;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9289;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x928A;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x928B;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x928C;'" select="'cun4'"/>
                <xsl:map-entry key="'&#x928D;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x928E;'" select="'qiong1	qiong2'"/>
                <xsl:map-entry key="'&#x928F;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9290;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x9291;'" select="'xian3	xi3'"/>
                <xsl:map-entry key="'&#x9292;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x9293;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x9294;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x9295;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9296;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x9297;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x9298;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x9299;'" select="'kua3'"/>
                <xsl:map-entry key="'&#x929A;'" select="'yao2	diao4'"/>
                <xsl:map-entry key="'&#x929B;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x929C;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x929D;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x929E;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x929F;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x92A0;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x92A1;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x92A2;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x92A3;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x92A4;'" select="'mi3'"/>
                <xsl:map-entry key="'&#x92A5;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x92A6;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x92A7;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x92A8;'" select="'an1	an3'"/>
                <xsl:map-entry key="'&#x92A9;'" select="'diu1'"/>
                <xsl:map-entry key="'&#x92AA;'" select="'you3'"/>
                <xsl:map-entry key="'&#x92AB;'" select="'se4'"/>
                <xsl:map-entry key="'&#x92AC;'" select="'kao4'"/>
                <xsl:map-entry key="'&#x92AD;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x92AE;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x92AF;'" select="''"/>
                <xsl:map-entry key="'&#x92B0;'" select="'ai1'"/>
                <xsl:map-entry key="'&#x92B1;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x92B2;'" select="'han4'"/>
                <xsl:map-entry key="'&#x92B3;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x92B4;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x92B5;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x92B6;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x92B7;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x92B8;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x92B9;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x92BA;'" select="'zang4'"/>
                <xsl:map-entry key="'&#x92BB;'" select="'ti4	ti1'"/>
                <xsl:map-entry key="'&#x92BC;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x92BD;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x92BE;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x92BF;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x92C0;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x92C1;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x92C2;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x92C3;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x92C4;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x92C5;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x92C6;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x92C7;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x92C8;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x92C9;'" select="'su4'"/>
                <xsl:map-entry key="'&#x92CA;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x92CB;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x92CC;'" select="'ting3	ding4'"/>
                <xsl:map-entry key="'&#x92CD;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x92CE;'" select="'han4'"/>
                <xsl:map-entry key="'&#x92CF;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x92D0;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x92D1;'" select="'cuan1'"/>
                <xsl:map-entry key="'&#x92D2;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x92D3;'" select="'chan1'"/>
                <xsl:map-entry key="'&#x92D4;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x92D5;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x92D6;'" select="'si1'"/>
                <xsl:map-entry key="'&#x92D7;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x92D8;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x92D9;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x92DA;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x92DB;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x92DC;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x92DD;'" select="'lu:e4'"/>
                <xsl:map-entry key="'&#x92DE;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x92DF;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x92E0;'" select="'shen4'"/>
                <xsl:map-entry key="'&#x92E1;'" select="'han2'"/>
                <xsl:map-entry key="'&#x92E2;'" select="'l��4'"/>
                <xsl:map-entry key="'&#x92E3;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x92E4;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x92E5;'" select="'zeng4'"/>
                <xsl:map-entry key="'&#x92E6;'" select="'ju2	ju1'"/>
                <xsl:map-entry key="'&#x92E7;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x92E8;'" select="'e2'"/>
                <xsl:map-entry key="'&#x92E9;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x92EA;'" select="'pu1	pu4'"/>
                <xsl:map-entry key="'&#x92EB;'" select="'li2'"/>
                <xsl:map-entry key="'&#x92EC;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x92ED;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x92EE;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x92EF;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x92F0;'" select="'li3'"/>
                <xsl:map-entry key="'&#x92F1;'" select="'te4'"/>
                <xsl:map-entry key="'&#x92F2;'" select="''"/>
                <xsl:map-entry key="'&#x92F3;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x92F4;'" select="''"/>
                <xsl:map-entry key="'&#x92F5;'" select="'tu1'"/>
                <xsl:map-entry key="'&#x92F6;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x92F7;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x92F8;'" select="'ju4	ju1'"/>
                <xsl:map-entry key="'&#x92F9;'" select="'chang3'"/>
                <xsl:map-entry key="'&#x92FA;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x92FB;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x92FC;'" select="'gang1	gang4'"/>
                <xsl:map-entry key="'&#x92FD;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x92FE;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x92FF;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x9300;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x9301;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x9302;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9303;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x9304;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9305;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9306;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x9307;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x9308;'" select="'juan3'"/>
                <xsl:map-entry key="'&#x9309;'" select="'min2'"/>
                <xsl:map-entry key="'&#x930A;'" select="'zui4'"/>
                <xsl:map-entry key="'&#x930B;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x930C;'" select="'an4'"/>
                <xsl:map-entry key="'&#x930D;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x930E;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x930F;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x9310;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x9311;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x9312;'" select="'a1	e1'"/>
                <xsl:map-entry key="'&#x9313;'" select="'kong1'"/>
                <xsl:map-entry key="'&#x9314;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x9315;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9316;'" select="'du3'"/>
                <xsl:map-entry key="'&#x9317;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x9318;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x9319;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x931A;'" select="'zheng1	zheng4'"/>
                <xsl:map-entry key="'&#x931B;'" select="'ben1'"/>
                <xsl:map-entry key="'&#x931C;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x931D;'" select="'cong2'"/>
                <xsl:map-entry key="'&#x931E;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x931F;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x9320;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x9321;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9322;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9323;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x9324;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9325;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9326;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x9327;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x9328;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x9329;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x932A;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x932B;'" select="'xi2	xi1'"/>
                <xsl:map-entry key="'&#x932C;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x932D;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x932E;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x932F;'" select="'cuo4	cu4'"/>
                <xsl:map-entry key="'&#x9330;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x9331;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x9332;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9333;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x9334;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9335;'" select="'hua1'"/>
                <xsl:map-entry key="'&#x9336;'" select="'biao3'"/>
                <xsl:map-entry key="'&#x9337;'" select="'ga2'"/>
                <xsl:map-entry key="'&#x9338;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x9339;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x933A;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x933B;'" select="''"/>
                <xsl:map-entry key="'&#x933C;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x933D;'" select="'wan3'"/>
                <xsl:map-entry key="'&#x933E;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x933F;'" select="''"/>
                <xsl:map-entry key="'&#x9340;'" select="'de2'"/>
                <xsl:map-entry key="'&#x9341;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x9342;'" select="''"/>
                <xsl:map-entry key="'&#x9343;'" select="'huo1'"/>
                <xsl:map-entry key="'&#x9344;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x9345;'" select="''"/>
                <xsl:map-entry key="'&#x9346;'" select="'men2'"/>
                <xsl:map-entry key="'&#x9347;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x9348;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9349;'" select="'di1'"/>
                <xsl:map-entry key="'&#x934A;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x934B;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x934C;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x934D;'" select="'du4'"/>
                <xsl:map-entry key="'&#x934E;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x934F;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x9350;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x9351;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x9352;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x9353;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x9354;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9355;'" select="'rou3'"/>
                <xsl:map-entry key="'&#x9356;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x9357;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9358;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x9359;'" select="'hong4'"/>
                <xsl:map-entry key="'&#x935A;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x935B;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x935C;'" select="'xia1'"/>
                <xsl:map-entry key="'&#x935D;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x935E;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x935F;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x9360;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9361;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x9362;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x9363;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x9364;'" select="'cha2	cha1'"/>
                <xsl:map-entry key="'&#x9365;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x9366;'" select="'she2'"/>
                <xsl:map-entry key="'&#x9367;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x9368;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x9369;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x936A;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x936B;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x936C;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x936D;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x936E;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x936F;'" select="'huo1'"/>
                <xsl:map-entry key="'&#x9370;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x9371;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9372;'" select="'min2'"/>
                <xsl:map-entry key="'&#x9373;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x9374;'" select="'duan3'"/>
                <xsl:map-entry key="'&#x9375;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x9376;'" select="'si1'"/>
                <xsl:map-entry key="'&#x9377;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x9378;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9379;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x937A;'" select="'zang1	zhe3'"/>
                <xsl:map-entry key="'&#x937B;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x937C;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x937D;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x937E;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x937F;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9380;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x9381;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x9382;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x9383;'" select="'pai4'"/>
                <xsl:map-entry key="'&#x9384;'" select="'ai1'"/>
                <xsl:map-entry key="'&#x9385;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x9386;'" select="''"/>
                <xsl:map-entry key="'&#x9387;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x9388;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x9389;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x938A;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x938B;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x938C;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x938D;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x938E;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x938F;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9390;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x9391;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9392;'" select="'nou4'"/>
                <xsl:map-entry key="'&#x9393;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x9394;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x9395;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x9396;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x9397;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x9398;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x9399;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x939A;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x939B;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x939C;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x939D;'" select="'ta3'"/>
                <xsl:map-entry key="'&#x939E;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x939F;'" select="'sang3'"/>
                <xsl:map-entry key="'&#x93A0;'" select="'gang1'"/>
                <xsl:map-entry key="'&#x93A1;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x93A2;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x93A3;'" select="'ying4	ying2'"/>
                <xsl:map-entry key="'&#x93A4;'" select="'huang3'"/>
                <xsl:map-entry key="'&#x93A5;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x93A6;'" select="'liu2	liu4'"/>
                <xsl:map-entry key="'&#x93A7;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x93A8;'" select="'sun3'"/>
                <xsl:map-entry key="'&#x93A9;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x93AA;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x93AB;'" select="'wan4'"/>
                <xsl:map-entry key="'&#x93AC;'" select="'hao4	gao3'"/>
                <xsl:map-entry key="'&#x93AD;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x93AE;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x93AF;'" select="'luo3'"/>
                <xsl:map-entry key="'&#x93B0;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x93B1;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x93B2;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x93B3;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x93B4;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x93B5;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x93B6;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x93B7;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x93B8;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x93B9;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x93BA;'" select="''"/>
                <xsl:map-entry key="'&#x93BB;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x93BC;'" select="''"/>
                <xsl:map-entry key="'&#x93BD;'" select="''"/>
                <xsl:map-entry key="'&#x93BE;'" select="''"/>
                <xsl:map-entry key="'&#x93BF;'" select="'na2'"/>
                <xsl:map-entry key="'&#x93C0;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x93C1;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x93C2;'" select="'kou1'"/>
                <xsl:map-entry key="'&#x93C3;'" select="'zu2	cu4'"/>
                <xsl:map-entry key="'&#x93C4;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x93C5;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x93C6;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x93C7;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x93C8;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x93C9;'" select="'shou4'"/>
                <xsl:map-entry key="'&#x93CA;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x93CB;'" select="'man3'"/>
                <xsl:map-entry key="'&#x93CC;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x93CD;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x93CE;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x93CF;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x93D0;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x93D1;'" select="'di2	di1'"/>
                <xsl:map-entry key="'&#x93D2;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x93D3;'" select="'huo1'"/>
                <xsl:map-entry key="'&#x93D4;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x93D5;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x93D6;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x93D7;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x93D8;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x93D9;'" select="'cui1'"/>
                <xsl:map-entry key="'&#x93DA;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x93DB;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x93DC;'" select="'tang1	tang2'"/>
                <xsl:map-entry key="'&#x93DD;'" select="'man4'"/>
                <xsl:map-entry key="'&#x93DE;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x93DF;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x93E0;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x93E1;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x93E2;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x93E3;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x93E4;'" select="'lou4'"/>
                <xsl:map-entry key="'&#x93E5;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x93E6;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x93E7;'" select="'long2'"/>
                <xsl:map-entry key="'&#x93E8;'" select="'zan4'"/>
                <xsl:map-entry key="'&#x93E9;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x93EA;'" select="'cao2'"/>
                <xsl:map-entry key="'&#x93EB;'" select="'li2'"/>
                <xsl:map-entry key="'&#x93EC;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x93ED;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x93EE;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x93EF;'" select="''"/>
                <xsl:map-entry key="'&#x93F0;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x93F1;'" select="''"/>
                <xsl:map-entry key="'&#x93F2;'" select="''"/>
                <xsl:map-entry key="'&#x93F3;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x93F4;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x93F5;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x93F6;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x93F7;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x93F8;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x93F9;'" select="'qiang1	qiang3'"/>
                <xsl:map-entry key="'&#x93FA;'" select="'po1'"/>
                <xsl:map-entry key="'&#x93FB;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x93FC;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x93FD;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x93FE;'" select="'san3'"/>
                <xsl:map-entry key="'&#x93FF;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x9400;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x9401;'" select="'san3'"/>
                <xsl:map-entry key="'&#x9402;'" select="'liu4	liu2'"/>
                <xsl:map-entry key="'&#x9403;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x9404;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9405;'" select="'pie3'"/>
                <xsl:map-entry key="'&#x9406;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x9407;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x9408;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x9409;'" select="'chuan1'"/>
                <xsl:map-entry key="'&#x940A;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x940B;'" select="'tang4	tang1'"/>
                <xsl:map-entry key="'&#x940C;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x940D;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x940E;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x940F;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x9410;'" select="'liao2	liao4'"/>
                <xsl:map-entry key="'&#x9411;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x9412;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x9413;'" select="'dui4	dun1'"/>
                <xsl:map-entry key="'&#x9414;'" select="'tan2	chan2	xin2'"/>
                <xsl:map-entry key="'&#x9415;'" select="'zan1'"/>
                <xsl:map-entry key="'&#x9416;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9417;'" select="'jian3	jian4'"/>
                <xsl:map-entry key="'&#x9418;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x9419;'" select="'deng1	deng4'"/>
                <xsl:map-entry key="'&#x941A;'" select="'lou4	lue2'"/>
                <xsl:map-entry key="'&#x941B;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x941C;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x941D;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x941E;'" select="'nou4'"/>
                <xsl:map-entry key="'&#x941F;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x9420;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x9421;'" select="'tie3'"/>
                <xsl:map-entry key="'&#x9422;'" select="''"/>
                <xsl:map-entry key="'&#x9423;'" select="''"/>
                <xsl:map-entry key="'&#x9424;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x9425;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9426;'" select="'kai1'"/>
                <xsl:map-entry key="'&#x9427;'" select="'jian3	jian4'"/>
                <xsl:map-entry key="'&#x9428;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x9429;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x942A;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x942B;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x942C;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x942D;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x942E;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x942F;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x9430;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x9431;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9432;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x9433;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x9434;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9435;'" select="'tie3'"/>
                <xsl:map-entry key="'&#x9436;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x9437;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9438;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x9439;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x943A;'" select="'dang1	cheng1'"/>
                <xsl:map-entry key="'&#x943B;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x943C;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x943D;'" select="'da2'"/>
                <xsl:map-entry key="'&#x943E;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x943F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9440;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x9441;'" select="'dang1	zheng1'"/>
                <xsl:map-entry key="'&#x9442;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x9443;'" select="'diao4	yao2'"/>
                <xsl:map-entry key="'&#x9444;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x9445;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x9446;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x9447;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9448;'" select="'nie1'"/>
                <xsl:map-entry key="'&#x9449;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x944A;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x944B;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x944C;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x944D;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x944E;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x944F;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x9450;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x9451;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x9452;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x9453;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x9454;'" select="'cha3'"/>
                <xsl:map-entry key="'&#x9455;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9456;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x9457;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9458;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x9459;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x945A;'" select="'zuan4	zuan1'"/>
                <xsl:map-entry key="'&#x945B;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x945C;'" select="'shang3'"/>
                <xsl:map-entry key="'&#x945D;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x945E;'" select="'la4'"/>
                <xsl:map-entry key="'&#x945F;'" select="'du2'"/>
                <xsl:map-entry key="'&#x9460;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x9461;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x9462;'" select="'lu:4'"/>
                <xsl:map-entry key="'&#x9463;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x9464;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x9465;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x9466;'" select="''"/>
                <xsl:map-entry key="'&#x9467;'" select="''"/>
                <xsl:map-entry key="'&#x9468;'" select="'long2'"/>
                <xsl:map-entry key="'&#x9469;'" select="'e4'"/>
                <xsl:map-entry key="'&#x946A;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x946B;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x946C;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x946D;'" select="'lan4	lan2'"/>
                <xsl:map-entry key="'&#x946E;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x946F;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9470;'" select="'yao4	yue4'"/>
                <xsl:map-entry key="'&#x9471;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x9472;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x9473;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x9474;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x9475;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x9476;'" select="'cang2'"/>
                <xsl:map-entry key="'&#x9477;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x9478;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x9479;'" select="'cuan1'"/>
                <xsl:map-entry key="'&#x947A;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x947B;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x947C;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x947D;'" select="'zuan4	zuan1	zuan3'"/>
                <xsl:map-entry key="'&#x947E;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x947F;'" select="'zao2	zuo4'"/>
                <xsl:map-entry key="'&#x9480;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x9481;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x9482;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x9483;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x9484;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x9485;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x9486;'" select="'ga2'"/>
                <xsl:map-entry key="'&#x9487;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x9488;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x9489;'" select="'ding1	ding4'"/>
                <xsl:map-entry key="'&#x948A;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x948B;'" select="'po1'"/>
                <xsl:map-entry key="'&#x948C;'" select="'liao3	liao4'"/>
                <xsl:map-entry key="'&#x948D;'" select="'tu3'"/>
                <xsl:map-entry key="'&#x948E;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x948F;'" select="'chuan4'"/>
                <xsl:map-entry key="'&#x9490;'" select="'shan4	shan1'"/>
                <xsl:map-entry key="'&#x9491;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x9492;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x9493;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x9494;'" select="'men2'"/>
                <xsl:map-entry key="'&#x9495;'" select="'nu:3'"/>
                <xsl:map-entry key="'&#x9496;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x9497;'" select="'chai1'"/>
                <xsl:map-entry key="'&#x9498;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x9499;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x949A;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x949B;'" select="'tai4'"/>
                <xsl:map-entry key="'&#x949C;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x949D;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x949E;'" select="'chao1'"/>
                <xsl:map-entry key="'&#x949F;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x94A0;'" select="'na4'"/>
                <xsl:map-entry key="'&#x94A1;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x94A2;'" select="'gang1	gang4'"/>
                <xsl:map-entry key="'&#x94A3;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x94A4;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x94A5;'" select="'yao4	yue4'"/>
                <xsl:map-entry key="'&#x94A6;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x94A7;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x94A8;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x94A9;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x94AA;'" select="'kang4'"/>
                <xsl:map-entry key="'&#x94AB;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x94AC;'" select="'huo3'"/>
                <xsl:map-entry key="'&#x94AD;'" select="'tou3'"/>
                <xsl:map-entry key="'&#x94AE;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x94AF;'" select="'ba3	pa2'"/>
                <xsl:map-entry key="'&#x94B0;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x94B1;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x94B2;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x94B3;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x94B4;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x94B5;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x94B6;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x94B7;'" select="'po3'"/>
                <xsl:map-entry key="'&#x94B8;'" select="'bu1'"/>
                <xsl:map-entry key="'&#x94B9;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x94BA;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x94BB;'" select="'zuan4	zuan1'"/>
                <xsl:map-entry key="'&#x94BC;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x94BD;'" select="'tan3'"/>
                <xsl:map-entry key="'&#x94BE;'" select="'jia3'"/>
                <xsl:map-entry key="'&#x94BF;'" select="'dian4	tian2'"/>
                <xsl:map-entry key="'&#x94C0;'" select="'you2'"/>
                <xsl:map-entry key="'&#x94C1;'" select="'tie3'"/>
                <xsl:map-entry key="'&#x94C2;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x94C3;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x94C4;'" select="'shuo4'"/>
                <xsl:map-entry key="'&#x94C5;'" select="'qian1	yan2'"/>
                <xsl:map-entry key="'&#x94C6;'" select="'mao3'"/>
                <xsl:map-entry key="'&#x94C7;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x94C8;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x94C9;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x94CA;'" select="'ta1	tuo2'"/>
                <xsl:map-entry key="'&#x94CB;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x94CC;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x94CD;'" select="'pi1	pi2'"/>
                <xsl:map-entry key="'&#x94CE;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x94CF;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x94D0;'" select="'kao4'"/>
                <xsl:map-entry key="'&#x94D1;'" select="'lao3'"/>
                <xsl:map-entry key="'&#x94D2;'" select="'er3'"/>
                <xsl:map-entry key="'&#x94D3;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x94D4;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x94D5;'" select="'you3'"/>
                <xsl:map-entry key="'&#x94D6;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x94D7;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x94D8;'" select="'ye2'"/>
                <xsl:map-entry key="'&#x94D9;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x94DA;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x94DB;'" select="'dang1	cheng1'"/>
                <xsl:map-entry key="'&#x94DC;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x94DD;'" select="'lu:3'"/>
                <xsl:map-entry key="'&#x94DE;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x94DF;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x94E0;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x94E1;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x94E2;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x94E3;'" select="'xian3	xi3'"/>
                <xsl:map-entry key="'&#x94E4;'" select="'ting3	ding4'"/>
                <xsl:map-entry key="'&#x94E5;'" select="'diu1'"/>
                <xsl:map-entry key="'&#x94E6;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x94E7;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x94E8;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x94E9;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x94EA;'" select="'ha1'"/>
                <xsl:map-entry key="'&#x94EB;'" select="'yao2	diao4'"/>
                <xsl:map-entry key="'&#x94EC;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x94ED;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x94EE;'" select="'zheng1	zheng4'"/>
                <xsl:map-entry key="'&#x94EF;'" select="'se4'"/>
                <xsl:map-entry key="'&#x94F0;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x94F1;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x94F2;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x94F3;'" select="'chong4'"/>
                <xsl:map-entry key="'&#x94F4;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x94F5;'" select="'an3'"/>
                <xsl:map-entry key="'&#x94F6;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x94F7;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x94F8;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x94F9;'" select="'lao2'"/>
                <xsl:map-entry key="'&#x94FA;'" select="'pu1	pu4'"/>
                <xsl:map-entry key="'&#x94FB;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x94FC;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x94FD;'" select="'te4'"/>
                <xsl:map-entry key="'&#x94FE;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x94FF;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x9500;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x9501;'" select="'suo3'"/>
                <xsl:map-entry key="'&#x9502;'" select="'li3'"/>
                <xsl:map-entry key="'&#x9503;'" select="'zeng4'"/>
                <xsl:map-entry key="'&#x9504;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x9505;'" select="'guo1'"/>
                <xsl:map-entry key="'&#x9506;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x9507;'" select="'e2'"/>
                <xsl:map-entry key="'&#x9508;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x9509;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x950A;'" select="'lu:e4'"/>
                <xsl:map-entry key="'&#x950B;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x950C;'" select="'xin1'"/>
                <xsl:map-entry key="'&#x950D;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x950E;'" select="'kai1'"/>
                <xsl:map-entry key="'&#x950F;'" select="'jian3	jian4'"/>
                <xsl:map-entry key="'&#x9510;'" select="'rui4'"/>
                <xsl:map-entry key="'&#x9511;'" select="'ti1'"/>
                <xsl:map-entry key="'&#x9512;'" select="'lang2'"/>
                <xsl:map-entry key="'&#x9513;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x9514;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x9515;'" select="'a1'"/>
                <xsl:map-entry key="'&#x9516;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x9517;'" select="'zang1	zhe3'"/>
                <xsl:map-entry key="'&#x9518;'" select="'nuo4'"/>
                <xsl:map-entry key="'&#x9519;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x951A;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x951B;'" select="'ben1'"/>
                <xsl:map-entry key="'&#x951C;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x951D;'" select="'de2'"/>
                <xsl:map-entry key="'&#x951E;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x951F;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9520;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x9521;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x9522;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x9523;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x9524;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x9525;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x9526;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x9527;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9528;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x9529;'" select="'juan3'"/>
                <xsl:map-entry key="'&#x952A;'" select="'huo1'"/>
                <xsl:map-entry key="'&#x952B;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x952C;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x952D;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x952E;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x952F;'" select="'ju4	ju1'"/>
                <xsl:map-entry key="'&#x9530;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x9531;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9532;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x9533;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9534;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x9535;'" select="'qiang1'"/>
                <xsl:map-entry key="'&#x9536;'" select="'si1'"/>
                <xsl:map-entry key="'&#x9537;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9538;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x9539;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x953A;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x953B;'" select="'duan4'"/>
                <xsl:map-entry key="'&#x953C;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x953D;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x953E;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x953F;'" select="'ai1'"/>
                <xsl:map-entry key="'&#x9540;'" select="'du4'"/>
                <xsl:map-entry key="'&#x9541;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x9542;'" select="'lou4'"/>
                <xsl:map-entry key="'&#x9543;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9544;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x9545;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x9546;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9547;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x9548;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x9549;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x954A;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x954B;'" select="'tang3'"/>
                <xsl:map-entry key="'&#x954C;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x954D;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x954E;'" select="'na2'"/>
                <xsl:map-entry key="'&#x954F;'" select="'liu2	liu4'"/>
                <xsl:map-entry key="'&#x9550;'" select="'hao4	gao3'"/>
                <xsl:map-entry key="'&#x9551;'" select="'bang4'"/>
                <xsl:map-entry key="'&#x9552;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9553;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x9554;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x9555;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x9556;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x9557;'" select="'tang1	tang2'"/>
                <xsl:map-entry key="'&#x9558;'" select="'man4'"/>
                <xsl:map-entry key="'&#x9559;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x955A;'" select="'beng4'"/>
                <xsl:map-entry key="'&#x955B;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x955C;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x955D;'" select="'di2	di1'"/>
                <xsl:map-entry key="'&#x955E;'" select="'zu2'"/>
                <xsl:map-entry key="'&#x955F;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x9560;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9561;'" select="'chan2	xin2'"/>
                <xsl:map-entry key="'&#x9562;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x9563;'" select="'liao4'"/>
                <xsl:map-entry key="'&#x9564;'" select="'pu2'"/>
                <xsl:map-entry key="'&#x9565;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x9566;'" select="'dui4	dun1'"/>
                <xsl:map-entry key="'&#x9567;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x9568;'" select="'pu3'"/>
                <xsl:map-entry key="'&#x9569;'" select="'cuan1'"/>
                <xsl:map-entry key="'&#x956A;'" select="'qiang1	qiang3'"/>
                <xsl:map-entry key="'&#x956B;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x956C;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x956D;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x956E;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x956F;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x9570;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x9571;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9572;'" select="'cha3'"/>
                <xsl:map-entry key="'&#x9573;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x9574;'" select="'la4'"/>
                <xsl:map-entry key="'&#x9575;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x9576;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x9577;'" select="'chang2	zhang3'"/>
                <xsl:map-entry key="'&#x9578;'" select="'chang2	zhang3'"/>
                <xsl:map-entry key="'&#x9579;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x957A;'" select="'ao3'"/>
                <xsl:map-entry key="'&#x957B;'" select="'die2'"/>
                <xsl:map-entry key="'&#x957C;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x957D;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x957E;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x957F;'" select="'chang2	zhang3'"/>
                <xsl:map-entry key="'&#x9580;'" select="'men2'"/>
                <xsl:map-entry key="'&#x9581;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x9582;'" select="'shuan1'"/>
                <xsl:map-entry key="'&#x9583;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x9584;'" select="'huo2'"/>
                <xsl:map-entry key="'&#x9585;'" select="'men2'"/>
                <xsl:map-entry key="'&#x9586;'" select="'yan4	yan2'"/>
                <xsl:map-entry key="'&#x9587;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9588;'" select="'han4'"/>
                <xsl:map-entry key="'&#x9589;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x958A;'" select="''"/>
                <xsl:map-entry key="'&#x958B;'" select="'kai1'"/>
                <xsl:map-entry key="'&#x958C;'" select="'kang4	kang1'"/>
                <xsl:map-entry key="'&#x958D;'" select="'beng1'"/>
                <xsl:map-entry key="'&#x958E;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x958F;'" select="'run4'"/>
                <xsl:map-entry key="'&#x9590;'" select="'san4'"/>
                <xsl:map-entry key="'&#x9591;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x9592;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x9593;'" select="'jian1	jian4'"/>
                <xsl:map-entry key="'&#x9594;'" select="'min3'"/>
                <xsl:map-entry key="'&#x9595;'" select="'xia1'"/>
                <xsl:map-entry key="'&#x9596;'" select="'min3'"/>
                <xsl:map-entry key="'&#x9597;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x9598;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x9599;'" select="'nao4'"/>
                <xsl:map-entry key="'&#x959A;'" select="'jian'"/>
                <xsl:map-entry key="'&#x959B;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x959C;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x959D;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x959E;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x959F;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x95A0;'" select="'run4'"/>
                <xsl:map-entry key="'&#x95A1;'" select="'he2'"/>
                <xsl:map-entry key="'&#x95A2;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x95A3;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x95A4;'" select="'he2'"/>
                <xsl:map-entry key="'&#x95A5;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x95A6;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x95A7;'" select="'hong4	hong1	hong3'"/>
                <xsl:map-entry key="'&#x95A8;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x95A9;'" select="'min3'"/>
                <xsl:map-entry key="'&#x95AA;'" select="''"/>
                <xsl:map-entry key="'&#x95AB;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x95AC;'" select="'lang3	lang2	lang4'"/>
                <xsl:map-entry key="'&#x95AD;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x95AE;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x95AF;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x95B0;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x95B1;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x95B2;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x95B3;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x95B4;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x95B5;'" select="'lin4'"/>
                <xsl:map-entry key="'&#x95B6;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x95B7;'" select="'shai4'"/>
                <xsl:map-entry key="'&#x95B8;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x95B9;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x95BA;'" select="'min2	wen2'"/>
                <xsl:map-entry key="'&#x95BB;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x95BC;'" select="'e4	yan1'"/>
                <xsl:map-entry key="'&#x95BD;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x95BE;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x95BF;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x95C0;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x95C1;'" select="'bao'"/>
                <xsl:map-entry key="'&#x95C2;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x95C3;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x95C4;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x95C5;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x95C6;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x95C7;'" select="'an4'"/>
                <xsl:map-entry key="'&#x95C8;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x95C9;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x95CA;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x95CB;'" select="'que4'"/>
                <xsl:map-entry key="'&#x95CC;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x95CD;'" select="'du1'"/>
                <xsl:map-entry key="'&#x95CE;'" select="''"/>
                <xsl:map-entry key="'&#x95CF;'" select="''"/>
                <xsl:map-entry key="'&#x95D0;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x95D1;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x95D2;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x95D3;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x95D4;'" select="'he2'"/>
                <xsl:map-entry key="'&#x95D5;'" select="'que4	que1'"/>
                <xsl:map-entry key="'&#x95D6;'" select="'chuang3	chuang4'"/>
                <xsl:map-entry key="'&#x95D7;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x95D8;'" select="'dou4	dou3'"/>
                <xsl:map-entry key="'&#x95D9;'" select="'qi3'"/>
                <xsl:map-entry key="'&#x95DA;'" select="'kui1'"/>
                <xsl:map-entry key="'&#x95DB;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x95DC;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x95DD;'" select="'piao2'"/>
                <xsl:map-entry key="'&#x95DE;'" select="'kan4	han3	kan3'"/>
                <xsl:map-entry key="'&#x95DF;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x95E0;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x95E1;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x95E2;'" select="'pi4	bi4	pi1'"/>
                <xsl:map-entry key="'&#x95E3;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x95E4;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x95E5;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x95E6;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x95E7;'" select="''"/>
                <xsl:map-entry key="'&#x95E8;'" select="'men2'"/>
                <xsl:map-entry key="'&#x95E9;'" select="'shuan1'"/>
                <xsl:map-entry key="'&#x95EA;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x95EB;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x95EC;'" select="'han4'"/>
                <xsl:map-entry key="'&#x95ED;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x95EE;'" select="'wen4'"/>
                <xsl:map-entry key="'&#x95EF;'" select="'chuang3'"/>
                <xsl:map-entry key="'&#x95F0;'" select="'run4'"/>
                <xsl:map-entry key="'&#x95F1;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x95F2;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x95F3;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x95F4;'" select="'jian1	jian4'"/>
                <xsl:map-entry key="'&#x95F5;'" select="'min3'"/>
                <xsl:map-entry key="'&#x95F6;'" select="'kang4	kang1'"/>
                <xsl:map-entry key="'&#x95F7;'" select="'men4	men1'"/>
                <xsl:map-entry key="'&#x95F8;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x95F9;'" select="'nao4'"/>
                <xsl:map-entry key="'&#x95FA;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x95FB;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x95FC;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x95FD;'" select="'min3'"/>
                <xsl:map-entry key="'&#x95FE;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x95FF;'" select="'kai1'"/>
                <xsl:map-entry key="'&#x9600;'" select="'fa2'"/>
                <xsl:map-entry key="'&#x9601;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x9602;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9603;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x9604;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x9605;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x9606;'" select="'lang2	lang4'"/>
                <xsl:map-entry key="'&#x9607;'" select="'du1'"/>
                <xsl:map-entry key="'&#x9608;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9609;'" select="'yan1'"/>
                <xsl:map-entry key="'&#x960A;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x960B;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x960C;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x960D;'" select="'hun1'"/>
                <xsl:map-entry key="'&#x960E;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x960F;'" select="'e4	yan1'"/>
                <xsl:map-entry key="'&#x9610;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x9611;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x9612;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x9613;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x9614;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x9615;'" select="'que4'"/>
                <xsl:map-entry key="'&#x9616;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9617;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x9618;'" select="'da2'"/>
                <xsl:map-entry key="'&#x9619;'" select="'que1'"/>
                <xsl:map-entry key="'&#x961A;'" select="'han3	kan3'"/>
                <xsl:map-entry key="'&#x961B;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x961C;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x961D;'" select="'fu4	yi4'"/>
                <xsl:map-entry key="'&#x961E;'" select="'le4'"/>
                <xsl:map-entry key="'&#x961F;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x9620;'" select="'xin4	shen1'"/>
                <xsl:map-entry key="'&#x9621;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9622;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x9623;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9624;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9625;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x9626;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x9627;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x9628;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9629;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x962A;'" select="'ban3'"/>
                <xsl:map-entry key="'&#x962B;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x962C;'" select="'keng1'"/>
                <xsl:map-entry key="'&#x962D;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x962E;'" select="'ruan3'"/>
                <xsl:map-entry key="'&#x962F;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x9630;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x9631;'" select="'jing3'"/>
                <xsl:map-entry key="'&#x9632;'" select="'fang2'"/>
                <xsl:map-entry key="'&#x9633;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x9634;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x9635;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x9636;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x9637;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x9638;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9639;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x963A;'" select="'di3'"/>
                <xsl:map-entry key="'&#x963B;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x963C;'" select="'zuo4'"/>
                <xsl:map-entry key="'&#x963D;'" select="'dian4	yan2'"/>
                <xsl:map-entry key="'&#x963E;'" select="'ling3'"/>
                <xsl:map-entry key="'&#x963F;'" select="'a1	e1'"/>
                <xsl:map-entry key="'&#x9640;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9641;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9642;'" select="'po1	bei1	pi2'"/>
                <xsl:map-entry key="'&#x9643;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x9644;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x9645;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9646;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x9647;'" select="'long3'"/>
                <xsl:map-entry key="'&#x9648;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x9649;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x964A;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x964B;'" select="'lou4'"/>
                <xsl:map-entry key="'&#x964C;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x964D;'" select="'jiang4	xiang2'"/>
                <xsl:map-entry key="'&#x964E;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x964F;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x9650;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x9651;'" select="'er2'"/>
                <xsl:map-entry key="'&#x9652;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x9653;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x9654;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x9655;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x9656;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x9657;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x9658;'" select="'xing2'"/>
                <xsl:map-entry key="'&#x9659;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x965A;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x965B;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x965C;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x965D;'" select="'shan3	xia2'"/>
                <xsl:map-entry key="'&#x965E;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x965F;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9660;'" select="'pu1'"/>
                <xsl:map-entry key="'&#x9661;'" select="'dou3'"/>
                <xsl:map-entry key="'&#x9662;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x9663;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x9664;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x9665;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x9666;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9667;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x9668;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x9669;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x966A;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x966B;'" select="'pei2'"/>
                <xsl:map-entry key="'&#x966C;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x966D;'" select="'yi1	yi3'"/>
                <xsl:map-entry key="'&#x966E;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x966F;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x9670;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x9671;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x9672;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x9673;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x9674;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x9675;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9676;'" select="'tao2	yao2'"/>
                <xsl:map-entry key="'&#x9677;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x9678;'" select="'lu4	liu4'"/>
                <xsl:map-entry key="'&#x9679;'" select="'sheng'"/>
                <xsl:map-entry key="'&#x967A;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x967B;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x967C;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x967D;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x967E;'" select="'reng2'"/>
                <xsl:map-entry key="'&#x967F;'" select="'shan3'"/>
                <xsl:map-entry key="'&#x9680;'" select="'chong2'"/>
                <xsl:map-entry key="'&#x9681;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9682;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x9683;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9684;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9685;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9686;'" select="'long2	long1'"/>
                <xsl:map-entry key="'&#x9687;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x9688;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x9689;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x968A;'" select="'dui4'"/>
                <xsl:map-entry key="'&#x968B;'" select="'sui2'"/>
                <xsl:map-entry key="'&#x968C;'" select="'an3'"/>
                <xsl:map-entry key="'&#x968D;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x968E;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x968F;'" select="'sui2'"/>
                <xsl:map-entry key="'&#x9690;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x9691;'" select="'gai1'"/>
                <xsl:map-entry key="'&#x9692;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9693;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x9694;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x9695;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x9696;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x9697;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x9698;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x9699;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x969A;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x969B;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x969C;'" select="'zhang4'"/>
                <xsl:map-entry key="'&#x969D;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x969E;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x969F;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x96A0;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x96A1;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x96A2;'" select="'rao4'"/>
                <xsl:map-entry key="'&#x96A3;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x96A4;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x96A5;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x96A6;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x96A7;'" select="'sui4'"/>
                <xsl:map-entry key="'&#x96A8;'" select="'sui2'"/>
                <xsl:map-entry key="'&#x96A9;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x96AA;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x96AB;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x96AC;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x96AD;'" select="'er2'"/>
                <xsl:map-entry key="'&#x96AE;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x96AF;'" select="'dao3'"/>
                <xsl:map-entry key="'&#x96B0;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x96B1;'" select="'yin3	yin4'"/>
                <xsl:map-entry key="'&#x96B2;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x96B3;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x96B4;'" select="'long3'"/>
                <xsl:map-entry key="'&#x96B5;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x96B6;'" select="'li4'"/>
                <xsl:map-entry key="'&#x96B7;'" select="'li4'"/>
                <xsl:map-entry key="'&#x96B8;'" select="'li4'"/>
                <xsl:map-entry key="'&#x96B9;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x96BA;'" select="'he4'"/>
                <xsl:map-entry key="'&#x96BB;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x96BC;'" select="'zhun3	sun3'"/>
                <xsl:map-entry key="'&#x96BD;'" select="'jun4	juan4'"/>
                <xsl:map-entry key="'&#x96BE;'" select="'nan2	nan4'"/>
                <xsl:map-entry key="'&#x96BF;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x96C0;'" select="'que4	qiao1'"/>
                <xsl:map-entry key="'&#x96C1;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x96C2;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x96C3;'" select="'ya3'"/>
                <xsl:map-entry key="'&#x96C4;'" select="'xiong2'"/>
                <xsl:map-entry key="'&#x96C5;'" select="'ya3	ya1'"/>
                <xsl:map-entry key="'&#x96C6;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x96C7;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x96C8;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x96C9;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x96CA;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x96CB;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x96CC;'" select="'ci2	ci1'"/>
                <xsl:map-entry key="'&#x96CD;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x96CE;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x96CF;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x96D0;'" select="'hu1'"/>
                <xsl:map-entry key="'&#x96D1;'" select="'za2'"/>
                <xsl:map-entry key="'&#x96D2;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x96D3;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x96D4;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x96D5;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x96D6;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x96D7;'" select="'han4'"/>
                <xsl:map-entry key="'&#x96D8;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x96D9;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x96DA;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x96DB;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x96DC;'" select="'za2'"/>
                <xsl:map-entry key="'&#x96DD;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x96DE;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x96DF;'" select="'sui2'"/>
                <xsl:map-entry key="'&#x96E0;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x96E1;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x96E2;'" select="'li2'"/>
                <xsl:map-entry key="'&#x96E3;'" select="'nan2	nan4'"/>
                <xsl:map-entry key="'&#x96E4;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x96E5;'" select="'za2'"/>
                <xsl:map-entry key="'&#x96E6;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x96E7;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x96E8;'" select="'yu3	yu4'"/>
                <xsl:map-entry key="'&#x96E9;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x96EA;'" select="'xue3	xue4'"/>
                <xsl:map-entry key="'&#x96EB;'" select="'na3'"/>
                <xsl:map-entry key="'&#x96EC;'" select="'fou3'"/>
                <xsl:map-entry key="'&#x96ED;'" select="'se4'"/>
                <xsl:map-entry key="'&#x96EE;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x96EF;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x96F0;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x96F1;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x96F2;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x96F3;'" select="'li4'"/>
                <xsl:map-entry key="'&#x96F4;'" select="'li4'"/>
                <xsl:map-entry key="'&#x96F5;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x96F6;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x96F7;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x96F8;'" select="'an2'"/>
                <xsl:map-entry key="'&#x96F9;'" select="'bao2'"/>
                <xsl:map-entry key="'&#x96FA;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x96FB;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x96FC;'" select="'dang4'"/>
                <xsl:map-entry key="'&#x96FD;'" select="'hang2	yu2'"/>
                <xsl:map-entry key="'&#x96FE;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x96FF;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x9700;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x9701;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9702;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x9703;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x9704;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x9705;'" select="'zha2'"/>
                <xsl:map-entry key="'&#x9706;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x9707;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x9708;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x9709;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x970A;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x970B;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x970C;'" select="'chou1'"/>
                <xsl:map-entry key="'&#x970D;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x970E;'" select="'sha4'"/>
                <xsl:map-entry key="'&#x970F;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x9710;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x9711;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x9712;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9713;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x9714;'" select="'chou4'"/>
                <xsl:map-entry key="'&#x9715;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x9716;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x9717;'" select="''"/>
                <xsl:map-entry key="'&#x9718;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x9719;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x971A;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x971B;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x971C;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x971D;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x971E;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x971F;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x9720;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x9721;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x9722;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9723;'" select="'yun3'"/>
                <xsl:map-entry key="'&#x9724;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x9725;'" select="'meng4'"/>
                <xsl:map-entry key="'&#x9726;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x9727;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x9728;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x9729;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x972A;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x972B;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x972C;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x972D;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x972E;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x972F;'" select="'deng4'"/>
                <xsl:map-entry key="'&#x9730;'" select="'xian4	san3'"/>
                <xsl:map-entry key="'&#x9731;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9732;'" select="'lu4	lou4'"/>
                <xsl:map-entry key="'&#x9733;'" select="'long2	long1'"/>
                <xsl:map-entry key="'&#x9734;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x9735;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x9736;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x9737;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x9738;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x9739;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x973A;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x973B;'" select="''"/>
                <xsl:map-entry key="'&#x973C;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x973D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x973E;'" select="'mai2'"/>
                <xsl:map-entry key="'&#x973F;'" select="'meng4'"/>
                <xsl:map-entry key="'&#x9740;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x9741;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x9742;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9743;'" select="'huo4	sui3'"/>
                <xsl:map-entry key="'&#x9744;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x9745;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x9746;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x9747;'" select="'long2'"/>
                <xsl:map-entry key="'&#x9748;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9749;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x974A;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x974B;'" select="'li4'"/>
                <xsl:map-entry key="'&#x974C;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x974D;'" select="''"/>
                <xsl:map-entry key="'&#x974E;'" select="'he4'"/>
                <xsl:map-entry key="'&#x974F;'" select="'he4'"/>
                <xsl:map-entry key="'&#x9750;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x9751;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x9752;'" select="'qing1'"/>
                <xsl:map-entry key="'&#x9753;'" select="'liang4'"/>
                <xsl:map-entry key="'&#x9754;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9755;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x9756;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x9757;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x9758;'" select="'qing4'"/>
                <xsl:map-entry key="'&#x9759;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x975A;'" select="'jing4	liang4'"/>
                <xsl:map-entry key="'&#x975B;'" select="'dian4'"/>
                <xsl:map-entry key="'&#x975C;'" select="'jing4'"/>
                <xsl:map-entry key="'&#x975D;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x975E;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x975F;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x9760;'" select="'kao4'"/>
                <xsl:map-entry key="'&#x9761;'" select="'mi3	mi2'"/>
                <xsl:map-entry key="'&#x9762;'" select="'mian4'"/>
                <xsl:map-entry key="'&#x9763;'" select="'mian4'"/>
                <xsl:map-entry key="'&#x9764;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x9765;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9766;'" select="'tian3	mian3'"/>
                <xsl:map-entry key="'&#x9767;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x9768;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9769;'" select="'ge2	ji2	ji3'"/>
                <xsl:map-entry key="'&#x976A;'" select="'ding1'"/>
                <xsl:map-entry key="'&#x976B;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x976C;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x976D;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x976E;'" select="'di2'"/>
                <xsl:map-entry key="'&#x976F;'" select="'du4'"/>
                <xsl:map-entry key="'&#x9770;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x9771;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x9772;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x9773;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x9774;'" select="'xue1'"/>
                <xsl:map-entry key="'&#x9775;'" select="'niu3'"/>
                <xsl:map-entry key="'&#x9776;'" select="'ba3'"/>
                <xsl:map-entry key="'&#x9777;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x9778;'" select="'sa3'"/>
                <xsl:map-entry key="'&#x9779;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x977A;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x977B;'" select="'zu3'"/>
                <xsl:map-entry key="'&#x977C;'" select="'da2'"/>
                <xsl:map-entry key="'&#x977D;'" select="'ban4'"/>
                <xsl:map-entry key="'&#x977E;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x977F;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x9780;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x9781;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9782;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x9783;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x9784;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x9785;'" select="'yang3	yang1	yang4'"/>
                <xsl:map-entry key="'&#x9786;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9787;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x9788;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x9789;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x978A;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x978B;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x978C;'" select="'an1'"/>
                <xsl:map-entry key="'&#x978D;'" select="'an1'"/>
                <xsl:map-entry key="'&#x978E;'" select="'hen2'"/>
                <xsl:map-entry key="'&#x978F;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x9790;'" select="'gong3'"/>
                <xsl:map-entry key="'&#x9791;'" select="'da2'"/>
                <xsl:map-entry key="'&#x9792;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x9793;'" select="'ting1'"/>
                <xsl:map-entry key="'&#x9794;'" select="'wan3	man2'"/>
                <xsl:map-entry key="'&#x9795;'" select="'ying4'"/>
                <xsl:map-entry key="'&#x9796;'" select="'sui1'"/>
                <xsl:map-entry key="'&#x9797;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x9798;'" select="'qiao4	shao1'"/>
                <xsl:map-entry key="'&#x9799;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x979A;'" select="'kong4'"/>
                <xsl:map-entry key="'&#x979B;'" select="'beng3'"/>
                <xsl:map-entry key="'&#x979C;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x979D;'" select="'zhang3'"/>
                <xsl:map-entry key="'&#x979E;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x979F;'" select="'kuo4'"/>
                <xsl:map-entry key="'&#x97A0;'" select="'ju2	ju1'"/>
                <xsl:map-entry key="'&#x97A1;'" select="'la1'"/>
                <xsl:map-entry key="'&#x97A2;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x97A3;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x97A4;'" select="'bang1'"/>
                <xsl:map-entry key="'&#x97A5;'" select="'yi4	eng1'"/>
                <xsl:map-entry key="'&#x97A6;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x97A7;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x97A8;'" select="'he2'"/>
                <xsl:map-entry key="'&#x97A9;'" select="'xiao4'"/>
                <xsl:map-entry key="'&#x97AA;'" select="'mu4'"/>
                <xsl:map-entry key="'&#x97AB;'" select="'ju2	ju1'"/>
                <xsl:map-entry key="'&#x97AC;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x97AD;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x97AE;'" select="'di1'"/>
                <xsl:map-entry key="'&#x97AF;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x97B0;'" select="''"/>
                <xsl:map-entry key="'&#x97B1;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x97B2;'" select="'gou1'"/>
                <xsl:map-entry key="'&#x97B3;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x97B4;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x97B5;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x97B6;'" select="'pan2'"/>
                <xsl:map-entry key="'&#x97B7;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x97B8;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x97B9;'" select="'kuo4	kui1'"/>
                <xsl:map-entry key="'&#x97BA;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x97BB;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x97BC;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x97BD;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x97BE;'" select="'xue1'"/>
                <xsl:map-entry key="'&#x97BF;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x97C0;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x97C1;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x97C2;'" select="'chan4'"/>
                <xsl:map-entry key="'&#x97C3;'" select="'da2'"/>
                <xsl:map-entry key="'&#x97C4;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x97C5;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x97C6;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x97C7;'" select="'du2'"/>
                <xsl:map-entry key="'&#x97C8;'" select="'wa4'"/>
                <xsl:map-entry key="'&#x97C9;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x97CA;'" select="'lan2'"/>
                <xsl:map-entry key="'&#x97CB;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x97CC;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x97CD;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x97CE;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x97CF;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x97D0;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x97D1;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x97D2;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x97D3;'" select="'han2'"/>
                <xsl:map-entry key="'&#x97D4;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x97D5;'" select="''"/>
                <xsl:map-entry key="'&#x97D6;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x97D7;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x97D8;'" select="'she4'"/>
                <xsl:map-entry key="'&#x97D9;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x97DA;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x97DB;'" select="'bei4'"/>
                <xsl:map-entry key="'&#x97DC;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x97DD;'" select="'gou4'"/>
                <xsl:map-entry key="'&#x97DE;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x97DF;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x97E0;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x97E1;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x97E2;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x97E3;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x97E4;'" select="'wa4'"/>
                <xsl:map-entry key="'&#x97E5;'" select="'du2'"/>
                <xsl:map-entry key="'&#x97E6;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x97E7;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x97E8;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x97E9;'" select="'han2'"/>
                <xsl:map-entry key="'&#x97EA;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x97EB;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x97EC;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x97ED;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x97EE;'" select="'jiu3'"/>
                <xsl:map-entry key="'&#x97EF;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x97F0;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x97F1;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x97F2;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x97F3;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x97F4;'" select="'za2'"/>
                <xsl:map-entry key="'&#x97F5;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x97F6;'" select="'shao2'"/>
                <xsl:map-entry key="'&#x97F7;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x97F8;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x97F9;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x97FA;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x97FB;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x97FC;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x97FD;'" select="'yin1	an1'"/>
                <xsl:map-entry key="'&#x97FE;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x97FF;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x9800;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x9801;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9802;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x9803;'" select="'qing3'"/>
                <xsl:map-entry key="'&#x9804;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x9805;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x9806;'" select="'shun4'"/>
                <xsl:map-entry key="'&#x9807;'" select="'han1'"/>
                <xsl:map-entry key="'&#x9808;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x9809;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x980A;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x980B;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x980C;'" select="'song4'"/>
                <xsl:map-entry key="'&#x980D;'" select="'kui3'"/>
                <xsl:map-entry key="'&#x980E;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x980F;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x9810;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9811;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x9812;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x9813;'" select="'dun4	du2'"/>
                <xsl:map-entry key="'&#x9814;'" select="'di2'"/>
                <xsl:map-entry key="'&#x9815;'" select="'dan1'"/>
                <xsl:map-entry key="'&#x9816;'" select="'pan4'"/>
                <xsl:map-entry key="'&#x9817;'" select="'po3'"/>
                <xsl:map-entry key="'&#x9818;'" select="'ling3'"/>
                <xsl:map-entry key="'&#x9819;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x981A;'" select="'jing3	geng3'"/>
                <xsl:map-entry key="'&#x981B;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x981C;'" select="'he2	han4'"/>
                <xsl:map-entry key="'&#x981D;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x981E;'" select="'e4'"/>
                <xsl:map-entry key="'&#x981F;'" select="'e2'"/>
                <xsl:map-entry key="'&#x9820;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x9821;'" select="'jie2	xie2'"/>
                <xsl:map-entry key="'&#x9822;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x9823;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x9824;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9825;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9826;'" select="'ke1	ke2'"/>
                <xsl:map-entry key="'&#x9827;'" select="'dui1'"/>
                <xsl:map-entry key="'&#x9828;'" select="'pian1'"/>
                <xsl:map-entry key="'&#x9829;'" select="'ping1'"/>
                <xsl:map-entry key="'&#x982A;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x982B;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x982C;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x982D;'" select="'tou2'"/>
                <xsl:map-entry key="'&#x982E;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x982F;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x9830;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x9831;'" select="'le4'"/>
                <xsl:map-entry key="'&#x9832;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x9833;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x9834;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x9835;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x9836;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9837;'" select="'han4'"/>
                <xsl:map-entry key="'&#x9838;'" select="'jing3	geng3'"/>
                <xsl:map-entry key="'&#x9839;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x983A;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x983B;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x983C;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x983D;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x983E;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x983F;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9840;'" select="'chui2'"/>
                <xsl:map-entry key="'&#x9841;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x9842;'" select="'lai4'"/>
                <xsl:map-entry key="'&#x9843;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x9844;'" select="'han4'"/>
                <xsl:map-entry key="'&#x9845;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9846;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x9847;'" select="'cui4'"/>
                <xsl:map-entry key="'&#x9848;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x9849;'" select="'qin3'"/>
                <xsl:map-entry key="'&#x984A;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x984B;'" select="'sai1'"/>
                <xsl:map-entry key="'&#x984C;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x984D;'" select="'e2'"/>
                <xsl:map-entry key="'&#x984E;'" select="'e4'"/>
                <xsl:map-entry key="'&#x984F;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x9850;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x9851;'" select="'kan3'"/>
                <xsl:map-entry key="'&#x9852;'" select="'yong2'"/>
                <xsl:map-entry key="'&#x9853;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x9854;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x9855;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x9856;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x9857;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x9858;'" select="'yuan4'"/>
                <xsl:map-entry key="'&#x9859;'" select="'sang3'"/>
                <xsl:map-entry key="'&#x985A;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x985B;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x985C;'" select="'jiang3'"/>
                <xsl:map-entry key="'&#x985D;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x985E;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x985F;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x9860;'" select="'piao4'"/>
                <xsl:map-entry key="'&#x9861;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9862;'" select="'man2	man1'"/>
                <xsl:map-entry key="'&#x9863;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x9864;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x9865;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x9866;'" select="'qiao2'"/>
                <xsl:map-entry key="'&#x9867;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x9868;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x9869;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x986A;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x986B;'" select="'zhan4	chan4'"/>
                <xsl:map-entry key="'&#x986C;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x986D;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x986E;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x986F;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x9870;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x9871;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9872;'" select="'lan3'"/>
                <xsl:map-entry key="'&#x9873;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x9874;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x9875;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9876;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x9877;'" select="'qing3'"/>
                <xsl:map-entry key="'&#x9878;'" select="'han1'"/>
                <xsl:map-entry key="'&#x9879;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x987A;'" select="'shun4'"/>
                <xsl:map-entry key="'&#x987B;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x987C;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x987D;'" select="'wan2'"/>
                <xsl:map-entry key="'&#x987E;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x987F;'" select="'dun4	du2'"/>
                <xsl:map-entry key="'&#x9880;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9881;'" select="'ban1'"/>
                <xsl:map-entry key="'&#x9882;'" select="'song4'"/>
                <xsl:map-entry key="'&#x9883;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x9884;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9885;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9886;'" select="'ling3'"/>
                <xsl:map-entry key="'&#x9887;'" select="'po1'"/>
                <xsl:map-entry key="'&#x9888;'" select="'jing3	geng3'"/>
                <xsl:map-entry key="'&#x9889;'" select="'jie2	xie2'"/>
                <xsl:map-entry key="'&#x988A;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x988B;'" select="'ting3'"/>
                <xsl:map-entry key="'&#x988C;'" select="'he2'"/>
                <xsl:map-entry key="'&#x988D;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x988E;'" select="'jiong3'"/>
                <xsl:map-entry key="'&#x988F;'" select="'ke1	ke2'"/>
                <xsl:map-entry key="'&#x9890;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9891;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x9892;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x9893;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x9894;'" select="'han4'"/>
                <xsl:map-entry key="'&#x9895;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x9896;'" select="'ying3'"/>
                <xsl:map-entry key="'&#x9897;'" select="'ke1'"/>
                <xsl:map-entry key="'&#x9898;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9899;'" select="'yong2'"/>
                <xsl:map-entry key="'&#x989A;'" select="'e4'"/>
                <xsl:map-entry key="'&#x989B;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x989C;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x989D;'" select="'e2'"/>
                <xsl:map-entry key="'&#x989E;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x989F;'" select="'man1'"/>
                <xsl:map-entry key="'&#x98A0;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x98A1;'" select="'sang3'"/>
                <xsl:map-entry key="'&#x98A2;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x98A3;'" select="'lei4'"/>
                <xsl:map-entry key="'&#x98A4;'" select="'zhan4	chan4'"/>
                <xsl:map-entry key="'&#x98A5;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x98A6;'" select="'pin2'"/>
                <xsl:map-entry key="'&#x98A7;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x98A8;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x98A9;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98AA;'" select="''"/>
                <xsl:map-entry key="'&#x98AB;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x98AC;'" select="'xia1'"/>
                <xsl:map-entry key="'&#x98AD;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x98AE;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98AF;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x98B0;'" select="'fa1'"/>
                <xsl:map-entry key="'&#x98B1;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x98B2;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x98B3;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x98B4;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x98B5;'" select="'shao4'"/>
                <xsl:map-entry key="'&#x98B6;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x98B7;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98B8;'" select="'si1'"/>
                <xsl:map-entry key="'&#x98B9;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x98BA;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x98BB;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x98BC;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x98BD;'" select="'kai3'"/>
                <xsl:map-entry key="'&#x98BE;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x98BF;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x98C0;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x98C1;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x98C2;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x98C3;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x98C4;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x98C5;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x98C6;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98C7;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98C8;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98C9;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x98CA;'" select="''"/>
                <xsl:map-entry key="'&#x98CB;'" select="'se4'"/>
                <xsl:map-entry key="'&#x98CC;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x98CD;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98CE;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x98CF;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x98D0;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x98D1;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98D2;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x98D3;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x98D4;'" select="'si1'"/>
                <xsl:map-entry key="'&#x98D5;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x98D6;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x98D7;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x98D8;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x98D9;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98DA;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x98DB;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x98DC;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x98DD;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x98DE;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x98DF;'" select="'shi2	si4'"/>
                <xsl:map-entry key="'&#x98E0;'" select="'shi2	si4'"/>
                <xsl:map-entry key="'&#x98E1;'" select="'can1'"/>
                <xsl:map-entry key="'&#x98E2;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x98E3;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x98E4;'" select="'si4'"/>
                <xsl:map-entry key="'&#x98E5;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x98E6;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x98E7;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x98E8;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x98E9;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x98EA;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x98EB;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x98EC;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x98ED;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x98EE;'" select="'yin3	yin4'"/>
                <xsl:map-entry key="'&#x98EF;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x98F0;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x98F1;'" select="'sun1'"/>
                <xsl:map-entry key="'&#x98F2;'" select="'yin3	yin4'"/>
                <xsl:map-entry key="'&#x98F3;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x98F4;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x98F5;'" select="'zhai3'"/>
                <xsl:map-entry key="'&#x98F6;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x98F7;'" select="'jie3'"/>
                <xsl:map-entry key="'&#x98F8;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x98F9;'" select="'liu3'"/>
                <xsl:map-entry key="'&#x98FA;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x98FB;'" select="'tie4'"/>
                <xsl:map-entry key="'&#x98FC;'" select="'si4'"/>
                <xsl:map-entry key="'&#x98FD;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x98FE;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x98FF;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x9900;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x9901;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x9902;'" select="'tian3'"/>
                <xsl:map-entry key="'&#x9903;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x9904;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x9905;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x9906;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x9907;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x9908;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x9909;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x990A;'" select="'yang3	yang4'"/>
                <xsl:map-entry key="'&#x990B;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x990C;'" select="'er3'"/>
                <xsl:map-entry key="'&#x990D;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x990E;'" select="'le5'"/>
                <xsl:map-entry key="'&#x990F;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x9910;'" select="'can1'"/>
                <xsl:map-entry key="'&#x9911;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x9912;'" select="'nei3'"/>
                <xsl:map-entry key="'&#x9913;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9914;'" select="'bu1'"/>
                <xsl:map-entry key="'&#x9915;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x9916;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x9917;'" select="'su4'"/>
                <xsl:map-entry key="'&#x9918;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9919;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x991A;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x991B;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x991C;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x991D;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x991E;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x991F;'" select="'zhui4'"/>
                <xsl:map-entry key="'&#x9920;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x9921;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x9922;'" select="'bu4'"/>
                <xsl:map-entry key="'&#x9923;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9924;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x9925;'" select="'fei3'"/>
                <xsl:map-entry key="'&#x9926;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x9927;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x9928;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x9929;'" select="'e4'"/>
                <xsl:map-entry key="'&#x992A;'" select="'nuan3'"/>
                <xsl:map-entry key="'&#x992B;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x992C;'" select="'hu2	hu1	hu4'"/>
                <xsl:map-entry key="'&#x992D;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x992E;'" select="'tie4'"/>
                <xsl:map-entry key="'&#x992F;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x9930;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9931;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x9932;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9933;'" select="'xing2	tang2'"/>
                <xsl:map-entry key="'&#x9934;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x9935;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x9936;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x9937;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x9938;'" select="'song4'"/>
                <xsl:map-entry key="'&#x9939;'" select="'tang2	xing2'"/>
                <xsl:map-entry key="'&#x993A;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x993B;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x993C;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x993D;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x993E;'" select="'liu4	liu2'"/>
                <xsl:map-entry key="'&#x993F;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x9940;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x9941;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9942;'" select="'yun2'"/>
                <xsl:map-entry key="'&#x9943;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x9944;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x9945;'" select="'man2'"/>
                <xsl:map-entry key="'&#x9946;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9947;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9948;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x9949;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x994A;'" select="'san3'"/>
                <xsl:map-entry key="'&#x994B;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x994C;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x994D;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x994E;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x994F;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x9950;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9951;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9952;'" select="'rao2'"/>
                <xsl:map-entry key="'&#x9953;'" select="'cheng1'"/>
                <xsl:map-entry key="'&#x9954;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x9955;'" select="'tao1'"/>
                <xsl:map-entry key="'&#x9956;'" select="'hui4'"/>
                <xsl:map-entry key="'&#x9957;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x9958;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x9959;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x995A;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x995B;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x995C;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x995D;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x995E;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x995F;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x9960;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x9961;'" select="'zuan4	zan4'"/>
                <xsl:map-entry key="'&#x9962;'" select="'nang3	nang2'"/>
                <xsl:map-entry key="'&#x9963;'" select="'shi2	si4'"/>
                <xsl:map-entry key="'&#x9964;'" select="'ding4'"/>
                <xsl:map-entry key="'&#x9965;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9966;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x9967;'" select="'xing2	tang2'"/>
                <xsl:map-entry key="'&#x9968;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x9969;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x996A;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x996B;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x996C;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x996D;'" select="'fan4'"/>
                <xsl:map-entry key="'&#x996E;'" select="'yin3	yin4'"/>
                <xsl:map-entry key="'&#x996F;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x9970;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x9971;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x9972;'" select="'si4'"/>
                <xsl:map-entry key="'&#x9973;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x9974;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9975;'" select="'er3'"/>
                <xsl:map-entry key="'&#x9976;'" select="'rao2'"/>
                <xsl:map-entry key="'&#x9977;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x9978;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9979;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x997A;'" select="'jiao3	jia3'"/>
                <xsl:map-entry key="'&#x997B;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x997C;'" select="'bing3'"/>
                <xsl:map-entry key="'&#x997D;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x997E;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x997F;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9980;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9981;'" select="'nei3'"/>
                <xsl:map-entry key="'&#x9982;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x9983;'" select="'guo3'"/>
                <xsl:map-entry key="'&#x9984;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x9985;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x9986;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x9987;'" select="'cha1'"/>
                <xsl:map-entry key="'&#x9988;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x9989;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x998A;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x998B;'" select="'chan2'"/>
                <xsl:map-entry key="'&#x998C;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x998D;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x998E;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x998F;'" select="'liu4	liu2'"/>
                <xsl:map-entry key="'&#x9990;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x9991;'" select="'jin3'"/>
                <xsl:map-entry key="'&#x9992;'" select="'man2'"/>
                <xsl:map-entry key="'&#x9993;'" select="'san3'"/>
                <xsl:map-entry key="'&#x9994;'" select="'zhuan4'"/>
                <xsl:map-entry key="'&#x9995;'" select="'nang3	nang2'"/>
                <xsl:map-entry key="'&#x9996;'" select="'shou3'"/>
                <xsl:map-entry key="'&#x9997;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x9998;'" select="'guo2'"/>
                <xsl:map-entry key="'&#x9999;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x999A;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x999B;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x999C;'" select="'ni3'"/>
                <xsl:map-entry key="'&#x999D;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x999E;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x999F;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x99A0;'" select="'han1'"/>
                <xsl:map-entry key="'&#x99A1;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x99A2;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x99A3;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x99A4;'" select="'ai3'"/>
                <xsl:map-entry key="'&#x99A5;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x99A6;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x99A7;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x99A8;'" select="'xin1	xing1'"/>
                <xsl:map-entry key="'&#x99A9;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x99AA;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x99AB;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x99AC;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x99AD;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x99AE;'" select="'feng2	ping2'"/>
                <xsl:map-entry key="'&#x99AF;'" select="'han4'"/>
                <xsl:map-entry key="'&#x99B0;'" select="'di4'"/>
                <xsl:map-entry key="'&#x99B1;'" select="'tuo2	duo4'"/>
                <xsl:map-entry key="'&#x99B2;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x99B3;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x99B4;'" select="'xun2	xun4'"/>
                <xsl:map-entry key="'&#x99B5;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x99B6;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x99B7;'" select="'pei4'"/>
                <xsl:map-entry key="'&#x99B8;'" select="'xin4'"/>
                <xsl:map-entry key="'&#x99B9;'" select="'ri4'"/>
                <xsl:map-entry key="'&#x99BA;'" select="'sa4'"/>
                <xsl:map-entry key="'&#x99BB;'" select="'yin3'"/>
                <xsl:map-entry key="'&#x99BC;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x99BD;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x99BE;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x99BF;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x99C0;'" select="'you2'"/>
                <xsl:map-entry key="'&#x99C1;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x99C2;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x99C3;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x99C4;'" select="'tuo2	duo4'"/>
                <xsl:map-entry key="'&#x99C5;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x99C6;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x99C7;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x99C8;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x99C9;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x99CA;'" select="'bo3'"/>
                <xsl:map-entry key="'&#x99CB;'" select="'zhao1'"/>
                <xsl:map-entry key="'&#x99CC;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x99CD;'" select="'peng1'"/>
                <xsl:map-entry key="'&#x99CE;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x99CF;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x99D0;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x99D1;'" select="'nu2'"/>
                <xsl:map-entry key="'&#x99D2;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x99D3;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x99D4;'" select="'zang3'"/>
                <xsl:map-entry key="'&#x99D5;'" select="'jia4'"/>
                <xsl:map-entry key="'&#x99D6;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x99D7;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x99D8;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x99D9;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x99DA;'" select="'yang3'"/>
                <xsl:map-entry key="'&#x99DB;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x99DC;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x99DD;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x99DE;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x99DF;'" select="'si4'"/>
                <xsl:map-entry key="'&#x99E0;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x99E1;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x99E2;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x99E3;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x99E4;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x99E5;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x99E6;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x99E7;'" select="'dong4'"/>
                <xsl:map-entry key="'&#x99E8;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x99E9;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x99EA;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x99EB;'" select="'jiong1'"/>
                <xsl:map-entry key="'&#x99EC;'" select="'er3'"/>
                <xsl:map-entry key="'&#x99ED;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x99EE;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x99EF;'" select="'zhu'"/>
                <xsl:map-entry key="'&#x99F0;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x99F1;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x99F2;'" select="''"/>
                <xsl:map-entry key="'&#x99F3;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x99F4;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x99F5;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x99F6;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x99F7;'" select="'song3'"/>
                <xsl:map-entry key="'&#x99F8;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x99F9;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x99FA;'" select="'liang2'"/>
                <xsl:map-entry key="'&#x99FB;'" select="'han4'"/>
                <xsl:map-entry key="'&#x99FC;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x99FD;'" select="'xuan4'"/>
                <xsl:map-entry key="'&#x99FE;'" select="'tui4'"/>
                <xsl:map-entry key="'&#x99FF;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x9A00;'" select="'e2'"/>
                <xsl:map-entry key="'&#x9A01;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x9A02;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x9A03;'" select="'ai2'"/>
                <xsl:map-entry key="'&#x9A04;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9A05;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x9A06;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x9A07;'" select="'she4'"/>
                <xsl:map-entry key="'&#x9A08;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x9A09;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9A0A;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x9A0B;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x9A0C;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x9A0D;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x9A0E;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9A0F;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9A10;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9A11;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x9A12;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9A13;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9A14;'" select="'jie2	ge3'"/>
                <xsl:map-entry key="'&#x9A15;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x9A16;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x9A17;'" select="'pian4'"/>
                <xsl:map-entry key="'&#x9A18;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x9A19;'" select="'pian4'"/>
                <xsl:map-entry key="'&#x9A1A;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9A1B;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x9A1C;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9A1D;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9A1E;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x9A1F;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9A20;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9A21;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x9A22;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x9A23;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x9A24;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x9A25;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x9A26;'" select="'si1'"/>
                <xsl:map-entry key="'&#x9A27;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x9A28;'" select="'tuo2	tan2'"/>
                <xsl:map-entry key="'&#x9A29;'" select="'kui4'"/>
                <xsl:map-entry key="'&#x9A2A;'" select="'sou1'"/>
                <xsl:map-entry key="'&#x9A2B;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9A2C;'" select="'cheng2'"/>
                <xsl:map-entry key="'&#x9A2D;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9A2E;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9A2F;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x9A30;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x9A31;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x9A32;'" select="'cao3'"/>
                <xsl:map-entry key="'&#x9A33;'" select="'du2'"/>
                <xsl:map-entry key="'&#x9A34;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9A35;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x9A36;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x9A37;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9A38;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9A39;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9A3A;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9A3B;'" select="'shuang3'"/>
                <xsl:map-entry key="'&#x9A3C;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9A3D;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x9A3E;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x9A3F;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x9A40;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9A41;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x9A42;'" select="'can1'"/>
                <xsl:map-entry key="'&#x9A43;'" select="'piao4	biao1'"/>
                <xsl:map-entry key="'&#x9A44;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x9A45;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9A46;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9A47;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9A48;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9A49;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x9A4A;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x9A4B;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x9A4C;'" select="'su4'"/>
                <xsl:map-entry key="'&#x9A4D;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x9A4E;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x9A4F;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x9A50;'" select="'dun1'"/>
                <xsl:map-entry key="'&#x9A51;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9A52;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9A53;'" select="'zeng1'"/>
                <xsl:map-entry key="'&#x9A54;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x9A55;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9A56;'" select="'tie3'"/>
                <xsl:map-entry key="'&#x9A57;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9A58;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x9A59;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x9A5A;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9A5B;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9A5C;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9A5D;'" select="'tuo4'"/>
                <xsl:map-entry key="'&#x9A5E;'" select="'bin1'"/>
                <xsl:map-entry key="'&#x9A5F;'" select="'zou4	zhou4'"/>
                <xsl:map-entry key="'&#x9A60;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9A61;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x9A62;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x9A63;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x9A64;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x9A65;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9A66;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x9A67;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x9A68;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x9A69;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x9A6A;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9A6B;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x9A6C;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x9A6D;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9A6E;'" select="'tuo2	duo4'"/>
                <xsl:map-entry key="'&#x9A6F;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x9A70;'" select="'chi2'"/>
                <xsl:map-entry key="'&#x9A71;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9A72;'" select="'ri4'"/>
                <xsl:map-entry key="'&#x9A73;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x9A74;'" select="'lu:2'"/>
                <xsl:map-entry key="'&#x9A75;'" select="'zang3'"/>
                <xsl:map-entry key="'&#x9A76;'" select="'shi3'"/>
                <xsl:map-entry key="'&#x9A77;'" select="'si4'"/>
                <xsl:map-entry key="'&#x9A78;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x9A79;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x9A7A;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x9A7B;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x9A7C;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9A7D;'" select="'nu2'"/>
                <xsl:map-entry key="'&#x9A7E;'" select="'jia4'"/>
                <xsl:map-entry key="'&#x9A7F;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9A80;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x9A81;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x9A82;'" select="'ma4'"/>
                <xsl:map-entry key="'&#x9A83;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x9A84;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9A85;'" select="'hua2'"/>
                <xsl:map-entry key="'&#x9A86;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x9A87;'" select="'hai4'"/>
                <xsl:map-entry key="'&#x9A88;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x9A89;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x9A8A;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9A8B;'" select="'cheng3'"/>
                <xsl:map-entry key="'&#x9A8C;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9A8D;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x9A8E;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x9A8F;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x9A90;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9A91;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9A92;'" select="'ke4'"/>
                <xsl:map-entry key="'&#x9A93;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x9A94;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x9A95;'" select="'su4'"/>
                <xsl:map-entry key="'&#x9A96;'" select="'can1'"/>
                <xsl:map-entry key="'&#x9A97;'" select="'pian4'"/>
                <xsl:map-entry key="'&#x9A98;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9A99;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x9A9A;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9A9B;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x9A9C;'" select="'ao4'"/>
                <xsl:map-entry key="'&#x9A9D;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9A9E;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9A9F;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9AA0;'" select="'piao4	biao1'"/>
                <xsl:map-entry key="'&#x9AA1;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x9AA2;'" select="'cong1'"/>
                <xsl:map-entry key="'&#x9AA3;'" select="'chan3'"/>
                <xsl:map-entry key="'&#x9AA4;'" select="'zhou4'"/>
                <xsl:map-entry key="'&#x9AA5;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9AA6;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x9AA7;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x9AA8;'" select="'gu3	gu1'"/>
                <xsl:map-entry key="'&#x9AA9;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x9AAA;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x9AAB;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x9AAC;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9AAD;'" select="'gan4'"/>
                <xsl:map-entry key="'&#x9AAE;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9AAF;'" select="'ang1'"/>
                <xsl:map-entry key="'&#x9AB0;'" select="'tou2	shai3'"/>
                <xsl:map-entry key="'&#x9AB1;'" select="'xie4	jie4'"/>
                <xsl:map-entry key="'&#x9AB2;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x9AB3;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9AB4;'" select="'ci1'"/>
                <xsl:map-entry key="'&#x9AB5;'" select="'ti3'"/>
                <xsl:map-entry key="'&#x9AB6;'" select="'di3'"/>
                <xsl:map-entry key="'&#x9AB7;'" select="'ku1'"/>
                <xsl:map-entry key="'&#x9AB8;'" select="'hai2'"/>
                <xsl:map-entry key="'&#x9AB9;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x9ABA;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x9ABB;'" select="'kua4'"/>
                <xsl:map-entry key="'&#x9ABC;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x9ABD;'" select="'tui3'"/>
                <xsl:map-entry key="'&#x9ABE;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x9ABF;'" select="'pian2'"/>
                <xsl:map-entry key="'&#x9AC0;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9AC1;'" select="'ke1	ke4'"/>
                <xsl:map-entry key="'&#x9AC2;'" select="'ka4	qia4'"/>
                <xsl:map-entry key="'&#x9AC3;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9AC4;'" select="'sui3'"/>
                <xsl:map-entry key="'&#x9AC5;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x9AC6;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x9AC7;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x9AC8;'" select="'bang3'"/>
                <xsl:map-entry key="'&#x9AC9;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x9ACA;'" select="'cuo1'"/>
                <xsl:map-entry key="'&#x9ACB;'" select="'kuan1'"/>
                <xsl:map-entry key="'&#x9ACC;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x9ACD;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x9ACE;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x9ACF;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x9AD0;'" select="'nao2'"/>
                <xsl:map-entry key="'&#x9AD1;'" select="'du2'"/>
                <xsl:map-entry key="'&#x9AD2;'" select="'zang1'"/>
                <xsl:map-entry key="'&#x9AD3;'" select="'sui3'"/>
                <xsl:map-entry key="'&#x9AD4;'" select="'ti3'"/>
                <xsl:map-entry key="'&#x9AD5;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x9AD6;'" select="'kuan1'"/>
                <xsl:map-entry key="'&#x9AD7;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9AD8;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x9AD9;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x9ADA;'" select="'qiao4'"/>
                <xsl:map-entry key="'&#x9ADB;'" select="'kao1'"/>
                <xsl:map-entry key="'&#x9ADC;'" select="'qiao1'"/>
                <xsl:map-entry key="'&#x9ADD;'" select="'lao4'"/>
                <xsl:map-entry key="'&#x9ADE;'" select="'zao4'"/>
                <xsl:map-entry key="'&#x9ADF;'" select="'biao1	shan1'"/>
                <xsl:map-entry key="'&#x9AE0;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9AE1;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9AE2;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x9AE3;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x9AE4;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x9AE5;'" select="'ran2'"/>
                <xsl:map-entry key="'&#x9AE6;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x9AE7;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x9AE8;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9AE9;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x9AEA;'" select="'fa4'"/>
                <xsl:map-entry key="'&#x9AEB;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x9AEC;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x9AED;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9AEE;'" select="'fa4'"/>
                <xsl:map-entry key="'&#x9AEF;'" select="'ran2	ran3'"/>
                <xsl:map-entry key="'&#x9AF0;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x9AF1;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x9AF2;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x9AF3;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x9AF4;'" select="'fu2	fo2'"/>
                <xsl:map-entry key="'&#x9AF5;'" select="'er2'"/>
                <xsl:map-entry key="'&#x9AF6;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x9AF7;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9AF8;'" select="'gong'"/>
                <xsl:map-entry key="'&#x9AF9;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x9AFA;'" select="'gua4'"/>
                <xsl:map-entry key="'&#x9AFB;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9AFC;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x9AFD;'" select="'zhua1'"/>
                <xsl:map-entry key="'&#x9AFE;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x9AFF;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x9B00;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x9B01;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9B02;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x9B03;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x9B04;'" select="'ti4'"/>
                <xsl:map-entry key="'&#x9B05;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x9B06;'" select="'song1'"/>
                <xsl:map-entry key="'&#x9B07;'" select="'zheng1'"/>
                <xsl:map-entry key="'&#x9B08;'" select="'quan2	qian2'"/>
                <xsl:map-entry key="'&#x9B09;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x9B0A;'" select="'shun4'"/>
                <xsl:map-entry key="'&#x9B0B;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9B0C;'" select="'duo3'"/>
                <xsl:map-entry key="'&#x9B0D;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9B0E;'" select="'la4'"/>
                <xsl:map-entry key="'&#x9B0F;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x9B10;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9B11;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x9B12;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x9B13;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x9B14;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x9B15;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9B16;'" select="'san1'"/>
                <xsl:map-entry key="'&#x9B17;'" select="'man2'"/>
                <xsl:map-entry key="'&#x9B18;'" select="'man2'"/>
                <xsl:map-entry key="'&#x9B19;'" select="'seng1'"/>
                <xsl:map-entry key="'&#x9B1A;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x9B1B;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x9B1C;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9B1D;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9B1E;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x9B1F;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x9B20;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x9B21;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x9B22;'" select="'bin4'"/>
                <xsl:map-entry key="'&#x9B23;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x9B24;'" select="'rang2'"/>
                <xsl:map-entry key="'&#x9B25;'" select="'dou4	dou3'"/>
                <xsl:map-entry key="'&#x9B26;'" select="'dou4	dou3'"/>
                <xsl:map-entry key="'&#x9B27;'" select="'nao4'"/>
                <xsl:map-entry key="'&#x9B28;'" select="'hong4'"/>
                <xsl:map-entry key="'&#x9B29;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x9B2A;'" select="'dou4	dou3'"/>
                <xsl:map-entry key="'&#x9B2B;'" select="'kan4'"/>
                <xsl:map-entry key="'&#x9B2C;'" select="'dou4'"/>
                <xsl:map-entry key="'&#x9B2D;'" select="'dou4	dou3'"/>
                <xsl:map-entry key="'&#x9B2E;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x9B2F;'" select="'chang4'"/>
                <xsl:map-entry key="'&#x9B30;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9B31;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9B32;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9B33;'" select="'juan4'"/>
                <xsl:map-entry key="'&#x9B34;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x9B35;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9B36;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x9B37;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x9B38;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x9B39;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x9B3A;'" select="'shang1'"/>
                <xsl:map-entry key="'&#x9B3B;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9B3C;'" select="'gui3'"/>
                <xsl:map-entry key="'&#x9B3D;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x9B3E;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9B3F;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9B40;'" select="'jie4'"/>
                <xsl:map-entry key="'&#x9B41;'" select="'kui2'"/>
                <xsl:map-entry key="'&#x9B42;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x9B43;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x9B44;'" select="'po4	tuo4'"/>
                <xsl:map-entry key="'&#x9B45;'" select="'mei4'"/>
                <xsl:map-entry key="'&#x9B46;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x9B47;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9B48;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x9B49;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x9B4A;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9B4B;'" select="'tui2'"/>
                <xsl:map-entry key="'&#x9B4C;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x9B4D;'" select="'wang3'"/>
                <xsl:map-entry key="'&#x9B4E;'" select="'liang3'"/>
                <xsl:map-entry key="'&#x9B4F;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x9B50;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9B51;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x9B52;'" select="'piao1'"/>
                <xsl:map-entry key="'&#x9B53;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9B54;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x9B55;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x9B56;'" select="'xu1'"/>
                <xsl:map-entry key="'&#x9B57;'" select="'chou3'"/>
                <xsl:map-entry key="'&#x9B58;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9B59;'" select="'zhan3'"/>
                <xsl:map-entry key="'&#x9B5A;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9B5B;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x9B5C;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x9B5D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9B5E;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x9B5F;'" select="'hong1'"/>
                <xsl:map-entry key="'&#x9B60;'" select="'tuo1'"/>
                <xsl:map-entry key="'&#x9B61;'" select="'diao4'"/>
                <xsl:map-entry key="'&#x9B62;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x9B63;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9B64;'" select="'e2'"/>
                <xsl:map-entry key="'&#x9B65;'" select="'que4'"/>
                <xsl:map-entry key="'&#x9B66;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x9B67;'" select="'hang2'"/>
                <xsl:map-entry key="'&#x9B68;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x9B69;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9B6A;'" select="'gai4'"/>
                <xsl:map-entry key="'&#x9B6B;'" select="'shen3'"/>
                <xsl:map-entry key="'&#x9B6C;'" select="'fan3'"/>
                <xsl:map-entry key="'&#x9B6D;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x9B6E;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x9B6F;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x9B70;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x9B71;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9B72;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9B73;'" select="'za2'"/>
                <xsl:map-entry key="'&#x9B74;'" select="'fang2'"/>
                <xsl:map-entry key="'&#x9B75;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x9B76;'" select="'na4'"/>
                <xsl:map-entry key="'&#x9B77;'" select="'you2'"/>
                <xsl:map-entry key="'&#x9B78;'" select="''"/>
                <xsl:map-entry key="'&#x9B79;'" select="''"/>
                <xsl:map-entry key="'&#x9B7A;'" select="'he2	ge3'"/>
                <xsl:map-entry key="'&#x9B7B;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x9B7C;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9B7D;'" select="'han1'"/>
                <xsl:map-entry key="'&#x9B7E;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x9B7F;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9B80;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9B81;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x9B82;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x9B83;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x9B84;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9B85;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9B86;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9B87;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x9B88;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x9B89;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x9B8A;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x9B8B;'" select="'you2'"/>
                <xsl:map-entry key="'&#x9B8C;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x9B8D;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x9B8E;'" select="'nian2'"/>
                <xsl:map-entry key="'&#x9B8F;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x9B90;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x9B91;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x9B92;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x9B93;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x9B94;'" select="'ju4'"/>
                <xsl:map-entry key="'&#x9B95;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x9B96;'" select="''"/>
                <xsl:map-entry key="'&#x9B97;'" select="''"/>
                <xsl:map-entry key="'&#x9B98;'" select="''"/>
                <xsl:map-entry key="'&#x9B99;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x9B9A;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x9B9B;'" select="'shua1'"/>
                <xsl:map-entry key="'&#x9B9C;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x9B9D;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x9B9E;'" select="'er2'"/>
                <xsl:map-entry key="'&#x9B9F;'" select="'an4'"/>
                <xsl:map-entry key="'&#x9BA0;'" select="'wei2'"/>
                <xsl:map-entry key="'&#x9BA1;'" select="'tiao1'"/>
                <xsl:map-entry key="'&#x9BA2;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x9BA3;'" select="'yin4'"/>
                <xsl:map-entry key="'&#x9BA4;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x9BA5;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x9BA6;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x9BA7;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9BA8;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9BA9;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x9BAA;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x9BAB;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9BAC;'" select="'pu4'"/>
                <xsl:map-entry key="'&#x9BAD;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x9BAE;'" select="'xian1	xian3'"/>
                <xsl:map-entry key="'&#x9BAF;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x9BB0;'" select="'hui2'"/>
                <xsl:map-entry key="'&#x9BB1;'" select="''"/>
                <xsl:map-entry key="'&#x9BB2;'" select="''"/>
                <xsl:map-entry key="'&#x9BB3;'" select="'kao3'"/>
                <xsl:map-entry key="'&#x9BB4;'" select="''"/>
                <xsl:map-entry key="'&#x9BB5;'" select="'duo2'"/>
                <xsl:map-entry key="'&#x9BB6;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x9BB7;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9BB8;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x9BB9;'" select="'shao1'"/>
                <xsl:map-entry key="'&#x9BBA;'" select="'za3'"/>
                <xsl:map-entry key="'&#x9BBB;'" select="'suo1'"/>
                <xsl:map-entry key="'&#x9BBC;'" select="'qin1'"/>
                <xsl:map-entry key="'&#x9BBD;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9BBE;'" select="'nei3'"/>
                <xsl:map-entry key="'&#x9BBF;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x9BC0;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x9BC1;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x9BC2;'" select="'su'"/>
                <xsl:map-entry key="'&#x9BC3;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x9BC4;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x9BC5;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x9BC6;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x9BC7;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x9BC8;'" select="'chou2'"/>
                <xsl:map-entry key="'&#x9BC9;'" select="'li3'"/>
                <xsl:map-entry key="'&#x9BCA;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x9BCB;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x9BCC;'" select="'gao4'"/>
                <xsl:map-entry key="'&#x9BCD;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x9BCE;'" select="''"/>
                <xsl:map-entry key="'&#x9BCF;'" select="''"/>
                <xsl:map-entry key="'&#x9BD0;'" select="''"/>
                <xsl:map-entry key="'&#x9BD1;'" select="''"/>
                <xsl:map-entry key="'&#x9BD2;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x9BD3;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x9BD4;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9BD5;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9BD6;'" select="'qing1	zheng1'"/>
                <xsl:map-entry key="'&#x9BD7;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x9BD8;'" select="'nei3'"/>
                <xsl:map-entry key="'&#x9BD9;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x9BDA;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9BDB;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x9BDC;'" select="'qie4'"/>
                <xsl:map-entry key="'&#x9BDD;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x9BDE;'" select="'zhou3'"/>
                <xsl:map-entry key="'&#x9BDF;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x9BE0;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x9BE1;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x9BE2;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x9BE3;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9BE4;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9BE5;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9BE6;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x9BE7;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x9BE8;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9BE9;'" select="'lun2'"/>
                <xsl:map-entry key="'&#x9BEA;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9BEB;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x9BEC;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9BED;'" select="'meng3'"/>
                <xsl:map-entry key="'&#x9BEE;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x9BEF;'" select="'zhi2'"/>
                <xsl:map-entry key="'&#x9BF0;'" select="'nian2	nian3'"/>
                <xsl:map-entry key="'&#x9BF1;'" select="''"/>
                <xsl:map-entry key="'&#x9BF2;'" select="''"/>
                <xsl:map-entry key="'&#x9BF3;'" select="''"/>
                <xsl:map-entry key="'&#x9BF4;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x9BF5;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9BF6;'" select="'hun3'"/>
                <xsl:map-entry key="'&#x9BF7;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x9BF8;'" select="'hou2'"/>
                <xsl:map-entry key="'&#x9BF9;'" select="'xing1'"/>
                <xsl:map-entry key="'&#x9BFA;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x9BFB;'" select="'la4'"/>
                <xsl:map-entry key="'&#x9BFC;'" select="'zong1'"/>
                <xsl:map-entry key="'&#x9BFD;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9BFE;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x9BFF;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x9C00;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x9C01;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x9C02;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9C03;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x9C04;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x9C05;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9C06;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x9C07;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x9C08;'" select="'die2'"/>
                <xsl:map-entry key="'&#x9C09;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9C0A;'" select="'lian4'"/>
                <xsl:map-entry key="'&#x9C0B;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9C0C;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x9C0D;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x9C0E;'" select="'jian4'"/>
                <xsl:map-entry key="'&#x9C0F;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9C10;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9C11;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x9C12;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x9C13;'" select="'sai1	xi3'"/>
                <xsl:map-entry key="'&#x9C14;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x9C15;'" select="'ha2	xia1'"/>
                <xsl:map-entry key="'&#x9C16;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x9C17;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9C18;'" select="''"/>
                <xsl:map-entry key="'&#x9C19;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x9C1A;'" select="''"/>
                <xsl:map-entry key="'&#x9C1B;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x9C1C;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9C1D;'" select="'hao4'"/>
                <xsl:map-entry key="'&#x9C1E;'" select="'wu1	wu4'"/>
                <xsl:map-entry key="'&#x9C1F;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x9C20;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9C21;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9C22;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x9C23;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x9C24;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x9C25;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x9C26;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9C27;'" select="'teng2'"/>
                <xsl:map-entry key="'&#x9C28;'" select="'ta4	ta3'"/>
                <xsl:map-entry key="'&#x9C29;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x9C2A;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x9C2B;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x9C2C;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9C2D;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9C2E;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x9C2F;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x9C30;'" select="''"/>
                <xsl:map-entry key="'&#x9C31;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x9C32;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x9C33;'" select="'le4'"/>
                <xsl:map-entry key="'&#x9C34;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x9C35;'" select="'min3'"/>
                <xsl:map-entry key="'&#x9C36;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9C37;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x9C38;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9C39;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9C3A;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9C3B;'" select="'man2'"/>
                <xsl:map-entry key="'&#x9C3C;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x9C3D;'" select="'qiu2'"/>
                <xsl:map-entry key="'&#x9C3E;'" select="'biao4'"/>
                <xsl:map-entry key="'&#x9C3F;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9C40;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9C41;'" select="'zhu2'"/>
                <xsl:map-entry key="'&#x9C42;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x9C43;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x9C44;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x9C45;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x9C46;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x9C47;'" select="'kang1'"/>
                <xsl:map-entry key="'&#x9C48;'" select="'xue3'"/>
                <xsl:map-entry key="'&#x9C49;'" select="'bie1'"/>
                <xsl:map-entry key="'&#x9C4A;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x9C4B;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9C4C;'" select="'xiang4'"/>
                <xsl:map-entry key="'&#x9C4D;'" select="'bo1'"/>
                <xsl:map-entry key="'&#x9C4E;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9C4F;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x9C50;'" select="'su4'"/>
                <xsl:map-entry key="'&#x9C51;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9C52;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x9C53;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9C54;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9C55;'" select="'fan1'"/>
                <xsl:map-entry key="'&#x9C56;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x9C57;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x9C58;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x9C59;'" select="'miao2'"/>
                <xsl:map-entry key="'&#x9C5A;'" select="'xi3'"/>
                <xsl:map-entry key="'&#x9C5B;'" select="''"/>
                <xsl:map-entry key="'&#x9C5C;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x9C5D;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x9C5E;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x9C5F;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x9C60;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x9C61;'" select="'zei2'"/>
                <xsl:map-entry key="'&#x9C62;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9C63;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x9C64;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x9C65;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x9C66;'" select="'sheng2'"/>
                <xsl:map-entry key="'&#x9C67;'" select="'li3'"/>
                <xsl:map-entry key="'&#x9C68;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x9C69;'" select="''"/>
                <xsl:map-entry key="'&#x9C6A;'" select="''"/>
                <xsl:map-entry key="'&#x9C6B;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x9C6C;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x9C6D;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9C6E;'" select="'xu4'"/>
                <xsl:map-entry key="'&#x9C6F;'" select="'huo4'"/>
                <xsl:map-entry key="'&#x9C70;'" select="''"/>
                <xsl:map-entry key="'&#x9C71;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9C72;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x9C73;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9C74;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x9C75;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x9C76;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x9C77;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9C78;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9C79;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x9C7A;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9C7B;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x9C7C;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9C7D;'" select="'dao1'"/>
                <xsl:map-entry key="'&#x9C7E;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x9C7F;'" select="'you2'"/>
                <xsl:map-entry key="'&#x9C80;'" select="'tun2'"/>
                <xsl:map-entry key="'&#x9C81;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x9C82;'" select="'fang2'"/>
                <xsl:map-entry key="'&#x9C83;'" select="'ba1'"/>
                <xsl:map-entry key="'&#x9C84;'" select="'ke3'"/>
                <xsl:map-entry key="'&#x9C85;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x9C86;'" select="'ping2'"/>
                <xsl:map-entry key="'&#x9C87;'" select="'nian2'"/>
                <xsl:map-entry key="'&#x9C88;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9C89;'" select="'you2'"/>
                <xsl:map-entry key="'&#x9C8A;'" select="'zha4'"/>
                <xsl:map-entry key="'&#x9C8B;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x9C8C;'" select="'ba4'"/>
                <xsl:map-entry key="'&#x9C8D;'" select="'bao4'"/>
                <xsl:map-entry key="'&#x9C8E;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x9C8F;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x9C90;'" select="'tai2'"/>
                <xsl:map-entry key="'&#x9C91;'" select="'gui1'"/>
                <xsl:map-entry key="'&#x9C92;'" select="'jie2'"/>
                <xsl:map-entry key="'&#x9C93;'" select="'kao4'"/>
                <xsl:map-entry key="'&#x9C94;'" select="'wei3'"/>
                <xsl:map-entry key="'&#x9C95;'" select="'er2'"/>
                <xsl:map-entry key="'&#x9C96;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x9C97;'" select="'zei2'"/>
                <xsl:map-entry key="'&#x9C98;'" select="'hou4'"/>
                <xsl:map-entry key="'&#x9C99;'" select="'kuai4'"/>
                <xsl:map-entry key="'&#x9C9A;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9C9B;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9C9C;'" select="'xian1'"/>
                <xsl:map-entry key="'&#x9C9D;'" select="'zha3'"/>
                <xsl:map-entry key="'&#x9C9E;'" select="'xiang3'"/>
                <xsl:map-entry key="'&#x9C9F;'" select="'xun2'"/>
                <xsl:map-entry key="'&#x9CA0;'" select="'geng3'"/>
                <xsl:map-entry key="'&#x9CA1;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9CA2;'" select="'lian2'"/>
                <xsl:map-entry key="'&#x9CA3;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9CA4;'" select="'li3'"/>
                <xsl:map-entry key="'&#x9CA5;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x9CA6;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x9CA7;'" select="'gun3'"/>
                <xsl:map-entry key="'&#x9CA8;'" select="'sha1'"/>
                <xsl:map-entry key="'&#x9CA9;'" select="'huan4'"/>
                <xsl:map-entry key="'&#x9CAA;'" select="'jun1'"/>
                <xsl:map-entry key="'&#x9CAB;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9CAC;'" select="'yong3'"/>
                <xsl:map-entry key="'&#x9CAD;'" select="'qing1	zheng1'"/>
                <xsl:map-entry key="'&#x9CAE;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9CAF;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9CB0;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x9CB1;'" select="'fei1'"/>
                <xsl:map-entry key="'&#x9CB2;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9CB3;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x9CB4;'" select="'gu4'"/>
                <xsl:map-entry key="'&#x9CB5;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x9CB6;'" select="'nian2'"/>
                <xsl:map-entry key="'&#x9CB7;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x9CB8;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9CB9;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9CBA;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x9CBB;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9CBC;'" select="'fen4'"/>
                <xsl:map-entry key="'&#x9CBD;'" select="'die2'"/>
                <xsl:map-entry key="'&#x9CBE;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9CBF;'" select="'chang2'"/>
                <xsl:map-entry key="'&#x9CC0;'" select="'di4'"/>
                <xsl:map-entry key="'&#x9CC1;'" select="'wen1'"/>
                <xsl:map-entry key="'&#x9CC2;'" select="'wei1'"/>
                <xsl:map-entry key="'&#x9CC3;'" select="'sai1'"/>
                <xsl:map-entry key="'&#x9CC4;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9CC5;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x9CC6;'" select="'fu4'"/>
                <xsl:map-entry key="'&#x9CC7;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9CC8;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x9CC9;'" select="'jiang1'"/>
                <xsl:map-entry key="'&#x9CCA;'" select="'bian1'"/>
                <xsl:map-entry key="'&#x9CCB;'" select="'sao1'"/>
                <xsl:map-entry key="'&#x9CCC;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x9CCD;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9CCE;'" select="'ta3'"/>
                <xsl:map-entry key="'&#x9CCF;'" select="'guan1'"/>
                <xsl:map-entry key="'&#x9CD0;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x9CD1;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x9CD2;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9CD3;'" select="'le4'"/>
                <xsl:map-entry key="'&#x9CD4;'" select="'biao4'"/>
                <xsl:map-entry key="'&#x9CD5;'" select="'xue3'"/>
                <xsl:map-entry key="'&#x9CD6;'" select="'bie1'"/>
                <xsl:map-entry key="'&#x9CD7;'" select="'man2'"/>
                <xsl:map-entry key="'&#x9CD8;'" select="'min3'"/>
                <xsl:map-entry key="'&#x9CD9;'" select="'yong1'"/>
                <xsl:map-entry key="'&#x9CDA;'" select="'wei4'"/>
                <xsl:map-entry key="'&#x9CDB;'" select="'xi2'"/>
                <xsl:map-entry key="'&#x9CDC;'" select="'gui4'"/>
                <xsl:map-entry key="'&#x9CDD;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9CDE;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x9CDF;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x9CE0;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x9CE1;'" select="'gan3'"/>
                <xsl:map-entry key="'&#x9CE2;'" select="'li3'"/>
                <xsl:map-entry key="'&#x9CE3;'" select="'shan4'"/>
                <xsl:map-entry key="'&#x9CE4;'" select="'guan3'"/>
                <xsl:map-entry key="'&#x9CE5;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x9CE6;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x9CE7;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9CE8;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9CE9;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x9CEA;'" select="'bu3'"/>
                <xsl:map-entry key="'&#x9CEB;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9CEC;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9CED;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x9CEE;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9CEF;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x9CF0;'" select="''"/>
                <xsl:map-entry key="'&#x9CF1;'" select="'gan1'"/>
                <xsl:map-entry key="'&#x9CF2;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x9CF3;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x9CF4;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x9CF5;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x9CF6;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x9CF7;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x9CF8;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x9CF9;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9CFA;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x9CFB;'" select="'fen1'"/>
                <xsl:map-entry key="'&#x9CFC;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x9CFD;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9CFE;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x9CFF;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9D00;'" select="'fou3'"/>
                <xsl:map-entry key="'&#x9D01;'" select="'yiao1'"/>
                <xsl:map-entry key="'&#x9D02;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x9D03;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x9D04;'" select="'pi1'"/>
                <xsl:map-entry key="'&#x9D05;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x9D06;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x9D07;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x9D08;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9D09;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x9D0A;'" select="'zheng4'"/>
                <xsl:map-entry key="'&#x9D0B;'" select="'fang1'"/>
                <xsl:map-entry key="'&#x9D0C;'" select="'feng4'"/>
                <xsl:map-entry key="'&#x9D0D;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x9D0E;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x9D0F;'" select="'te4'"/>
                <xsl:map-entry key="'&#x9D10;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x9D11;'" select="'nu1'"/>
                <xsl:map-entry key="'&#x9D12;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9D13;'" select="'mie4'"/>
                <xsl:map-entry key="'&#x9D14;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9D15;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9D16;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x9D17;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9D18;'" select="'bian4'"/>
                <xsl:map-entry key="'&#x9D19;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9D1A;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x9D1B;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x9D1C;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9D1D;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x9D1E;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x9D1F;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x9D20;'" select="'dan4'"/>
                <xsl:map-entry key="'&#x9D21;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x9D22;'" select="'you4'"/>
                <xsl:map-entry key="'&#x9D23;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x9D24;'" select="'zhong1'"/>
                <xsl:map-entry key="'&#x9D25;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9D26;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x9D27;'" select="'rong4'"/>
                <xsl:map-entry key="'&#x9D28;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x9D29;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9D2A;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9D2B;'" select="''"/>
                <xsl:map-entry key="'&#x9D2C;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9D2D;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x9D2E;'" select="'wu1'"/>
                <xsl:map-entry key="'&#x9D2F;'" select="'er2'"/>
                <xsl:map-entry key="'&#x9D30;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x9D31;'" select="'ai4'"/>
                <xsl:map-entry key="'&#x9D32;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x9D33;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9D34;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x9D35;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9D36;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x9D37;'" select="'lie4'"/>
                <xsl:map-entry key="'&#x9D38;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x9D39;'" select="'ren2'"/>
                <xsl:map-entry key="'&#x9D3A;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9D3B;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x9D3C;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x9D3D;'" select="'ru2'"/>
                <xsl:map-entry key="'&#x9D3E;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x9D3F;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x9D40;'" select="'ren4'"/>
                <xsl:map-entry key="'&#x9D41;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9D42;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x9D43;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x9D44;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x9D45;'" select="'luo4'"/>
                <xsl:map-entry key="'&#x9D46;'" select="''"/>
                <xsl:map-entry key="'&#x9D47;'" select="''"/>
                <xsl:map-entry key="'&#x9D48;'" select="''"/>
                <xsl:map-entry key="'&#x9D49;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x9D4A;'" select="'jia2'"/>
                <xsl:map-entry key="'&#x9D4B;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9D4C;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9D4D;'" select="'huan1'"/>
                <xsl:map-entry key="'&#x9D4E;'" select="'tuo3'"/>
                <xsl:map-entry key="'&#x9D4F;'" select="'bu1'"/>
                <xsl:map-entry key="'&#x9D50;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x9D51;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x9D52;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9D53;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x9D54;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x9D55;'" select="'xun4'"/>
                <xsl:map-entry key="'&#x9D56;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9D57;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x9D58;'" select="'jun4'"/>
                <xsl:map-entry key="'&#x9D59;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x9D5A;'" select="'tu2	tu1'"/>
                <xsl:map-entry key="'&#x9D5B;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9D5C;'" select="'ti2	ti4'"/>
                <xsl:map-entry key="'&#x9D5D;'" select="'e2'"/>
                <xsl:map-entry key="'&#x9D5E;'" select="'e2'"/>
                <xsl:map-entry key="'&#x9D5F;'" select="'kuang2'"/>
                <xsl:map-entry key="'&#x9D60;'" select="'hu2	gu3'"/>
                <xsl:map-entry key="'&#x9D61;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x9D62;'" select="'shen1'"/>
                <xsl:map-entry key="'&#x9D63;'" select="'la4'"/>
                <xsl:map-entry key="'&#x9D64;'" select="''"/>
                <xsl:map-entry key="'&#x9D65;'" select="''"/>
                <xsl:map-entry key="'&#x9D66;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9D67;'" select="'bing4'"/>
                <xsl:map-entry key="'&#x9D68;'" select="'shu1'"/>
                <xsl:map-entry key="'&#x9D69;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9D6A;'" select="'an1'"/>
                <xsl:map-entry key="'&#x9D6B;'" select="'zhao4'"/>
                <xsl:map-entry key="'&#x9D6C;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x9D6D;'" select="'qin2'"/>
                <xsl:map-entry key="'&#x9D6E;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9D6F;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x9D70;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x9D71;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9D72;'" select="'que4'"/>
                <xsl:map-entry key="'&#x9D73;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9D74;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x9D75;'" select="'tu4'"/>
                <xsl:map-entry key="'&#x9D76;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x9D77;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x9D78;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9D79;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9D7A;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9D7B;'" select="'zhui1'"/>
                <xsl:map-entry key="'&#x9D7C;'" select="'kong1'"/>
                <xsl:map-entry key="'&#x9D7D;'" select="'duo4'"/>
                <xsl:map-entry key="'&#x9D7E;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9D7F;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x9D80;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9D81;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9D82;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x9D83;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9D84;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9D85;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9D86;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x9D87;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x9D88;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x9D89;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x9D8A;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x9D8B;'" select="'ju1'"/>
                <xsl:map-entry key="'&#x9D8C;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9D8D;'" select="''"/>
                <xsl:map-entry key="'&#x9D8E;'" select="''"/>
                <xsl:map-entry key="'&#x9D8F;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9D90;'" select="'shu4'"/>
                <xsl:map-entry key="'&#x9D91;'" select="''"/>
                <xsl:map-entry key="'&#x9D92;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x9D93;'" select="'miao2'"/>
                <xsl:map-entry key="'&#x9D94;'" select="'rou2'"/>
                <xsl:map-entry key="'&#x9D95;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9D96;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x9D97;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9D98;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9D99;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9D9A;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9D9B;'" select="'jie1'"/>
                <xsl:map-entry key="'&#x9D9C;'" select="'mao2'"/>
                <xsl:map-entry key="'&#x9D9D;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9D9E;'" select="'chun1'"/>
                <xsl:map-entry key="'&#x9D9F;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x9DA0;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9DA1;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9DA2;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x9DA3;'" select="'pian1	bin4'"/>
                <xsl:map-entry key="'&#x9DA4;'" select="'yun4'"/>
                <xsl:map-entry key="'&#x9DA5;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x9DA6;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9DA7;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9DA8;'" select="'dun4'"/>
                <xsl:map-entry key="'&#x9DA9;'" select="'mu4	wu4'"/>
                <xsl:map-entry key="'&#x9DAA;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x9DAB;'" select="''"/>
                <xsl:map-entry key="'&#x9DAC;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x9DAD;'" select="'fang3'"/>
                <xsl:map-entry key="'&#x9DAE;'" select="'ge4'"/>
                <xsl:map-entry key="'&#x9DAF;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9DB0;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x9DB1;'" select="'xuan1'"/>
                <xsl:map-entry key="'&#x9DB2;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x9DB3;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x9DB4;'" select="'he4'"/>
                <xsl:map-entry key="'&#x9DB5;'" select="'chu2'"/>
                <xsl:map-entry key="'&#x9DB6;'" select="'tang2'"/>
                <xsl:map-entry key="'&#x9DB7;'" select="'xia4'"/>
                <xsl:map-entry key="'&#x9DB8;'" select="'ruo4'"/>
                <xsl:map-entry key="'&#x9DB9;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9DBA;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x9DBB;'" select="'gu3	hu2'"/>
                <xsl:map-entry key="'&#x9DBC;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9DBD;'" select="'zhun3'"/>
                <xsl:map-entry key="'&#x9DBE;'" select="'han4'"/>
                <xsl:map-entry key="'&#x9DBF;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9DC0;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x9DC1;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9DC2;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x9DC3;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9DC4;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9DC5;'" select="'li4	piao3'"/>
                <xsl:map-entry key="'&#x9DC6;'" select="'tian2'"/>
                <xsl:map-entry key="'&#x9DC7;'" select="'kou4'"/>
                <xsl:map-entry key="'&#x9DC8;'" select="'si1'"/>
                <xsl:map-entry key="'&#x9DC9;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9DCA;'" select="'ni4'"/>
                <xsl:map-entry key="'&#x9DCB;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x9DCC;'" select="'ma3'"/>
                <xsl:map-entry key="'&#x9DCD;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9DCE;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9DCF;'" select="'zhen1'"/>
                <xsl:map-entry key="'&#x9DD0;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x9DD1;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9DD2;'" select="'zhuan1'"/>
                <xsl:map-entry key="'&#x9DD3;'" select="'zhe4'"/>
                <xsl:map-entry key="'&#x9DD4;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x9DD5;'" select="'yao3'"/>
                <xsl:map-entry key="'&#x9DD6;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x9DD7;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x9DD8;'" select="'chi4'"/>
                <xsl:map-entry key="'&#x9DD9;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9DDA;'" select="'liao2	liu4'"/>
                <xsl:map-entry key="'&#x9DDB;'" select="'rong2'"/>
                <xsl:map-entry key="'&#x9DDC;'" select="'lou2'"/>
                <xsl:map-entry key="'&#x9DDD;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9DDE;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x9DDF;'" select="'zhuo2'"/>
                <xsl:map-entry key="'&#x9DE0;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9DE1;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x9DE2;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x9DE3;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x9DE4;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x9DE5;'" select="'si1'"/>
                <xsl:map-entry key="'&#x9DE6;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9DE7;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9DE8;'" select="'hua1'"/>
                <xsl:map-entry key="'&#x9DE9;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9DEA;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9DEB;'" select="'su4'"/>
                <xsl:map-entry key="'&#x9DEC;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9DED;'" select="'fan2'"/>
                <xsl:map-entry key="'&#x9DEE;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9DEF;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x9DF0;'" select="'yan4'"/>
                <xsl:map-entry key="'&#x9DF1;'" select="'kao1'"/>
                <xsl:map-entry key="'&#x9DF2;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x9DF3;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x9DF4;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x9DF5;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x9DF6;'" select="'mai3'"/>
                <xsl:map-entry key="'&#x9DF7;'" select="'zun1'"/>
                <xsl:map-entry key="'&#x9DF8;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9DF9;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9DFA;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9DFB;'" select="'tuan2'"/>
                <xsl:map-entry key="'&#x9DFC;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x9DFD;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x9DFE;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9DFF;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x9E00;'" select="'shu2'"/>
                <xsl:map-entry key="'&#x9E01;'" select="'luo2'"/>
                <xsl:map-entry key="'&#x9E02;'" select="'qi1'"/>
                <xsl:map-entry key="'&#x9E03;'" select="'yi2'"/>
                <xsl:map-entry key="'&#x9E04;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9E05;'" select="'zhe2'"/>
                <xsl:map-entry key="'&#x9E06;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9E07;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x9E08;'" select="'ye4'"/>
                <xsl:map-entry key="'&#x9E09;'" select="'yang2'"/>
                <xsl:map-entry key="'&#x9E0A;'" select="'pi4'"/>
                <xsl:map-entry key="'&#x9E0B;'" select="'ning2'"/>
                <xsl:map-entry key="'&#x9E0C;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x9E0D;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x9E0E;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9E0F;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x9E10;'" select="'di2'"/>
                <xsl:map-entry key="'&#x9E11;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x9E12;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9E13;'" select="'lei3'"/>
                <xsl:map-entry key="'&#x9E14;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x9E15;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9E16;'" select="'he4'"/>
                <xsl:map-entry key="'&#x9E17;'" select="'long2'"/>
                <xsl:map-entry key="'&#x9E18;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x9E19;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x9E1A;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9E1B;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x9E1C;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x9E1D;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9E1E;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x9E1F;'" select="'niao3'"/>
                <xsl:map-entry key="'&#x9E20;'" select="'jiu1'"/>
                <xsl:map-entry key="'&#x9E21;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9E22;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x9E23;'" select="'ming2'"/>
                <xsl:map-entry key="'&#x9E24;'" select="'shi1'"/>
                <xsl:map-entry key="'&#x9E25;'" select="'ou1'"/>
                <xsl:map-entry key="'&#x9E26;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x9E27;'" select="'cang1'"/>
                <xsl:map-entry key="'&#x9E28;'" select="'bao3'"/>
                <xsl:map-entry key="'&#x9E29;'" select="'zhen4'"/>
                <xsl:map-entry key="'&#x9E2A;'" select="'gu1'"/>
                <xsl:map-entry key="'&#x9E2B;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x9E2C;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9E2D;'" select="'ya1'"/>
                <xsl:map-entry key="'&#x9E2E;'" select="'xiao1'"/>
                <xsl:map-entry key="'&#x9E2F;'" select="'yang1'"/>
                <xsl:map-entry key="'&#x9E30;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9E31;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x9E32;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x9E33;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x9E34;'" select="'xue2'"/>
                <xsl:map-entry key="'&#x9E35;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9E36;'" select="'si1'"/>
                <xsl:map-entry key="'&#x9E37;'" select="'zhi4'"/>
                <xsl:map-entry key="'&#x9E38;'" select="'er2'"/>
                <xsl:map-entry key="'&#x9E39;'" select="'gua1'"/>
                <xsl:map-entry key="'&#x9E3A;'" select="'xiu1'"/>
                <xsl:map-entry key="'&#x9E3B;'" select="'heng2'"/>
                <xsl:map-entry key="'&#x9E3C;'" select="'zhou1'"/>
                <xsl:map-entry key="'&#x9E3D;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x9E3E;'" select="'luan2'"/>
                <xsl:map-entry key="'&#x9E3F;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x9E40;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x9E41;'" select="'bo2'"/>
                <xsl:map-entry key="'&#x9E42;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9E43;'" select="'juan1'"/>
                <xsl:map-entry key="'&#x9E44;'" select="'hu2	gu3'"/>
                <xsl:map-entry key="'&#x9E45;'" select="'e2'"/>
                <xsl:map-entry key="'&#x9E46;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9E47;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x9E48;'" select="'ti2'"/>
                <xsl:map-entry key="'&#x9E49;'" select="'wu3'"/>
                <xsl:map-entry key="'&#x9E4A;'" select="'que4'"/>
                <xsl:map-entry key="'&#x9E4B;'" select="'miao2'"/>
                <xsl:map-entry key="'&#x9E4C;'" select="'an1'"/>
                <xsl:map-entry key="'&#x9E4D;'" select="'kun1'"/>
                <xsl:map-entry key="'&#x9E4E;'" select="'bei1'"/>
                <xsl:map-entry key="'&#x9E4F;'" select="'peng2'"/>
                <xsl:map-entry key="'&#x9E50;'" select="'qian1'"/>
                <xsl:map-entry key="'&#x9E51;'" select="'chun2'"/>
                <xsl:map-entry key="'&#x9E52;'" select="'geng1'"/>
                <xsl:map-entry key="'&#x9E53;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x9E54;'" select="'su4'"/>
                <xsl:map-entry key="'&#x9E55;'" select="'hu2'"/>
                <xsl:map-entry key="'&#x9E56;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9E57;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9E58;'" select="'gu3	hu2'"/>
                <xsl:map-entry key="'&#x9E59;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x9E5A;'" select="'ci2'"/>
                <xsl:map-entry key="'&#x9E5B;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x9E5C;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x9E5D;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9E5E;'" select="'yao4'"/>
                <xsl:map-entry key="'&#x9E5F;'" select="'weng1'"/>
                <xsl:map-entry key="'&#x9E60;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9E61;'" select="'ji2'"/>
                <xsl:map-entry key="'&#x9E62;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9E63;'" select="'jian1'"/>
                <xsl:map-entry key="'&#x9E64;'" select="'he4'"/>
                <xsl:map-entry key="'&#x9E65;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x9E66;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9E67;'" select="'zhe4'"/>
                <xsl:map-entry key="'&#x9E68;'" select="'liu4'"/>
                <xsl:map-entry key="'&#x9E69;'" select="'liao2'"/>
                <xsl:map-entry key="'&#x9E6A;'" select="'jiao1'"/>
                <xsl:map-entry key="'&#x9E6B;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x9E6C;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9E6D;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9E6E;'" select="'huan2'"/>
                <xsl:map-entry key="'&#x9E6F;'" select="'zhan1'"/>
                <xsl:map-entry key="'&#x9E70;'" select="'ying1'"/>
                <xsl:map-entry key="'&#x9E71;'" select="'hu4'"/>
                <xsl:map-entry key="'&#x9E72;'" select="'meng2'"/>
                <xsl:map-entry key="'&#x9E73;'" select="'guan4'"/>
                <xsl:map-entry key="'&#x9E74;'" select="'shuang1'"/>
                <xsl:map-entry key="'&#x9E75;'" select="'lu3'"/>
                <xsl:map-entry key="'&#x9E76;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x9E77;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9E78;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x9E79;'" select="'xian2'"/>
                <xsl:map-entry key="'&#x9E7A;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x9E7B;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x9E7C;'" select="'jian3'"/>
                <xsl:map-entry key="'&#x9E7D;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x9E7E;'" select="'cuo2'"/>
                <xsl:map-entry key="'&#x9E7F;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9E80;'" select="'you1'"/>
                <xsl:map-entry key="'&#x9E81;'" select="'cu1'"/>
                <xsl:map-entry key="'&#x9E82;'" select="'ji3'"/>
                <xsl:map-entry key="'&#x9E83;'" select="'biao1'"/>
                <xsl:map-entry key="'&#x9E84;'" select="'cu1'"/>
                <xsl:map-entry key="'&#x9E85;'" select="'pao2'"/>
                <xsl:map-entry key="'&#x9E86;'" select="'zhu4'"/>
                <xsl:map-entry key="'&#x9E87;'" select="'jun1	qun2'"/>
                <xsl:map-entry key="'&#x9E88;'" select="'zhu3'"/>
                <xsl:map-entry key="'&#x9E89;'" select="'jian1	qian1'"/>
                <xsl:map-entry key="'&#x9E8A;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x9E8B;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x9E8C;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x9E8D;'" select="'liu2'"/>
                <xsl:map-entry key="'&#x9E8E;'" select="'chen2'"/>
                <xsl:map-entry key="'&#x9E8F;'" select="'jun1	qun2'"/>
                <xsl:map-entry key="'&#x9E90;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x9E91;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x9E92;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9E93;'" select="'lu4'"/>
                <xsl:map-entry key="'&#x9E94;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x9E95;'" select="'jun1	qun2'"/>
                <xsl:map-entry key="'&#x9E96;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9E97;'" select="'li4	li2'"/>
                <xsl:map-entry key="'&#x9E98;'" select="'xiang1'"/>
                <xsl:map-entry key="'&#x9E99;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x9E9A;'" select="'jia1'"/>
                <xsl:map-entry key="'&#x9E9B;'" select="'mi2'"/>
                <xsl:map-entry key="'&#x9E9C;'" select="'li4'"/>
                <xsl:map-entry key="'&#x9E9D;'" select="'she4'"/>
                <xsl:map-entry key="'&#x9E9E;'" select="'zhang1'"/>
                <xsl:map-entry key="'&#x9E9F;'" select="'lin2'"/>
                <xsl:map-entry key="'&#x9EA0;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9EA1;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9EA2;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9EA3;'" select="'yan2'"/>
                <xsl:map-entry key="'&#x9EA4;'" select="'cu1'"/>
                <xsl:map-entry key="'&#x9EA5;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x9EA6;'" select="'mai4'"/>
                <xsl:map-entry key="'&#x9EA7;'" select="'ge1'"/>
                <xsl:map-entry key="'&#x9EA8;'" select="'chao3'"/>
                <xsl:map-entry key="'&#x9EA9;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x9EAA;'" select="'mian4'"/>
                <xsl:map-entry key="'&#x9EAB;'" select="'mian3'"/>
                <xsl:map-entry key="'&#x9EAC;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x9EAD;'" select="'pao4'"/>
                <xsl:map-entry key="'&#x9EAE;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x9EAF;'" select="'qu1	qu3'"/>
                <xsl:map-entry key="'&#x9EB0;'" select="'mou2'"/>
                <xsl:map-entry key="'&#x9EB1;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x9EB2;'" select="'xian4'"/>
                <xsl:map-entry key="'&#x9EB3;'" select="'lai2'"/>
                <xsl:map-entry key="'&#x9EB4;'" select="'qu2	qu1'"/>
                <xsl:map-entry key="'&#x9EB5;'" select="'mian4'"/>
                <xsl:map-entry key="'&#x9EB6;'" select="'chi1	li2'"/>
                <xsl:map-entry key="'&#x9EB7;'" select="'feng1'"/>
                <xsl:map-entry key="'&#x9EB8;'" select="'fu1'"/>
                <xsl:map-entry key="'&#x9EB9;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9EBA;'" select="'mian4'"/>
                <xsl:map-entry key="'&#x9EBB;'" select="'ma2	ma1'"/>
                <xsl:map-entry key="'&#x9EBC;'" select="'ma2'"/>
                <xsl:map-entry key="'&#x9EBD;'" select="'mo2'"/>
                <xsl:map-entry key="'&#x9EBE;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x9EBF;'" select="''"/>
                <xsl:map-entry key="'&#x9EC0;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x9EC1;'" select="'nen1'"/>
                <xsl:map-entry key="'&#x9EC2;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x9EC3;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9EC4;'" select="'huang2'"/>
                <xsl:map-entry key="'&#x9EC5;'" select="'jin1'"/>
                <xsl:map-entry key="'&#x9EC6;'" select="'guang1'"/>
                <xsl:map-entry key="'&#x9EC7;'" select="'tian1'"/>
                <xsl:map-entry key="'&#x9EC8;'" select="'tou3'"/>
                <xsl:map-entry key="'&#x9EC9;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x9ECA;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x9ECB;'" select="'kuang4'"/>
                <xsl:map-entry key="'&#x9ECC;'" select="'hong2'"/>
                <xsl:map-entry key="'&#x9ECD;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x9ECE;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9ECF;'" select="'nian2'"/>
                <xsl:map-entry key="'&#x9ED0;'" select="'chi1	li2'"/>
                <xsl:map-entry key="'&#x9ED1;'" select="'hei1	hei3'"/>
                <xsl:map-entry key="'&#x9ED2;'" select="'hei1'"/>
                <xsl:map-entry key="'&#x9ED3;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9ED4;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9ED5;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x9ED6;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x9ED7;'" select="'tuan3'"/>
                <xsl:map-entry key="'&#x9ED8;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9ED9;'" select="'mo4'"/>
                <xsl:map-entry key="'&#x9EDA;'" select="'qian2'"/>
                <xsl:map-entry key="'&#x9EDB;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x9EDC;'" select="'chu4'"/>
                <xsl:map-entry key="'&#x9EDD;'" select="'you3'"/>
                <xsl:map-entry key="'&#x9EDE;'" select="'dian3'"/>
                <xsl:map-entry key="'&#x9EDF;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x9EE0;'" select="'xia2'"/>
                <xsl:map-entry key="'&#x9EE1;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9EE2;'" select="'qu1'"/>
                <xsl:map-entry key="'&#x9EE3;'" select="'mei3'"/>
                <xsl:map-entry key="'&#x9EE4;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9EE5;'" select="'qing2'"/>
                <xsl:map-entry key="'&#x9EE6;'" select="'yu4'"/>
                <xsl:map-entry key="'&#x9EE7;'" select="'li2'"/>
                <xsl:map-entry key="'&#x9EE8;'" select="'dang3'"/>
                <xsl:map-entry key="'&#x9EE9;'" select="'du2'"/>
                <xsl:map-entry key="'&#x9EEA;'" select="'can3'"/>
                <xsl:map-entry key="'&#x9EEB;'" select="'yin1'"/>
                <xsl:map-entry key="'&#x9EEC;'" select="'an4'"/>
                <xsl:map-entry key="'&#x9EED;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9EEE;'" select="'tan2'"/>
                <xsl:map-entry key="'&#x9EEF;'" select="'an4'"/>
                <xsl:map-entry key="'&#x9EF0;'" select="'zhen3'"/>
                <xsl:map-entry key="'&#x9EF1;'" select="'dai4'"/>
                <xsl:map-entry key="'&#x9EF2;'" select="'can3'"/>
                <xsl:map-entry key="'&#x9EF3;'" select="'yi1'"/>
                <xsl:map-entry key="'&#x9EF4;'" select="'mei2'"/>
                <xsl:map-entry key="'&#x9EF5;'" select="'dan3'"/>
                <xsl:map-entry key="'&#x9EF6;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9EF7;'" select="'du2'"/>
                <xsl:map-entry key="'&#x9EF8;'" select="'lu2'"/>
                <xsl:map-entry key="'&#x9EF9;'" select="'zhi3'"/>
                <xsl:map-entry key="'&#x9EFA;'" select="'fen3'"/>
                <xsl:map-entry key="'&#x9EFB;'" select="'fu2'"/>
                <xsl:map-entry key="'&#x9EFC;'" select="'fu3'"/>
                <xsl:map-entry key="'&#x9EFD;'" select="'min3	mian3'"/>
                <xsl:map-entry key="'&#x9EFE;'" select="'min3	mian3'"/>
                <xsl:map-entry key="'&#x9EFF;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x9F00;'" select="'cu4'"/>
                <xsl:map-entry key="'&#x9F01;'" select="'qu4'"/>
                <xsl:map-entry key="'&#x9F02;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x9F03;'" select="'wa1'"/>
                <xsl:map-entry key="'&#x9F04;'" select="'zhu1'"/>
                <xsl:map-entry key="'&#x9F05;'" select="'zhi1'"/>
                <xsl:map-entry key="'&#x9F06;'" select="'mang2'"/>
                <xsl:map-entry key="'&#x9F07;'" select="'ao2'"/>
                <xsl:map-entry key="'&#x9F08;'" select="'bie1'"/>
                <xsl:map-entry key="'&#x9F09;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9F0A;'" select="'bi4'"/>
                <xsl:map-entry key="'&#x9F0B;'" select="'yuan2'"/>
                <xsl:map-entry key="'&#x9F0C;'" select="'chao2'"/>
                <xsl:map-entry key="'&#x9F0D;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9F0E;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x9F0F;'" select="'mi4'"/>
                <xsl:map-entry key="'&#x9F10;'" select="'nai4'"/>
                <xsl:map-entry key="'&#x9F11;'" select="'ding3'"/>
                <xsl:map-entry key="'&#x9F12;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9F13;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x9F14;'" select="'gu3'"/>
                <xsl:map-entry key="'&#x9F15;'" select="'dong1'"/>
                <xsl:map-entry key="'&#x9F16;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x9F17;'" select="'tao2'"/>
                <xsl:map-entry key="'&#x9F18;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x9F19;'" select="'pi2'"/>
                <xsl:map-entry key="'&#x9F1A;'" select="'chang1'"/>
                <xsl:map-entry key="'&#x9F1B;'" select="'gao1'"/>
                <xsl:map-entry key="'&#x9F1C;'" select="'qi4'"/>
                <xsl:map-entry key="'&#x9F1D;'" select="'yuan1'"/>
                <xsl:map-entry key="'&#x9F1E;'" select="'tang1'"/>
                <xsl:map-entry key="'&#x9F1F;'" select="'teng1'"/>
                <xsl:map-entry key="'&#x9F20;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x9F21;'" select="'shu3'"/>
                <xsl:map-entry key="'&#x9F22;'" select="'fen2'"/>
                <xsl:map-entry key="'&#x9F23;'" select="'fei4'"/>
                <xsl:map-entry key="'&#x9F24;'" select="'wen2'"/>
                <xsl:map-entry key="'&#x9F25;'" select="'ba2'"/>
                <xsl:map-entry key="'&#x9F26;'" select="'diao1'"/>
                <xsl:map-entry key="'&#x9F27;'" select="'tuo2'"/>
                <xsl:map-entry key="'&#x9F28;'" select="'tong2'"/>
                <xsl:map-entry key="'&#x9F29;'" select="'qu2'"/>
                <xsl:map-entry key="'&#x9F2A;'" select="'sheng1'"/>
                <xsl:map-entry key="'&#x9F2B;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x9F2C;'" select="'you4'"/>
                <xsl:map-entry key="'&#x9F2D;'" select="'shi2'"/>
                <xsl:map-entry key="'&#x9F2E;'" select="'ting2'"/>
                <xsl:map-entry key="'&#x9F2F;'" select="'wu2'"/>
                <xsl:map-entry key="'&#x9F30;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x9F31;'" select="'jing1'"/>
                <xsl:map-entry key="'&#x9F32;'" select="'hun2'"/>
                <xsl:map-entry key="'&#x9F33;'" select="'ju2'"/>
                <xsl:map-entry key="'&#x9F34;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9F35;'" select="'tu2'"/>
                <xsl:map-entry key="'&#x9F36;'" select="'si1'"/>
                <xsl:map-entry key="'&#x9F37;'" select="'xi1'"/>
                <xsl:map-entry key="'&#x9F38;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x9F39;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9F3A;'" select="'lei2'"/>
                <xsl:map-entry key="'&#x9F3B;'" select="'bi2'"/>
                <xsl:map-entry key="'&#x9F3C;'" select="'yao2'"/>
                <xsl:map-entry key="'&#x9F3D;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9F3E;'" select="'han1'"/>
                <xsl:map-entry key="'&#x9F3F;'" select="'hui1'"/>
                <xsl:map-entry key="'&#x9F40;'" select="'wu4'"/>
                <xsl:map-entry key="'&#x9F41;'" select="'hou1'"/>
                <xsl:map-entry key="'&#x9F42;'" select="'xi4'"/>
                <xsl:map-entry key="'&#x9F43;'" select="'ge2'"/>
                <xsl:map-entry key="'&#x9F44;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x9F45;'" select="'xiu4'"/>
                <xsl:map-entry key="'&#x9F46;'" select="'weng4'"/>
                <xsl:map-entry key="'&#x9F47;'" select="'zha1'"/>
                <xsl:map-entry key="'&#x9F48;'" select="'nong2'"/>
                <xsl:map-entry key="'&#x9F49;'" select="'nang4'"/>
                <xsl:map-entry key="'&#x9F4A;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9F4B;'" select="'zhai1'"/>
                <xsl:map-entry key="'&#x9F4C;'" select="'ji4'"/>
                <xsl:map-entry key="'&#x9F4D;'" select="'zi1	ji1'"/>
                <xsl:map-entry key="'&#x9F4E;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9F4F;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9F50;'" select="'qi2'"/>
                <xsl:map-entry key="'&#x9F51;'" select="'ji1'"/>
                <xsl:map-entry key="'&#x9F52;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x9F53;'" select="'chen3'"/>
                <xsl:map-entry key="'&#x9F54;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x9F55;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9F56;'" select="'ya2'"/>
                <xsl:map-entry key="'&#x9F57;'" select="'ken3'"/>
                <xsl:map-entry key="'&#x9F58;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x9F59;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x9F5A;'" select="'ze2'"/>
                <xsl:map-entry key="'&#x9F5B;'" select="'shi4'"/>
                <xsl:map-entry key="'&#x9F5C;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9F5D;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x9F5E;'" select="'nian4'"/>
                <xsl:map-entry key="'&#x9F5F;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x9F60;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x9F61;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9F62;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9F63;'" select="'chu1'"/>
                <xsl:map-entry key="'&#x9F64;'" select="'quan2'"/>
                <xsl:map-entry key="'&#x9F65;'" select="'xie4'"/>
                <xsl:map-entry key="'&#x9F66;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x9F67;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x9F68;'" select="'jiu4'"/>
                <xsl:map-entry key="'&#x9F69;'" select="'nie4'"/>
                <xsl:map-entry key="'&#x9F6A;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x9F6B;'" select="'kun3'"/>
                <xsl:map-entry key="'&#x9F6C;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x9F6D;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x9F6E;'" select="'yi3'"/>
                <xsl:map-entry key="'&#x9F6F;'" select="'ni2'"/>
                <xsl:map-entry key="'&#x9F70;'" select="'cuo4'"/>
                <xsl:map-entry key="'&#x9F71;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x9F72;'" select="'qu3'"/>
                <xsl:map-entry key="'&#x9F73;'" select="'nian3'"/>
                <xsl:map-entry key="'&#x9F74;'" select="'xian3'"/>
                <xsl:map-entry key="'&#x9F75;'" select="'yu2'"/>
                <xsl:map-entry key="'&#x9F76;'" select="'e4'"/>
                <xsl:map-entry key="'&#x9F77;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x9F78;'" select="'yi4'"/>
                <xsl:map-entry key="'&#x9F79;'" select="'chi1'"/>
                <xsl:map-entry key="'&#x9F7A;'" select="'zou1'"/>
                <xsl:map-entry key="'&#x9F7B;'" select="'dian1'"/>
                <xsl:map-entry key="'&#x9F7C;'" select="'chu3'"/>
                <xsl:map-entry key="'&#x9F7D;'" select="'jin4'"/>
                <xsl:map-entry key="'&#x9F7E;'" select="'ya4'"/>
                <xsl:map-entry key="'&#x9F7F;'" select="'chi3'"/>
                <xsl:map-entry key="'&#x9F80;'" select="'chen4'"/>
                <xsl:map-entry key="'&#x9F81;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9F82;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x9F83;'" select="'ju3'"/>
                <xsl:map-entry key="'&#x9F84;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9F85;'" select="'bao1'"/>
                <xsl:map-entry key="'&#x9F86;'" select="'tiao2'"/>
                <xsl:map-entry key="'&#x9F87;'" select="'zi1'"/>
                <xsl:map-entry key="'&#x9F88;'" select="'yin2'"/>
                <xsl:map-entry key="'&#x9F89;'" select="'yu3'"/>
                <xsl:map-entry key="'&#x9F8A;'" select="'chuo4'"/>
                <xsl:map-entry key="'&#x9F8B;'" select="'qu3'"/>
                <xsl:map-entry key="'&#x9F8C;'" select="'wo4'"/>
                <xsl:map-entry key="'&#x9F8D;'" select="'long2'"/>
                <xsl:map-entry key="'&#x9F8E;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x9F8F;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x9F90;'" select="'pang2'"/>
                <xsl:map-entry key="'&#x9F91;'" select="'yan3'"/>
                <xsl:map-entry key="'&#x9F92;'" select="'long2'"/>
                <xsl:map-entry key="'&#x9F93;'" select="'long2'"/>
                <xsl:map-entry key="'&#x9F94;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x9F95;'" select="'kan1'"/>
                <xsl:map-entry key="'&#x9F96;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x9F97;'" select="'ling2'"/>
                <xsl:map-entry key="'&#x9F98;'" select="'ta4'"/>
                <xsl:map-entry key="'&#x9F99;'" select="'long2'"/>
                <xsl:map-entry key="'&#x9F9A;'" select="'gong1'"/>
                <xsl:map-entry key="'&#x9F9B;'" select="'kan1'"/>
                <xsl:map-entry key="'&#x9F9C;'" select="'gui1	jun1	qiu1'"/>
                <xsl:map-entry key="'&#x9F9D;'" select="'qiu1'"/>
                <xsl:map-entry key="'&#x9F9E;'" select="'bie1'"/>
                <xsl:map-entry key="'&#x9F9F;'" select="'gui1	jun1	qiu1'"/>
                <xsl:map-entry key="'&#x9FA0;'" select="'yue4'"/>
                <xsl:map-entry key="'&#x9FA1;'" select="'chui1'"/>
                <xsl:map-entry key="'&#x9FA2;'" select="'he2'"/>
                <xsl:map-entry key="'&#x9FA3;'" select="'jue2'"/>
                <xsl:map-entry key="'&#x9FA4;'" select="'xie2'"/>
                <xsl:map-entry key="'&#x9FA5;'" select="'yue4'"/>
            </xsl:map>
        </xsl:variable>
        <xsl:variable name="finalContent">
            <!-- Split the text in consecutive characters -->
            <xsl:analyze-string select="$text" regex=".">
                <xsl:matching-substring>
                    <xsl:variable name="pinyinEquivalent" select="$map(.)"/>
                    <xsl:choose>
                        <xsl:when test="not($pinyinEquivalent)">
                            <xsl:value-of select="."/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$pinyinEquivalent"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:matching-substring>
                <xsl:fallback>
                    <xsl:value-of select="."/>
                </xsl:fallback>
            </xsl:analyze-string>
        </xsl:variable>
        <xsl:value-of select="string-join($finalContent)"/>
    </xsl:function>
</xsl:stylesheet>
